@@ -2,38 +2,27 @@
 // Name:        const_wxusedef.h
 // Purpose:     wxUSE preprocessor symbols
 // Author:      Tim Stahlhut
-// RCS-ID:      $Id: const_wxusedef.h 64943 2010-07-13 13:29:58Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 
 /**
 
-@page page_wxusedef wxUSE preprocessor symbols defined by wxWidgets
+@page page_wxusedef wxUSE Preprocessor Symbols
+
+@tableofcontents
 
 This section documents the wxUSE preprocessor symbols used in the wxWidgets
 source, grouped by category (and sorted by alphabetical order inside each
 category). These symbols are always defined and whether the given feature is
 active or not depends on their value: if defined as @c 1, feature is active,
-otherwise it is disabled. Because of this these symbols should be always tested
+otherwise it is disabled. Because of this these symbols should always be tested
 using @if_ and not @ifdef_.
 
-@li @ref page_wxusedef_important
-@li @ref page_wxusedef_multi
-@li @ref page_wxusedef_unix
-@li @ref page_wxusedef_x11
-@li @ref page_wxusedef_gtk
-@li @ref page_wxusedef_mac
-@li @ref page_wxusedef_motif
-@li @ref page_wxusedef_cocoa
-@li @ref page_wxusedef_os2
-@li @ref page_wxusedef_msw
-@li @ref page_wxusedef_univ
 
 
-<hr>
-
-@section page_wxusedef_important Most important wxUSE symbols
+@section page_wxusedef_important Most Important Symbols
 
 This table summarizes some of the global build features affecting the entire
 library:
@@ -60,15 +49,17 @@ library:
 @endDefList
 
 
-@section page_wxusedef_multi Generic wxUSE preprocessor symbols
+@section page_wxusedef_multi Generic Symbols
 
 @beginDefList
 @itemdef{wxUSE_ABOUTDLG, Use wxAboutDialogInfo class.}
 @itemdef{wxUSE_ACCEL, Use wxAcceleratorTable/Entry classes and support for them in wxMenu, wxMenuBar.}
 @itemdef{wxUSE_AFM_FOR_POSTSCRIPT, In wxPostScriptDC class use AFM (adobe font metrics) file for character widths.}
 @itemdef{wxUSE_ANIMATIONCTRL, Use wxAnimationCtrl class.}
+@itemdef{wxUSE_ARTPROVIDER_STD, Use standard low quality icons in wxArtProvider.}
+@itemdef{wxUSE_ARTPROVIDER_TANGO, Use Tango icons in wxArtProvider.}
 @itemdef{wxUSE_ANY, Use wxAny class.}
-@itemdef{wxUSE_APPLE_IEEE, IEEE Extended to/from double routines; see src/common/extended.c file.}
+@itemdef{wxUSE_APPLE_IEEE, IEEE Extended to/from double routines, see wxDataOutputStream.}
 @itemdef{wxUSE_ARCHIVE_STREAMS, Enable streams for archive formats.}
 @itemdef{wxUSE_AUI, Use AUI (dockable windows) library.}
 @itemdef{wxUSE_BASE64, Enables Base64 support.}
@@ -90,6 +81,7 @@ library:
 @itemdef{wxUSE_COLOURPICKERCTRL, Use wxColourPickerCtrl class.}
 @itemdef{wxUSE_COMBOBOX, Use wxComboBox class.}
 @itemdef{wxUSE_COMBOCTRL, Use wxComboCtrl class.}
+@itemdef{wxUSE_COMPILER_TLS, Can be set to 0 to prevent using compile thread-specific variables support.}
 @itemdef{wxUSE_CONFIG, Use wxConfig and related classes.}
 @itemdef{wxUSE_CONFIG_NATIVE, When enabled use native OS configuration instead of the wxFileConfig class.}
 @itemdef{wxUSE_CONSOLE_EVENTLOOP, Enable event loop in console programs.}
@@ -99,7 +91,8 @@ library:
 @itemdef{wxUSE_DATAVIEWCTRL, Use wxDataViewCtrl class.}
 @itemdef{wxUSE_DATEPICKCTRL, Use wxDatePickerCtrl class.}
 @itemdef{wxUSE_DATETIME, Use wxDateTime and related classes.}
-@itemdef{wxUSE_DBGHELP, Use wxDbgHelpDLL class.}
+@itemdef{wxUSE_DBGHELP, Predefine as 0 to avoid using wxDbgHelpDLL and related classes.}
+@itemdef{wxUSE_DC_TRANSFORM_MATRIX, Use wxDC::SetTransformMatrix() and related methods.}
 @itemdef{wxUSE_DEBUG_CONTEXT, Use wxDebugContext class.}
 @itemdef{wxUSE_DEBUG_NEW_ALWAYS, See @ref overview_debugging}
 @itemdef{wxUSE_DEBUGREPORT, Use wxDebugReport class.}
@@ -168,6 +161,7 @@ library:
 @itemdef{wxUSE_LONGLONG, Use wxLongLong class.}
 @itemdef{wxUSE_LONGLONG_NATIVE, Use native <tt>long long</tt> type in wxLongLong implementation.}
 @itemdef{wxUSE_LONGLONG_WX, Use generic wxLongLong implementation.}
+@itemdef{wxUSE_MARKUP, Provide wxControl::SetLabelMarkup() method.}
 @itemdef{wxUSE_MDI, Use wxMDIParentFrame, and wxMDIChildFrame}
 @itemdef{wxUSE_MDI_ARCHITECTURE, Use MDI-based document-view classes.}
 @itemdef{wxUSE_MEDIACTRL, Use wxMediaCtrl.}
@@ -247,11 +241,13 @@ library:
 @itemdef{wxUSE_TOOLTIPS, Use wxToolTip class.}
 @itemdef{wxUSE_TREEBOOK, Use wxTreebook class.}
 @itemdef{wxUSE_TREECTRL, Use wxTreeCtrl class.}
+@itemdef{wxUSE_TREELISTCTRL, Use wxTreeListCtrl class.}
 @itemdef{wxUSE_TTM_WINDOWFROMPOINT, Obsolete, do not use.}
 @itemdef{wxUSE_URL, Use wxURL class.}
 @itemdef{wxUSE_URL_NATIVE, Use native support for some operations with wxURL.}
 @itemdef{wxUSE_VALIDATORS, Use wxValidator class.}
 @itemdef{wxUSE_VARIANT, Use wxVariant class.}
+@itemdef{wxUSE_WEBVIEW, Use wxWebView class.}
 @itemdef{wxUSE_WIZARDDLG, Use wxWizard class.}
 @itemdef{wxUSE_WXHTML_HELP, Use wxHtmlHelpController and related classes.}
 @itemdef{wxUSE_XML, Use XML parsing classes.}
@@ -262,7 +258,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_unix wxUSE preprocessor symbols used only under Unix platforms
+@section page_wxusedef_unix Unix Platform Symbols
 
 @beginDefList
 @itemdef{wxUSE_EPOLL_DISPATCHER, Use wxEpollDispatcher class. See also wxUSE_SELECT_DISPATCHER.}
@@ -274,7 +270,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_x11 wxUSE preprocessor symbols used only in wxX11 Platform
+@section page_wxusedef_x11 wxX11 Symbols
 
 @beginDefList
 @itemdef{wxUSE_NANOX, Use NanoX.}
@@ -282,7 +278,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_gtk wxUSE preprocessor symbols used only in wxGTK port
+@section page_wxusedef_gtk wxGTK Symbols
 
 @beginDefList
 @itemdef{wxUSE_DETECT_SM, Use code to detect X11 session manager.}
@@ -293,7 +289,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_mac wxUSE preprocessor symbols used only in wxMac port
+@section page_wxusedef_mac wxMac Symbols
 
 @beginDefList
 @itemdef{wxUSE_MAC_CRITICAL_REGION_MUTEX, See src/osx/carbon/thread.cpp file.}
@@ -303,7 +299,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_motif wxUSE preprocessor symbols used only in wxMotif port
+@section page_wxusedef_motif wxMotif Symbols
 
 @beginDefList
 @itemdef{wxUSE_GADGETS, Use xmCascadeButtonGadgetClass, xmLabelGadgetClass, xmPushButtonGadgetClass and xmToggleButtonGadgetClass classes.}
@@ -311,14 +307,14 @@ library:
 @endDefList
 
 
-@section page_wxusedef_cocoa wxUSE preprocessor symbols used only in Cocoa port
+@section page_wxusedef_cocoa Cocoa Symbols
 
 @beginDefList
 @itemdef{wxUSE_OBJC_UNIQUIFYING, Enable Objective-C class name uniquifying.}
 @endDefList
 
 
-@section page_wxusedef_os2 wxUSE preprocessor symbols used only in OS2 port
+@section page_wxusedef_os2 OS2 Symbols
 
 @beginDefList
 @itemdef{wxUSE_CONSOLEDEBUG, See src/os2/app.cpp file.}
@@ -330,7 +326,7 @@ library:
 @endDefList
 
 
-@section page_wxusedef_msw wxUSE preprocessor symbols used only in wxMSW port
+@section page_wxusedef_msw wxMSW Symbols
 
 @beginDefList
 @itemdef{wxUSE_ACCESSIBILITY, Enable accessibility support}
@@ -341,13 +337,13 @@ library:
 @itemdef{wxUSE_CRASHREPORT, Use wxCrashReport class.}
 @itemdef{wxUSE_DATEPICKCTRL_GENERIC, Use generic wxDatePickerCtrl implementation in addition to the native one.}
 @itemdef{wxUSE_DC_CACHEING, cache temporary wxDC objects.}
-@itemdef{wxUSE_DIRECTDRAW, Enable use of the system include file ddraw.h.}
 @itemdef{wxUSE_DDE_FOR_IPC, See wx/ipc.h file.}
 @itemdef{wxUSE_ENH_METAFILE, Use wxEnhMetaFile.}
 @itemdef{wxUSE_HOTKEY, Use wxWindow::RegisterHotKey() and wxWindow::UnregisterHotKey}
 @itemdef{wxUSE_INKEDIT, Use InkEdit library. Related to Tablet PCs.}
 @itemdef{wxUSE_MS_HTML_HELP, Use wxCHMHelpController class.}
-@itemdef{wxUSE_NO_MANIFEST, Use to prevent the auto generation, under MSVC, of manifest file needed by windows XP and above.}
+@itemdef{wxUSE_NO_MANIFEST, Can be predefined to disable inclusion of the
+manifest from wxWidgets RC file. See also wxUSE_RC_MANIFEST.}
 @itemdef{wxUSE_NORLANDER_HEADERS, Using headers whose author is Anders Norlander.}
 @itemdef{wxUSE_OLE, Enables OLE helper routines.}
 @itemdef{wxUSE_OLE_AUTOMATION, Enable OLE automation utilities.}
@@ -355,6 +351,11 @@ library:
 @itemdef{wxUSE_PENWINDOWS, See src/msw/penwin.cpp file.}
 @itemdef{wxUSE_POSTSCRIPT_ARCHITECTURE_IN_MSW, Use PS printing in wxMSW.}
 @itemdef{wxUSE_PS_PRINTING, See src/msw/dcprint.cpp file.}
+@itemdef{wxUSE_RC_MANIFEST, Include manifest for common controls library v6
+from wxWidgets RC file. This may be needed to be defined explicitly for MSVC 6
+and 7 (98 and 2003) only as later versions of MSVC generate this manifest
+themselves and the manifest generation is enabled by default for the other
+compilers. See also wxUSE_NO_MANIFEST.}
 @itemdef{wxUSE_REGKEY, Use wxRegKey class.}
 @itemdef{wxUSE_RICHEDIT, Enable use of riched32.dll in wxTextCtrl}
 @itemdef{wxUSE_RICHEDIT2, Enable use of riched20.dll in wxTextCtrl}
@@ -363,11 +364,10 @@ library:
 @itemdef{wxUSE_UXTHEME, Enable support for XP themes.}
 @itemdef{wxUSE_WIN_METAFILES_ALWAYS, Use wxMetaFile even when wxUSE_ENH_METAFILE=1.}
 @itemdef{wxUSE_WXDIB, Use wxDIB class.}
-@itemdef{wxUSE_XPM_IN_MSW, See also wxUSE_XPM}
 @endDefList
 
 
-@section page_wxusedef_univ wxUSE preprocessor symbols used only in wxUniversal
+@section page_wxusedef_univ wxUniversal Symbols
 
 @beginDefList
 @itemdef{wxUSE_ALL_THEMES, Use all themes in wxUniversal; See wx/univ/theme.h file.}
