@@ -2,21 +2,22 @@
 // Name:        main.h
 // Purpose:     Main page of the Doxygen manual
 // Author:      wxWidgets team
-// RCS-ID:      $Id: manual.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-
 /**
 
-@mainpage Reference manual
+@mainpage Documentation
+
 @author Julian Smart, Robert Roebling, Vadim Zeitlin, Robin Dunn,
         Stefan Csomor, Francesco Montorsi, Bryan Petty, et al
-@date July, 2010
+
+@date July, 2013
 
 @image html main_wxlogo.png
 
-@section manual_user User Manual:
+@section manual_user User Manual
 
 @li @subpage page_introduction
 @li @subpage page_copyright
@@ -28,7 +29,7 @@
 @li @subpage page_multiplatform
 @li @subpage page_port
 
-@section manual_reference Reference Manual:
+@section manual_reference Reference Manual
 
 @li @subpage page_constants
 @li @subpage page_translations
