@@ -2,7 +2,7 @@
 // Name:        envvars.h
 // Purpose:     topic overview
 // Author:      wxWidgets team
-// RCS-ID:      $Id: envvars.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -27,13 +27,5 @@ wxWidgets programs.
         @c /usr/local or @c /usr). You can set WXPREFIX if you are for example
         distributing a binary version of an application and you don't know in advance
         where it will be installed.}
-@itemdef{WXMODE,
-        (wxMGL only.)
-        Sets MGL video mode. The value must be in form
-        @e widthx@e height-@e depth. The default is @c 640x480-16.}
-@itemdef{WXSTDERR,
-        (wxMGL only.)
-        Redirects stderr output to a file.}
-
 */
 
