@@ -2,7 +2,7 @@
 // Name:        listctrl.h
 // Purpose:     topic overview
 // Author:      wxWidgets team
-// RCS-ID:      $Id: listctrl.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -10,9 +10,10 @@
 
 @page overview_listctrl wxListCtrl Overview
 
-Classes: wxListCtrl, wxImageList
+@tableofcontents
 
-@todo Sorry, this topic hasn't been written yet.
+@todo The wxListCtrl topic overview still needs to be written, sorry.
+
+@see wxListCtrl, wxImageList, wxListItem
 
 */
-
