@@ -2,7 +2,7 @@
 // Name:        referencenotes.h
 // Purpose:     topic overview
 // Author:      wxWidgets team
-// RCS-ID:      $Id: referencenotes.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -10,14 +10,15 @@
 
 @page overview_referencenotes Notes on Using this Reference Manual
 
+@tableofcontents
+
 In the descriptions of the wxWidgets classes and their member functions, note
 that descriptions of inherited member functions are not duplicated in derived
 classes unless their behaviour is different. So in using a class such as
 wxScrolledWindow, be aware that wxWindow functions may be relevant.
 
-Where not explicitely stated, size and position arguments may usually be given a
+Where not explicitly stated, size and position arguments may usually be given a
 value of ::wxDefaultSize and ::wxDefaultPosition, in which case wxWidgets will
 choose suitable values.
 
 */
-
