@@ -2,7 +2,7 @@
 // Name:        sizer.h
 // Purpose:     topic overview
 // Author:      wxWidgets team
-// RCS-ID:      $Id: sizer.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -10,8 +10,7 @@
 
 @page overview_sizer Sizers Overview
 
-Classes: wxSizer, wxBoxSizer, wxStaticBoxSizer, wxGridSizer, wxFlexGridSizer,
-         wxGridBagSizer
+@tableofcontents
 
 Sizers, as represented by the wxSizer class and its descendants in the
 wxWidgets class hierarchy, have become the method of choice to define the
@@ -29,16 +28,10 @@ classes.
 For information about the wxWidgets resource system, which can describe
 sizer-based dialogs, see the @ref overview_xrc.
 
-@li @ref overview_sizer_idea
-@li @ref overview_sizer_features
-@li @ref overview_sizer_hiding
-@li @ref overview_sizer_box
-@li @ref overview_sizer_types
-@li @ref overview_sizer_button
+@see wxSizer, wxBoxSizer, wxStaticBoxSizer, wxGridSizer, wxFlexGridSizer,
+     wxGridBagSizer
 
 
-<hr>
-
 
 @section overview_sizer_idea The Idea Behind Sizers
 
@@ -368,4 +361,3 @@ wxMORE      // return wxID_MORE
 @endcode
 
 */
-
