@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     07/07/03
-// RCS-ID:      $Id: afterstd.h 64943 2010-07-13 13:29:58Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -13,7 +13,7 @@
     See the comments in beforestd.h.
  */
 
-#if defined(__WXMSW__)
+#if defined(__WINDOWS__)
     #include "wx/msw/winundef.h"
 #endif
 
