@@ -4,7 +4,7 @@
 // Author:      Julian Smart (extracted from docview.h by VZ)
 // Modified by:
 // Created:     05.11.00
-// RCS-ID:      $Id: cmdproc.h 58757 2009-02-08 11:45:59Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -97,10 +97,7 @@ public:
     virtual void ClearCommands();
 
     // Has the current project been changed?
-    virtual bool IsDirty() const
-    {
-        return m_currentCommand && (m_lastSavedCommand != m_currentCommand);
-    }
+    virtual bool IsDirty() const;
 
     // Mark the current command as the one where the last save took place
     void MarkAsSaved()
