@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        sound.h
+// Name:        wx/cocoa/sound.h
 // Purpose:     wxSound class (loads and plays short Windows .wav files).
 //              Optional on non-Windows platforms.
 // Authors:     David Elliott, Ryan Norton
 // Modified by:
 // Created:     2004-10-02
-// RCS-ID:      $Id: sound.h 61724 2009-08-21 10:41:26Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2004 David Elliott, Ryan Norton
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,7 +25,7 @@ public:
     wxSound(const wxString& fileName, bool isResource = false)
     :   m_cocoaNSSound(NULL)
     {   Create(fileName, isResource); }
-    wxSound(int size, const wxByte* data)
+    wxSound(size_t size, const void* data)
     :   m_cocoaNSSound(NULL)
     {   LoadWAV(data,size,true); }
     wxSound(const wxSound& sound); // why not?
@@ -43,7 +43,7 @@ public:
     {   return m_cocoaNSSound; }
 protected:
     bool DoPlay(unsigned flags) const;
-    bool LoadWAV(const wxUint8 *data, size_t length, bool copyData);
+    bool LoadWAV(const void* data, size_t length, bool copyData);
 private:
     WX_NSSound m_cocoaNSSound;
     static const wxObjcAutoRefFromAlloc<struct objc_object *> sm_cocoaDelegate;
