@@ -5,7 +5,7 @@
 // Modified by:
 // Created:     14/4/2006
 // Copyright:   (c) Francesco Montorsi
-// RCS-ID:      $Id: fontpickerg.h 51533 2008-02-03 21:32:11Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows Licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -13,7 +13,7 @@
 #define _WX_FONTPICKER_H_
 
 #include "wx/button.h"
-#include "wx/cmndata.h"
+#include "wx/fontdata.h"
 
 //-----------------------------------------------------------------------------
 // wxGenericFontButton: a button which brings up a wxFontDialog
