@@ -4,7 +4,7 @@
 // Author:      Michael Bedward (based on code by Julian Smart, Robin Dunn)
 // Modified by: Santiago Palacios
 // Created:     1/08/1999
-// RCS-ID:      $Id: grid.h 64937 2010-07-13 12:38:00Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Michael Bedward
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,6 +16,10 @@
 
 #if wxUSE_GRID
 
+// Internally used (and hence intentionally not exported) event telling wxGrid
+// to hide the currently shown editor.
+wxDECLARE_EVENT( wxEVT_GRID_HIDE_EDITOR, wxCommandEvent );
+
 // ----------------------------------------------------------------------------
 // array classes
 // ----------------------------------------------------------------------------
@@ -301,6 +305,8 @@ public:
         m_owner = owner;
     }
 
+    virtual wxWindow *GetMainWindowOfCompositeControl() { return m_owner; }
+
     virtual bool AcceptsFocus() const { return false; }
 
     wxGrid *GetOwner() { return m_owner; }
@@ -544,8 +550,15 @@ public:
     //
     // NB: currently this is always identity for the rows as reordering is only
     //     implemented for the lines
-    virtual int GetLineAt(const wxGrid *grid, int line) const = 0;
+    virtual int GetLineAt(const wxGrid *grid, int pos) const = 0;
 
+    // Return the display position of the line with the given index.
+    //
+    // NB: As GetLineAt(), currently this is always identity for rows.
+    virtual int GetLinePos(const wxGrid *grid, int line) const = 0;
+
+    // Return the index of the line just before the given one or wxNOT_FOUND.
+    virtual int GetLineBefore(const wxGrid* grid, int line) const = 0;
 
     // Get the row or column label window
     virtual wxWindow *GetHeaderWindow(wxGrid *grid) const = 0;
@@ -611,9 +624,14 @@ public:
     virtual void SetDefaultLineSize(wxGrid *grid, int size, bool resizeExisting) const
         {  grid->SetDefaultRowSize(size, resizeExisting); }
 
-    virtual int GetLineAt(const wxGrid * WXUNUSED(grid), int line) const
+    virtual int GetLineAt(const wxGrid * WXUNUSED(grid), int pos) const
+        { return pos; } // TODO: implement row reordering
+    virtual int GetLinePos(const wxGrid * WXUNUSED(grid), int line) const
         { return line; } // TODO: implement row reordering
 
+    virtual int GetLineBefore(const wxGrid* WXUNUSED(grid), int line) const
+        { return line - 1; }
+
     virtual wxWindow *GetHeaderWindow(wxGrid *grid) const
         { return grid->GetGridRowLabelWindow(); }
     virtual int GetHeaderWindowSize(wxGrid *grid) const
@@ -672,8 +690,16 @@ public:
     virtual void SetDefaultLineSize(wxGrid *grid, int size, bool resizeExisting) const
         {  grid->SetDefaultColSize(size, resizeExisting); }
 
-    virtual int GetLineAt(const wxGrid *grid, int line) const
-        { return grid->GetColAt(line); }
+    virtual int GetLineAt(const wxGrid *grid, int pos) const
+        { return grid->GetColAt(pos); }
+    virtual int GetLinePos(const wxGrid *grid, int line) const
+        { return grid->GetColPos(line); }
+
+    virtual int GetLineBefore(const wxGrid* grid, int line) const
+    {
+        int posBefore = grid->GetColPos(line) - 1;
+        return posBefore >= 0 ? grid->GetColAt(posBefore) : wxNOT_FOUND;
+    }
 
     virtual wxWindow *GetHeaderWindow(wxGrid *grid) const
         { return grid->GetGridColLabelWindow(); }
@@ -683,7 +709,10 @@ public:
 
 // This class abstracts the difference between operations going forward
 // (down/right) and backward (up/left) and allows to use the same code for
-// functions which differ only in the direction of grid traversal
+// functions which differ only in the direction of grid traversal.
+//
+// Notice that all operations in this class work with display positions and not
+// internal indices which can be different if the columns were reordered.
 //
 // Like wxGridOperations it's an ABC with two concrete subclasses below. Unlike
 // it, this is a normal object and not just a function dispatch table and has a
@@ -712,6 +741,12 @@ public:
     // Find the line at the given distance, in pixels, away from this one
     // (this uses clipping, i.e. anything after the last line is counted as the
     // last one and anything before the first one as 0)
+    //
+    // TODO: Implementation of this method currently doesn't support column
+    //       reordering as it mixes up indices and positions. But this doesn't
+    //       really matter as it's only called for rows (Page Up/Down only work
+    //       vertically) and row reordering is not currently supported. We'd
+    //       need to fix it if this ever changes however.
     virtual int MoveByPixelDistance(int line, int distance) const = 0;
 
     // This class is never used polymorphically but give it a virtual dtor
@@ -719,6 +754,28 @@ public:
     virtual ~wxGridDirectionOperations() { }
 
 protected:
+    // Get the position of the row or column from the given coordinates pair.
+    //
+    // This is just a shortcut to avoid repeating m_oper and m_grid multiple
+    // times in the derived classes code.
+    int GetLinePos(const wxGridCellCoords& coords) const
+    {
+        return m_oper.GetLinePos(m_grid, m_oper.Select(coords));
+    }
+
+    // Get the index of the row or column from the position.
+    int GetLineAt(int pos) const
+    {
+        return m_oper.GetLineAt(m_grid, pos);
+    }
+
+    // Check if the given line is visible, i.e. has non 0 size.
+    bool IsLineVisible(int line) const
+    {
+        return m_oper.GetLineSize(m_grid, line) != 0;
+    }
+
+
     wxGrid * const m_grid;
     const wxGridOperations& m_oper;
 };
@@ -735,14 +792,38 @@ public:
     {
         wxASSERT_MSG( m_oper.Select(coords) >= 0, "invalid row/column" );
 
-        return m_oper.Select(coords) == 0;
+        int pos = GetLinePos(coords);
+        while ( pos )
+        {
+            // Check the previous line.
+            int line = GetLineAt(--pos);
+            if ( IsLineVisible(line) )
+            {
+                // There is another visible line before this one, hence it's
+                // not at boundary.
+                return false;
+            }
+        }
+
+        // We reached the boundary without finding any visible lines.
+        return true;
     }
 
     virtual void Advance(wxGridCellCoords& coords) const
     {
-        wxASSERT( !IsAtBoundary(coords) );
+        int pos = GetLinePos(coords);
+        for ( ;; )
+        {
+            // This is not supposed to happen if IsAtBoundary() returned false.
+            wxCHECK_RET( pos, "can't advance when already at boundary" );
 
-        m_oper.Set(coords, m_oper.Select(coords) - 1);
+            int line = GetLineAt(--pos);
+            if ( IsLineVisible(line) )
+            {
+                m_oper.Set(coords, line);
+                break;
+            }
+        }
     }
 
     virtual int MoveByPixelDistance(int line, int distance) const
@@ -752,6 +833,8 @@ public:
     }
 };
 
+// Please refer to the comments above when reading this class code, it's
+// absolutely symmetrical to wxGridBackwardOperations.
 class wxGridForwardOperations : public wxGridDirectionOperations
 {
 public:
@@ -765,14 +848,32 @@ public:
     {
         wxASSERT_MSG( m_oper.Select(coords) < m_numLines, "invalid row/column" );
 
-        return m_oper.Select(coords) == m_numLines - 1;
+        int pos = GetLinePos(coords);
+        while ( pos < m_numLines - 1 )
+        {
+            int line = GetLineAt(++pos);
+            if ( IsLineVisible(line) )
+                return false;
+        }
+
+        return true;
     }
 
     virtual void Advance(wxGridCellCoords& coords) const
     {
-        wxASSERT( !IsAtBoundary(coords) );
+        int pos = GetLinePos(coords);
+        for ( ;; )
+        {
+            wxCHECK_RET( pos < m_numLines - 1,
+                         "can't advance when already at boundary" );
 
-        m_oper.Set(coords, m_oper.Select(coords) + 1);
+            int line = GetLineAt(++pos);
+            if ( IsLineVisible(line) )
+            {
+                m_oper.Set(coords, line);
+                break;
+            }
+        }
     }
 
     virtual int MoveByPixelDistance(int line, int distance) const
@@ -785,23 +886,6 @@ private:
     const int m_numLines;
 };
 
-// ----------------------------------------------------------------------------
-// private helpers
-// ----------------------------------------------------------------------------
-
-namespace
-{
-
-// ensure that first is less or equal to second, swapping the values if
-// necessary
-void EnsureFirstLessThanSecond(int& first, int& second)
-{
-    if ( first > second )
-        wxSwap(first, second);
-}
-
-} // anonymous namespace
-
 // ----------------------------------------------------------------------------
 // data structures used for the data type registry
 // ----------------------------------------------------------------------------
