@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: VZ at 05.02.00 to derive from wxStatusBarBase
 // Created:     01/02/97
-// RCS-ID:      $Id: statusbr.h 61624 2009-08-06 00:01:43Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -81,7 +81,7 @@ protected:
     virtual void InitColours();
 
     // true if the status bar shows the size grip: for this it must have
-    // wxSTB_SIZEGRIP style and the window it is attached to must be resizeable
+    // wxSTB_SIZEGRIP style and the window it is attached to must be resizable
     // and not maximized
     bool ShowsSizeGrip() const;
 
@@ -91,8 +91,9 @@ protected:
     // common part of all ctors
     void Init();
 
-    // the last known height of the client rect
-    int               m_lastClientHeight;
+    // the last known size, fields widths must be updated whenever it's out of
+    // date
+    wxSize m_lastClientSize;
 
     // the absolute widths of the status bar panes in pixels
     wxArrayInt        m_widthsAbs;
@@ -106,6 +107,9 @@ protected:
     virtual wxSize DoGetBestSize() const;
 
 private:
+    // Update m_lastClientSize and m_widthsAbs from the current size.
+    void DoUpdateFieldWidths();
+
     DECLARE_EVENT_TABLE()
     DECLARE_DYNAMIC_CLASS_NO_COPY(wxStatusBarGeneric)
 };
