@@ -2,7 +2,7 @@
 // Name:        wx/gtk/colour.h
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: colour.h 50897 2007-12-22 15:03:58Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -10,6 +10,10 @@
 #ifndef _WX_GTK_COLOUR_H_
 #define _WX_GTK_COLOUR_H_
 
+#ifdef __WXGTK3__
+typedef struct _GdkRGBA GdkRGBA;
+#endif
+
 //-----------------------------------------------------------------------------
 // wxColour
 //-----------------------------------------------------------------------------
@@ -21,6 +25,9 @@ public:
     // ------------
     DEFINE_STD_WXCOLOUR_CONSTRUCTORS
     wxColour(const GdkColor& gdkColor);
+#ifdef __WXGTK3__
+    wxColour(const GdkRGBA& gdkRGBA);
+#endif
 
     virtual ~wxColour();
 
@@ -33,8 +40,12 @@ public:
     unsigned char Alpha() const;
 
     // Implementation part
+#ifdef __WXGTK3__
+    operator const GdkRGBA*() const;
+#else
     void CalcPixel( GdkColormap *cmap );
     int GetPixel() const;
+#endif
     const GdkColor *GetColor() const;
 
 protected:
@@ -43,7 +54,6 @@ protected:
 
     virtual bool FromString(const wxString& str);
 
-private:
     DECLARE_DYNAMIC_CLASS(wxColour)
 };
 
