@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        dcscreen.h
+// Name:        wx/gtk/dcscreen.h
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: dcscreen.h 50547 2007-12-06 16:22:00Z PC $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,7 +25,7 @@ public:
 
     virtual void DoGetSize(int *width, int *height) const;
 
-protected:
+private:
     void Init();
 
     DECLARE_ABSTRACT_CLASS(wxScreenDCImpl)
