@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        popupwin.h
+// Name:        wx/gtk/popupwin.h
 // Purpose:
 // Author:      Robert Roebling
 // Created:
-// Id:          $Id: popupwin.h 61724 2009-08-21 10:41:26Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 2001 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -42,9 +42,6 @@ protected:
 
     virtual void DoMoveWindow(int x, int y, int width, int height);
 
-private:
-    virtual void AddChildGTK(wxWindowGTK* child);
-
 #ifdef __WXUNIVERSAL__
     DECLARE_EVENT_TABLE()
 #endif
