@@ -1,9 +1,9 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        wx/gtk/private/msgtype.h
+// Name:        wx/gtk/private/messagetype.h
 // Purpose:     translate between wx and GtkMessageType
 // Author:      Vadim Zeitlin
 // Created:     2009-09-27
-// RCS-ID:      $Id: messagetype.h 64140 2010-04-25 21:33:16Z FM $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2009 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -11,7 +11,7 @@
 #ifndef _GTK_PRIVATE_MSGTYPE_H_
 #define _GTK_PRIVATE_MSGTYPE_H_
 
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 
 namespace wxGTKImpl
 {
