@@ -1,8 +1,8 @@
 /* ///////////////////////////////////////////////////////////////////////////
-// Name:        win_gtk.h
+// Name:        wx/gtk/private/win_gtk.h
 // Purpose:     native GTK+ widget for wxWindow
 // Author:      Robert Roebling
-// Id:          $Id: win_gtk.h 64431 2010-05-30 06:38:28Z PC $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////// */
@@ -10,8 +10,6 @@
 #ifndef _WX_GTK_PIZZA_H_
 #define _WX_GTK_PIZZA_H_
 
-#include <gtk/gtkfixed.h>
-
 #define WX_PIZZA(obj) G_TYPE_CHECK_INSTANCE_CAST(obj, wxPizza::type(), wxPizza)
 #define WX_IS_PIZZA(obj) G_TYPE_CHECK_INSTANCE_TYPE(obj, wxPizza::type())
 
@@ -23,16 +21,16 @@ struct WXDLLIMPEXP_CORE wxPizza
 
     static GtkWidget* New(long windowStyle = 0);
     static GType type();
-    void move(GtkWidget* widget, int x, int y);
-    void put(GtkWidget* widget, int x, int y);
+    void move(GtkWidget* widget, int x, int y, int width, int height);
+    void put(GtkWidget* widget, int x, int y, int width, int height);
     void scroll(int dx, int dy);
-    void get_border_widths(int& x, int& y);
+    void get_border(GtkBorder& border);
 
     GtkFixed m_fixed;
+    GList* m_children;
     int m_scroll_x;
     int m_scroll_y;
-    int m_border_style;
-    bool m_is_scrollable;
+    int m_windowStyle;
 };
 
 #endif // _WX_GTK_PIZZA_H_
