@@ -2,7 +2,7 @@
 // Name:        wx/gtk/region.h
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: region.h 61724 2009-08-21 10:41:26Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -10,6 +10,10 @@
 #ifndef _WX_GTK_REGION_H_
 #define _WX_GTK_REGION_H_
 
+#ifdef __WXGTK3__
+typedef struct _cairo_region cairo_region_t;
+#endif
+
 // ----------------------------------------------------------------------------
 // wxRegion
 // ----------------------------------------------------------------------------
@@ -56,12 +60,12 @@ public:
     virtual void Clear();
     virtual bool IsEmpty() const;
 
-public:
-    // Init with GdkRegion, set ref count to 2 so that
-    // the C++ class will not destroy the region!
-    wxRegion( GdkRegion *region );
-
+#ifdef __WXGTK3__
+    cairo_region_t* GetRegion() const;
+#else
+    wxRegion(const GdkRegion* region);
     GdkRegion *GetRegion() const;
+#endif
 
 protected:
     virtual wxGDIRefData *CreateGDIRefData() const;
@@ -122,13 +126,11 @@ private:
     void Init();
     void CreateRects( const wxRegion& r );
 
-    size_t   m_current;
     wxRegion m_region;
-
     wxRect *m_rects;
-    size_t  m_numRects;
+    int m_numRects;
+    int m_current;
 
-private:
     DECLARE_DYNAMIC_CLASS(wxRegionIterator)
 };
 
