@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        helpctrl.h
+// Name:        wx/html/helpctrl.h
 // Purpose:     wxHtmlHelpController
 // Notes:       Based on htmlhelp.cpp, implementing a monolithic
 //              HTML Help controller class,  by Vaclav Slavik
 // Author:      Harm van der Heijden and Vaclav Slavik
-// RCS-ID:      $Id: helpctrl.h 63548 2010-02-25 00:03:30Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Harm van der Heijden and Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -46,8 +46,12 @@ class WXDLLIMPEXP_HTML wxHtmlHelpController : public wxHelpControllerBase // wxE
 
 public:
     wxHtmlHelpController(int style = wxHF_DEFAULT_STYLE, wxWindow* parentWindow = NULL);
+    wxHtmlHelpController(wxWindow* parentWindow, int style = wxHF_DEFAULT_STYLE);
+    
     virtual ~wxHtmlHelpController();
 
+    void SetShouldPreventAppExit(bool enable);
+
     void SetTitleFormat(const wxString& format);
     void SetTempDir(const wxString& path) { m_helpData.SetTempDir(path); }
     bool AddBook(const wxString& book_url, bool show_wait_msg = false);
@@ -87,7 +91,7 @@ public:
     virtual bool DisplayBlock(long blockNo) { return DisplaySection(blockNo); }
     virtual bool DisplayTextPopup(const wxString& text, const wxPoint& pos);
 
-    virtual void SetFrameParameters(const wxString& title,
+    virtual void SetFrameParameters(const wxString& titleFormat,
                                const wxSize& size,
                                const wxPoint& pos = wxDefaultPosition,
                                bool newFrameEachTime = false);
@@ -113,6 +117,8 @@ public:
     wxWindow* FindTopLevelWindow();
 
 protected:
+    void Init(int style);
+    
     virtual wxWindow* CreateHelpWindow();
     virtual wxHtmlHelpFrame* CreateHelpFrame(wxHtmlHelpData *data);
     virtual wxHtmlHelpDialog* CreateHelpDialog(wxHtmlHelpData *data);
@@ -129,6 +135,8 @@ protected:
     wxHtmlHelpFrame*    m_helpFrame;
     wxHtmlHelpDialog*   m_helpDialog;
 
+    bool                m_shouldPreventAppExit;
+
     wxDECLARE_NO_COPY_CLASS(wxHtmlHelpController);
 };
 
