@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.02.99
-// RCS-ID:      $Id: ioswrap.h 47644 2007-07-22 09:20:42Z VS $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -21,7 +21,7 @@
 
 #include "wx/afterstd.h"
 
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
 #   include "wx/msw/winundef.h"
 #endif
 
