@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        joystick.h
+// Name:        wx/joystick.h
 // Purpose:     wxJoystick base header
 // Author:      wxWidgets Team
 // Modified by:
 // Created:
 // Copyright:   (c) wxWidgets Team
-// RCS-ID:      $Id: joystick.h 54125 2008-06-11 19:17:41Z SC $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -16,7 +16,7 @@
 
 #if wxUSE_JOYSTICK
 
-#if defined(__WXMSW__)
+#if defined(__WINDOWS__)
 #include "wx/msw/joystick.h"
 #elif defined(__WXMOTIF__)
 #include "wx/unix/joystick.h"
