@@ -3,7 +3,7 @@
 // Purpose:     wxMSWFileSystemWatcher
 // Author:      Bartosz Bekier
 // Created:     2009-05-26
-// RCS-ID:      $Id: fswatcher.h 62474 2009-10-22 11:35:43Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2009 Bartosz Bekier <bartosz.bekier@gmail.com>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -23,6 +23,12 @@ public:
     wxMSWFileSystemWatcher(const wxFileName& path,
                            int events = wxFSW_EVENT_ALL);
 
+    // Override the base class function to provide a much more efficient
+    // implementation for it using the platform native support for watching the
+    // entire directory trees.
+    virtual bool AddTree(const wxFileName& path, int events = wxFSW_EVENT_ALL,
+                         const wxString& filter = wxEmptyString);
+
 protected:
     bool Init();
 };
