@@ -5,7 +5,7 @@
 // Modified by: Vadim Zeitlin on 13.05.99: complete refont of message handling,
 //              elimination of Default(), ...
 // Created:     01/02/97
-// RCS-ID:      $Id: window.h 64372 2010-05-21 12:51:20Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -67,6 +67,9 @@ public:
     virtual void Raise();
     virtual void Lower();
 
+    virtual bool BeginRepositioningChildren();
+    virtual void EndRepositioningChildren();
+
     virtual bool Show(bool show = true);
     virtual bool ShowWithEffect(wxShowEffect effect,
                                 unsigned timeout = 0)
@@ -345,12 +348,13 @@ public:
 
     bool HandleMouseEvent(WXUINT msg, int x, int y, WXUINT flags);
     bool HandleMouseMove(int x, int y, WXUINT flags);
-    bool HandleMouseWheel(WXWPARAM wParam, WXLPARAM lParam);
+    bool HandleMouseWheel(wxMouseWheelAxis axis,
+                          WXWPARAM wParam, WXLPARAM lParam);
 
-    bool HandleChar(WXWPARAM wParam, WXLPARAM lParam, bool isASCII = false);
+    bool HandleChar(WXWPARAM wParam, WXLPARAM lParam);
     bool HandleKeyDown(WXWPARAM wParam, WXLPARAM lParam);
     bool HandleKeyUp(WXWPARAM wParam, WXLPARAM lParam);
-#if wxUSE_ACCEL
+#if wxUSE_HOTKEY
     bool HandleHotKey(WXWPARAM wParam, WXLPARAM lParam);
 #endif
 #ifdef __WIN32__
@@ -366,7 +370,21 @@ public:
     bool HandlePower(WXWPARAM wParam, WXLPARAM lParam, bool *vetoed);
 
 
-    // Window procedure
+    // The main body of common window proc for all wxWindow objects. It tries
+    // to handle the given message and returns true if it was handled (the
+    // appropriate return value is then put in result, which must be non-NULL)
+    // or false if it wasn't.
+    //
+    // This function should be overridden in any new code instead of
+    // MSWWindowProc() even if currently most of the code overrides
+    // MSWWindowProc() as it had been written before this function was added.
+    virtual bool MSWHandleMessage(WXLRESULT *result,
+                                  WXUINT message,
+                                  WXWPARAM wParam,
+                                  WXLPARAM lParam);
+
+    // Common Window procedure for all wxWindow objects: forwards to
+    // MSWHandleMessage() and MSWDefWindowProc() if the message wasn't handled.
     virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
 
     // Calls an appropriate default window procedure
@@ -388,6 +406,29 @@ public:
     virtual void MSWDestroyWindow();
 
 
+    // Functions dealing with painting the window background. The derived
+    // classes should normally only need to reimplement MSWGetBgBrush() if they
+    // need to use a non-solid brush for erasing their background. This
+    // function is called by MSWGetBgBrushForChild() which only exists for the
+    // weird wxToolBar case and MSWGetBgBrushForChild() itself is used by
+    // MSWGetBgBrush() to actually find the right brush to use.
+
+    // Adjust the origin for the brush returned by MSWGetBgBrushForChild().
+    //
+    // This needs to be overridden for scrolled windows to ensure that the
+    // scrolling of their associated DC is taken into account.
+    //
+    // Both parameters must be non-NULL.
+    virtual void MSWAdjustBrushOrg(int* WXUNUSED(xOrg),
+                                   int* WXUNUSED(yOrg)) const
+    {
+    }
+
+    // The brush returned from here must remain valid at least until the next
+    // event loop iteration. Returning 0, as is done by default, indicates
+    // there is no custom background brush.
+    virtual WXHBRUSH MSWGetCustomBgBrush() { return 0; }
+
     // this function should return the brush to paint the children controls
     // background or 0 if this window doesn't impose any particular background
     // on its children
@@ -438,6 +479,17 @@ public:
         return true;
     }
 
+    // This should be overridden to return true for the controls which have
+    // themed background that should through their children. Currently only
+    // wxNotebook uses this.
+    //
+    // The base class version already returns true if we have a solid
+    // background colour that should be propagated to our children.
+    virtual bool MSWHasInheritableBackground() const
+    {
+        return InheritsBackgroundColour();
+    }
+
 #if !defined(__WXWINCE__) && !defined(__WXUNIVERSAL__)
     #define wxHAS_MSW_BACKGROUND_ERASE_HOOK
 #endif
@@ -566,9 +618,29 @@ protected:
                              const wxString& ttip);
 #endif // wxUSE_TOOLTIPS
 
-    // the helper functions used by HandleChar/KeyXXX methods
-    wxKeyEvent CreateKeyEvent(wxEventType evType, int id,
-                              WXLPARAM lParam = 0, WXWPARAM wParam = 0) const;
+    // This is used by CreateKeyEvent() and also for wxEVT_CHAR[_HOOK] event
+    // creation. Notice that this method doesn't initialize wxKeyEvent
+    // m_keyCode and m_uniChar fields.
+    void InitAnyKeyEvent(wxKeyEvent& event,
+                         WXWPARAM wParam,
+                         WXLPARAM lParam) const;
+
+    // Helper functions used by HandleKeyXXX() methods and some derived
+    // classes, wParam and lParam have the same meaning as in WM_KEY{DOWN,UP}.
+    //
+    // NB: evType here must be wxEVT_KEY_{DOWN,UP} as wParam here contains the
+    //     virtual key code, not character!
+    wxKeyEvent CreateKeyEvent(wxEventType evType,
+                              WXWPARAM wParam,
+                              WXLPARAM lParam = 0) const;
+
+    // Another helper for creating wxKeyEvent for wxEVT_CHAR and related types.
+    //
+    // The wParam and lParam here must come from WM_CHAR event parameters, i.e.
+    // wParam must be a character and not a virtual code.
+    wxKeyEvent CreateCharEvent(wxEventType evType,
+                               WXWPARAM wParam,
+                               WXLPARAM lParam) const;
 
 
     // default OnEraseBackground() implementation, return true if we did erase
@@ -590,6 +662,8 @@ protected:
                                           int& x, int& y,
                                           int& w, int& h) const;
 
+    bool MSWEnableHWND(WXHWND hWnd, bool enable);
+
 private:
     // common part of all ctors
     void Init();
@@ -629,18 +703,6 @@ private:
     DECLARE_EVENT_TABLE()
 };
 
-// ----------------------------------------------------------------------------
-// inline functions
-// ----------------------------------------------------------------------------
-
-// ---------------------------------------------------------------------------
-// global functions
-// ---------------------------------------------------------------------------
-
-// kbd code translation
-WXDLLIMPEXP_CORE int wxCharCodeMSWToWX(int keySym, WXLPARAM lParam = 0);
-WXDLLIMPEXP_CORE WXWORD wxCharCodeWXToMSW(int id);
-
 // window creation helper class: before creating a new HWND, instantiate an
 // object of this class on stack - this allows to process the messages sent to
 // the window even before CreateWindow() returns
