@@ -4,7 +4,7 @@
  * Author:      Stefan Csomor
  * Modified by:
  * Created:     2008-07-30
- * RCS-ID:      $Id: chkconf.h 54833 2008-07-29 21:03:18Z SC $
+ * RCS-ID:      $Id$
  * Copyright:   (c) Stefan Csomor
  * Licence:     wxWindows licence
  */
@@ -12,6 +12,17 @@
 #ifndef _WX_OSX_COCOA_CHKCONF_H_
 #define _WX_OSX_COCOA_CHKCONF_H_
 
+/* Many wchar functions (and also strnlen(), for some reason) are only
+   available since 10.7 so don't use them if we want to build the applications
+   that would run under 10.6 and earlier. */
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_7
+#define HAVE_STRNLEN 1
+#define HAVE_WCSDUP 1
+#define HAVE_WCSNLEN 1
+#define HAVE_WCSCASECMP 1
+#define HAVE_WCSNCASECMP 1
+#endif
+
 /*
  * native (1) or emulated (0) toolbar
  */
@@ -20,6 +31,13 @@
     #define wxOSX_USE_NATIVE_TOOLBAR 1
 #endif
 
+/*
+ * leave is isFlipped and don't override
+ */
+#ifndef wxOSX_USE_NATIVE_FLIPPED 
+    #define wxOSX_USE_NATIVE_FLIPPED 1
+#endif
+
 /*
  * text rendering system
  */
