@@ -6,7 +6,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: private.h 53819 2008-05-29 14:11:45Z SC $
+// RCS-ID:      $Id$
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -32,13 +32,43 @@
     #define wxOSX_10_6_AND_LATER(x)
 #endif
 
-#if wxOSX_USE_COCOA_OR_CARBON
+// platform specific Clang analyzer support
+#ifndef NS_RETURNS_RETAINED
+#   if WX_HAS_CLANG_FEATURE(attribute_ns_returns_retained)
+#       define NS_RETURNS_RETAINED __attribute__((ns_returns_retained))
+#   else
+#       define NS_RETURNS_RETAINED
+#   endif
+#endif
+
+#ifndef CF_RETURNS_RETAINED
+#   if WX_HAS_CLANG_FEATURE(attribute_cf_returns_retained)
+#       define CF_RETURNS_RETAINED __attribute__((cf_returns_retained))
+#   else
+#       define CF_RETURNS_RETAINED
+#   endif
+#endif
+
+#if ( !wxUSE_GUI && !wxOSX_USE_IPHONE ) || wxOSX_USE_COCOA_OR_CARBON
+
+// Carbon functions are currently still used in wxOSX/Cocoa too (including
+// wxBase part of it).
+#include <Carbon/Carbon.h>
 
 WXDLLIMPEXP_BASE long UMAGetSystemVersion() ;
 
 void WXDLLIMPEXP_CORE wxMacStringToPascal( const wxString&from , unsigned char * to );
 wxString WXDLLIMPEXP_CORE wxMacMakeStringFromPascal( const unsigned char * from );
 
+WXDLLIMPEXP_BASE wxString wxMacFSRefToPath( const FSRef *fsRef , CFStringRef additionalPathComponent = NULL );
+WXDLLIMPEXP_BASE OSStatus wxMacPathToFSRef( const wxString&path , FSRef *fsRef );
+WXDLLIMPEXP_BASE wxString wxMacHFSUniStrToString( ConstHFSUniStr255Param uniname );
+
+// keycode utils from app.cpp
+
+WXDLLIMPEXP_BASE CGKeyCode wxCharCodeWXToOSX(wxKeyCode code);
+WXDLLIMPEXP_BASE long wxMacTranslateKey(unsigned char key, unsigned char code);
+
 #endif
 
 #if wxUSE_GUI
@@ -92,7 +122,9 @@ WXDLLIMPEXP_CORE CGDataProviderRef wxMacCGDataProviderCreateWithCFData( CFDataRe
 WXDLLIMPEXP_CORE CGDataConsumerRef wxMacCGDataConsumerCreateWithCFData( CFMutableDataRef data );
 WXDLLIMPEXP_CORE CGDataProviderRef wxMacCGDataProviderCreateWithMemoryBuffer( const wxMemoryBuffer& buf );
 
-CGColorSpaceRef WXDLLIMPEXP_CORE wxMacGetGenericRGBColorSpace(void);
+WXDLLIMPEXP_CORE CGColorSpaceRef wxMacGetGenericRGBColorSpace(void);
+
+WXDLLIMPEXP_CORE double wxOSXGetMainScreenContentScaleFactor();
 
 class wxWindowMac;
 // to
@@ -103,8 +135,10 @@ class wxNonOwnedWindow;
 
 class wxMacControl;
 class wxWidgetImpl;
+class wxComboBox;
 class wxNotebook;
 class wxTextCtrl;
+class wxSearchCtrl;
 
 WXDLLIMPEXP_CORE wxWindowMac * wxFindWindowFromWXWidget(WXWidget inControl );
 
@@ -140,7 +174,9 @@ public :
                        const wxString& strHelp,
                        wxItemKind kind,
                        wxMenu *pSubMenu );
-
+    
+    // handle OS specific menu items if they weren't handled during normal processing
+    virtual bool DoDefault() { return false; }
 protected :
     wxMenuItem* m_peer;
 
@@ -181,13 +217,15 @@ protected :
 class WXDLLIMPEXP_CORE wxWidgetImpl : public wxObject
 {
 public :
-    wxWidgetImpl( wxWindowMac* peer , bool isRootControl = false );
+    wxWidgetImpl( wxWindowMac* peer , bool isRootControl = false, bool isUserPane = false );
     wxWidgetImpl();
     virtual ~wxWidgetImpl();
 
     void Init();
 
     bool                IsRootControl() const { return m_isRootControl; }
+    
+    bool                IsUserPane() const { return m_isUserPane; }
 
     wxWindowMac*        GetWXPeer() const { return m_wxPeer; }
 
@@ -223,6 +261,17 @@ public :
     virtual void        GetPosition( int &x, int &y ) const = 0;
     virtual void        GetSize( int &width, int &height ) const = 0;
     virtual void        SetControlSize( wxWindowVariant variant ) = 0;
+    virtual double      GetContentScaleFactor() const
+    {
+        return 1.0;
+    }
+    
+    // the native coordinates may have an 'aura' for shadows etc, if this is the case the layout
+    // inset indicates on which insets the real control is drawn
+    virtual void        GetLayoutInset(int &left , int &top , int &right, int &bottom) const
+    {
+        left = top = right = bottom = 0;
+    }
 
     // native view coordinates are topleft to bottom right (flipped regarding CoreGraphics origin)
     virtual bool        IsFlipped() const { return true; }
@@ -235,6 +284,8 @@ public :
 
     virtual bool        NeedsFrame() const;
     virtual void        SetNeedsFrame( bool needs );
+    
+    virtual void        SetDrawingEnabled(bool enabled);
 
     virtual bool        CanFocus() const = 0;
     // return true if successful
@@ -247,17 +298,23 @@ public :
     virtual void        SetDefaultButton( bool isDefault ) = 0;
     virtual void        PerformClick() = 0;
     virtual void        SetLabel( const wxString& title, wxFontEncoding encoding ) = 0;
+#if wxUSE_MARKUP && wxOSX_USE_COCOA
+    virtual void        SetLabelMarkup( const wxString& WXUNUSED(markup) ) { }
+#endif
 
     virtual void        SetCursor( const wxCursor & cursor ) = 0;
     virtual void        CaptureMouse() = 0;
     virtual void        ReleaseMouse() = 0;
+    
+    virtual void        SetDropTarget( wxDropTarget * WXUNUSED(dropTarget) ) {}
 
     virtual wxInt32     GetValue() const = 0;
     virtual void        SetValue( wxInt32 v ) = 0;
     virtual wxBitmap    GetBitmap() const = 0;
     virtual void        SetBitmap( const wxBitmap& bitmap ) = 0;
     virtual void        SetBitmapPosition( wxDirection dir ) = 0;
-    virtual void        SetupTabs( const wxNotebook &notebook ) =0;
+    virtual void        SetupTabs( const wxNotebook& WXUNUSED(notebook) ) {}
+    virtual int         TabHitTest( const wxPoint & WXUNUSED(pt), long *flags ) {*flags=1; return -1;};
     virtual void        GetBestRect( wxRect *r ) const = 0;
     virtual bool        IsEnabled() const = 0;
     virtual void        Enable( bool enable ) = 0;
@@ -269,7 +326,7 @@ public :
     virtual void        SetScrollThumb( wxInt32 value, wxInt32 thumbSize ) = 0;
 
     virtual void        SetFont( const wxFont & font , const wxColour& foreground , long windowStyle, bool ignoreBlack = true ) = 0;
-    
+
     virtual void        SetToolTip(wxToolTip* WXUNUSED(tooltip)) { }
 
     // is the clicked event sent AFTER the state already changed, so no additional
@@ -277,7 +334,7 @@ public :
     virtual bool        ButtonClickDidStateChange() = 0;
 
     virtual void        InstallEventHandler( WXWidget control = NULL ) = 0;
-    
+
     // Mechanism used to keep track of whether a change should send an event
     // Do SendEvents(false) when starting actions that would trigger programmatic events
     // and SendEvents(true) at the end of the block.
@@ -286,9 +343,16 @@ public :
 
     // static methods for associating native controls and their implementations
 
+    // finds the impl associated with this native control
     static wxWidgetImpl*
                         FindFromWXWidget(WXWidget control);
 
+    // finds the impl associated with this native control, if the native control itself is not known
+    // also checks whether its parent is eg a registered scrollview, ie whether the control is a native subpart
+    // of a known control
+    static wxWidgetImpl*
+                        FindBestFromWXWidget(WXWidget control);
+    
     static void         RemoveAssociations( wxWidgetImpl* impl);
 
     static void         Associate( WXWidget control, wxWidgetImpl *impl );
@@ -359,7 +423,7 @@ public :
                                     long style,
                                     long extraStyle) ;
 
-    static wxWidgetImplType*    CreateSearchControl( wxTextCtrl* wxpeer,
+    static wxWidgetImplType*    CreateSearchControl( wxSearchCtrl* wxpeer,
                                     wxWindowMac* parent,
                                     wxWindowID id,
                                     const wxString& content,
@@ -480,13 +544,13 @@ public :
                                     long extraStyle);
 
 #if wxOSX_USE_COCOA
-    static wxWidgetImplType*    CreateComboBox( wxWindowMac* wxpeer, 
-                                    wxWindowMac* parent, 
-                                    wxWindowID id, 
+    static wxWidgetImplType*    CreateComboBox( wxComboBox* wxpeer,
+                                    wxWindowMac* parent,
+                                    wxWindowID id,
                                     wxMenu* menu,
-                                    const wxPoint& pos, 
+                                    const wxPoint& pos,
                                     const wxSize& size,
-                                    long style, 
+                                    long style,
                                     long extraStyle);
 #endif
 
@@ -494,6 +558,7 @@ public :
     static void Convert( wxPoint *pt , wxWidgetImpl *from , wxWidgetImpl *to );
 protected :
     bool                m_isRootControl;
+    bool                m_isUserPane;
     wxWindowMac*        m_wxPeer;
     bool                m_needsFocusRect;
     bool                m_needsFrame;
@@ -563,7 +628,7 @@ public:
     // accessing content
 
     virtual unsigned int    ListGetCount() const = 0;
-    
+
     virtual int             DoListHitTest( const wxPoint& inpoint ) const = 0;
 };
 
@@ -572,16 +637,23 @@ public:
 //
 
 class WXDLLIMPEXP_FWD_CORE wxTextAttr;
+class WXDLLIMPEXP_FWD_CORE wxTextEntry;
 
 // common interface for all implementations
 class WXDLLIMPEXP_CORE wxTextWidgetImpl
 
 {
 public :
-    wxTextWidgetImpl() {}
+    // Any widgets implementing this interface must be associated with a
+    // wxTextEntry so instead of requiring the derived classes to implement
+    // another (pure) virtual function, just take the pointer to this entry in
+    // our ctor and implement GetTextEntry() ourselves.
+    wxTextWidgetImpl(wxTextEntry *entry) : m_entry(entry) {}
 
     virtual ~wxTextWidgetImpl() {}
 
+    wxTextEntry *GetTextEntry() const { return m_entry; }
+
     virtual bool CanFocus() const { return true; }
 
     virtual wxString GetStringValue() const = 0 ;
@@ -590,6 +662,9 @@ public :
     virtual void GetSelection( long* from, long* to ) const = 0 ;
     virtual void WriteText( const wxString& str ) = 0 ;
 
+    virtual bool CanClipMaxLength() const { return false; }
+    virtual void SetMaxLength(unsigned long WXUNUSED(len)) {}
+    
     virtual bool GetStyle( long position, wxTextAttr& style);
     virtual void SetStyle( long start, long end, const wxTextAttr& style ) ;
     virtual void Copy() ;
@@ -620,8 +695,14 @@ public :
     virtual int GetLineLength(long lineNo) const ;
     virtual wxString GetLineText(long lineNo) const ;
     virtual void CheckSpelling(bool WXUNUSED(check)) { }
-    
+
     virtual wxSize GetBestSize() const { return wxDefaultSize; }
+
+    virtual bool SetHint(const wxString& WXUNUSED(hint)) { return false; }
+private:
+    wxTextEntry * const m_entry;
+
+    wxDECLARE_NO_COPY_CLASS(wxTextWidgetImpl);
 };
 
 // common interface for all implementations
@@ -632,20 +713,22 @@ public :
     wxComboWidgetImpl() {}
 
     virtual ~wxComboWidgetImpl() {}
-  
-    virtual int GetSelectedItem() const { return -1; };
-    virtual void SetSelectedItem(int WXUNUSED(item)) {};
-    
-    virtual int GetNumberOfItems() const { return -1; };
-    
+
+    virtual int GetSelectedItem() const { return -1; }
+    virtual void SetSelectedItem(int WXUNUSED(item)) {}
+
+    virtual int GetNumberOfItems() const { return -1; }
+
     virtual void InsertItem(int WXUNUSED(pos), const wxString& WXUNUSED(item)) {}
-    
+
     virtual void RemoveItem(int WXUNUSED(pos)) {}
-    
+
     virtual void Clear() {}
-    
+    virtual void Popup() {}
+    virtual void Dismiss() {}
+
     virtual wxString GetStringAtIndex(int WXUNUSED(pos)) const { return wxEmptyString; }
-    
+
     virtual int FindString(const wxString& WXUNUSED(text)) const { return -1; }
 };
 
@@ -658,7 +741,7 @@ class wxButtonImpl
     public :
     wxButtonImpl(){}
     virtual ~wxButtonImpl(){}
-    
+
     virtual void SetPressedBitmap( const wxBitmap& bitmap ) = 0;
 } ;
 
@@ -746,11 +829,11 @@ public :
     virtual void SetExtraStyle( long WXUNUSED(exStyle) )
     {
     }
-    
+
     virtual void SetWindowStyleFlag( long WXUNUSED(style) )
     {
     }
-    
+
     virtual bool SetBackgroundStyle(wxBackgroundStyle WXUNUSED(style))
     {
         return false ;
@@ -784,7 +867,7 @@ public :
     virtual bool IsFullScreen() const= 0;
 
     virtual void ShowWithoutActivating() { Show(true); }
-    
+
     virtual bool ShowFullScreen(bool show, long style)= 0;
 
     virtual void RequestUserAttention(int flags) = 0;
@@ -792,28 +875,36 @@ public :
     virtual void ScreenToWindow( int *x, int *y ) = 0;
 
     virtual void WindowToScreen( int *x, int *y ) = 0;
-
-    virtual bool IsActive() = 0;
     
+    virtual bool IsActive() = 0;
+
     wxNonOwnedWindow*   GetWXPeer() { return m_wxPeer; }
 
     static wxNonOwnedWindowImpl*
                 FindFromWXWindow(WXWindow window);
-    
+
     static void  RemoveAssociations( wxNonOwnedWindowImpl* impl);
-    
+
     static void  Associate( WXWindow window, wxNonOwnedWindowImpl *impl );
-    
+
     // static creation methods, must be implemented by all toolkits
 
     static wxNonOwnedWindowImpl* CreateNonOwnedWindow( wxNonOwnedWindow* wxpeer, wxWindow* parent, WXWindow native) ;
-    
+
     static wxNonOwnedWindowImpl* CreateNonOwnedWindow( wxNonOwnedWindow* wxpeer, wxWindow* parent, const wxPoint& pos, const wxSize& size,
     long style, long extraStyle, const wxString& name  ) ;
-    
+
     virtual void SetModified(bool WXUNUSED(modified)) { }
     virtual bool IsModified() const { return false; }
 
+    virtual void SetRepresentedFilename(const wxString& WXUNUSED(filename)) { }
+
+#if wxOSX_USE_IPHONE
+    virtual CGFloat GetWindowLevel() const { return 0.0; }
+#else
+    virtual CGWindowLevel GetWindowLevel() const { return kCGNormalWindowLevel; }
+#endif
+    virtual void RestoreWindowLevel() {}
 protected :
     wxNonOwnedWindow*   m_wxPeer;
     DECLARE_ABSTRACT_CLASS(wxNonOwnedWindowImpl)
