@@ -1,10 +1,10 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        mac/dataobj.h
+// Name:        wx/osx/dataobj.h
 // Purpose:     declaration of the wxDataObject
 // Author:      Stefan Csomor (adapted from Robert Roebling's gtk port)
 // Modified by:
 // Created:     10/21/99
-// RCS-ID:      $Id: dataobj.h 58168 2009-01-17 10:43:43Z SC $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998, 1999 Vadim Zeitlin, Robert Roebling
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -31,6 +31,10 @@ public:
     // returns true if any of the accepted formats of this dataobj is in the pasteboard
     bool HasDataInPasteboard( void * pasteboardRef );
     bool GetFromPasteboard( void * pasteboardRef );
+    
+#if wxOSX_USE_COCOA
+    virtual void AddSupportedTypes( void* cfarray);
+#endif
 };
 
 #endif // _WX_MAC_DATAOBJ_H_
