@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:
-// RCS-ID:      $Id: popupwin.h 64943 2010-07-13 13:29:58Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2006 Stefan Csomor
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -26,10 +26,7 @@ public:
         { (void)Create(parent, flags); }
 
     bool Create(wxWindow *parent, int flags = wxBORDER_NONE);
-    
-    virtual bool Show(bool show);
 
-protected:
     DECLARE_DYNAMIC_CLASS_NO_COPY(wxPopupWindow)
 };
 
