@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        wx/propgeid/propgridiface.h
+// Name:        wx/propgrid/propgridiface.h
 // Purpose:     wxPropertyGridInterface class
 // Author:      Jaakko Salli
 // Modified by:
 // Created:     2008-08-24
-// RCS-ID:      $Id: propgridiface.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Jaakko Salli
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -12,6 +12,8 @@
 #ifndef __WX_PROPGRID_PROPGRIDIFACE_H__
 #define __WX_PROPGRID_PROPGRIDIFACE_H__
 
+#include "wx/defs.h"
+
 #if wxUSE_PROPGRID
 
 #include "wx/propgrid/property.h"
@@ -27,7 +29,6 @@
 class WXDLLIMPEXP_PROPGRID wxPGPropArgCls
 {
 public:
-    wxPGPropArgCls() { }
     wxPGPropArgCls( const wxPGProperty* property )
     {
         m_ptr.property = (wxPGProperty*) property;
@@ -267,7 +268,11 @@ public:
     */
     wxPGProperty* RemoveProperty( wxPGPropArg id );
 
-    /** Disables property. */
+    /**
+        Disables a property.
+
+        @see EnableProperty(), wxPGProperty::Enable()
+    */
     bool DisableProperty( wxPGPropArg id ) { return EnableProperty(id,false); }
 
     /**
@@ -280,7 +285,14 @@ public:
 
     /**
         Enables or disables property, depending on whether enable is true or
-        false.
+        false. Disabled property usually appears as having grey text.
+
+        @param id
+            Name or pointer to a property.
+        @param enable
+            If @false, property is disabled instead.
+
+        @see wxPGProperty::Enable()
      */
     bool EnableProperty( wxPGPropArg id, bool enable = true );
 
@@ -368,7 +380,7 @@ public:
         If there is no property with such name, @NULL pointer is returned.
 
         @remarks Properties which have non-category, non-root parent
-                 can not be accessed globally by their name. Instead, use
+                 cannot be accessed globally by their name. Instead, use
                  "<property>.<subproperty>" instead of "<subproperty>".
     */
     wxPGProperty* GetProperty( const wxString& name ) const
@@ -866,7 +878,7 @@ public:
     wxString SaveEditableState( int includedStates = AllStates ) const;
 
     /**
-        Lets user to set the strings listed in the choice dropdown of a
+        Lets user set the strings listed in the choice dropdown of a
         wxBoolProperty. Defaults are "True" and "False", so changing them to,
         say, "Yes" and "No" may be useful in some less technical applications.
     */
@@ -876,7 +888,7 @@ public:
     /**
         Set proportion of a auto-stretchable column. wxPG_SPLITTER_AUTO_CENTER
         window style needs to be used to indicate that columns are auto-
-        resizeable.
+        resizable.
 
         @returns Returns @false on failure.
 
@@ -1372,7 +1384,8 @@ private:
     {
         if ( !m_pState )
             return NULL;
-        return static_cast<const wxPropertyGrid*>(m_pState->GetGrid());
+
+        return m_pState->GetGrid();
     }
 
     friend class wxPropertyGrid;
