@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     10/5/2006 3:11:58 PM
-// RCS-ID:      $Id: richtextsymboldlg.h 62699 2009-11-23 09:34:07Z JS $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,6 +16,7 @@
  * Includes
  */
 
+#include "wx/richtext/richtextuicustomization.h"
 #include "wx/dialog.h"
 #include "wx/vscroll.h"
 
@@ -29,6 +30,7 @@ class WXDLLIMPEXP_FWD_CORE wxTextCtrl;
 
 ////@begin forward declarations
 class wxSymbolListCtrl;
+class wxStdDialogButtonSizer;
 ////@end forward declarations
 
 // __UNICODE__ is a symbol used by DialogBlocks-generated code.
@@ -56,6 +58,7 @@ class WXDLLIMPEXP_RICHTEXT wxSymbolPickerDialog: public wxDialog
 {
     DECLARE_DYNAMIC_CLASS( wxSymbolPickerDialog )
     DECLARE_EVENT_TABLE()
+    DECLARE_HELP_PROVISION()
 
 public:
     /// Constructors
@@ -105,11 +108,11 @@ public:
 
 ////@begin wxSymbolPickerDialog event handler declarations
 
-    /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_SYMBOLPICKERDIALOG_FONT
+    /// wxEVT_COMBOBOX event handler for ID_SYMBOLPICKERDIALOG_FONT
     void OnFontCtrlSelected( wxCommandEvent& event );
 
 #if defined(__UNICODE__)
-    /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_SYMBOLPICKERDIALOG_SUBSET
+    /// wxEVT_COMBOBOX event handler for ID_SYMBOLPICKERDIALOG_SUBSET
     void OnSubsetSelected( wxCommandEvent& event );
 
     /// wxEVT_UPDATE_UI event handler for ID_SYMBOLPICKERDIALOG_SUBSET
@@ -117,13 +120,19 @@ public:
 
 #endif
 #if defined(__UNICODE__)
-    /// wxEVT_COMMAND_COMBOBOX_SELECTED event handler for ID_SYMBOLPICKERDIALOG_FROM
+    /// wxEVT_COMBOBOX event handler for ID_SYMBOLPICKERDIALOG_FROM
     void OnFromUnicodeSelected( wxCommandEvent& event );
 
 #endif
     /// wxEVT_UPDATE_UI event handler for wxID_OK
     void OnOkUpdate( wxUpdateUIEvent& event );
 
+    /// wxEVT_BUTTON event handler for wxID_HELP
+    void OnHelpClick( wxCommandEvent& event );
+
+    /// wxEVT_UPDATE_UI event handler for wxID_HELP
+    void OnHelpUpdate( wxUpdateUIEvent& event );
+
 ////@end wxSymbolPickerDialog event handler declarations
 
 ////@begin wxSymbolPickerDialog member function declarations
@@ -131,15 +140,15 @@ public:
     wxString GetFontName() const { return m_fontName ; }
     void SetFontName(wxString value) { m_fontName = value ; }
 
-    wxString GetSymbol() const { return m_symbol ; }
-    void SetSymbol(wxString value) { m_symbol = value ; }
-
     bool GetFromUnicode() const { return m_fromUnicode ; }
     void SetFromUnicode(bool value) { m_fromUnicode = value ; }
 
     wxString GetNormalTextFontName() const { return m_normalTextFontName ; }
     void SetNormalTextFontName(wxString value) { m_normalTextFontName = value ; }
 
+    wxString GetSymbol() const { return m_symbol ; }
+    void SetSymbol(wxString value) { m_symbol = value ; }
+
     /// Retrieves bitmap resources
     wxBitmap GetBitmapResource( const wxString& name );
 
@@ -158,10 +167,11 @@ public:
 #if defined(__UNICODE__)
     wxComboBox* m_fromUnicodeCtrl;
 #endif
+    wxStdDialogButtonSizer* m_stdButtonSizer;
     wxString m_fontName;
-    wxString m_symbol;
     bool m_fromUnicode;
     wxString m_normalTextFontName;
+    wxString m_symbol;
     /// Control identifiers
     enum {
         ID_SYMBOLPICKERDIALOG = 10600,
@@ -299,7 +309,7 @@ protected:
     // common part of all ctors
     void Init();
 
-    // send the wxEVT_COMMAND_LISTBOX_SELECTED event
+    // send the wxEVT_LISTBOX event
     void SendSelectedEvent();
 
     // change the current item (in single selection listbox it also implicitly
