@@ -3,7 +3,7 @@
 // Purpose:     Private declarations for wxSTC
 // Author:      Robin Dunn
 // Created:     2007-07-15
-// RCS-ID:      $Id: private.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2000 by Total Control Software
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -21,7 +21,16 @@
 
 extern wxString stc2wx(const char* str);
 extern wxString stc2wx(const char* str, size_t len);
-extern const wxWX2MBbuf wx2stc(const wxString& str);
+extern wxCharBuffer wx2stc(const wxString& str);
+
+// This function takes both wxString and wxCharBuffer because it uses either
+// one or the other of them depending on the build mode. In Unicode it uses the
+// length of the already converted buffer to avoid doing the conversion again
+// just to compute the length.
+inline size_t wx2stclen(const wxString& WXUNUSED(str), const wxCharBuffer& buf)
+{
+    return buf.length() - 1;
+}
 
 #else // not UNICODE
 
@@ -31,10 +40,16 @@ inline wxString stc2wx(const char* str) {
 inline wxString stc2wx(const char* str, size_t len) {
     return wxString(str, len);
 }
-inline const wxWX2MBbuf wx2stc(const wxString& str) {
+inline const char* wx2stc(const wxString& str) {
     return str.mbc_str();
 }
 
+// As explained above, the buffer argument is only used in Unicode build.
+inline size_t wx2stclen(const wxString& str, const char* WXUNUSED(buf))
+{
+    return str.length();
+}
+
 #endif // UNICODE
 
 #endif // _WX_STC_PRIVATE_H_
