@@ -2,7 +2,7 @@
 // Name:        dateevt.h
 // Purpose:     interface of wxDateEvent
 // Author:      wxWidgets team
-// RCS-ID:      $Id: dateevt.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -19,6 +19,9 @@
 class wxDateEvent : public wxCommandEvent
 {
 public:
+    wxDateEvent();
+    wxDateEvent(wxWindow *win, const wxDateTime& dt, wxEventType type);
+
     /**
         Returns the date.
     */
@@ -31,3 +34,5 @@ public:
     void SetDate(const wxDateTime& date);
 };
 
+wxEventType wxEVT_DATE_CHANGED;
+wxEventType wxEVT_TIME_CHANGED;
