@@ -2,7 +2,7 @@
 // Name:        dcgraph.h
 // Purpose:     interface of wxGCDC
 // Author:      wxWidgets team
-// RCS-ID:      $Id: dcgraph.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -23,21 +23,46 @@ public:
     /**
        Constructs a wxGCDC from a wxWindowDC.
     */
-    wxGCDC( const wxWindowDC& dc );
+    wxGCDC( const wxWindowDC& windowDC );
 
     /**
        Constructs a wxGCDC from a wxMemoryDC.
     */
-    wxGCDC( const wxMemoryDC& dc );
+    wxGCDC( const wxMemoryDC& memoryDC );
 
     /**
        Constructs a wxGCDC from a wxPrinterDC.
     */
-    wxGCDC( const wxPrinterDC& dc );
+    wxGCDC( const wxPrinterDC& printerDC );
 
+    /**
+       Construct a wxGCDC from an existing graphics context.
+    */
+    wxGCDC(wxGraphicsContext* context);
+
+    /**
+       Constructs a wxGCDC from a wxEnhMetaFileDC.
+
+       This constructor is only available in wxMSW port and when @c
+       wxUSE_ENH_METAFILE build option is enabled, i.e. when wxEnhMetaFileDC
+       class itself is available.
+
+       @since 2.9.3
+    */
+    wxGCDC( const wxEnhMetaFileDC& emfDC );
+
+    wxGCDC();
+    virtual ~wxGCDC();
+    
     /**
        Retrieves associated wxGraphicsContext
     */
-    wxGraphicsContext* GetGraphicsContext();
+    wxGraphicsContext* GetGraphicsContext() const;
+
+    /**
+       Set the grpahics context to be used for this wxGCDC.
+    */
+    void SetGraphicsContext( wxGraphicsContext* ctx );
+
 };
 
