@@ -2,7 +2,7 @@
 // Name:        dynarray.h
 // Purpose:     interface of wxArray<T>
 // Author:      wxWidgets team
-// RCS-ID:      $Id: dynarray.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -32,7 +32,7 @@
     functions are inline, so they do not take @e any space at all.
 
     wxWidgets has three different kinds of array. All of them derive from
-    wxBaseArray class which works with untyped data and can not be used
+    wxBaseArray class which works with untyped data and cannot be used
     directly. The standard macros WX_DEFINE_ARRAY(), WX_DEFINE_SORTED_ARRAY()
     and WX_DEFINE_OBJARRAY() are used to define a new class deriving from it.
     The classes declared will be called in this documentation wxArray,
@@ -124,7 +124,7 @@
     however dumb, C++ compiler in the world.
 
     Remember to include @<wx/arrimpl.cpp@> just before each
-    WX_DEFINE_OBJARRAY() ocurrence in your code, even if you have several in
+    WX_DEFINE_OBJARRAY() occurrence in your code, even if you have several in
     the same file.
 
     Things are much simpler for wxArray and wxSortedArray however: it is enough
@@ -254,36 +254,36 @@ public:
     wxSortedArray(int (*)(T first, T second)compareFunction);
 
     /**
-        Performs a shallow array copy (i.e. doesn't copy the objects pointed to
+        Performs a shallow array copy (i.e.\ doesn't copy the objects pointed to
         even if the source array contains the items of pointer type).
     */
     wxArray(const wxArray& array);
 
     /**
-        Performs a shallow array copy (i.e. doesn't copy the objects pointed to
+        Performs a shallow array copy (i.e.\ doesn't copy the objects pointed to
         even if the source array contains the items of pointer type).
     */
     wxSortedArray(const wxSortedArray& array);
 
     /**
-        Performs a deep copy (i.e. the array element are copied too).
+        Performs a deep copy (i.e.\ the array element are copied too).
     */
     wxObjArray(const wxObjArray& array);
 
     /**
-        Performs a shallow array copy (i.e. doesn't copy the objects pointed to
+        Performs a shallow array copy (i.e.\ doesn't copy the objects pointed to
         even if the source array contains the items of pointer type).
     */
     wxArray& operator=(const wxArray& array);
 
     /**
-        Performs a shallow array copy (i.e. doesn't copy the objects pointed to
+        Performs a shallow array copy (i.e.\ doesn't copy the objects pointed to
         even if the source array contains the items of pointer type).
     */
     wxSortedArray& operator=(const wxSortedArray& array);
 
     /**
-        Performs a deep copy (i.e. the array element are copied too).
+        Performs a deep copy (i.e.\ the array element are copied too).
     */
     wxObjArray& operator=(const wxObjArray& array);
 
@@ -376,7 +376,7 @@ public:
     T& Item(size_t index) const;
 
     /**
-        Returns the last element in the array, i.e. is the same as calling
+        Returns the last element in the array, i.e.\ is the same as calling
         "Item(GetCount() - 1)". An assert failure is raised in the debug mode
         if the array is empty.
 
@@ -659,7 +659,7 @@ public:
 
 /**
     This macro may be used to delete all elements of the array before emptying
-    it. It can not be used with wxObjArrays - but they will delete their
+    it. It cannot be used with wxObjArrays - but they will delete their
     elements anyway when you call Empty().
 */
 #define WX_CLEAR_ARRAY(wxArray_arrayToBeCleared)
