@@ -2,7 +2,7 @@
 // Name:        iconloc.h
 // Purpose:     interface of wxIconLocation
 // Author:      wxWidgets team
-// RCS-ID:      $Id: iconloc.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -30,9 +30,13 @@ class wxIconLocation
 {
 public:
     /**
-        Returns @true if the object is valid, i.e. was properly initialized, and
+        Returns @true if the object is valid, i.e.\ was properly initialized, and
         @false otherwise.
     */
     bool IsOk() const;
+
+    void SetFileName(const wxString& filename);
+    const wxString& GetFileName() const;
+
 };
 
