@@ -2,7 +2,7 @@
 // Name:        wx/persist/treebook.h
 // Purpose:     interface of wxPersistentTreeBook
 // Author:      Vadim Zeitlin
-// RCS-ID:      $Id: treebook.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2009 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -14,7 +14,7 @@
     used by wxTreebook, in addition to saving and restoring the selection as
     implemented by the base wxPersistentBookCtrl class.
  */
-class wxPersistentTreeBook : public wxPersistentBookCtrl
+class wxPersistentTreeBookCtrl : public wxPersistentBookCtrl
 {
 public:
     /**
@@ -23,7 +23,7 @@ public:
         @param book
             The associated tree book control.
      */
-    wxPersistentTreeBook(wxTreebook *book);
+    wxPersistentTreeBookCtrl(wxTreebook *book);
 
     /**
         Save the currently opened branches.
