@@ -2,7 +2,7 @@
 // Name:        manager.h
 // Purpose:     interface of wxPropertyGridManager
 // Author:      wxWidgets team
-// RCS-ID:      $Id: manager.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -147,7 +147,7 @@ public:
     acquire the internal grid (GetGrid()) or wxPropertyGridPage object (GetPage()).
 
     wxPropertyGridManager constructor has exact same format as wxPropertyGrid
-    constructor, and basicly accepts same extra window style flags (albeit also
+    constructor, and basically accepts same extra window style flags (albeit also
     has some extra ones).
 
     Here's some example code for creating and populating a wxPropertyGridManager:
@@ -256,7 +256,7 @@ public:
                  const wxPoint& pos = wxDefaultPosition,
                  const wxSize& size = wxDefaultSize,
                  long style = wxPGMAN_DEFAULT_STYLE,
-                 const wxChar* name = wxPropertyGridManagerNameStr );
+                 const wxString& name = wxPropertyGridManagerNameStr );
 
     /**
         Enables or disables (shows/hides) categories according to parameter enable.
@@ -340,11 +340,11 @@ public:
     /** Returns index to currently selected page. */
     int GetSelectedPage() const;
 
-    /** Shortcut for GetGrid()->GetSelection(). */
+    /** Alias for GetSelection(). */
     wxPGProperty* GetSelectedProperty() const;
 
-    /** Synonyme for GetSelectedPage. */
-    int GetSelection() const;
+    /** Shortcut for GetGrid()->GetSelection(). */
+    wxPGProperty* GetSelection() const;
 
     /**
         Returns a pointer to the toolbar currently associated with the
@@ -467,6 +467,11 @@ public:
     */
     void SetSplitterLeft( bool subProps = false, bool allPages = true );
 
+    /** Moves splitter as left as possible on an individual page, while still allowing all
+        labels to be shown in full.
+    */
+    void SetPageSplitterLeft(int page, bool subProps = false);
+
     /**
         Sets splitter position on individual page.
 
@@ -490,9 +495,6 @@ public:
     */
     void SetSplitterPosition( int pos, int column = 0 );
 
-    /** Synonyme for SelectPage(name). */
-    void SetStringSelection( const wxChar* name );
-
     /**
         Show or hide the property grid header control. It is hidden
         by the default.
@@ -510,7 +512,7 @@ protected:
 
     /**
         Creates property grid for the manager. Reimplement in derived class to
-        use subclassed wxPropertyGrid. However, if you you do this then you
+        use subclassed wxPropertyGrid. However, if you do this then you
         must also use the two-step construction (ie. default constructor and
         Create() instead of constructor with arguments) when creating the
         manager.
