@@ -2,7 +2,7 @@
 // Name:        property.h
 // Purpose:     interface of wxPGProperty
 // Author:      wxWidgets team
-// RCS-ID:      $Id: propgridiface.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -153,6 +153,8 @@ public:
 
     /**
         Disables a property.
+
+        @see EnableProperty(), wxPGProperty::Enable()
     */
     bool DisableProperty( wxPGPropArg id );
 
@@ -164,12 +166,15 @@ public:
     bool EditorValidate();
 
     /**
-        Enables or disables property.
+        Enables or disables property. Disabled property usually appears as
+        having grey text.
 
         @param id
             Name or pointer to a property.
         @param enable
             If @false, property is disabled instead.
+
+        @see wxPGProperty::Enable()
     */
     bool EnableProperty( wxPGPropArg id, bool enable = true );
 
@@ -225,14 +230,6 @@ public:
             Property to start iteration from. If @NULL, then first child of root
             is used.
 
-        @beginWxPythonOnly
-        Instead of ++ operator, use Next() method, and instead of * operator,
-        use GetProperty() method. There is also GetPyIterator() method (which
-        takes the same arguments but instead returns a pythonic iterator),
-        @c Properties attribute which is an iterator over all non-category,
-        non-private properties, and finally @c Items attribute which is an
-        iterator over all items except private child properties.
-        @endWxPythonOnly
     */
     wxPropertyGridIterator GetIterator( int flags = wxPG_ITERATE_DEFAULT,
                                         wxPGProperty* firstProp = NULL );
@@ -252,14 +249,6 @@ public:
             from the first property from the top, and wxBOTTOM means that the
             iteration will instead begin from bottommost valid item.
 
-        @beginWxPythonOnly
-        Instead of ++ operator, use Next() method, and instead of * operator,
-        use GetProperty() method. There is also GetPyIterator() method (which
-        takes the same arguments but instead returns a pythonic iterator),
-        @c Properties attribute which is an iterator over all non-category,
-        non-private properties, and finally @c Items attribute which is an
-        iterator over all items except private child properties.
-        @endWxPythonOnly
     */
     wxPropertyGridIterator GetIterator( int flags, int startPos );
     wxPropertyGridConstIterator GetIterator( int flags, int startPos ) const;
@@ -278,7 +267,7 @@ public:
         If there is no property with such name, @NULL pointer is returned.
 
         @remarks Properties which have non-category, non-root parent
-                 can not be accessed globally by their name. Instead, use
+                 cannot be accessed globally by their name. Instead, use
                  "<property>.<subproperty>" instead of "<subproperty>".
     */
     wxPGProperty* GetProperty( const wxString& name ) const;
@@ -334,7 +323,7 @@ public:
         If there is no property with such name, @NULL pointer is returned.
 
         @remarks Properties which have non-category, non-root parent
-                 can not be accessed globally by their name. Instead, use
+                 cannot be accessed globally by their name. Instead, use
                  "<property>.<subproperty>" instead of "<subproperty>".
     */
     wxPGProperty* GetPropertyByName( const wxString& name ) const;
@@ -464,15 +453,6 @@ public:
 
         @param flags
             See @ref propgrid_iterator_flags.
-
-        @beginWxPythonOnly
-        Instead of ++ operator, use Next() method, and instead of * operator,
-        use GetProperty() method. There is also GetPyVIterator() method (which
-        takes the same arguments but instead returns a pythonic iterator),
-        @c Properties attribute which is an iterator over all non-category,
-        non-private properties, and finally @c Items attribute which is an
-        iterator over all items except private child properties.
-        @endWxPythonOnly
     */
     virtual wxPGVIterator GetVIterator( int flags ) const;
 
@@ -715,7 +695,7 @@ public:
     /**
         Set proportion of a auto-stretchable column. wxPG_SPLITTER_AUTO_CENTER
         window style needs to be used to indicate that columns are auto-
-        resizeable.
+        resizable.
 
         @returns Returns @false on failure.
 
