@@ -2,7 +2,7 @@
 // Name:        renderer.h
 // Purpose:     interface of wxRendererNative
 // Author:      wxWidgets team
-// RCS-ID:      $Id: renderer.h 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -43,7 +43,7 @@ enum
     /** Mouse is currently over the control. */
     wxCONTROL_CURRENT    = 0x00000010,
 
-    /** Selected item in e.g. listbox. */
+    /** Selected item in e.g.\ listbox. */
     wxCONTROL_SELECTED   = 0x00000020,
 
     /** (Check/radio button) is checked. */
@@ -77,7 +77,7 @@ enum wxTitleBarButton
     It doesn't have any methods and all of its fields are constant, so they can
     only be examined but not modified.
 
-    @library{wxbase}
+    @library{wxcore}
     @category{gdi}
 */
 struct wxSplitterRenderParams
@@ -111,7 +111,7 @@ struct wxSplitterRenderParams
     wxRendererNative::DrawHeaderButton() to specify custom values used to draw
     the text or bitmap label.
 
-    @library{wxbase}
+    @library{wxcore}
     @category{gdi}
 */
 struct wxHeaderButtonParams
@@ -204,6 +204,8 @@ public:
 
     virtual int GetHeaderButtonHeight(wxWindow *win);
 
+    virtual int GetHeaderButtonMargin(wxWindow *win);
+
     virtual void DrawTreeItemButton(wxWindow *win, wxDC& dc,
                                     const wxRect& rect, int flags = 0);
 
@@ -348,7 +350,7 @@ public:
         @c wxCONTROL_DISABLED and @c wxCONTROL_CURRENT bits, see @ref wxCONTROL_FLAGS.
 
         @return
-        The optimal width to contain the the unabreviated label text or
+        The optimal width to contain the unabbreviated label text or
         bitmap, the sort arrow if present, and internal margins.
     */
     virtual int DrawHeaderButton(wxWindow* win, wxDC& dc, const wxRect& rect,
@@ -363,7 +365,7 @@ public:
         @c wxCONTROL_DISABLED and @c wxCONTROL_CURRENT bits, see @ref wxCONTROL_FLAGS.
 
         @return
-        The optimal width to contain the the unabreviated label text or
+        The optimal width to contain the unabbreviated label text or
         bitmap, the sort arrow if present, and internal margins.
     */
     virtual int DrawHeaderButtonContents(wxWindow* win, wxDC& dc,
@@ -378,7 +380,7 @@ public:
         which are selected (e.g. often a blue rectangle) and @c wxCONTROL_CURRENT
         for the item that has the focus (often a dotted line around the item's text).
         @c wxCONTROL_FOCUSED may be used to indicate if the control has the focus
-        (othewise the the selection rectangle is e.g. often grey and not blue).
+        (otherwise the selection rectangle is e.g. often grey and not blue).
         This may be ignored by the renderer or deduced by the code directly from
         the @a win.
     */
@@ -494,6 +496,14 @@ public:
     */
     virtual int GetHeaderButtonHeight(wxWindow* win) = 0;
 
+    /**
+        Returns the horizontal margin on the left and right sides of header
+        button's label.
+
+        @since 2.9.2
+     */
+    virtual int GetHeaderButtonMargin(wxWindow *win) = 0;
+
     /**
         Get the splitter parameters, see wxSplitterRenderParams.
         The @a win parameter should be a wxSplitterWindow.
@@ -551,11 +561,13 @@ public:
     renderer which has the age greater or equal to its age. This verification is
     done by IsCompatible() method.
 
-    @library{wxbase}
+    @library{wxcore}
     @category{gdi}
 */
 struct wxRendererVersion
 {
+    wxRendererVersion(int version_, int age_);
+    
     /**
         Checks if the main program is compatible with the renderer having the version
         @e ver, returns @true if it is and @false otherwise.
