@@ -4,7 +4,7 @@
 // Author:      Robert Roebling
 // Modified by: Francesco Montorsi, Bo Yang
 // Created:     06/01/06
-// RCS-ID:      $Id: dataview.cpp 64957 2010-07-14 11:12:03Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -41,7 +41,7 @@
 // resources
 // ----------------------------------------------------------------------------
 
-#ifndef __WXMSW__
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -88,6 +88,8 @@ private:
     void OnSelectNinth(wxCommandEvent& event);
     void OnCollapse(wxCommandEvent& event);
     void OnExpand(wxCommandEvent& event);
+    void OnShowCurrent(wxCommandEvent& event);
+    void OnSetNinthCurrent(wxCommandEvent& event);
 
     void OnPrependList(wxCommandEvent& event);
     void OnDeleteList(wxCommandEvent& event);
@@ -117,15 +119,16 @@ private:
 
     void OnContextMenu( wxDataViewEvent &event );
 
-    void OnRightClick( wxMouseEvent &event );
     void OnGoto( wxCommandEvent &event);
     void OnAddMany( wxCommandEvent &event);
     void OnHideAttributes( wxCommandEvent &event);
     void OnShowAttributes( wxCommandEvent &event);
 
+#if wxUSE_DRAG_AND_DROP
     void OnBeginDrag( wxDataViewEvent &event );
     void OnDropPossible( wxDataViewEvent &event );
     void OnDrop( wxDataViewEvent &event );
+#endif // wxUSE_DRAG_AND_DROP
 
     void OnDataViewChar(wxKeyEvent& event);
 
@@ -186,21 +189,18 @@ public:
         return true;
     }
 
-    virtual bool Activate( wxRect WXUNUSED(cell),
-                           wxDataViewModel *WXUNUSED(model),
-                           const wxDataViewItem &WXUNUSED(item),
-                           unsigned int WXUNUSED(col) )
+    virtual bool ActivateCell(const wxRect& WXUNUSED(cell),
+                              wxDataViewModel *WXUNUSED(model),
+                              const wxDataViewItem &WXUNUSED(item),
+                              unsigned int WXUNUSED(col),
+                              const wxMouseEvent *mouseEvent)
     {
-        wxLogMessage( "MyCustomRenderer Activate()" );
-        return false;
-    }
-
-    virtual bool LeftClick( wxPoint cursor, wxRect WXUNUSED(cell),
-                           wxDataViewModel *WXUNUSED(model),
-                           const wxDataViewItem &WXUNUSED(item),
-                           unsigned int WXUNUSED(col) )
-    {
-        wxLogMessage( "MyCustomRenderer LeftClick( %d, %d )", cursor.x, cursor.y );
+        wxString position;
+        if ( mouseEvent )
+            position = wxString::Format("via mouse at %d, %d", mouseEvent->m_x, mouseEvent->m_y);
+        else
+            position = "from keyboard";
+        wxLogMessage("MyCustomRenderer ActivateCell() %s", position);
         return false;
     }
 
@@ -239,7 +239,6 @@ bool MyApp::OnInit()
 
     MyFrame *frame =
         new MyFrame(NULL, "wxDataViewCtrl sample", 40, 40, 1000, 540);
-    SetTopWindow(frame);
 
     frame->Show(true);
     return true;
@@ -281,6 +280,8 @@ enum
     ID_SELECT_NINTH     = 103,
     ID_COLLAPSE         = 104,
     ID_EXPAND           = 105,
+    ID_SHOW_CURRENT,
+    ID_SET_NINTH_CURRENT,
 
     ID_PREPEND_LIST     = 200,
     ID_DELETE_LIST      = 201,
@@ -288,7 +289,7 @@ enum
     ID_ADD_MANY         = 203,
     ID_HIDE_ATTRIBUTES  = 204,
     ID_SHOW_ATTRIBUTES  = 205,
-    
+
     // Fourth page.
     ID_DELETE_TREE_ITEM = 400,
     ID_DELETE_ALL_TREE_ITEMS = 401,
@@ -313,6 +314,8 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_BUTTON( ID_SELECT_NINTH, MyFrame::OnSelectNinth )
     EVT_BUTTON( ID_COLLAPSE, MyFrame::OnCollapse )
     EVT_BUTTON( ID_EXPAND, MyFrame::OnExpand )
+    EVT_BUTTON( ID_SHOW_CURRENT, MyFrame::OnShowCurrent )
+    EVT_BUTTON( ID_SET_NINTH_CURRENT, MyFrame::OnSetNinthCurrent )
 
     EVT_BUTTON( ID_PREPEND_LIST, MyFrame::OnPrependList )
     EVT_BUTTON( ID_DELETE_LIST, MyFrame::OnDeleteList )
@@ -340,19 +343,19 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_DATAVIEW_ITEM_EDITING_DONE(ID_MUSIC_CTRL, MyFrame::OnEditingDone)
 
     EVT_DATAVIEW_COLUMN_HEADER_CLICK(ID_MUSIC_CTRL, MyFrame::OnHeaderClick)
-    EVT_DATAVIEW_COLUMN_HEADER_RIGHT_CLICKED(ID_MUSIC_CTRL, MyFrame::OnHeaderRightClick)
+    EVT_DATAVIEW_COLUMN_HEADER_RIGHT_CLICK(ID_MUSIC_CTRL, MyFrame::OnHeaderRightClick)
     EVT_DATAVIEW_COLUMN_SORTED(ID_MUSIC_CTRL, MyFrame::OnSorted)
 
     EVT_DATAVIEW_ITEM_CONTEXT_MENU(ID_MUSIC_CTRL, MyFrame::OnContextMenu)
 
+#if wxUSE_DRAG_AND_DROP
     EVT_DATAVIEW_ITEM_BEGIN_DRAG( ID_MUSIC_CTRL, MyFrame::OnBeginDrag )
     EVT_DATAVIEW_ITEM_DROP_POSSIBLE( ID_MUSIC_CTRL, MyFrame::OnDropPossible )
     EVT_DATAVIEW_ITEM_DROP( ID_MUSIC_CTRL, MyFrame::OnDrop )
+#endif // wxUSE_DRAG_AND_DROP
 
-    EVT_RIGHT_UP(MyFrame::OnRightClick)
-    
     EVT_DATAVIEW_COLUMN_HEADER_CLICK(ID_ATTR_CTRL, MyFrame::OnAttrHeaderClick)
-    
+
 END_EVENT_TABLE()
 
 MyFrame::MyFrame(wxFrame *frame, const wxString &title, int x, int y, int w, int h):
@@ -398,6 +401,14 @@ MyFrame::MyFrame(wxFrame *frame, const wxString &title, int x, int y, int w, int
     CreateStatusBar();
 
 
+    // redirect logs from our event handlers to text control
+    m_log = new wxTextCtrl( this, wxID_ANY, wxString(), wxDefaultPosition,
+                            wxDefaultSize, wxTE_MULTILINE );
+    m_log->SetMinSize(wxSize(-1, 100));
+    m_logOld = wxLog::SetActiveTarget(new wxLogTextCtrl(m_log));
+    wxLogMessage( "This is the log window" );
+
+
     // first page of the notebook
     // --------------------------
 
@@ -407,13 +418,21 @@ MyFrame::MyFrame(wxFrame *frame, const wxString &title, int x, int y, int w, int
 
     BuildDataViewCtrl(firstPanel, 0);    // sets m_ctrl[0]
 
+    const wxSizerFlags border = wxSizerFlags().DoubleBorder();
+
     wxBoxSizer *button_sizer = new wxBoxSizer( wxHORIZONTAL );
-    button_sizer->Add( new wxButton( firstPanel, ID_ADD_MOZART,  "Add Mozart"),             0, wxALL, 10 );
-    button_sizer->Add( new wxButton( firstPanel, ID_DELETE_SEL,  "Delete selected"),        0, wxALL, 10 );
-    button_sizer->Add( new wxButton( firstPanel, ID_DELETE_YEAR, "Delete \"Year\" column"), 0, wxALL, 10 );
-    button_sizer->Add( new wxButton( firstPanel, ID_SELECT_NINTH,"Select ninth symphony"),  0, wxALL, 10 );
-    button_sizer->Add( new wxButton( firstPanel, ID_COLLAPSE,    "Collapse"),               0, wxALL, 10 );
-    button_sizer->Add( new wxButton( firstPanel, ID_EXPAND,      "Expand"),                 0, wxALL, 10 );
+    button_sizer->Add( new wxButton( firstPanel, ID_ADD_MOZART,  "Add Mozart"),             border );
+    button_sizer->Add( new wxButton( firstPanel, ID_DELETE_SEL,  "Delete selected"),        border );
+    button_sizer->Add( new wxButton( firstPanel, ID_DELETE_YEAR, "Delete \"Year\" column"), border );
+    button_sizer->Add( new wxButton( firstPanel, ID_SELECT_NINTH,"Select ninth symphony"),  border );
+    button_sizer->Add( new wxButton( firstPanel, ID_COLLAPSE,    "Collapse"),               border );
+    button_sizer->Add( new wxButton( firstPanel, ID_EXPAND,      "Expand"),                 border );
+
+    wxBoxSizer *sizerCurrent = new wxBoxSizer(wxHORIZONTAL);
+    sizerCurrent->Add(new wxButton(firstPanel, ID_SHOW_CURRENT,
+                                   "&Show current"), border);
+    sizerCurrent->Add(new wxButton(firstPanel, ID_SET_NINTH_CURRENT,
+                                   "Make &ninth symphony current"), border);
 
     wxSizer *firstPanelSz = new wxBoxSizer( wxVERTICAL );
     m_ctrl[0]->SetMinSize(wxSize(-1, 200));
@@ -422,6 +441,7 @@ MyFrame::MyFrame(wxFrame *frame, const wxString &title, int x, int y, int w, int
         new wxStaticText(firstPanel, wxID_ANY, "Most of the cells above are editable!"),
         0, wxGROW|wxALL, 5);
     firstPanelSz->Add(button_sizer);
+    firstPanelSz->Add(sizerCurrent);
     firstPanel->SetSizerAndFit(firstPanelSz);
 
 
@@ -488,12 +508,6 @@ MyFrame::MyFrame(wxFrame *frame, const wxString &title, int x, int y, int w, int
 
     wxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
 
-    m_log = new wxTextCtrl( this, wxID_ANY, wxString(), wxDefaultPosition,
-                            wxDefaultSize, wxTE_MULTILINE );
-    m_log->SetMinSize(wxSize(-1, 100));
-    m_logOld = wxLog::SetActiveTarget(new wxLogTextCtrl(m_log));
-    wxLogMessage( "This is the log window" );
-
     mainSizer->Add( m_notebook, 1, wxGROW );
     mainSizer->Add( m_log, 0, wxGROW );
 
@@ -522,8 +536,10 @@ void MyFrame::BuildDataViewCtrl(wxPanel* parent, unsigned int nPanel, unsigned l
             m_music_model = new MyMusicTreeModel;
             m_ctrl[0]->AssociateModel( m_music_model.get() );
 
+#if wxUSE_DRAG_AND_DROP && wxUSE_UNICODE
             m_ctrl[0]->EnableDragSource( wxDF_UNICODETEXT );
             m_ctrl[0]->EnableDropTarget( wxDF_UNICODETEXT );
+#endif // wxUSE_DRAG_AND_DROP && wxUSE_UNICODE
 
             // column 0 of the view control:
 
@@ -601,16 +617,18 @@ void MyFrame::BuildDataViewCtrl(wxPanel* parent, unsigned int nPanel, unsigned l
             // the various columns
             m_ctrl[1]->AppendTextColumn("editable string",
                                         MyListModel::Col_EditableText,
-                                        wxDATAVIEW_CELL_EDITABLE);
+                                        wxDATAVIEW_CELL_EDITABLE,
+                                        wxCOL_WIDTH_AUTOSIZE);
             m_ctrl[1]->AppendIconTextColumn("icon",
                                             MyListModel::Col_IconText,
-                                            wxDATAVIEW_CELL_EDITABLE);
-                                            
-            m_attributes = 
+                                            wxDATAVIEW_CELL_EDITABLE,
+                                            wxCOL_WIDTH_AUTOSIZE);
+
+            m_attributes =
                 new wxDataViewColumn("attributes",
                                      new wxDataViewTextRenderer,
                                      MyListModel::Col_TextWithAttr,
-                                     80,
+                                     wxCOL_WIDTH_AUTOSIZE,
                                      wxALIGN_RIGHT,
                                      wxDATAVIEW_COL_REORDERABLE | wxDATAVIEW_COL_RESIZABLE );
             m_ctrl[1]->AppendColumn( m_attributes );
@@ -631,6 +649,10 @@ void MyFrame::BuildDataViewCtrl(wxPanel* parent, unsigned int nPanel, unsigned l
                                         wxDefaultSize, style );
             m_ctrl[2] = lc;
 
+            MyListStoreDerivedModel* page2_model = new MyListStoreDerivedModel();
+            lc->AssociateModel(page2_model);
+            page2_model->DecRef();
+
             lc->AppendToggleColumn( "Toggle" );
             lc->AppendTextColumn( "Text" );
             lc->AppendProgressColumn( "Progress" );
@@ -658,7 +680,7 @@ void MyFrame::BuildDataViewCtrl(wxPanel* parent, unsigned int nPanel, unsigned l
 
             wxImageList *ilist = new wxImageList( 16, 16 );
             ilist->Add( wxIcon(wx_small_xpm) );
-            tc->SetImageList( ilist );
+            tc->AssignImageList( ilist );
 
             wxDataViewItem parent =
                 tc->AppendContainer( wxDataViewItem(0), "The Root", 0 );
@@ -802,6 +824,8 @@ void MyFrame::OnAbout( wxCommandEvent& WXUNUSED(event) )
 // MyFrame - event handlers for the first page
 // ----------------------------------------------------------------------------
 
+#if wxUSE_DRAG_AND_DROP
+
 void MyFrame::OnBeginDrag( wxDataViewEvent &event )
 {
     wxDataViewItem item( event.GetItem() );
@@ -817,6 +841,7 @@ void MyFrame::OnBeginDrag( wxDataViewEvent &event )
     wxTextDataObject *obj = new wxTextDataObject;
     obj->SetText( node->m_title );
     event.SetDataObject( obj );
+    event.SetDragFlags(wxDrag_AllowMove); // allows both copy and move
 }
 
 void MyFrame::OnDropPossible( wxDataViewEvent &event )
@@ -854,9 +879,11 @@ void MyFrame::OnDrop( wxDataViewEvent &event )
     wxLogMessage( "Text dropped: %s", obj.GetText() );
 }
 
+#endif // wxUSE_DRAG_AND_DROP
+
 void MyFrame::OnAddMozart( wxCommandEvent& WXUNUSED(event) )
 {
-    m_music_model->AddToClassical( "Kleine Nachtmusik", "Wolfgang Mozart", 1787 );
+    m_music_model->AddToClassical( "Eine kleine Nachtmusik", "Wolfgang Mozart", 1787 );
 }
 
 void MyFrame::DeleteSelectedItems()
@@ -904,11 +931,46 @@ void MyFrame::OnExpand( wxCommandEvent& WXUNUSED(event) )
         m_ctrl[0]->Expand( item );
 }
 
+void MyFrame::OnShowCurrent(wxCommandEvent& WXUNUSED(event))
+{
+    wxDataViewItem item = m_ctrl[0]->GetCurrentItem();
+    if ( item.IsOk() )
+    {
+        wxLogMessage("Current item: \"%s\" by %s",
+                     m_music_model->GetTitle(item),
+                     m_music_model->GetArtist(item));
+    }
+    else
+    {
+        wxLogMessage("There is no current item.");
+    }
+
+    wxDataViewColumn *col = m_ctrl[0]->GetCurrentColumn();
+    if ( col )
+    {
+        wxLogMessage("Current column: %d",
+                     m_ctrl[0]->GetColumnPosition(col));
+    }
+    else
+    {
+        wxLogMessage("There is no current column.");
+    }
+}
+
+void MyFrame::OnSetNinthCurrent(wxCommandEvent& WXUNUSED(event))
+{
+    wxDataViewItem item(m_music_model->GetNinthItem());
+    if ( !item.IsOk() )
+    {
+        wxLogError( "Cannot make the ninth symphony current: it was removed!" );
+        return;
+    }
+
+    m_ctrl[0]->SetCurrentItem(item);
+}
+
 void MyFrame::OnValueChanged( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
     wxLogMessage( "wxEVT_DATAVIEW_ITEM_VALUE_CHANGED, Item Id: %s;  Column: %d",
                   title, event.GetColumn() );
@@ -916,11 +978,8 @@ void MyFrame::OnValueChanged( wxDataViewEvent &event )
 
 void MyFrame::OnActivated( wxDataViewEvent &event )
 {
-    if(!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_ACTIVATED, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_ACTIVATED, Item: %s", title );
 
     if (m_ctrl[0]->IsExpanded( event.GetItem() ))
     {
@@ -930,23 +989,17 @@ void MyFrame::OnActivated( wxDataViewEvent &event )
 
 void MyFrame::OnSelectionChanged( wxDataViewEvent &event )
 {
-    if(!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
     if (title.empty())
         title = "None";
 
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_SELECTION_CHANGED, First selected Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_SELECTION_CHANGED, First selected Item: %s", title );
 }
 
 void MyFrame::OnExpanding( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_EXPANDING, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_EXPANDING, Item: %s", title );
 }
 
 
@@ -957,68 +1010,47 @@ void MyFrame::OnStartEditing( wxDataViewEvent &event )
     {
         event.Veto();
 
-        if (!m_log)
-           return;
-
-        wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_START_EDITING vetoed. Artist: %s", artist );
+        wxLogMessage( "wxEVT_DATAVIEW_ITEM_START_EDITING vetoed. Artist: %s", artist );
     }
     else
-        wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_START_EDITING not vetoed. Artist: %s", artist );
+        wxLogMessage( "wxEVT_DATAVIEW_ITEM_START_EDITING not vetoed. Artist: %s", artist );
 
 }
 
 void MyFrame::OnEditingStarted( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_EDITING_STARTED, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_EDITING_STARTED, Item: %s", title );
 }
 
 void MyFrame::OnEditingDone( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_EDITING_DONE, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_EDITING_DONE, Item: %s", title );
 }
 
 void MyFrame::OnExpanded( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_EXPANDED, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_EXPANDED, Item: %s", title );
 }
 
 void MyFrame::OnCollapsing( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_COLLAPSING, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_COLLAPSING, Item: %s", title );
 }
 
 void MyFrame::OnCollapsed( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_COLLAPSED, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_COLLAPSED, Item: %s", title );
 }
 
 void MyFrame::OnContextMenu( wxDataViewEvent &event )
 {
-    if (!m_log)
-        return;
-
     wxString title = m_music_model->GetTitle( event.GetItem() );
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_ITEM_CONTEXT_MENU, Item: %s", title );
+    wxLogMessage( "wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, Item: %s", title );
 
     wxMenu menu;
     menu.Append( 1, "menuitem 1" );
@@ -1034,12 +1066,9 @@ void MyFrame::OnAttrHeaderClick( wxDataViewEvent &event )
     // this column when it is clicked to take place
     event.Skip();
 
-    if (!m_log)
-        return;
-
     int pos = m_ctrl[1]->GetColumnPosition( event.GetDataViewColumn() );
 
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_COLUMN_HEADER_CLICK, Column position: %d", pos );
+    wxLogMessage( "wxEVT_DATAVIEW_COLUMN_HEADER_CLICK, Column position: %d", pos );
     wxLogMessage( "Column title: %s  Column width: %d", event.GetDataViewColumn()->GetTitle(), event.GetDataViewColumn()->GetWidth() );
 }
 
@@ -1049,42 +1078,24 @@ void MyFrame::OnHeaderClick( wxDataViewEvent &event )
     // this column when it is clicked to take place
     event.Skip();
 
-    if (!m_log)
-        return;
-
     int pos = m_ctrl[0]->GetColumnPosition( event.GetDataViewColumn() );
 
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_COLUMN_HEADER_CLICK, Column position: %d", pos );
+    wxLogMessage( "wxEVT_DATAVIEW_COLUMN_HEADER_CLICK, Column position: %d", pos );
     wxLogMessage( "Column width: %d", event.GetDataViewColumn()->GetWidth() );
 }
 
 void MyFrame::OnHeaderRightClick( wxDataViewEvent &event )
 {
-    if(!m_log)
-        return;
-
     int pos = m_ctrl[0]->GetColumnPosition( event.GetDataViewColumn() );
 
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_COLUMN_HEADER_RIGHT_CLICK, Column position: %d", pos );
+    wxLogMessage( "wxEVT_DATAVIEW_COLUMN_HEADER_RIGHT_CLICK, Column position: %d", pos );
 }
 
 void MyFrame::OnSorted( wxDataViewEvent &event )
 {
-    if(!m_log)
-        return;
-
     int pos = m_ctrl[0]->GetColumnPosition( event.GetDataViewColumn() );
 
-    wxLogMessage( "wxEVT_COMMAND_DATAVIEW_COLUMN_SORTED, Column position: %d", pos );
-}
-
-void MyFrame::OnRightClick( wxMouseEvent &event )
-{
-    if(!m_log)
-        return;
-
-    wxLogMessage( "wxEVT_MOUSE_RIGHT_UP, Click Point is X: %d, Y: %d",
-                 event.GetX(), event.GetY() );
+    wxLogMessage( "wxEVT_DATAVIEW_COLUMN_SORTED, Column position: %d", pos );
 }
 
 void MyFrame::OnDataViewChar(wxKeyEvent& event)
@@ -1143,7 +1154,7 @@ void MyFrame::OnDeleteTreeItem(wxCommandEvent& WXUNUSED(event))
     wxDataViewItem selected = ctrl->GetSelection();
     if (!selected.IsOk())
         return;
-        
+
     ctrl->DeleteItem(selected);
 }
 
@@ -1157,8 +1168,11 @@ void MyFrame::OnAddTreeItem(wxCommandEvent& WXUNUSED(event))
 {
     wxDataViewTreeCtrl* ctrl = (wxDataViewTreeCtrl*) m_ctrl[3];
     wxDataViewItem selected = ctrl->GetSelection();
-    if (ctrl->IsContainer(selected))
-        ctrl->AppendItem( selected, "Item", 0 );
+    if (ctrl->IsContainer(selected)) {
+        wxDataViewItem newitem = ctrl->AppendItem( selected, "Item", 0 );
+        ctrl->Select(newitem);
+        ctrl->EditItem(newitem, ctrl->GetColumn(0));
+    }
 }
 
 void MyFrame::OnAddTreeContainerItem(wxCommandEvent& WXUNUSED(event))
