@@ -3,7 +3,7 @@
 // Purpose:     Erase wxWidgets sample
 // Author:      Robert Roebling, Vadim Zeitlin
 // Created:     04/01/98
-// RCS-ID:      $Id: erase.cpp 64645 2010-06-20 17:42:33Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Robert Roebling
 //              (c) 2009 Vadim Zeitlin
 // Licence:     wxWindows licence
@@ -30,14 +30,16 @@
     #include "wx/wx.h"
 #endif
 
+#include "wx/custombgwin.h"
 #include "wx/dcbuffer.h"
+#include "wx/artprov.h"
 
 // ----------------------------------------------------------------------------
 // resources
 // ----------------------------------------------------------------------------
 
 // the application icon
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -52,7 +54,7 @@ public:
 };
 
 
-class MyCanvas : public wxScrolledWindow
+class MyCanvas : public wxCustomBackgroundWindow<wxScrolledWindow>
 {
 public:
     MyCanvas(wxFrame *parent);
@@ -60,22 +62,49 @@ public:
     void UseBuffer(bool useBuffer) { m_useBuffer = useBuffer; Refresh(); }
     bool UsesBuffer() const { return m_useBuffer; }
 
+    void UseBgBitmap(bool useBgBmp)
+    {
+        m_useBgBmp = useBgBmp;
+        SetBackgroundBitmap(m_useBgBmp ? GetBgBitmap() : wxBitmap());
+
+        Refresh();
+    }
+
     void EraseBgInPaint(bool erase) { m_eraseBgInPaint = erase; Refresh(); }
 
 private:
     void OnPaint( wxPaintEvent &event );
-    void OnChar( wxKeyEvent &event );
     void OnEraseBackground( wxEraseEvent &event );
 
     void DoPaint(wxDC& dc);
 
+    // Create an easily recognizable background bitmap.
+    static wxBitmap GetBgBitmap()
+    {
+        static const int BMP_SIZE = 40;
+
+        wxBitmap bmp(BMP_SIZE, BMP_SIZE);
+        wxMemoryDC dc(bmp);
+        dc.SetBackground(*wxCYAN);
+        dc.Clear();
+
+        dc.SetPen(*wxBLUE_PEN);
+        dc.DrawLine(0, BMP_SIZE/2, BMP_SIZE/2, 0);
+        dc.DrawLine(BMP_SIZE/2, 0, BMP_SIZE, BMP_SIZE/2);
+        dc.DrawLine(BMP_SIZE, BMP_SIZE/2, BMP_SIZE/2, BMP_SIZE);
+        dc.DrawLine(BMP_SIZE/2, BMP_SIZE, 0, BMP_SIZE/2);
+
+        return bmp;
+    }
 
     wxBitmap    m_bitmap;
-    wxString    m_text;
 
     // use wxMemoryDC in OnPaint()?
     bool m_useBuffer;
 
+    // use background bitmap?
+    bool m_useBgBmp;
+
     // erase background in OnPaint()?
     bool m_eraseBgInPaint;
 
@@ -90,6 +119,7 @@ public:
 
 private:
     void OnUseBuffer(wxCommandEvent& event);
+    void OnUseBgBitmap(wxCommandEvent& event);
     void OnEraseBgInPaint(wxCommandEvent& event);
     void OnChangeBgStyle(wxCommandEvent& event);
     void OnQuit(wxCommandEvent& event);
@@ -111,6 +141,58 @@ private:
     DECLARE_EVENT_TABLE()
 };
 
+class ControlWithTransparency : public wxWindow
+{
+public:
+    ControlWithTransparency(wxWindow *parent,
+                            const wxPoint& pos,
+                            const wxSize& size)
+    {
+        wxString reason;
+        if ( parent->IsTransparentBackgroundSupported(&reason) )
+        {
+            SetBackgroundStyle (wxBG_STYLE_TRANSPARENT);
+            m_message = "This is custom control with transparency";
+        }
+        else
+        {
+            m_message = "Transparency not supported, check tooltip.";
+        }
+
+        Create (parent, wxID_ANY, pos, size, wxBORDER_NONE);
+        Connect(wxEVT_PAINT,
+                wxPaintEventHandler(ControlWithTransparency::OnPaint));
+
+        if ( !reason.empty() )
+        {
+            // This can be only done now, after creating the window.
+            SetToolTip(reason);
+        }
+    }
+
+private:
+    void OnPaint( wxPaintEvent& WXUNUSED(event) )
+    {
+        wxPaintDC dc(this);
+
+        dc.SetPen(*wxRED_PEN);
+        dc.SetBrush(*wxTRANSPARENT_BRUSH);
+        dc.DrawRectangle(GetClientSize());
+
+        dc.SetTextForeground(*wxBLUE);
+        dc.SetBackgroundMode(wxTRANSPARENT);
+        dc.DrawText(m_message, 0, 2);
+
+        // Draw some bitmap/icon to ensure transparent bitmaps are indeed
+        //  transparent on transparent windows
+        wxBitmap bmp(wxArtProvider::GetBitmap(wxART_WARNING, wxART_MENU));
+        wxIcon icon(wxArtProvider::GetIcon(wxART_GOTO_LAST, wxART_MENU));
+        dc.DrawBitmap (bmp, GetSize().x - 1 - bmp.GetWidth(), 2);
+        dc.DrawIcon(icon, GetSize().x - 1 - bmp.GetWidth()-icon.GetWidth(), 2);
+    }
+
+    wxString m_message;
+};
 
 // ----------------------------------------------------------------------------
 // constants
@@ -120,6 +202,7 @@ enum
 {
     // menu items
     Erase_Menu_UseBuffer = 100,
+    Erase_Menu_UseBgBitmap,
     Erase_Menu_EraseBgInPaint,
     Erase_Menu_BgStyleErase,
     Erase_Menu_BgStyleSystem,
@@ -153,6 +236,7 @@ bool MyApp::OnInit()
 
 BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Erase_Menu_UseBuffer, MyFrame::OnUseBuffer)
+    EVT_MENU(Erase_Menu_UseBgBitmap, MyFrame::OnUseBgBitmap)
     EVT_MENU(Erase_Menu_EraseBgInPaint, MyFrame::OnEraseBgInPaint)
     EVT_MENU_RANGE(Erase_Menu_BgStyleErase, Erase_Menu_BgStylePaint,
                    MyFrame::OnChangeBgStyle)
@@ -174,6 +258,8 @@ MyFrame::MyFrame()
 
     wxMenu *menuFile = new wxMenu("", wxMENU_TEAROFF);
     menuFile->AppendCheckItem(Erase_Menu_UseBuffer, "&Use memory DC\tCtrl-M");
+    menuFile->AppendCheckItem(Erase_Menu_UseBgBitmap,
+                              "Use background &bitmap\tCtrl-B");
     menuFile->AppendCheckItem(Erase_Menu_EraseBgInPaint,
                               "&Erase background in EVT_PAINT\tCtrl-R");
     menuFile->AppendSeparator();
@@ -188,7 +274,7 @@ MyFrame::MyFrame()
 
 
     wxMenu *helpMenu = new wxMenu;
-    helpMenu->Append(Erase_Menu_About, "&About...\tCtrl-A", "Show about dialog");
+    helpMenu->Append(Erase_Menu_About, "&About\tCtrl-A", "Show about dialog");
 
     wxMenuBar *menuBar = new wxMenuBar();
     menuBar->Append(menuFile, "&File");
@@ -205,6 +291,11 @@ void MyFrame::OnUseBuffer(wxCommandEvent& event)
     m_canvas->UseBuffer(event.IsChecked());
 }
 
+void MyFrame::OnUseBgBitmap(wxCommandEvent& event)
+{
+    m_canvas->UseBgBitmap(event.IsChecked());
+}
+
 void MyFrame::OnEraseBgInPaint(wxCommandEvent& event)
 {
     m_canvas->EraseBgInPaint(event.IsChecked());
@@ -241,14 +332,15 @@ void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
 
 BEGIN_EVENT_TABLE(MyCanvas, wxScrolledWindow)
     EVT_PAINT(MyCanvas::OnPaint)
-    EVT_CHAR(MyCanvas::OnChar)
     EVT_ERASE_BACKGROUND(MyCanvas::OnEraseBackground)
 END_EVENT_TABLE()
 
 MyCanvas::MyCanvas(wxFrame *parent)
-        : wxScrolledWindow(parent, wxID_ANY)
 {
+    Create(parent, wxID_ANY);
+
     m_useBuffer = false;
+    m_useBgBmp = false;
     m_eraseBgInPaint = false;
 
     SetScrollbars( 10, 10, 40, 100, 0, 0 );
@@ -257,39 +349,29 @@ MyCanvas::MyCanvas(wxFrame *parent)
 
     new wxStaticBitmap( this, wxID_ANY, m_bitmap, wxPoint(80,20) );
 
+    new wxStaticText(this, wxID_ANY,
+                     "Left bitmap is a wxStaticBitmap,\n"
+                     "right one drawn directly",
+                     wxPoint(150, 20));
+
+    new ControlWithTransparency(this, wxPoint(65, 125), wxSize(350, 22));
+
     SetFocusIgnoringChildren();
-    SetBackgroundColour(*wxBLUE);
-}
-
-void MyCanvas::OnChar( wxKeyEvent &event )
-{
-#if wxUSE_UNICODE
-    if (event.m_uniChar)
-    {
-        m_text += event.m_uniChar;
-        Refresh();
-        return;
-    }
-#endif
-
-    // some test cases
-    switch (event.m_keyCode)
-    {
-        case WXK_UP: m_text += wxT( "<UP>" ); break;
-        case WXK_LEFT: m_text += wxT( "<LEFT>" ); break;
-        case WXK_RIGHT: m_text += wxT( "<RIGHT>" ); break;
-        case WXK_DOWN: m_text += wxT( "<DOWN>" ); break;
-        case WXK_RETURN: m_text += wxT( "<ENTER>" ); break;
-        default: m_text += (wxChar)event.m_keyCode; break;
-    }
+    SetBackgroundColour(*wxCYAN);
 }
 
 void MyCanvas::DoPaint(wxDC& dc)
 {
+    PrepareDC(dc);
+
     if ( m_eraseBgInPaint )
     {
         dc.SetBackground(*wxLIGHT_GREY);
-        dc.Clear();
+
+        // Erase the entire virtual area, not just the client area.
+        dc.SetPen(*wxTRANSPARENT_PEN);
+        dc.SetBrush(GetBackgroundColour());
+        dc.DrawRectangle(GetVirtualSize());
 
         dc.DrawText("Background erased in OnPaint", 65, 110);
     }
@@ -300,21 +382,10 @@ void MyCanvas::DoPaint(wxDC& dc)
                     "display corruption", 65, 110);
     }
 
-    dc.SetBrush( *wxBLACK_BRUSH );
-    dc.DrawRectangle( 10,10,60,50 );
-
     dc.DrawBitmap( m_bitmap, 20, 20, true );
 
-    dc.SetTextForeground(*wxWHITE);
+    dc.SetTextForeground(*wxRED);
     dc.DrawText("This text is drawn from OnPaint", 65, 65);
-
-    wxString tmp;
-    tmp.Printf("Hit any key to display more text: %s", m_text);
-
-    int w,h;
-    dc.GetTextExtent( tmp, &w, &h );
-    dc.DrawRectangle( 65, 85, w, h );
-    dc.DrawText( tmp, 65, 85 );
 }
 
 void MyCanvas::OnPaint( wxPaintEvent &WXUNUSED(event) )
@@ -322,21 +393,25 @@ void MyCanvas::OnPaint( wxPaintEvent &WXUNUSED(event) )
     if ( m_useBuffer )
     {
         wxAutoBufferedPaintDC dc(this);
-        PrepareDC(dc);
-
         DoPaint(dc);
     }
     else
     {
         wxPaintDC dc(this);
-        PrepareDC(dc);
-
         DoPaint(dc);
     }
 }
 
 void MyCanvas::OnEraseBackground( wxEraseEvent& event )
 {
+    // We must not erase the background ourselves if we asked wxPanel to erase
+    // it using a background bitmap.
+    if ( m_useBgBmp )
+    {
+        event.Skip();
+        return;
+    }
+
     wxASSERT_MSG
     (
         GetBackgroundStyle() == wxBG_STYLE_ERASE,
@@ -346,12 +421,12 @@ void MyCanvas::OnEraseBackground( wxEraseEvent& event )
     wxDC& dc = *event.GetDC();
     dc.SetPen(*wxGREEN_PEN);
 
-    PrepareDC( dc );
-
     // clear any junk currently displayed
     dc.Clear();
 
-    const wxSize size = GetClientSize();
+    PrepareDC( dc );
+
+    const wxSize size = GetVirtualSize();
     for ( int x = 0; x < size.x; x += 15 )
     {
         dc.DrawLine(x, 0, x, size.y);
