@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: tbtest.h 50402 2007-12-01 16:25:30Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c)
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -12,8 +12,8 @@
 class MyTaskBarIcon : public wxTaskBarIcon
 {
 public:
-#if defined(__WXCOCOA__)
-    MyTaskBarIcon(wxTaskBarIconType iconType = DEFAULT_TYPE)
+#if defined(__WXOSX__) && wxOSX_USE_COCOA
+    MyTaskBarIcon(wxTaskBarIconType iconType = wxTBI_DEFAULT_TYPE)
     :   wxTaskBarIcon(iconType)
 #else
     MyTaskBarIcon()
@@ -53,7 +53,7 @@ protected:
     void OnCloseWindow(wxCloseEvent& event);
 
     MyTaskBarIcon   *m_taskBarIcon;
-#if defined(__WXCOCOA__)
+#if defined(__WXOSX__) && wxOSX_USE_COCOA
     MyTaskBarIcon   *m_dockIcon;
 #endif
 
