@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by: Brad Anderson
 // Created:     04/01/98
-// RCS-ID:      $Id: vstest.cpp 61508 2009-07-23 20:30:22Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwidgets.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -40,7 +40,7 @@
 // ----------------------------------------------------------------------------
 
 // the application icon (under Windows and OS/2 it is in resources)
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -513,7 +513,7 @@ VarScrollFrame::VarScrollFrame()
 
     // the "About" item should be in the help menu
     wxMenu *menuHelp = new wxMenu;
-    menuHelp->Append(VScroll_About, wxT("&About...\tF1"), wxT("Show about dialog"));
+    menuHelp->Append(VScroll_About, wxT("&About\tF1"), wxT("Show about dialog"));
 
 #ifdef wxHAS_RADIO_MENU_ITEMS
     menuMode->AppendRadioItem(VScroll_VScrollMode, wxT("&Vertical\tAlt-V"),
