@@ -4,7 +4,7 @@
 // Purpose:     Part of the widgets sample showing wxOwnerDrawnComboBox
 // Author:      Jaakko Salli (based on combobox page by Vadim Zeitlin)
 // Created:     Jul-28-2006
-// Id:          $Id: odcombobox.cpp 64940 2010-07-13 13:29:13Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 2006 Jaakko Salli
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -541,7 +541,7 @@ void ODComboboxWidgetsPage::CreateCombo()
 
     if ( m_chkBitmapbutton->GetValue() )
     {
-        wxBitmap bmpNormal = CreateBitmap(wxColour(0,0,255));
+        wxBitmap bmpNormal = CreateBitmap(*wxBLUE);
         wxBitmap bmpPressed = CreateBitmap(wxColour(0,0,128));
         wxBitmap bmpHover = CreateBitmap(wxColour(128,128,255));
         m_combobox->SetButtonBitmaps(bmpNormal,m_chkStdbutton->GetValue(),bmpPressed,bmpHover);
@@ -764,7 +764,7 @@ void ODComboboxWidgetsPage::OnComboText(wxCommandEvent& event)
     wxASSERT_MSG( s == m_combobox->GetValue(),
                   wxT("event and combobox values should be the same") );
 
-    if (event.GetEventType() == wxEVT_COMMAND_TEXT_ENTER)
+    if (event.GetEventType() == wxEVT_TEXT_ENTER)
     {
         wxLogMessage(wxT("OwnerDrawnCombobox enter pressed (now '%s')"), s.c_str());
     }
