@@ -4,7 +4,7 @@
 // Purpose:     Part of the widgets sample showing various static controls
 // Author:      Vadim Zeitlin
 // Created:     11.04.01
-// Id:          $Id: static.cpp 64940 2010-07-13 13:29:13Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 2001 Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -41,6 +41,7 @@
 
 #include "wx/statline.h"
 #include "wx/generic/stattextg.h"
+#include "wx/wupdlock.h"
 
 #include "widgets.h"
 #include "icons/statbox.xpm"
@@ -94,6 +95,20 @@ public:
     virtual ~StaticWidgetsPage(){};
 
     virtual wxControl *GetWidget() const { return m_statText; }
+    virtual Widgets GetWidgets() const
+    {
+        Widgets widgets;
+        widgets.push_back(m_sizerStatBox->GetStaticBox());
+        widgets.push_back(m_statText);
+#if wxUSE_MARKUP
+        widgets.push_back(m_statMarkup);
+#endif // wxUSE_MARKUP
+#if wxUSE_STATLINE
+        widgets.push_back(m_statLine);
+#endif // wxUSE_STATLINE
+
+        return widgets;
+    }
     virtual void RecreateWidget() { CreateStatic(); }
 
     // lazy creation of the content
@@ -106,7 +121,9 @@ protected:
     void OnButtonReset(wxCommandEvent& event);
     void OnButtonBoxText(wxCommandEvent& event);
     void OnButtonLabelText(wxCommandEvent& event);
+#if wxUSE_MARKUP
     void OnButtonLabelWithMarkupText(wxCommandEvent& event);
+#endif // wxUSE_MARKUP
     void OnMouseEvent(wxMouseEvent& event);
 
     // reset all parameters
@@ -122,9 +139,12 @@ protected:
     wxCheckBox *m_chkVert,
                *m_chkGeneric,
                *m_chkAutoResize,
-               *m_chkEllipsize,
-               *m_chkMarkup,
+               *m_chkEllipsize;
+
+#if wxUSE_MARKUP
+    wxCheckBox *m_chkMarkup,
                *m_chkGreen;
+#endif // wxUSE_MARKUP
 
     wxRadioBox *m_radioHAlign,
                *m_radioVAlign,
@@ -132,8 +152,12 @@ protected:
 
     // the controls and the sizer containing them
     wxStaticBoxSizer *m_sizerStatBox;
-    wxStaticTextBase *m_statText,
-                     *m_statMarkup;
+    wxStaticTextBase *m_statText;
+
+#if wxUSE_MARKUP
+    wxStaticTextBase *m_statMarkup;
+#endif // wxUSE_MARKUP
+
 #if wxUSE_STATLINE
     wxStaticLine *m_statLine;
 #endif // wxUSE_STATLINE
@@ -141,8 +165,11 @@ protected:
 
     // the text entries for command parameters
     wxTextCtrl *m_textBox,
-               *m_textLabel,
-               *m_textLabelWithMarkup;
+               *m_textLabel;
+
+#if wxUSE_MARKUP
+    wxTextCtrl *m_textLabelWithMarkup;
+#endif // wxUSE_MARKUP
 
 private:
     DECLARE_EVENT_TABLE()
@@ -156,7 +183,9 @@ private:
 BEGIN_EVENT_TABLE(StaticWidgetsPage, WidgetsPage)
     EVT_BUTTON(StaticPage_Reset, StaticWidgetsPage::OnButtonReset)
     EVT_BUTTON(StaticPage_LabelText, StaticWidgetsPage::OnButtonLabelText)
+#if wxUSE_MARKUP
     EVT_BUTTON(StaticPage_LabelTextWithMarkup, StaticWidgetsPage::OnButtonLabelWithMarkupText)
+#endif // wxUSE_MARKUP
     EVT_BUTTON(StaticPage_BoxText, StaticWidgetsPage::OnButtonBoxText)
 
     EVT_CHECKBOX(wxID_ANY, StaticWidgetsPage::OnCheckOrRadioBox)
@@ -177,9 +206,12 @@ StaticWidgetsPage::StaticWidgetsPage(WidgetsBookCtrl *book,
 {
     // init everything
     m_chkVert =
-    m_chkAutoResize = (wxCheckBox *)NULL;
-    m_chkGeneric = NULL;
-    m_chkGreen = NULL;
+    m_chkAutoResize =
+    m_chkGeneric =
+#if wxUSE_MARKUP
+    m_chkGreen =
+#endif // wxUSE_MARKUP
+                NULL;
 
     m_radioHAlign =
     m_radioVAlign = (wxRadioBox *)NULL;
@@ -187,12 +219,19 @@ StaticWidgetsPage::StaticWidgetsPage(WidgetsBookCtrl *book,
 #if wxUSE_STATLINE
     m_statLine = (wxStaticLine *)NULL;
 #endif // wxUSE_STATLINE
+#if wxUSE_MARKUP
     m_statText = m_statMarkup = NULL;
+#endif // wxUSE_MARKUP
 
     m_sizerStatBox = (wxStaticBoxSizer *)NULL;
     m_sizerStatic = (wxSizer *)NULL;
 
-    m_textBox = m_textLabel = m_textLabelWithMarkup = NULL;
+    m_textBox =
+    m_textLabel =
+#if wxUSE_MARKUP
+    m_textLabelWithMarkup =
+#endif // wxUSE_MARKUP
+                            NULL;
 }
 
 void StaticWidgetsPage::CreateContent()
@@ -204,7 +243,6 @@ void StaticWidgetsPage::CreateContent()
 
     m_chkGeneric = CreateCheckBoxAndAddToSizer(sizerLeft,
                                                "&Generic wxStaticText");
-    m_chkMarkup = CreateCheckBoxAndAddToSizer(sizerLeft, "Support &markup");
     m_chkVert = CreateCheckBoxAndAddToSizer(sizerLeft, "&Vertical line");
     m_chkAutoResize = CreateCheckBoxAndAddToSizer(sizerLeft, "&Fit to text");
     sizerLeft->Add(5, 5, 0, wxGROW | wxALL, 5); // spacer
@@ -261,7 +299,7 @@ void StaticWidgetsPage::CreateContent()
 
     m_textBox = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     wxButton *b1 = new wxButton(this, wxID_ANY, "Change &box label");
-    b1->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b1->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonBoxText),
                 NULL, this);
     sizerMiddle->Add(m_textBox, 0, wxEXPAND|wxALL, 5);
@@ -271,38 +309,41 @@ void StaticWidgetsPage::CreateContent()
                                  wxDefaultPosition, wxDefaultSize,
                                  wxTE_MULTILINE|wxHSCROLL);
     wxButton *b2 = new wxButton(this, wxID_ANY, "Change &text label");
-    b2->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b2->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelText),
                 NULL, this);
     sizerMiddle->Add(m_textLabel, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b2, 0, wxLEFT|wxBOTTOM, 5);
 
+#if wxUSE_MARKUP
     m_textLabelWithMarkup = new wxTextCtrl(this, wxID_ANY, wxEmptyString,
                                            wxDefaultPosition, wxDefaultSize,
                                            wxTE_MULTILINE|wxHSCROLL);
 
     wxButton *b3 = new wxButton(this, wxID_ANY, "Change decorated text label");
-    b3->Connect(wxEVT_COMMAND_BUTTON_CLICKED,
+    b3->Connect(wxEVT_BUTTON,
                 wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelWithMarkupText),
                 NULL, this);
     sizerMiddle->Add(m_textLabelWithMarkup, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b3, 0, wxLEFT|wxBOTTOM, 5);
 
-    m_chkGreen = CreateCheckBoxAndAddToSizer(sizerLeft,
+    m_chkGreen = CreateCheckBoxAndAddToSizer(sizerMiddle,
                                              "Decorated label on g&reen");
-    sizerMiddle->Add(m_chkGreen, 0, wxALL, 5);
+#endif // wxUSE_MARKUP
 
     // final initializations
     // NB: must be done _before_ calling CreateStatic()
     Reset();
 
-    m_textBox->SetValue(wxT("This is a box"));
+    m_textBox->SetValue(wxT("This is a &box"));
     m_textLabel->SetValue(wxT("And this is a\n\tlabel inside the box with a &mnemonic.\n")
                           wxT("Only this text is affected by the ellipsize settings."));
+#if wxUSE_MARKUP
     m_textLabelWithMarkup->SetValue(wxT("Another label, this time <b>decorated</b> ")
                                     wxT("with <u>markup</u>; here you need entities ")
                                     wxT("for the symbols: &lt; &gt; &amp; &apos; &quot; ")
                                     wxT(" but you can still place &mnemonics..."));
+#endif // wxUSE_MARKUP
 
     // right pane
     wxSizer *sizerRight = new wxBoxSizer(wxHORIZONTAL);
@@ -329,7 +370,6 @@ void StaticWidgetsPage::Reset()
     m_chkVert->SetValue(false);
     m_chkAutoResize->SetValue(true);
     m_chkEllipsize->SetValue(true);
-    m_chkMarkup->SetValue(true);
 
     m_radioHAlign->SetSelection(StaticHAlign_Left);
     m_radioVAlign->SetSelection(StaticVAlign_Top);
@@ -337,6 +377,8 @@ void StaticWidgetsPage::Reset()
 
 void StaticWidgetsPage::CreateStatic()
 {
+    wxWindowUpdateLocker lock(this);
+
     bool isVert = m_chkVert->GetValue();
 
     if ( m_sizerStatBox )
@@ -344,7 +386,9 @@ void StaticWidgetsPage::CreateStatic()
         // delete m_sizerStatBox; -- deleted by Remove()
         m_sizerStatic->Remove(m_sizerStatBox);
         delete m_statText;
+#if wxUSE_MARKUP
         delete m_statMarkup;
+#endif // wxUSE_MARKUP
 #if wxUSE_STATLINE
         delete m_statLine;
 #endif // wxUSE_STATLINE
@@ -360,12 +404,6 @@ void StaticWidgetsPage::CreateStatic()
         flagsDummyText |= wxST_NO_AUTORESIZE;
     }
 
-    if ( m_chkMarkup->GetValue() )
-    {
-        flagsText |= wxST_MARKUP;
-        flagsDummyText |= wxST_MARKUP;
-    }
-
     int align = 0;
     switch ( m_radioHAlign->GetSelection() )
     {
@@ -444,10 +482,12 @@ void StaticWidgetsPage::CreateStatic()
                                              m_textLabel->GetValue(),
                                              wxDefaultPosition, wxDefaultSize,
                                              flagsDummyText);
+#if wxUSE_MARKUP
         m_statMarkup = new wxGenericStaticText(this, wxID_ANY,
-                                             m_textLabelWithMarkup->GetValue(),
+                                             wxString(),
                                              wxDefaultPosition, wxDefaultSize,
                                              flagsText);
+#endif // wxUSE_MARKUP
     }
     else // use native versions
     {
@@ -455,13 +495,21 @@ void StaticWidgetsPage::CreateStatic()
                                       m_textLabel->GetValue(),
                                       wxDefaultPosition, wxDefaultSize,
                                       flagsDummyText);
+#if wxUSE_MARKUP
         m_statMarkup = new wxStaticText(this, wxID_ANY,
-                                        m_textLabelWithMarkup->GetValue(),
+                                        wxString(),
                                         wxDefaultPosition, wxDefaultSize,
                                         flagsText);
+#endif // wxUSE_MARKUP
     }
+
+#if wxUSE_MARKUP
+    m_statMarkup->SetLabelMarkup(m_textLabelWithMarkup->GetValue());
+
     if ( m_chkGreen->GetValue() )
         m_statMarkup->SetBackgroundColour(*wxGREEN);
+#endif // wxUSE_MARKUP
+
 #if wxUSE_STATLINE
     m_statLine = new wxStaticLine(this, wxID_ANY,
                                   wxDefaultPosition, wxDefaultSize,
@@ -472,7 +520,9 @@ void StaticWidgetsPage::CreateStatic()
 #if wxUSE_STATLINE
     m_sizerStatBox->Add(m_statLine, 0, wxGROW | wxALL, 5);
 #endif // wxUSE_STATLINE
+#if wxUSE_MARKUP
     m_sizerStatBox->Add(m_statMarkup, 1, wxGROW | wxALL, 5);
+#endif // wxUSE_MARKUP
 
     m_sizerStatic->Add(m_sizerStatBox, 1, wxGROW);
 
@@ -525,9 +575,10 @@ void StaticWidgetsPage::OnButtonLabelText(wxCommandEvent& WXUNUSED(event))
                  m_statText->GetLabelText());
 }
 
+#if wxUSE_MARKUP
 void StaticWidgetsPage::OnButtonLabelWithMarkupText(wxCommandEvent& WXUNUSED(event))
 {
-    m_statMarkup->SetLabel(m_textLabelWithMarkup->GetValue());
+    m_statMarkup->SetLabelMarkup(m_textLabelWithMarkup->GetValue());
 
     // test GetLabel() and GetLabelText(); the first should return the
     // label as it is written in the relative text control; the second should
@@ -537,6 +588,7 @@ void StaticWidgetsPage::OnButtonLabelWithMarkupText(wxCommandEvent& WXUNUSED(eve
     wxLogMessage(wxT("The label text is '%s'"),
                  m_statMarkup->GetLabelText());
 }
+#endif // wxUSE_MARKUP
 
 void StaticWidgetsPage::OnMouseEvent(wxMouseEvent& event)
 {
