@@ -4,7 +4,7 @@
 // Author:      David Elliott
 // Modified by:
 // Created:     2003/03/16
-// RCS-ID:      $Id: bmpbuttn.mm 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 David Elliott
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -24,7 +24,6 @@
 #import <AppKit/NSButton.h>
 #import <Foundation/NSString.h>
 
-IMPLEMENT_DYNAMIC_CLASS(wxBitmapButton, wxControl)
 BEGIN_EVENT_TABLE(wxBitmapButton, wxBitmapButtonBase)
 END_EVENT_TABLE()
 WX_IMPLEMENT_COCOA_OWNER(wxBitmapButton,NSButton,NSControl,NSView)
@@ -64,7 +63,7 @@ wxBitmapButton::~wxBitmapButton()
 void wxBitmapButton::Cocoa_wxNSButtonAction(void)
 {
     wxLogTrace(wxTRACE_COCOA,wxT("YAY!"));
-    wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, GetId());
+    wxCommandEvent event(wxEVT_BUTTON, GetId());
     InitCommandEvent(event); //    event.SetEventObject(this);
     Command(event);
 }
