@@ -4,7 +4,7 @@
 // Author:      David Elliott
 // Modified by:
 // Created:     2003/04/01
-// RCS-ID:      $Id: dc.mm 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 David Elliott
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -362,11 +362,11 @@ void wxCocoaDCImpl::DoDrawPoint( int x, int y )
 {
 };
 
-void wxCocoaDCImpl::DoDrawPolygon( int, wxPoint *, int, int, wxPolygonFillMode)
+void wxCocoaDCImpl::DoDrawPolygon( int, const wxPoint *, int, int, wxPolygonFillMode)
 {
 };
 
-void wxCocoaDCImpl::DoDrawLines( int, wxPoint *, int, int )
+void wxCocoaDCImpl::DoDrawLines( int, const wxPoint *, int, int )
 {
 }
 
@@ -456,7 +456,7 @@ void wxCocoaDCImpl::DoDrawBitmap(const wxBitmap &bmp, wxCoord x, wxCoord y, bool
 {
     wxAutoNSAutoreleasePool pool;
     if(!CocoaTakeFocus()) return;
-    if(!bmp.Ok())
+    if(!bmp.IsOk())
         return;
 
 #if 0
@@ -540,13 +540,13 @@ void wxCocoaDCImpl::DoGetSizeMM( int* width, int* height ) const
 
 void wxCocoaDCImpl::SetTextForeground( const wxColour &col )
 {
-//  if (!Ok()) return;
+//  if (!IsOk()) return;
   m_textForegroundColour = col;
 };
 
 void wxCocoaDCImpl::SetTextBackground( const wxColour &col )
 {
-//  if (!Ok()) return;
+//  if (!IsOk()) return;
   m_textBackgroundColour = col;
 };
 
