@@ -4,7 +4,7 @@
 // Author:      David Elliott
 // Modified by:
 // Created:     2003/08/17
-// RCS-ID:      $Id: toolbar.mm 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 David Elliott
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -100,7 +100,7 @@ void wxToolBarTool::Init()
 void wxToolBar::CocoaToolClickEnded()
 {
     wxASSERT(m_mouseDownTool);
-    wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED, m_mouseDownTool->GetId());
+    wxCommandEvent event(wxEVT_MENU, m_mouseDownTool->GetId());
     InitCommandEvent(event);
     Command(event);
 }
