@@ -4,7 +4,7 @@
 // Author:      Ryan Norton
 // Modified by:
 // Created:     2004-10-05
-// RCS-ID:      $Id: utilsexc.mm 48106 2007-08-15 16:10:19Z DE $
+// RCS-ID:      $Id$
 // Copyright:   (c) Ryan Norton
 // Licence:     wxWindows licence
 // Notes:       This code may be useful on platforms other than Darwin.
@@ -17,8 +17,6 @@
     #include "wx/utils.h"
 #endif
 
-#include "wx/unix/execute.h"
-
 #if 0
 
 #ifndef   WX_PRECOMP
@@ -142,7 +140,8 @@ WX_IMPLEMENT_GET_OBJC_CLASS(wxTaskHandler,NSObject)
 
 long wxExecute(const wxString& command,
                int sync,
-               wxProcess *handle)
+               wxProcess *handle,
+               const wxExecuteEnv *env)
 {
     NSTask* theTask = [[NSTask alloc] init];
 
