@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     06.08.01
-// RCS-ID:      $Id: containr.cpp 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2001 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -47,14 +47,20 @@
 // wxControlContainerBase
 // ----------------------------------------------------------------------------
 
-void wxControlContainerBase::SetCanFocus(bool acceptsFocus)
+bool wxControlContainerBase::UpdateCanFocusChildren()
 {
-    if ( acceptsFocus == m_acceptsFocus )
-        return;
+    const bool acceptsFocusChildren = HasAnyFocusableChildren();
+    if ( acceptsFocusChildren != m_acceptsFocusChildren )
+    {
+        m_acceptsFocusChildren = acceptsFocusChildren;
 
-    m_acceptsFocus = acceptsFocus;
+        // In the ports where it does something non trivial, the parent window
+        // should only be focusable if it doesn't have any focusable children
+        // (e.g. native focus handling in wxGTK totally breaks down otherwise).
+        m_winParent->SetCanFocus(m_acceptsFocusSelf && !m_acceptsFocusChildren);
+    }
 
-    m_winParent->SetCanFocus(m_acceptsFocus);
+    return m_acceptsFocusChildren;
 }
 
 bool wxControlContainerBase::HasAnyFocusableChildren() const
@@ -174,19 +180,6 @@ void wxControlContainer::SetLastFocus(wxWindow *win)
             wxLogTrace(TRACE_FOCUS, wxT("No more last focus"));
         }
     }
-
-    // propagate the last focus upwards so that our parent can set focus back
-    // to us if it loses it now and regains later; do *not* do this if we are
-    // a toplevel window (e.g. wxDialog) that has another frame as its parent
-    if ( !m_winParent->IsTopLevel() )
-    {
-        wxWindow *parent = m_winParent->GetParent();
-        if ( parent )
-        {
-            wxChildFocusEvent eventFocus(m_winParent);
-            parent->GetEventHandler()->ProcessEvent(eventFocus);
-        }
-    }
 }
 
 // --------------------------------------------------------------------
@@ -194,7 +187,7 @@ void wxControlContainer::SetLastFocus(wxWindow *win)
 // within the same group. Used by wxSetFocusToChild on wxMSW
 // --------------------------------------------------------------------
 
-#if defined(__WXMSW__) && wxUSE_RADIOBTN
+#if wxUSE_RADIOBTN 
 
 wxRadioButton* wxGetPreviousButtonInGroup(wxRadioButton *btn)
 {
@@ -472,18 +465,21 @@ void wxControlContainer::HandleOnNavigationKey( wxNavigationKeyEvent& event )
                 // looping inside this panel (normally, the focus will go to
                 // the next/previous item after this panel in the parent
                 // panel).
-                wxWindow *focussed_child_of_parent = m_winParent;
+                wxWindow *focusedParent = m_winParent;
                 while ( parent )
                 {
-                    // we don't want to tab into a different dialog or frame
-                    if ( focussed_child_of_parent->IsTopLevel() )
+                    // We don't want to tab into a different dialog or frame or
+                    // even an MDI child frame, so test for this explicitly
+                    // (and in particular don't just use IsTopLevel() which
+                    // would return false in the latter case).
+                    if ( focusedParent->IsTopNavigationDomain() )
                         break;
 
-                    event.SetCurrentFocus( focussed_child_of_parent );
+                    event.SetCurrentFocus( focusedParent );
                     if ( parent->GetEventHandler()->ProcessEvent( event ) )
                         return;
 
-                    focussed_child_of_parent = parent;
+                    focusedParent = parent;
 
                     parent = parent->GetParent();
                 }
@@ -650,14 +646,36 @@ bool wxSetFocusToChild(wxWindow *win, wxWindow **childLastFocused)
         // It might happen that the window got reparented
         if ( (*childLastFocused)->GetParent() == win )
         {
-            wxLogTrace(TRACE_FOCUS,
-                       wxT("SetFocusToChild() => last child (0x%p)."),
-                       (*childLastFocused)->GetHandle());
+            // And it also could have become hidden in the meanwhile
+            // We want to focus on the deepest widget visible
+            wxWindow *deepestVisibleWindow = NULL;
 
-            // not SetFocusFromKbd(): we're restoring focus back to the old
-            // window and not setting it as the result of a kbd action
-            (*childLastFocused)->SetFocus();
-            return true;
+            while ( *childLastFocused )
+            {
+                if ( (*childLastFocused)->IsShown() )
+                {
+                    if ( !deepestVisibleWindow )
+                        deepestVisibleWindow = *childLastFocused;
+                }
+                else
+                    deepestVisibleWindow = NULL;
+
+                *childLastFocused = (*childLastFocused)->GetParent();
+            }
+
+            if ( deepestVisibleWindow )
+            {
+                *childLastFocused = deepestVisibleWindow;
+
+                wxLogTrace(TRACE_FOCUS,
+                           wxT("SetFocusToChild() => last child (0x%p)."),
+                           (*childLastFocused)->GetHandle());
+
+                // not SetFocusFromKbd(): we're restoring focus back to the old
+                // window and not setting it as the result of a kbd action
+                (*childLastFocused)->SetFocus();
+                return true;
+            }
         }
         else
         {
