@@ -1,9 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        svg.cpp
+// Name:        src/common/svg.cpp
 // Purpose:     SVG sample
 // Author:      Chris Elliott
 // Modified by:
-// RCS-ID:      $Id: dcsvg.cpp 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -28,54 +28,86 @@
 #include "wx/wfstream.h"
 #include "wx/filename.h"
 
-#define wxSVG_DEBUG false
-// or true to see the calls being executed
-
-#define newline        wxString(wxT("\n"))
-#define space          wxString(wxT(" "))
-#define semicolon      wxString(wxT(";"))
-#define wx_round(a)    (int)((a)+.5)
-
-
 // ----------------------------------------------------------
 // Global utilities
 // ----------------------------------------------------------
 
-static inline double DegToRad(double deg) { return (deg * M_PI) / 180.0; }
-
-wxString wxColStr ( wxColour c )
+namespace
 {
-    unsigned char r, g, b;
-    r = c.Red ();
-    g = c.Green ();
-    b = c. Blue ();
 
-    // possible Unicode bug here
-    wxString s = wxDecToHex(r) + wxDecToHex(g) + wxDecToHex(b);
-    return s;
+inline double DegToRad(double deg) { return (deg * M_PI) / 180.0; }
+
+// This function returns a string representation of a floating point number in
+// C locale (i.e. always using "." for the decimal separator) and with the
+// fixed precision (which is 2 for some unknown reason but this is what it was
+// in this code originally).
+inline wxString NumStr(double f)
+{
+    return wxString::FromCDouble(f, 2);
 }
 
-
-wxString wxBrushString ( wxColour c, int style )
+// Return the colour representation as HTML-like "#rrggbb" string and also
+// returns its alpha as opacity number in 0..1 range.
+wxString Col2SVG(wxColour c, float *opacity)
 {
-    wxString s = wxT("fill:#") + wxColStr (c)  + semicolon + space;
+    if ( c.Alpha() != wxALPHA_OPAQUE )
+    {
+        *opacity = c.Alpha()/255.;
+
+        // Remove the alpha before using GetAsString(wxC2S_HTML_SYNTAX) as it
+        // doesn't support colours with alpha channel.
+        c = wxColour(c.GetRGB());
+    }
+    else // No alpha.
+    {
+        *opacity = 1.;
+    }
+
+    return c.GetAsString(wxC2S_HTML_SYNTAX);
+}
+
+wxString wxPenString(wxColour c, int style = wxPENSTYLE_SOLID)
+{
+    float opacity;
+    wxString s = wxT("stroke:") + Col2SVG(c, &opacity)  + wxT("; ");
+
     switch ( style )
     {
-        case wxBRUSHSTYLE_SOLID :
-            s = s + wxT("fill-opacity:1.0; ");
+        case wxPENSTYLE_SOLID:
+            s += wxString::Format(wxT("stroke-opacity:%s; "), NumStr(opacity));
             break;
-        case wxBRUSHSTYLE_TRANSPARENT:
-            s = s + wxT("fill-opacity:0.0; ");
+        case wxPENSTYLE_TRANSPARENT:
+            s += wxT("stroke-opacity:0.0; ");
             break;
-
         default :
-            wxASSERT_MSG(false, wxT("wxSVGFileDC::Requested Brush Style not available"));
-
+            wxASSERT_MSG(false, wxT("wxSVGFileDC::Requested Pen Style not available"));
     }
-    s = s + newline;
+
     return s;
 }
 
+wxString wxBrushString(wxColour c, int style = wxBRUSHSTYLE_SOLID)
+{
+    float opacity;
+    wxString s = wxT("fill:") + Col2SVG(c, &opacity)  + wxT("; ");
+
+    switch ( style )
+    {
+        case wxBRUSHSTYLE_SOLID:
+            s += wxString::Format(wxT("fill-opacity:%s; "), NumStr(opacity));
+            break;
+        case wxBRUSHSTYLE_TRANSPARENT:
+            s += wxT("fill-opacity:0.0; ");
+            break;
+        default :
+            wxASSERT_MSG(false, wxT("wxSVGFileDC::Requested Brush Style not available"));
+    }
+
+    return s;
+}
+
+} // anonymous namespace
+
 // ----------------------------------------------------------
 // wxSVGFileDCImpl
 // ----------------------------------------------------------
@@ -98,6 +130,9 @@ void wxSVGFileDCImpl::Init (const wxString &filename, int Width, int Height, dou
 
     m_OK = true;
 
+    m_clipUniqueId = 0;
+    m_clipNestingLevel = 0;
+
     m_mm_to_pix_x = dpi/25.4;
     m_mm_to_pix_y = dpi/25.4;
 
@@ -115,27 +150,27 @@ void wxSVGFileDCImpl::Init (const wxString &filename, int Width, int Height, dou
     ////////////////////code here
 
     m_outfile = new wxFileOutputStream(filename);
-    m_OK = m_outfile->Ok ();
+    m_OK = m_outfile->IsOk();
     if (m_OK)
     {
         m_filename = filename;
         m_sub_images = 0;
         wxString s;
-        s = wxT("<?xml version=\"1.0\" standalone=\"no\"?>"); s = s + newline;
+        s = wxT("<?xml version=\"1.0\" standalone=\"no\"?>\n");
         write(s);
-        s = wxT("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\" ") + newline;
+        s = wxT("<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20010904//EN\"\n");
         write(s);
-        s = wxT("\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\"> ") + newline;
+        s = wxT("\"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n");
         write(s);
-        s =  wxT("<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" ") + newline;
+        s = wxT("<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n");
         write(s);
-        s.Printf( wxT("    width=\"%.2gcm\" height=\"%.2gcm\" viewBox=\"0 0 %d %d \"> \n"), float(Width)/dpi*2.54, float(Height)/dpi*2.54, Width, Height );
+        s.Printf( wxT("    width=\"%scm\" height=\"%scm\" viewBox=\"0 0 %d %d \">\n"), NumStr(float(Width)/dpi*2.54), NumStr(float(Height)/dpi*2.54), Width, Height );
         write(s);
-        s = wxT("<title>SVG Picture created as ") + wxFileName(filename).GetFullName() + wxT(" </title>") + newline;
+        s = wxT("<title>SVG Picture created as ") + wxFileName(filename).GetFullName() + wxT(" </title>\n");
         write(s);
-        s = wxString (wxT("<desc>Picture generated by wxSVG ")) + wxSVGVersion + wxT(" </desc>")+ newline;
+        s = wxString (wxT("<desc>Picture generated by wxSVG ")) + wxSVGVersion + wxT(" </desc>\n");
         write(s);
-        s =  wxT("<g style=\"fill:black; stroke:black; stroke-width:1\">") + newline;
+        s =  wxT("<g style=\"fill:black; stroke:black; stroke-width:1\">\n");
         write(s);
     }
 }
@@ -163,20 +198,18 @@ wxSize wxSVGFileDCImpl::GetPPI() const
 
 void wxSVGFileDCImpl::DoDrawLine (wxCoord x1, wxCoord y1, wxCoord x2, wxCoord y2)
 {
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
     wxString s;
     s.Printf ( wxT("<path d=\"M%d %d L%d %d\" /> \n"), x1,y1,x2,y2 );
     if (m_OK)
     {
         write(s);
     }
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DrawLine Call executed"));
     CalcBoundingBox(x1, y1);
     CalcBoundingBox(x2, y2);
-    return;
 }
 
-void wxSVGFileDCImpl::DoDrawLines(int n, wxPoint points[], wxCoord xoffset , wxCoord yoffset )
+void wxSVGFileDCImpl::DoDrawLines(int n, const wxPoint points[], wxCoord xoffset , wxCoord yoffset )
 {
     for ( int i = 1; i < n; i++ )
     {
@@ -188,8 +221,8 @@ void wxSVGFileDCImpl::DoDrawLines(int n, wxPoint points[], wxCoord xoffset , wxC
 void wxSVGFileDCImpl::DoDrawPoint (wxCoord x1, wxCoord y1)
 {
     wxString s;
-    if (m_graphics_changed) NewGraphics ();
-    s = wxT("<g style = \"stroke-linecap:round;\" > ") + newline;
+    NewGraphicsIfNeeded();
+    s = wxT("<g style = \"stroke-linecap:round;\" > \n");
     write(s);
     DoDrawLine ( x1,y1,x1,y1 );
     s = wxT("</g>");
@@ -204,13 +237,12 @@ void wxSVGFileDCImpl::DoDrawCheckMark(wxCoord x1, wxCoord y1, wxCoord width, wxC
 void wxSVGFileDCImpl::DoDrawText(const wxString& text, wxCoord x1, wxCoord y1)
 {
     DoDrawRotatedText(text, x1,y1,0.0);
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DrawText Call executed"));
 }
 
 void wxSVGFileDCImpl::DoDrawRotatedText(const wxString& sText, wxCoord x, wxCoord y, double angle)
 {
     //known bug; if the font is drawn in a scaled DC, it will not behave exactly as wxMSW
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
     wxString s, sTmp;
 
     // calculate bounding box
@@ -224,47 +256,49 @@ void wxSVGFileDCImpl::DoDrawRotatedText(const wxString& sText, wxCoord x, wxCoor
     CalcBoundingBox((wxCoord)(x + w*cos(rad)), (wxCoord)(y - h*sin(rad)));
 
     // wxT("bottom left") and wxT("bottom right")
-    x += (wxCoord)(h*sin(rad));
-    y += (wxCoord)(h*cos(rad));
-    CalcBoundingBox(x, y);
     CalcBoundingBox((wxCoord)(x + h*sin(rad)), (wxCoord)(y + h*cos(rad)));
+    CalcBoundingBox((wxCoord)(x + h*sin(rad) + w*cos(rad)), (wxCoord)(y + h*cos(rad) - w*sin(rad)));
 
     if (m_backgroundMode == wxBRUSHSTYLE_SOLID)
     {
         // draw background first
         // just like DoDrawRectangle except we pass the text color to it and set the border to a 1 pixel wide text background
 
-        wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::Draw Rotated Text Call plotting text background"));
-        sTmp.Printf ( wxT(" <rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\"  "), x,y+desc-h, w, h );
-        s = sTmp + wxT("style=\"fill:#") + wxColStr (m_textBackgroundColour) + wxT("; ");
-        s = s + wxT("stroke-width:1; stroke:#") + wxColStr (m_textBackgroundColour) + wxT("; ");
-        sTmp.Printf ( wxT("\" transform=\"rotate( %.2g %d %d )  \">"), -angle, x,y );
-        s = s + sTmp + newline;
+        sTmp.Printf ( wxT(" <rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" "), x, y, w, h );
+        s = sTmp + wxT("style=\"") + wxBrushString(m_textBackgroundColour);
+        s += wxT("stroke-width:1; ") + wxPenString(m_textBackgroundColour);
+        sTmp.Printf ( wxT("\" transform=\"rotate( %s %d %d )  \" />"), NumStr(-angle), x,y );
+        s += sTmp + wxT("\n");
         write(s);
     }
+
+    // convert x,y to SVG text x,y (the coordinates of the text baseline)
+    x = (wxCoord)(x + (h-desc)*sin(rad));
+    y = (wxCoord)(y + (h-desc)*cos(rad));
+
     //now do the text itself
     s.Printf (wxT(" <text x=\"%d\" y=\"%d\" "),x,y );
 
-    sTmp = m_font.GetFaceName ();
-    if (sTmp.Len () > 0)  s = s + wxT("style=\"font-family:") + sTmp + wxT("; ");
-    else s = s + wxT("style=\" ");
+    sTmp = m_font.GetFaceName();
+    if (sTmp.Len() > 0)  s += wxT("style=\"font-family:") + sTmp + wxT("; ");
+    else s += wxT("style=\" ");
 
     wxString fontweights [3] = { wxT("normal"), wxT("lighter"), wxT("bold") };
-    s = s + wxT("font-weight:") + fontweights[m_font.GetWeight() - wxNORMAL] + semicolon + space;
+    s += wxT("font-weight:") + fontweights[m_font.GetWeight() - wxNORMAL] + wxT("; ");
 
     wxString fontstyles [5] = { wxT("normal"), wxT("style error"), wxT("style error"), wxT("italic"), wxT("oblique") };
-    s = s + wxT("font-style:") + fontstyles[m_font.GetStyle() - wxNORMAL] + semicolon  + space;
+    s += wxT("font-style:") + fontstyles[m_font.GetStyle() - wxNORMAL] + wxT("; ");
 
-    sTmp.Printf (wxT("font-size:%dpt; fill:#"), m_font.GetPointSize () );
-    s = s + sTmp;
-    s = s + wxColStr (m_textForegroundColour) + wxT("; stroke:#") + wxColStr (m_textForegroundColour) + wxT("; ");
-    sTmp.Printf ( wxT("stroke-width:0;\"  transform=\"rotate( %.2g %d %d )  \" >"),  -angle, x,y );
-    s = s + sTmp + sText + wxT("</text> ") + newline;
+    sTmp.Printf (wxT("font-size:%dpt; "), m_font.GetPointSize() );
+    s += sTmp;
+    //text will be solid, unless alpha value isn't opaque in the foreground colour
+    s += wxBrushString(m_textForegroundColour) + wxPenString(m_textForegroundColour);
+    sTmp.Printf ( wxT("stroke-width:0;\"  transform=\"rotate( %s %d %d )  \" >"),  NumStr(-angle), x,y );
+    s += sTmp + sText + wxT("</text> ") + wxT("\n");
     if (m_OK)
     {
         write(s);
     }
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DrawRotatedText Call executed"));
 }
 
 void wxSVGFileDCImpl::DoDrawRectangle(wxCoord x, wxCoord y, wxCoord width, wxCoord height)
@@ -275,62 +309,57 @@ void wxSVGFileDCImpl::DoDrawRectangle(wxCoord x, wxCoord y, wxCoord width, wxCoo
 void wxSVGFileDCImpl::DoDrawRoundedRectangle(wxCoord x, wxCoord y, wxCoord width, wxCoord height, double radius )
 
 {
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
     wxString s;
 
-    s.Printf ( wxT(" <rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"%.2g\" "),
-            x, y, width, height, radius );
+    s.Printf ( wxT(" <rect x=\"%d\" y=\"%d\" width=\"%d\" height=\"%d\" rx=\"%s\" "),
+            x, y, width, height, NumStr(radius) );
 
-    s = s + wxT(" /> ") + newline;
+    s += wxT(" /> \n");
     write(s);
 
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawRoundedRectangle Call executed"));
     CalcBoundingBox(x, y);
     CalcBoundingBox(x + width, y + height);
 }
 
-void wxSVGFileDCImpl::DoDrawPolygon(int n, wxPoint points[],
+void wxSVGFileDCImpl::DoDrawPolygon(int n, const wxPoint points[],
                                     wxCoord xoffset, wxCoord yoffset,
                                     wxPolygonFillMode fillStyle)
 {
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
     wxString s, sTmp;
     s = wxT("<polygon style=\"");
     if ( fillStyle == wxODDEVEN_RULE )
-        s = s + wxT("fill-rule:evenodd; ");
+        s += wxT("fill-rule:evenodd; ");
     else
-        s = s + wxT("fill-rule:nonzero; ");
+        s += wxT("fill-rule:nonzero; ");
 
-    s = s  + wxT("\" \npoints=\"");
+    s += wxT("\" \npoints=\"");
 
     for (int i = 0; i < n;  i++)
     {
         sTmp.Printf ( wxT("%d,%d"), points [i].x+xoffset, points[i].y+yoffset );
-        s = s + sTmp + newline;
+        s += sTmp + wxT("\n");
         CalcBoundingBox ( points [i].x+xoffset, points[i].y+yoffset);
     }
-    s = s + wxT("\" /> ");
-    s = s + newline;
+    s += wxT("\" /> \n");
     write(s);
-
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawPolygon Call executed"));
 }
 
 void wxSVGFileDCImpl::DoDrawEllipse (wxCoord x, wxCoord y, wxCoord width, wxCoord height)
 
 {
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
 
     int rh = height /2;
     int rw = width  /2;
 
     wxString s;
     s.Printf ( wxT("<ellipse cx=\"%d\" cy=\"%d\" rx=\"%d\" ry=\"%d\" "), x+rw,y+rh, rw, rh );
-    s = s + wxT(" /> ") + newline;
+    s += wxT(" /> \n");
 
     write(s);
 
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawEllipse Call executed"));
     CalcBoundingBox(x, y);
     CalcBoundingBox(x + width, y + height);
 }
@@ -346,7 +375,7 @@ void wxSVGFileDCImpl::DoDrawArc(wxCoord x1, wxCoord y1, wxCoord x2, wxCoord y2,
 
     Might be better described as Pie drawing */
 
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
     wxString s;
 
     // we need the radius of the circle which has two estimates
@@ -371,19 +400,17 @@ void wxSVGFileDCImpl::DoDrawArc(wxCoord x1, wxCoord y1, wxCoord x2, wxCoord y2,
 
     int fSweep = 0;             // flag for sweep always 0
 
-    s.Printf ( wxT("<path d=\"M%d %d A%.2g %.2g 0.0 %d %d %d %d L%d %d z "),
-        x1,y1, r1, r2, fArc, fSweep, x2, y2, xc, yc );
+    s.Printf ( wxT("<path d=\"M%d %d A%s %s 0.0 %d %d %d %d L%d %d z "),
+        x1,y1, NumStr(r1), NumStr(r2), fArc, fSweep, x2, y2, xc, yc );
 
     // the z means close the path and fill
-    s = s + wxT(" \" /> ") + newline;
+    s += wxT(" \" /> \n");
 
 
     if (m_OK)
     {
         write(s);
     }
-
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawArc Call executed"));
 }
 
 void wxSVGFileDCImpl::DoDrawEllipticArc(wxCoord x,wxCoord y,wxCoord w,wxCoord h,double sa,double ea)
@@ -407,7 +434,7 @@ void wxSVGFileDCImpl::DoDrawEllipticArc(wxCoord x,wxCoord y,wxCoord w,wxCoord h,
 
     //known bug: SVG draws with the current pen along the radii, but this does not happen in wxMSW
 
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
 
     wxString s;
     //radius
@@ -438,15 +465,65 @@ void wxSVGFileDCImpl::DoDrawEllipticArc(wxCoord x,wxCoord y,wxCoord w,wxCoord h,
         int(xs), int(ys), int(rx), int(ry),
         fArc, fSweep, int(xe), int(ye), int(xc), int(yc)  );
 
-
-    s = s + wxT(" \" /> ") + newline;
+    s += wxT(" \" /> \n");
 
     if (m_OK)
     {
         write(s);
     }
+}
 
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawEllipticArc Call executed"));
+void wxSVGFileDCImpl::DoSetClippingRegion( int x,  int y, int width, int height )
+{
+    wxString svg;
+
+    // End current graphics group to ensure proper xml nesting (e.g. so that
+    // graphics can be subsequently changed inside the clipping region)
+    svg << "</g>\n"
+           "<defs>\n"
+           "<clipPath id=\"clip" << m_clipNestingLevel << "\">\n"
+           "<rect id=\"cliprect" << m_clipNestingLevel << "\" "
+                "x=\"" << x << "\" "
+                "y=\"" << y << "\" "
+                "width=\"" << width << "\" "
+                "height=\"" << height << "\" "
+                "style=\"stroke: gray; fill: none;\"/>\n"
+           "</clipPath>\n"
+           "</defs>\n"
+           "<g style=\"clip-path: url(#clip" << m_clipNestingLevel << ");\">\n";
+
+    write(svg);
+
+    // Re-apply current graphics to ensure proper xml nesting
+    DoStartNewGraphics();
+
+    m_clipUniqueId++;
+    m_clipNestingLevel++;
+}
+
+void wxSVGFileDCImpl::DestroyClippingRegion()
+{
+    wxString svg;
+
+    // End current graphics element to ensure proper xml nesting (e.g. graphics
+    // might have been changed inside the clipping region)
+    svg << "</g>\n";
+
+    // Close clipping group elements
+    for ( size_t i = 0; i < m_clipUniqueId; i++ )
+    {
+        svg << "</g>";
+    }
+    svg << "\n";
+
+    write(svg);
+
+    // Re-apply current graphics (e.g. brush may have been changed inside one
+    // of the clipped regions - that change will have been lost after xml
+    // elements for the clipped region have been closed).
+    DoStartNewGraphics();
+
+    m_clipUniqueId = 0;
 }
 
 void wxSVGFileDCImpl::DoGetTextExtent(const wxString& string, wxCoord *w, wxCoord *h, wxCoord *descent , wxCoord *externalLeading , const wxFont *font) const
@@ -457,17 +534,14 @@ void wxSVGFileDCImpl::DoGetTextExtent(const wxString& string, wxCoord *w, wxCoor
     sDC.SetFont (m_font);
     if ( font != NULL ) sDC.SetFont ( *font );
     sDC.GetTextExtent(string, w,  h, descent, externalLeading );
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::GetTextExtent Call executed"));
 }
 
 wxCoord wxSVGFileDCImpl::GetCharHeight() const
-
 {
     wxScreenDC sDC;
     sDC.SetFont (m_font);
 
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::GetCharHeight Call executing"));
-    return ( sDC.GetCharHeight() );
+    return sDC.GetCharHeight();
 
 }
 
@@ -476,9 +550,7 @@ wxCoord wxSVGFileDCImpl::GetCharWidth() const
     wxScreenDC sDC;
     sDC.SetFont (m_font);
 
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::GetCharWidth Call executing"));
-    return ( sDC.GetCharWidth() );
-
+    return sDC.GetCharWidth();
 }
 
 
@@ -488,26 +560,21 @@ wxCoord wxSVGFileDCImpl::GetCharWidth() const
 
 void wxSVGFileDCImpl::SetBackground( const wxBrush &brush )
 {
-
     m_backgroundBrush = brush;
-    return;
 }
 
 
 void wxSVGFileDCImpl::SetBackgroundMode( int mode )
 {
     m_backgroundMode = mode;
-    return;
 }
 
 
 void wxSVGFileDCImpl::SetBrush(const wxBrush& brush)
-
 {
     m_brush = brush;
 
     m_graphics_changed = true;
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::SetBrush Call executed"));
 }
 
 
@@ -518,21 +585,28 @@ void wxSVGFileDCImpl::SetPen(const wxPen& pen)
     m_pen = pen;
 
     m_graphics_changed = true;
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::SetPen Call executed"));
 }
 
-void wxSVGFileDCImpl::NewGraphics ()
+void wxSVGFileDCImpl::NewGraphicsIfNeeded()
 {
+    if ( !m_graphics_changed )
+        return;
 
-    int w = m_pen.GetWidth ();
-    wxColour c = m_pen.GetColour ();
+    m_graphics_changed = false;
 
-    wxString s, sBrush, sPenCap, sPenJoin, sPenStyle, sLast, sWarn;
+    write(wxS("</g>\n"));
 
-    sBrush = wxT("</g>\n<g style=\"") + wxBrushString ( m_brush.GetColour (), m_brush.GetStyle () )
-            + wxT("  stroke:#") + wxColStr (c) + wxT("; ");
+    DoStartNewGraphics();
+}
 
-    switch ( m_pen.GetCap () )
+void wxSVGFileDCImpl::DoStartNewGraphics()
+{
+    wxString s, sBrush, sPenCap, sPenJoin, sPenStyle, sLast;
+
+    sBrush = wxS("<g style=\"") + wxBrushString ( m_brush.GetColour(), m_brush.GetStyle() )
+            + wxPenString(m_pen.GetColour(), m_pen.GetStyle());
+
+    switch ( m_pen.GetCap() )
     {
         case  wxCAP_PROJECTING :
             sPenCap = wxT("stroke-linecap:square; ");
@@ -543,8 +617,9 @@ void wxSVGFileDCImpl::NewGraphics ()
         case    wxCAP_ROUND :
         default :
             sPenCap = wxT("stroke-linecap:round; ");
-    };
-    switch ( m_pen.GetJoin () )
+    }
+
+    switch ( m_pen.GetJoin() )
     {
         case  wxJOIN_BEVEL :
             sPenJoin = wxT("stroke-linejoin:bevel; ");
@@ -555,28 +630,13 @@ void wxSVGFileDCImpl::NewGraphics ()
         case    wxJOIN_ROUND :
         default :
             sPenJoin = wxT("stroke-linejoin:round; ");
-    };
-
-    switch ( m_pen.GetStyle () )
-    {
-        case  wxPENSTYLE_SOLID :
-            sPenStyle = wxT("stroke-opacity:1.0; stroke-opacity:1.0; ");
-            break;
-        case  wxPENSTYLE_TRANSPARENT :
-            sPenStyle = wxT("stroke-opacity:0.0; stroke-opacity:0.0; ");
-            break;
-        default :
-            wxASSERT_MSG(false, wxT("wxSVGFileDC::SetPen Call called to set a Style which is not available"));
-            sWarn = sWarn + wxT("<!--- wxSVGFileDC::SetPen Call called to set a Style which is not available --> \n");
     }
 
-    sLast.Printf( wxT("stroke-width:%d\" \n   transform=\"translate(%.2g %.2g) scale(%.2g %.2g)\">"),
-                w, double(m_logicalOriginX), double(m_logicalOriginY), m_scaleX, m_scaleY  );
+    sLast.Printf( wxT("stroke-width:%d\" \n   transform=\"translate(%s %s) scale(%s %s)\">"),
+                m_pen.GetWidth(), NumStr(m_logicalOriginX), NumStr(m_logicalOriginY), NumStr(m_scaleX), NumStr(m_scaleY)  );
 
-    s = sBrush + sPenCap + sPenJoin + sPenStyle + sLast + newline + sWarn;
+    s = sBrush + sPenCap + sPenJoin + sPenStyle + sLast + wxT("\n");
     write(s);
-    m_graphics_changed = false;
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::NewGraphics Call executed"));
 }
 
 
@@ -584,8 +644,6 @@ void wxSVGFileDCImpl::SetFont(const wxFont& font)
 
 {
     m_font = font;
-
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::SetFont Call executed"));
 }
 
 // export a bitmap as a raster image in png
@@ -610,7 +668,6 @@ bool wxSVGFileDCImpl::DoBlit(wxCoord xdest, wxCoord ydest, wxCoord width, wxCoor
     memDC.Blit(0, 0, width, height, source, xsrc, ysrc);
     memDC.SelectObject( wxNullBitmap );
     DoDrawBitmap(myBitmap, xdest, ydest);
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoBlit Call executed"));
     return false;
 }
 
@@ -622,13 +679,11 @@ void wxSVGFileDCImpl::DoDrawIcon(const class wxIcon & myIcon, wxCoord x, wxCoord
     memDC.DrawIcon(myIcon,0,0);
     memDC.SelectObject( wxNullBitmap );
     DoDrawBitmap(myBitmap, x, y);
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawIcon Call executed"));
-    return;
 }
 
 void wxSVGFileDCImpl::DoDrawBitmap(const class wxBitmap & bmp, wxCoord x, wxCoord y , bool  WXUNUSED(bTransparent) /*=0*/ )
 {
-    if (m_graphics_changed) NewGraphics ();
+    NewGraphicsIfNeeded();
 
     wxString sTmp, s, sPNG;
     if ( wxImage::FindHandler(wxBITMAP_TYPE_PNG) == NULL )
@@ -657,25 +712,22 @@ void wxSVGFileDCImpl::DoDrawBitmap(const class wxBitmap & bmp, wxCoord x, wxCoor
     int w = myBitmap.GetWidth();
     int h = myBitmap.GetHeight();
     sTmp.Printf ( wxT(" <image x=\"%d\" y=\"%d\" width=\"%dpx\" height=\"%dpx\" "), x,y,w,h );
-    s = s + sTmp;
+    s += sTmp;
     sTmp.Printf ( wxT(" xlink:href=\"%s\"> \n"), sPNG.c_str() );
-    s = s + sTmp + wxT("<title>Image from wxSVG</title>  </image>") + newline;
+    s += sTmp + wxT("<title>Image from wxSVG</title>  </image>") + wxT("\n");
 
     if (m_OK && bPNG_OK)
     {
         write(s);
     }
-    m_OK = m_outfile->Ok () && bPNG_OK;
-    wxASSERT_MSG(!wxSVG_DEBUG, wxT("wxSVGFileDC::DoDrawBitmap Call executed"));
-
-    return;
+    m_OK = m_outfile->IsOk() && bPNG_OK;
 }
 
 void wxSVGFileDCImpl::write(const wxString &s)
 {
     const wxCharBuffer buf = s.utf8_str();
     m_outfile->Write(buf, strlen((const char *)buf));
-    m_OK = m_outfile->Ok();
+    m_OK = m_outfile->IsOk();
 }
 
 
