@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     04.11.99
-// RCS-ID:      $Id: fontmap.cpp 61508 2009-07-23 20:30:22Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999-2003 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -52,61 +52,64 @@
 
 #include "wx/encconv.h"
 
-#if wxUSE_EXTENDED_RTTI
+// ----------------------------------------------------------------------------
+// XTI
+// ----------------------------------------------------------------------------
 
 wxBEGIN_ENUM( wxFontEncoding )
-    wxENUM_MEMBER( wxFONTENCODING_SYSTEM )
-    wxENUM_MEMBER( wxFONTENCODING_DEFAULT )
+wxENUM_MEMBER( wxFONTENCODING_SYSTEM )
+wxENUM_MEMBER( wxFONTENCODING_DEFAULT )
 
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_1 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_2 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_3 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_4 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_5 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_6 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_7 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_8 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_9 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_10 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_11 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_12 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_13 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_14 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_15 )
-    wxENUM_MEMBER( wxFONTENCODING_ISO8859_MAX )
-    wxENUM_MEMBER( wxFONTENCODING_KOI8 )
-    wxENUM_MEMBER( wxFONTENCODING_KOI8_U )
-    wxENUM_MEMBER( wxFONTENCODING_ALTERNATIVE )
-    wxENUM_MEMBER( wxFONTENCODING_BULGARIAN )
-    wxENUM_MEMBER( wxFONTENCODING_CP437 )
-    wxENUM_MEMBER( wxFONTENCODING_CP850 )
-    wxENUM_MEMBER( wxFONTENCODING_CP852 )
-    wxENUM_MEMBER( wxFONTENCODING_CP855 )
-    wxENUM_MEMBER( wxFONTENCODING_CP866 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_1 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_2 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_3 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_4 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_5 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_6 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_7 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_8 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_9 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_10 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_11 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_12 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_13 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_14 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_15 )
+wxENUM_MEMBER( wxFONTENCODING_ISO8859_MAX )
+wxENUM_MEMBER( wxFONTENCODING_KOI8 )
+wxENUM_MEMBER( wxFONTENCODING_KOI8_U )
+wxENUM_MEMBER( wxFONTENCODING_ALTERNATIVE )
+wxENUM_MEMBER( wxFONTENCODING_BULGARIAN )
+wxENUM_MEMBER( wxFONTENCODING_CP437 )
+wxENUM_MEMBER( wxFONTENCODING_CP850 )
+wxENUM_MEMBER( wxFONTENCODING_CP852 )
+wxENUM_MEMBER( wxFONTENCODING_CP855 )
+wxENUM_MEMBER( wxFONTENCODING_CP866 )
 
-    wxENUM_MEMBER( wxFONTENCODING_CP874 )
-    wxENUM_MEMBER( wxFONTENCODING_CP932 )
-    wxENUM_MEMBER( wxFONTENCODING_CP936 )
-    wxENUM_MEMBER( wxFONTENCODING_CP949 )
-    wxENUM_MEMBER( wxFONTENCODING_CP950 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1250 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1251 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1252 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1253 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1254 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1255 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1256 )
-    wxENUM_MEMBER( wxFONTENCODING_CP1257 )
-    wxENUM_MEMBER( wxFONTENCODING_CP12_MAX )
-    wxENUM_MEMBER( wxFONTENCODING_UTF7 )
-    wxENUM_MEMBER( wxFONTENCODING_UTF8 )
-    wxENUM_MEMBER( wxFONTENCODING_GB2312 )
-    wxENUM_MEMBER( wxFONTENCODING_BIG5 )
-    wxENUM_MEMBER( wxFONTENCODING_SHIFT_JIS )
-    wxENUM_MEMBER( wxFONTENCODING_EUC_JP )
-    wxENUM_MEMBER( wxFONTENCODING_UNICODE )
+wxENUM_MEMBER( wxFONTENCODING_CP874 )
+wxENUM_MEMBER( wxFONTENCODING_CP932 )
+wxENUM_MEMBER( wxFONTENCODING_CP936 )
+wxENUM_MEMBER( wxFONTENCODING_CP949 )
+wxENUM_MEMBER( wxFONTENCODING_CP950 )
+wxENUM_MEMBER( wxFONTENCODING_CP1250 )
+wxENUM_MEMBER( wxFONTENCODING_CP1251 )
+wxENUM_MEMBER( wxFONTENCODING_CP1252 )
+wxENUM_MEMBER( wxFONTENCODING_CP1253 )
+wxENUM_MEMBER( wxFONTENCODING_CP1254 )
+wxENUM_MEMBER( wxFONTENCODING_CP1255 )
+wxENUM_MEMBER( wxFONTENCODING_CP1256 )
+wxENUM_MEMBER( wxFONTENCODING_CP1257 )
+wxENUM_MEMBER( wxFONTENCODING_CP1258 )
+wxENUM_MEMBER( wxFONTENCODING_CP1361 )
+wxENUM_MEMBER( wxFONTENCODING_CP12_MAX )
+wxENUM_MEMBER( wxFONTENCODING_UTF7 )
+wxENUM_MEMBER( wxFONTENCODING_UTF8 )
+wxENUM_MEMBER( wxFONTENCODING_GB2312 )
+wxENUM_MEMBER( wxFONTENCODING_BIG5 )
+wxENUM_MEMBER( wxFONTENCODING_SHIFT_JIS )
+wxENUM_MEMBER( wxFONTENCODING_EUC_JP )
+wxENUM_MEMBER( wxFONTENCODING_UNICODE )
 wxEND_ENUM( wxFontEncoding )
-#endif
 
 // ----------------------------------------------------------------------------
 // constants
