@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     22.10.99
-// RCS-ID:      $Id: lboxcmn.cpp 64496 2010-06-05 22:57:53Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -31,9 +31,10 @@
 #ifndef WX_PRECOMP
     #include "wx/dynarray.h"
     #include "wx/arrstr.h"
+    #include "wx/log.h"
 #endif
 
-#include "wx/log.h"
+extern WXDLLEXPORT_DATA(const char) wxListBoxNameStr[] = "listBox";
 
 // ============================================================================
 // implementation
@@ -44,6 +45,78 @@ wxListBoxBase::~wxListBoxBase()
     // this destructor is required for Darwin
 }
 
+// ----------------------------------------------------------------------------
+// XTI
+// ----------------------------------------------------------------------------
+
+wxDEFINE_FLAGS( wxListBoxStyle )
+wxBEGIN_FLAGS( wxListBoxStyle )
+// new style border flags, we put them first to
+// use them for streaming out
+wxFLAGS_MEMBER(wxBORDER_SIMPLE)
+wxFLAGS_MEMBER(wxBORDER_SUNKEN)
+wxFLAGS_MEMBER(wxBORDER_DOUBLE)
+wxFLAGS_MEMBER(wxBORDER_RAISED)
+wxFLAGS_MEMBER(wxBORDER_STATIC)
+wxFLAGS_MEMBER(wxBORDER_NONE)
+
+// old style border flags
+wxFLAGS_MEMBER(wxSIMPLE_BORDER)
+wxFLAGS_MEMBER(wxSUNKEN_BORDER)
+wxFLAGS_MEMBER(wxDOUBLE_BORDER)
+wxFLAGS_MEMBER(wxRAISED_BORDER)
+wxFLAGS_MEMBER(wxSTATIC_BORDER)
+wxFLAGS_MEMBER(wxBORDER)
+
+// standard window styles
+wxFLAGS_MEMBER(wxTAB_TRAVERSAL)
+wxFLAGS_MEMBER(wxCLIP_CHILDREN)
+wxFLAGS_MEMBER(wxTRANSPARENT_WINDOW)
+wxFLAGS_MEMBER(wxWANTS_CHARS)
+wxFLAGS_MEMBER(wxFULL_REPAINT_ON_RESIZE)
+wxFLAGS_MEMBER(wxALWAYS_SHOW_SB )
+wxFLAGS_MEMBER(wxVSCROLL)
+wxFLAGS_MEMBER(wxHSCROLL)
+
+wxFLAGS_MEMBER(wxLB_SINGLE)
+wxFLAGS_MEMBER(wxLB_MULTIPLE)
+wxFLAGS_MEMBER(wxLB_EXTENDED)
+wxFLAGS_MEMBER(wxLB_HSCROLL)
+wxFLAGS_MEMBER(wxLB_ALWAYS_SB)
+wxFLAGS_MEMBER(wxLB_NEEDED_SB)
+wxFLAGS_MEMBER(wxLB_SORT)
+wxEND_FLAGS( wxListBoxStyle )
+
+wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxListBox, wxControl, "wx/listbox.h")
+
+wxBEGIN_PROPERTIES_TABLE(wxListBox)
+wxEVENT_PROPERTY( Select, wxEVT_LISTBOX, wxCommandEvent )
+wxEVENT_PROPERTY( DoubleClick, wxEVT_LISTBOX_DCLICK, wxCommandEvent )
+
+wxPROPERTY( Font, wxFont, SetFont, GetFont , wxEMPTY_PARAMETER_VALUE, 0 /*flags*/, \
+           wxT("Helpstring"), wxT("group"))
+wxPROPERTY_COLLECTION( Choices, wxArrayString, wxString, AppendString, \
+                      GetStrings, 0 /*flags*/, wxT("Helpstring"), wxT("group") )
+wxPROPERTY( Selection, int, SetSelection, GetSelection, wxEMPTY_PARAMETER_VALUE, \
+           0 /*flags*/, wxT("Helpstring"), wxT("group") )
+
+wxPROPERTY_FLAGS( WindowStyle, wxListBoxStyle, long, SetWindowStyleFlag, \
+                 GetWindowStyleFlag, wxEMPTY_PARAMETER_VALUE, 0 /*flags*/, \
+                 wxT("Helpstring"), wxT("group")) // style
+wxEND_PROPERTIES_TABLE()
+
+wxEMPTY_HANDLERS_TABLE(wxListBox)
+
+wxCONSTRUCTOR_4( wxListBox, wxWindow*, Parent, wxWindowID, Id, \
+                wxPoint, Position, wxSize, Size )
+
+/*
+ TODO PROPERTIES
+ selection
+ content
+ item
+ */
+
 // ----------------------------------------------------------------------------
 // selection
 // ----------------------------------------------------------------------------
@@ -59,6 +132,14 @@ bool wxListBoxBase::SetStringSelection(const wxString& s, bool select)
     return true;
 }
 
+void wxListBoxBase::SetSelection(int n)
+{
+    if ( !HasMultipleSelection() )
+        DoChangeSingleSelection(n);
+
+    DoSetSelection(n, true);
+}
+
 void wxListBoxBase::DeselectAll(int itemToLeaveSelected)
 {
     if ( HasMultipleSelection() )
@@ -86,8 +167,23 @@ void wxListBoxBase::DeselectAll(int itemToLeaveSelected)
 
 void wxListBoxBase::UpdateOldSelections()
 {
+    // When the control becomes empty, any previously remembered selections are
+    // invalid anyhow, so just forget them.
+    if ( IsEmpty() )
+    {
+        m_oldSelections.clear();
+        return;
+    }
+
+    // We need to remember the selection even in single-selection case on
+    // Windows, so that we don't send an event when the user clicks on an
+    // already selected item.
+#ifndef __WXMSW__
     if (HasFlag(wxLB_MULTIPLE) || HasFlag(wxLB_EXTENDED))
+#endif
+    {
         GetSelections( m_oldSelections );
+    }
 }
 
 bool wxListBoxBase::SendEvent(wxEventType evtType, int item, bool selected)
@@ -107,6 +203,24 @@ bool wxListBoxBase::SendEvent(wxEventType evtType, int item, bool selected)
     return HandleWindowEvent(event);
 }
 
+bool wxListBoxBase::DoChangeSingleSelection(int item)
+{
+    // As we don't use m_oldSelections in single selection mode, we store the
+    // last item that we notified the user about in it in this case because we
+    // need to remember it to be able to filter out the dummy selection changes
+    // that we get when the user clicks on an already selected item.
+    if ( !m_oldSelections.empty() && *m_oldSelections.begin() == item )
+    {
+        // Same item as the last time.
+        return false;
+    }
+
+    m_oldSelections.clear();
+    m_oldSelections.push_back(item);
+
+    return true;
+}
+
 bool wxListBoxBase::CalcAndSendEvent()
 {
     wxArrayInt selections;
@@ -189,7 +303,7 @@ bool wxListBoxBase::CalcAndSendEvent()
 
     m_oldSelections = selections;
 
-    return SendEvent(wxEVT_COMMAND_LISTBOX_SELECTED, item, selected);
+    return SendEvent(wxEVT_LISTBOX, item, selected);
 }
 
 // ----------------------------------------------------------------------------
