@@ -4,7 +4,7 @@
 // Author:      Robin Dunn
 // Modified by:
 // Created:     27-Mar-2003
-// RCS-ID:      $Id: rgncmn.cpp 64874 2010-07-11 10:43:18Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Robin Dunn
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -33,7 +33,7 @@
 // ============================================================================
 
 // ----------------------------------------------------------------------------
-// region comparision
+// region comparison
 // ----------------------------------------------------------------------------
 
 bool wxRegionBase::IsEqual(const wxRegion& region) const
@@ -60,7 +60,7 @@ bool wxRegionBase::IsEqual(const wxRegion& region) const
 wxBitmap wxRegionBase::ConvertToBitmap() const
 {
     wxRect box = GetBox();
-    wxBitmap bmp(box.GetWidth(), box.GetHeight());
+    wxBitmap bmp(box.GetRight() + 1, box.GetBottom() + 1);
     wxMemoryDC dc;
     dc.SelectObject(bmp);
     dc.SetBackground(*wxBLACK_BRUSH);
