@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     13.07.99
-// RCS-ID:      $Id: textcmn.cpp 63880 2010-04-06 18:46:24Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -35,6 +35,8 @@
 
 #include "wx/ffile.h"
 
+extern WXDLLEXPORT_DATA(const char) wxTextCtrlNameStr[] = "text";
+
 // ----------------------------------------------------------------------------
 // macros
 // ----------------------------------------------------------------------------
@@ -47,12 +49,86 @@
 // implementation
 // ============================================================================
 
+// ----------------------------------------------------------------------------
+// XTI
+// ----------------------------------------------------------------------------
+
+wxDEFINE_FLAGS( wxTextCtrlStyle )
+wxBEGIN_FLAGS( wxTextCtrlStyle )
+// new style border flags, we put them first to
+// use them for streaming out
+wxFLAGS_MEMBER(wxBORDER_SIMPLE)
+wxFLAGS_MEMBER(wxBORDER_SUNKEN)
+wxFLAGS_MEMBER(wxBORDER_DOUBLE)
+wxFLAGS_MEMBER(wxBORDER_RAISED)
+wxFLAGS_MEMBER(wxBORDER_STATIC)
+wxFLAGS_MEMBER(wxBORDER_NONE)
+
+// old style border flags
+wxFLAGS_MEMBER(wxSIMPLE_BORDER)
+wxFLAGS_MEMBER(wxSUNKEN_BORDER)
+wxFLAGS_MEMBER(wxDOUBLE_BORDER)
+wxFLAGS_MEMBER(wxRAISED_BORDER)
+wxFLAGS_MEMBER(wxSTATIC_BORDER)
+wxFLAGS_MEMBER(wxBORDER)
+
+// standard window styles
+wxFLAGS_MEMBER(wxTAB_TRAVERSAL)
+wxFLAGS_MEMBER(wxCLIP_CHILDREN)
+wxFLAGS_MEMBER(wxTRANSPARENT_WINDOW)
+wxFLAGS_MEMBER(wxWANTS_CHARS)
+wxFLAGS_MEMBER(wxFULL_REPAINT_ON_RESIZE)
+wxFLAGS_MEMBER(wxALWAYS_SHOW_SB )
+wxFLAGS_MEMBER(wxVSCROLL)
+wxFLAGS_MEMBER(wxHSCROLL)
+
+wxFLAGS_MEMBER(wxTE_PROCESS_ENTER)
+wxFLAGS_MEMBER(wxTE_PROCESS_TAB)
+wxFLAGS_MEMBER(wxTE_MULTILINE)
+wxFLAGS_MEMBER(wxTE_PASSWORD)
+wxFLAGS_MEMBER(wxTE_READONLY)
+wxFLAGS_MEMBER(wxHSCROLL)
+wxFLAGS_MEMBER(wxTE_RICH)
+wxFLAGS_MEMBER(wxTE_RICH2)
+wxFLAGS_MEMBER(wxTE_AUTO_URL)
+wxFLAGS_MEMBER(wxTE_NOHIDESEL)
+wxFLAGS_MEMBER(wxTE_LEFT)
+wxFLAGS_MEMBER(wxTE_CENTRE)
+wxFLAGS_MEMBER(wxTE_RIGHT)
+wxFLAGS_MEMBER(wxTE_DONTWRAP)
+wxFLAGS_MEMBER(wxTE_CHARWRAP)
+wxFLAGS_MEMBER(wxTE_WORDWRAP)
+wxEND_FLAGS( wxTextCtrlStyle )
+
+wxIMPLEMENT_DYNAMIC_CLASS_XTI(wxTextCtrl, wxControl, "wx/textctrl.h")
+
+wxBEGIN_PROPERTIES_TABLE(wxTextCtrl)
+wxEVENT_PROPERTY( TextUpdated, wxEVT_TEXT, wxCommandEvent )
+wxEVENT_PROPERTY( TextEnter, wxEVT_TEXT_ENTER, wxCommandEvent )
+
+wxPROPERTY( Font, wxFont, SetFont, GetFont , wxEMPTY_PARAMETER_VALUE, \
+           0 /*flags*/, wxT("Helpstring"), wxT("group") )
+wxPROPERTY( Value, wxString, SetValue, GetValue, wxString(), \
+           0 /*flags*/, wxT("Helpstring"), wxT("group"))
+
+wxPROPERTY_FLAGS( WindowStyle, wxTextCtrlStyle, long, SetWindowStyleFlag, \
+                 GetWindowStyleFlag, wxEMPTY_PARAMETER_VALUE, 0 /*flags*/, \
+                 wxT("Helpstring"), wxT("group")) // style
+wxEND_PROPERTIES_TABLE()
+
+wxEMPTY_HANDLERS_TABLE(wxTextCtrl)
+
+wxCONSTRUCTOR_6( wxTextCtrl, wxWindow*, Parent, wxWindowID, Id, \
+                wxString, Value, wxPoint, Position, wxSize, Size, \
+                long, WindowStyle)
+
+
 IMPLEMENT_DYNAMIC_CLASS(wxTextUrlEvent, wxCommandEvent)
 
-wxDEFINE_EVENT( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEvent );
-wxDEFINE_EVENT( wxEVT_COMMAND_TEXT_ENTER, wxCommandEvent );
-wxDEFINE_EVENT( wxEVT_COMMAND_TEXT_URL, wxTextUrlEvent );
-wxDEFINE_EVENT( wxEVT_COMMAND_TEXT_MAXLEN, wxCommandEvent );
+wxDEFINE_EVENT( wxEVT_TEXT, wxCommandEvent );
+wxDEFINE_EVENT( wxEVT_TEXT_ENTER, wxCommandEvent );
+wxDEFINE_EVENT( wxEVT_TEXT_URL, wxTextUrlEvent );
+wxDEFINE_EVENT( wxEVT_TEXT_MAXLEN, wxCommandEvent );
 
 IMPLEMENT_ABSTRACT_CLASS(wxTextCtrlBase, wxControl)
 
@@ -67,8 +143,8 @@ wxTextAttr::wxTextAttr(const wxColour& colText,
 {
     Init();
 
-    if (m_colText.Ok()) m_flags |= wxTEXT_ATTR_TEXT_COLOUR;
-    if (m_colBack.Ok()) m_flags |= wxTEXT_ATTR_BACKGROUND_COLOUR;
+    if (m_colText.IsOk()) m_flags |= wxTEXT_ATTR_TEXT_COLOUR;
+    if (m_colBack.IsOk()) m_flags |= wxTEXT_ATTR_BACKGROUND_COLOUR;
     if (alignment != wxTEXT_ALIGNMENT_DEFAULT)
         m_flags |= wxTEXT_ATTR_ALIGNMENT;
 
@@ -88,6 +164,7 @@ void wxTextAttr::Init()
     m_fontStyle = wxFONTSTYLE_NORMAL;
     m_fontWeight = wxFONTWEIGHT_NORMAL;
     m_fontUnderlined = false;
+    m_fontStrikethrough = false;
     m_fontEncoding = wxFONTENCODING_DEFAULT;
     m_fontFamily = wxFONTFAMILY_DEFAULT;
 
@@ -117,6 +194,7 @@ void wxTextAttr::Copy(const wxTextAttr& attr)
     m_fontStyle = attr.m_fontStyle;
     m_fontWeight = attr.m_fontWeight;
     m_fontUnderlined = attr.m_fontUnderlined;
+    m_fontStrikethrough = attr.m_fontStrikethrough;
     m_fontFaceName = attr.m_fontFaceName;
     m_fontEncoding = attr.m_fontEncoding;
     m_fontFamily = attr.m_fontFamily;
@@ -188,116 +266,139 @@ bool wxTextAttr::operator== (const wxTextAttr& attr) const
             GetURL() == attr.GetURL();
 }
 
-// Partial equality test taking flags into account
-bool wxTextAttr::EqPartial(const wxTextAttr& attr, int flags) const
+// Partial equality test. Only returns false if an attribute doesn't match.
+bool wxTextAttr::EqPartial(const wxTextAttr& attr, bool weakTest) const
 {
-    if ((flags & wxTEXT_ATTR_TEXT_COLOUR) && GetTextColour() != attr.GetTextColour())
+    int flags = attr.GetFlags();
+
+    if (!weakTest &&
+        ((!HasTextColour() && attr.HasTextColour()) ||
+         (!HasBackgroundColour() && attr.HasBackgroundColour()) ||
+         (!HasFontFaceName() && attr.HasFontFaceName()) ||
+         (!HasFontSize() && attr.HasFontSize()) ||
+         (!HasFontWeight() && attr.HasFontWeight()) ||
+         (!HasFontItalic() && attr.HasFontItalic()) ||
+         (!HasFontUnderlined() && attr.HasFontUnderlined()) ||
+         (!HasFontStrikethrough() && attr.HasFontStrikethrough()) ||
+         (!HasFontEncoding() && attr.HasFontEncoding()) ||
+         (!HasFontFamily() && attr.HasFontFamily()) ||
+         (!HasURL() && attr.HasURL()) ||
+         (!HasAlignment() && attr.HasAlignment()) ||
+         (!HasLeftIndent() && attr.HasLeftIndent()) ||
+         (!HasParagraphSpacingAfter() && attr.HasParagraphSpacingAfter()) ||
+         (!HasParagraphSpacingBefore() && attr.HasParagraphSpacingBefore()) ||
+         (!HasLineSpacing() && attr.HasLineSpacing()) ||
+         (!HasCharacterStyleName() && attr.HasCharacterStyleName()) ||
+         (!HasParagraphStyleName() && attr.HasParagraphStyleName()) ||
+         (!HasListStyleName() && attr.HasListStyleName()) ||
+         (!HasBulletStyle() && attr.HasBulletStyle()) ||
+         (!HasBulletNumber() && attr.HasBulletNumber()) ||
+         (!HasBulletText() && attr.HasBulletText()) ||
+         (!HasBulletName() && attr.HasBulletName()) ||
+         (!HasTabs() && attr.HasTabs()) ||
+         (!HasTextEffects() && attr.HasTextEffects()) ||
+         (!HasOutlineLevel() && attr.HasOutlineLevel())))
+    {
+        return false;
+    }
+
+    if (HasTextColour() && attr.HasTextColour() && GetTextColour() != attr.GetTextColour())
         return false;
 
-    if ((flags & wxTEXT_ATTR_BACKGROUND_COLOUR) && GetBackgroundColour() != attr.GetBackgroundColour())
+    if (HasBackgroundColour() && attr.HasBackgroundColour() && GetBackgroundColour() != attr.GetBackgroundColour())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_FACE) &&
-        GetFontFaceName() != attr.GetFontFaceName())
+    if (HasFontFaceName() && attr.HasFontFaceName() && GetFontFaceName() != attr.GetFontFaceName())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_SIZE) &&
-        GetFontSize() != attr.GetFontSize())
+    // This checks whether the two objects have the same font size dimension (px versus pt)
+    if (HasFontSize() && attr.HasFontSize() && (flags & wxTEXT_ATTR_FONT) != (GetFlags() & wxTEXT_ATTR_FONT))
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_WEIGHT) &&
-        GetFontWeight() != attr.GetFontWeight())
+    if (HasFontPointSize() && attr.HasFontPointSize() && GetFontSize() != attr.GetFontSize())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_ITALIC) &&
-        GetFontStyle() != attr.GetFontStyle())
+    if (HasFontPixelSize() && attr.HasFontPixelSize() && GetFontSize() != attr.GetFontSize())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_UNDERLINE) &&
-        GetFontUnderlined() != attr.GetFontUnderlined())
+    if (HasFontWeight() && attr.HasFontWeight() && GetFontWeight() != attr.GetFontWeight())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_ENCODING) &&
-        GetFontEncoding() != attr.GetFontEncoding())
+    if (HasFontItalic() && attr.HasFontItalic() && GetFontStyle() != attr.GetFontStyle())
         return false;
 
-    if ((flags & wxTEXT_ATTR_FONT_FAMILY) &&
-        GetFontFamily() != attr.GetFontFamily())
+    if (HasFontUnderlined() && attr.HasFontUnderlined() && GetFontUnderlined() != attr.GetFontUnderlined())
         return false;
 
-    if ((flags & wxTEXT_ATTR_URL) && GetURL() != attr.GetURL())
+    if (HasFontStrikethrough() && attr.HasFontStrikethrough() && GetFontStrikethrough() != attr.GetFontStrikethrough())
         return false;
 
-    if ((flags & wxTEXT_ATTR_ALIGNMENT) && GetAlignment() != attr.GetAlignment())
+    if (HasFontEncoding() && attr.HasFontEncoding() && GetFontEncoding() != attr.GetFontEncoding())
         return false;
 
-    if ((flags & wxTEXT_ATTR_LEFT_INDENT) &&
+    if (HasFontFamily() && attr.HasFontFamily() && GetFontFamily() != attr.GetFontFamily())
+        return false;
+
+    if (HasURL() && attr.HasURL() && GetURL() != attr.GetURL())
+        return false;
+
+    if (HasAlignment() && attr.HasAlignment() && GetAlignment() != attr.GetAlignment())
+        return false;
+
+    if (HasLeftIndent() && attr.HasLeftIndent() &&
         ((GetLeftIndent() != attr.GetLeftIndent()) || (GetLeftSubIndent() != attr.GetLeftSubIndent())))
         return false;
 
-    if ((flags & wxTEXT_ATTR_RIGHT_INDENT) &&
-        (GetRightIndent() != attr.GetRightIndent()))
+    if (HasRightIndent() && attr.HasRightIndent() && (GetRightIndent() != attr.GetRightIndent()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_PARA_SPACING_AFTER) &&
+    if (HasParagraphSpacingAfter() && attr.HasParagraphSpacingAfter() &&
         (GetParagraphSpacingAfter() != attr.GetParagraphSpacingAfter()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_PARA_SPACING_BEFORE) &&
+    if (HasParagraphSpacingBefore() && attr.HasParagraphSpacingBefore() &&
         (GetParagraphSpacingBefore() != attr.GetParagraphSpacingBefore()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_LINE_SPACING) &&
-        (GetLineSpacing() != attr.GetLineSpacing()))
+    if (HasLineSpacing() && attr.HasLineSpacing() && (GetLineSpacing() != attr.GetLineSpacing()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_CHARACTER_STYLE_NAME) &&
-        (GetCharacterStyleName() != attr.GetCharacterStyleName()))
+    if (HasCharacterStyleName() && attr.HasCharacterStyleName() && (GetCharacterStyleName() != attr.GetCharacterStyleName()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_PARAGRAPH_STYLE_NAME) &&
-        (GetParagraphStyleName() != attr.GetParagraphStyleName()))
+    if (HasParagraphStyleName() && attr.HasParagraphStyleName() && (GetParagraphStyleName() != attr.GetParagraphStyleName()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_LIST_STYLE_NAME) &&
-        (GetListStyleName() != attr.GetListStyleName()))
+    if (HasListStyleName() && attr.HasListStyleName() && (GetListStyleName() != attr.GetListStyleName()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_BULLET_STYLE) &&
-        (GetBulletStyle() != attr.GetBulletStyle()))
+    if (HasBulletStyle() && attr.HasBulletStyle() && (GetBulletStyle() != attr.GetBulletStyle()))
          return false;
 
-    if ((flags & wxTEXT_ATTR_BULLET_NUMBER) &&
-        (GetBulletNumber() != attr.GetBulletNumber()))
+    if (HasBulletNumber() && attr.HasBulletNumber() && (GetBulletNumber() != attr.GetBulletNumber()))
          return false;
 
-    if ((flags & wxTEXT_ATTR_BULLET_TEXT) &&
+    if (HasBulletText() && attr.HasBulletText() &&
         (GetBulletText() != attr.GetBulletText()) &&
         (GetBulletFont() != attr.GetBulletFont()))
          return false;
 
-    if ((flags & wxTEXT_ATTR_BULLET_NAME) &&
-        (GetBulletName() != attr.GetBulletName()))
+    if (HasBulletName() && attr.HasBulletName() && (GetBulletName() != attr.GetBulletName()))
          return false;
 
-    if ((flags & wxTEXT_ATTR_TABS) &&
-        !TabsEq(GetTabs(), attr.GetTabs()))
+    if (HasTabs() && attr.HasTabs() && !TabsEq(GetTabs(), attr.GetTabs()))
         return false;
 
-    if ((flags & wxTEXT_ATTR_PAGE_BREAK) &&
-        (HasPageBreak() != attr.HasPageBreak()))
+    if ((HasPageBreak() != attr.HasPageBreak()))
          return false;
 
-    if (flags & wxTEXT_ATTR_EFFECTS)
+    if (HasTextEffects() && attr.HasTextEffects())
     {
-        if (HasTextEffects() != attr.HasTextEffects())
-            return false;
         if (!BitlistsEqPartial(GetTextEffects(), attr.GetTextEffects(), attr.GetTextEffectFlags()))
             return false;
     }
 
-    if ((flags & wxTEXT_ATTR_OUTLINE_LEVEL) &&
-        (GetOutlineLevel() != attr.GetOutlineLevel()))
+    if (HasOutlineLevel() && attr.HasOutlineLevel() && (GetOutlineLevel() != attr.GetOutlineLevel()))
          return false;
 
     return true;
@@ -313,11 +414,11 @@ wxFont wxTextAttr::GetFont() const
     if (HasFontSize())
         fontSize = GetFontSize();
 
-    int fontStyle = wxNORMAL;
+    wxFontStyle fontStyle = wxFONTSTYLE_NORMAL;
     if (HasFontItalic())
         fontStyle = GetFontStyle();
 
-    int fontWeight = wxNORMAL;
+    wxFontWeight fontWeight = wxFONTWEIGHT_NORMAL;
     if (HasFontWeight())
         fontWeight = GetFontWeight();
 
@@ -325,6 +426,10 @@ wxFont wxTextAttr::GetFont() const
     if (HasFontUnderlined())
         underlined = GetFontUnderlined();
 
+    bool strikethrough = false;
+    if (HasFontStrikethrough())
+        strikethrough = GetFontStrikethrough();
+
     wxString fontFaceName;
     if (HasFontFaceName())
         fontFaceName = GetFontFaceName();
@@ -337,18 +442,52 @@ wxFont wxTextAttr::GetFont() const
     if (HasFontFamily())
         fontFamily = GetFontFamily();
 
-    wxFont font(fontSize, fontFamily, fontStyle, fontWeight, underlined, fontFaceName, encoding);
-    return font;
+    if (HasFontPixelSize())
+    {
+        wxFont font(wxSize(0, fontSize), fontFamily, fontStyle, fontWeight, underlined, fontFaceName, encoding);
+        if (strikethrough)
+            font.SetStrikethrough(true);
+        return font;
+    }
+    else
+    {
+        wxFont font(fontSize, fontFamily, fontStyle, fontWeight, underlined, fontFaceName, encoding);
+        if (strikethrough)
+            font.SetStrikethrough(true);
+        return font;
+    }
 }
 
 // Get attributes from font.
 bool wxTextAttr::GetFontAttributes(const wxFont& font, int flags)
 {
-    if (!font.Ok())
+    if (!font.IsOk())
         return false;
 
-    if (flags & wxTEXT_ATTR_FONT_SIZE)
+    // If we pass both pixel and point size attributes, this is an indication
+    // to choose the most appropriate units.
+    if ((flags & wxTEXT_ATTR_FONT) == wxTEXT_ATTR_FONT)
+    {
+        if (font.IsUsingSizeInPixels())
+        {
+            m_fontSize = font.GetPixelSize().y;
+            flags &= ~wxTEXT_ATTR_FONT_POINT_SIZE;
+        }
+        else
+        {
+            m_fontSize = font.GetPointSize();
+            flags &= ~wxTEXT_ATTR_FONT_PIXEL_SIZE;
+        }
+    }
+    else if (flags & wxTEXT_ATTR_FONT_POINT_SIZE)
+    {
         m_fontSize = font.GetPointSize();
+        flags &= ~wxTEXT_ATTR_FONT_PIXEL_SIZE;
+    }
+    else if (flags & wxTEXT_ATTR_FONT_PIXEL_SIZE)
+    {
+        m_fontSize = font.GetPixelSize().y;
+    }
 
     if (flags & wxTEXT_ATTR_FONT_ITALIC)
         m_fontStyle = font.GetStyle();
@@ -359,6 +498,9 @@ bool wxTextAttr::GetFontAttributes(const wxFont& font, int flags)
     if (flags & wxTEXT_ATTR_FONT_UNDERLINE)
         m_fontUnderlined = font.GetUnderlined();
 
+    if (flags & wxTEXT_ATTR_FONT_STRIKETHROUGH)
+        m_fontStrikethrough = font.GetStrikethrough();
+
     if (flags & wxTEXT_ATTR_FONT_FACE)
         m_fontFaceName = font.GetFaceName();
 
@@ -404,10 +546,15 @@ bool wxTextAttr::Apply(const wxTextAttr& style, const wxTextAttr* compareWith)
             destStyle.SetFontWeight(style.GetFontWeight());
     }
 
-    if (style.HasFontSize())
+    if (style.HasFontPointSize())
     {
-        if (!(compareWith && compareWith->HasFontSize() && compareWith->GetFontSize() == style.GetFontSize()))
-            destStyle.SetFontSize(style.GetFontSize());
+        if (!(compareWith && compareWith->HasFontPointSize() && compareWith->GetFontSize() == style.GetFontSize()))
+            destStyle.SetFontPointSize(style.GetFontSize());
+    }
+    else if (style.HasFontPixelSize())
+    {
+        if (!(compareWith && compareWith->HasFontPixelSize() && compareWith->GetFontSize() == style.GetFontSize()))
+            destStyle.SetFontPixelSize(style.GetFontSize());
     }
 
     if (style.HasFontItalic())
@@ -422,6 +569,12 @@ bool wxTextAttr::Apply(const wxTextAttr& style, const wxTextAttr* compareWith)
             destStyle.SetFontUnderlined(style.GetFontUnderlined());
     }
 
+    if (style.HasFontStrikethrough())
+    {
+        if (!(compareWith && compareWith->HasFontStrikethrough() && compareWith->GetFontStrikethrough() == style.GetFontStrikethrough()))
+            destStyle.SetFontStrikethrough(style.GetFontStrikethrough());
+    }
+
     if (style.HasFontFaceName())
     {
         if (!(compareWith && compareWith->HasFontFaceName() && compareWith->GetFontFaceName() == style.GetFontFaceName()))
@@ -440,13 +593,13 @@ bool wxTextAttr::Apply(const wxTextAttr& style, const wxTextAttr* compareWith)
             destStyle.SetFontFamily(style.GetFontFamily());
     }
 
-    if (style.GetTextColour().Ok() && style.HasTextColour())
+    if (style.GetTextColour().IsOk() && style.HasTextColour())
     {
         if (!(compareWith && compareWith->HasTextColour() && compareWith->GetTextColour() == style.GetTextColour()))
             destStyle.SetTextColour(style.GetTextColour());
     }
 
-    if (style.GetBackgroundColour().Ok() && style.HasBackgroundColour())
+    if (style.GetBackgroundColour().IsOk() && style.HasBackgroundColour())
     {
         if (!(compareWith && compareWith->HasBackgroundColour() && compareWith->GetBackgroundColour() == style.GetBackgroundColour()))
             destStyle.SetBackgroundColour(style.GetBackgroundColour());
@@ -592,30 +745,30 @@ wxTextAttr wxTextAttr::Combine(const wxTextAttr& attr,
     if (attr.HasFont())
         font = attr.GetFont();
 
-    if ( !font.Ok() )
+    if ( !font.IsOk() )
     {
         if (attrDef.HasFont())
             font = attrDef.GetFont();
 
-        if ( text && !font.Ok() )
+        if ( text && !font.IsOk() )
             font = text->GetFont();
     }
 
     wxColour colFg = attr.GetTextColour();
-    if ( !colFg.Ok() )
+    if ( !colFg.IsOk() )
     {
         colFg = attrDef.GetTextColour();
 
-        if ( text && !colFg.Ok() )
+        if ( text && !colFg.IsOk() )
             colFg = text->GetForegroundColour();
     }
 
     wxColour colBg = attr.GetBackgroundColour();
-    if ( !colBg.Ok() )
+    if ( !colBg.IsOk() )
     {
         colBg = attrDef.GetBackgroundColour();
 
-        if ( text && !colBg.Ok() )
+        if ( text && !colBg.IsOk() )
             colBg = text->GetBackgroundColour();
     }
 
@@ -695,7 +848,7 @@ bool wxTextAttr::BitlistsEqPartial(int valueA, int valueB, int flags)
 {
     int relevantBitsA = valueA & flags;
     int relevantBitsB = valueB & flags;
-    return (relevantBitsA != relevantBitsB);
+    return relevantBitsA == relevantBitsB;
 }
 
 /// Split into paragraph and character styles
@@ -756,23 +909,16 @@ bool wxTextAreaBase::DoLoadFile(const wxString& filename, int WXUNUSED(fileType)
         {
             SetValue(text);
 
+            DiscardEdits();
+            m_filename = filename;
+
             return true;
         }
     }
 #endif // wxUSE_FFILE
 
-    return false;
-}
-
-bool wxTextCtrlBase::DoLoadFile(const wxString& filename, int fileType)
-{
-    if ( wxTextAreaBase::DoLoadFile(filename, fileType) )
-    {
-        DiscardEdits();
-        m_filename = filename;
-        return true;
-    }
     wxLogError(_("File couldn't be loaded."));
+
     return false;
 }
 
@@ -780,10 +926,19 @@ bool wxTextAreaBase::DoSaveFile(const wxString& filename, int WXUNUSED(fileType)
 {
 #if wxUSE_FFILE
     wxFFile file(filename, wxT("w"));
-    return file.IsOpened() && file.Write(GetValue(), *wxConvCurrent);
-#else
-    return false;
+    if ( file.IsOpened() && file.Write(GetValue(), *wxConvCurrent) )
+    {
+        // if it worked, save for future calls
+        m_filename = filename;
+
+        // it's not modified any longer
+        DiscardEdits();
+
+        return true;
+    }
 #endif // wxUSE_FFILE
+
+    return false;
 }
 
 bool wxTextAreaBase::SaveFile(const wxString& filename, int fileType)
@@ -800,21 +955,6 @@ bool wxTextAreaBase::SaveFile(const wxString& filename, int fileType)
     return DoSaveFile(filenameToUse, fileType);
 }
 
-bool wxTextCtrlBase::DoSaveFile(const wxString& filename, int fileType)
-{
-    if ( wxTextAreaBase::DoSaveFile(filename, fileType) )
-    {
-        // if it worked, save for future calls
-        m_filename = filename;
-
-        // it's not modified any longer
-        DiscardEdits();
-
-        return true;
-    }
-    return false;
-}
-
 // ----------------------------------------------------------------------------
 // stream-like insertion operator
 // ----------------------------------------------------------------------------
@@ -862,12 +1002,47 @@ int wxTextCtrlBase::overflow(int c)
 
 bool wxTextCtrlBase::EmulateKeyPress(const wxKeyEvent& event)
 {
+    bool handled = false;
     // we have a native implementation for Win32 and so don't need this one
 #ifndef __WIN32__
     wxChar ch = 0;
     int keycode = event.GetKeyCode();
+
+    long from, to;
+    GetSelection(&from,&to);
+    long insert = GetInsertionPoint();
+    long last = GetLastPosition();
+
+    // catch arrow left and right
+
     switch ( keycode )
     {
+        case WXK_LEFT:
+            if ( event.ShiftDown() )
+                SetSelection( (from > 0 ? from - 1 : 0) , to );
+            else
+            {
+                if ( from != to )
+                    insert = from;
+                else if ( insert > 0 )
+                    insert -= 1;
+                SetInsertionPoint( insert );
+            }
+            handled = true;
+            break;
+        case WXK_RIGHT:
+            if ( event.ShiftDown() )
+                SetSelection( from, (to < last ? to + 1 : last) );
+            else
+            {
+                if ( from != to )
+                    insert = to;
+                else if ( insert < last )
+                    insert += 1;
+                SetInsertionPoint( insert );
+            }
+            handled = true;
+            break;
         case WXK_NUMPAD0:
         case WXK_NUMPAD1:
         case WXK_NUMPAD2:
@@ -913,6 +1088,7 @@ bool wxTextCtrlBase::EmulateKeyPress(const wxKeyEvent& event)
                 const long pos = GetInsertionPoint();
                 if ( pos < GetLastPosition() )
                     Remove(pos, pos + 1);
+                handled = true;
             }
             break;
 
@@ -922,6 +1098,7 @@ bool wxTextCtrlBase::EmulateKeyPress(const wxKeyEvent& event)
                 const long pos = GetInsertionPoint();
                 if ( pos > 0 )
                     Remove(pos - 1, pos);
+                handled = true;
             }
             break;
 
@@ -953,13 +1130,25 @@ bool wxTextCtrlBase::EmulateKeyPress(const wxKeyEvent& event)
     {
         WriteText(ch);
 
-        return true;
+        handled = true;
     }
 #else // __WIN32__
     wxUnusedVar(event);
 #endif // !__WIN32__/__WIN32__
 
-    return false;
+    return handled;
+}
+
+// ----------------------------------------------------------------------------
+// Other miscellaneous stuff
+// ----------------------------------------------------------------------------
+
+bool wxTextCtrlBase::SetHint(const wxString& hint)
+{
+    wxCHECK_MSG( IsSingleLine(), false,
+                 wxS("Hints can only be set for single line text controls") );
+
+    return wxTextEntry::SetHint(hint);
 }
 
 // do the window-specific processing after processing the update event
@@ -1006,11 +1195,24 @@ wxTextAreaBase::HitTest(const wxPoint& WXUNUSED(pt), long * WXUNUSED(pos)) const
     return wxTE_HT_UNKNOWN;
 }
 
+wxPoint wxTextAreaBase::PositionToCoords(long pos) const
+{
+    wxCHECK_MSG( IsValidPosition(pos), wxDefaultPosition,
+                 wxS("Position argument out of range.") );
+
+    return DoPositionToCoords(pos);
+}
+
+wxPoint wxTextAreaBase::DoPositionToCoords(long WXUNUSED(pos)) const
+{
+    return wxDefaultPosition;
+}
+
 #else // !wxUSE_TEXTCTRL
 
 // define this one even if !wxUSE_TEXTCTRL because it is also used by other
 // controls (wxComboBox and wxSpinCtrl)
 
-wxDEFINE_EVENT( wxEVT_COMMAND_TEXT_UPDATED, wxCommandEvent );
+wxDEFINE_EVENT( wxEVT_TEXT, wxCommandEvent );
 
 #endif // wxUSE_TEXTCTRL/!wxUSE_TEXTCTRL
