@@ -4,7 +4,7 @@
 // Author:      Francesco Montorsi (readapted code written by Vadim Zeitlin)
 // Modified by:
 // Created:     15/04/2006
-// RCS-ID:      $Id: clrpickerg.cpp 52836 2008-03-26 15:49:31Z JS $
+// RCS-ID:      $Id$
 // Copyright:   (c) Vadim Zeitlin, Francesco Montorsi
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -58,7 +58,7 @@ bool wxGenericColourButton::Create( wxWindow *parent, wxWindowID id,
     }
 
     // and handle user clicks on it
-    Connect(GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
+    Connect(GetId(), wxEVT_BUTTON,
             wxCommandEventHandler(wxGenericColourButton::OnButtonClick),
             NULL, this);
 
@@ -83,7 +83,7 @@ void wxGenericColourButton::InitColourData()
 
 void wxGenericColourButton::OnButtonClick(wxCommandEvent& WXUNUSED(ev))
 {
-    // update the wxColouData to be shown in the the dialog
+    // update the wxColouData to be shown in the dialog
     ms_data.SetColour(m_colour);
 
     // create the colour dialog and display it
