@@ -4,7 +4,7 @@
 // Author:      Francesco Montorsi
 // Modified by:
 // Created:     15/04/2006
-// RCS-ID:      $Id: filepickerg.cpp 63654 2010-03-08 19:37:08Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Francesco Montorsi
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -26,8 +26,11 @@
 
 #if wxUSE_FILEPICKERCTRL || wxUSE_DIRPICKERCTRL
 
+#include "wx/filename.h"
 #include "wx/filepicker.h"
 
+#include "wx/scopedptr.h"
+
 
 // ============================================================================
 // implementation
@@ -54,15 +57,31 @@ bool wxGenericFileDirButton::Create(wxWindow *parent,
 {
     m_pickerStyle = style;
 
+    // If the special wxPB_SMALL flag is used, ignore the provided label and
+    // use the shortest possible label and the smallest possible button fitting
+    // it.
+    long styleButton = 0;
+    wxString labelButton;
+    if ( m_pickerStyle & wxPB_SMALL )
+    {
+        labelButton = _("...");
+        styleButton = wxBU_EXACTFIT;
+    }
+    else
+    {
+        labelButton = label;
+    }
+
     // create this button
-    if ( !wxButton::Create(parent, id, label, pos, size, 0, validator, name) )
+    if ( !wxButton::Create(parent, id, labelButton,
+                           pos, size, styleButton, validator, name) )
     {
         wxFAIL_MSG( wxT("wxGenericFileButton creation failed") );
         return false;
     }
 
     // and handle user clicks on it
-    Connect(GetId(), wxEVT_COMMAND_BUTTON_CLICKED,
+    Connect(GetId(), wxEVT_BUTTON,
             wxCommandEventHandler(wxGenericFileDirButton::OnButtonClick),
             NULL, this);
 
@@ -76,18 +95,62 @@ bool wxGenericFileDirButton::Create(wxWindow *parent,
 
 void wxGenericFileDirButton::OnButtonClick(wxCommandEvent& WXUNUSED(ev))
 {
-    wxDialog *p = CreateDialog();
+    wxScopedPtr<wxDialog> p(CreateDialog());
     if (p->ShowModal() == wxID_OK)
     {
         // save updated path in m_path
-        UpdatePathFromDialog(p);
+        UpdatePathFromDialog(p.get());
 
         // fire an event
         wxFileDirPickerEvent event(GetEventType(), this, GetId(), m_path);
         GetEventHandler()->ProcessEvent(event);
     }
+}
 
-    wxDELETE(p);
+void wxGenericFileDirButton::SetInitialDirectory(const wxString& dir)
+{
+    m_initialDir = dir;
+}
+
+// ----------------------------------------------------------------------------
+// wxGenericFileButton
+// ----------------------------------------------------------------------------
+
+wxDialog *wxGenericFileButton::CreateDialog()
+{
+    // Determine the initial directory for the dialog: it comes either from the
+    // default path, if it has it, or from the separately specified initial
+    // directory that can be set even if the path is e.g. empty.
+    wxFileName fn(m_path);
+    wxString initialDir = fn.GetPath();
+    if ( initialDir.empty() )
+        initialDir = m_initialDir;
+
+    return new wxFileDialog
+               (
+                    GetDialogParent(),
+                    m_message,
+                    initialDir,
+                    fn.GetFullName(),
+                    m_wildcard,
+                    GetDialogStyle()
+               );
+}
+
+// ----------------------------------------------------------------------------
+// wxGenericDirButton
+// ----------------------------------------------------------------------------
+
+wxDialog *wxGenericDirButton::CreateDialog()
+{
+    wxDirDialog* const dialog = new wxDirDialog
+                                    (
+                                        GetDialogParent(),
+                                        m_message,
+                                        m_path.empty() ? m_initialDir : m_path,
+                                        GetDialogStyle()
+                                    );
+    return dialog;
 }
 
 #endif      // wxUSE_FILEPICKERCTRL || wxUSE_DIRPICKERCTRL
