@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: fontdlgg.cpp 64019 2010-04-18 00:05:37Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -32,7 +32,6 @@
     #include "wx/checkbox.h"
     #include "wx/intl.h"
     #include "wx/settings.h"
-    #include "wx/cmndata.h"
     #include "wx/sizer.h"
 #endif
 
@@ -77,7 +76,7 @@ void wxFontPreviewer::OnPaint(wxPaintEvent& WXUNUSED(event))
     dc.SetBrush(*wxWHITE_BRUSH);
     dc.DrawRectangle(0, 0, size.x, size.y);
 
-    if ( font.Ok() )
+    if ( font.IsOk() )
     {
         dc.SetFont(font);
         dc.SetTextForeground(GetForegroundColour());
@@ -507,10 +506,10 @@ void wxGenericFontDialog::CreateWidgets()
     if (m_colourChoice)
     {
         wxString name(wxTheColourDatabase->FindName(m_fontData.GetColour()));
-        if (name.length())
-            m_colourChoice->SetStringSelection(name);
-        else
+        if ( name.empty() )
             m_colourChoice->SetStringSelection(wxT("BLACK"));
+        else
+            m_colourChoice->SetStringSelection(name);
     }
 
     if (m_underLineCheckBox)
@@ -551,7 +550,7 @@ void wxGenericFontDialog::InitializeFont()
     int fontSize = 12;
     bool fontUnderline = false;
 
-    if (m_fontData.m_initialFont.Ok())
+    if (m_fontData.m_initialFont.IsOk())
     {
         fontFamily = m_fontData.m_initialFont.GetFamily();
         fontWeight = m_fontData.m_initialFont.GetWeight();
@@ -603,7 +602,7 @@ void wxGenericFontDialog::DoChangeFont()
         if ( !m_colourChoice->GetStringSelection().empty() )
         {
             wxColour col = wxTheColourDatabase->Find(m_colourChoice->GetStringSelection());
-            if (col.Ok())
+            if (col.IsOk())
             {
                 m_fontData.m_fontColour = col;
             }
@@ -611,7 +610,7 @@ void wxGenericFontDialog::DoChangeFont()
     }
     // Update color here so that we can also use the color originally passed in
     // (EnableEffects may be false)
-    if (m_fontData.m_fontColour.Ok())
+    if (m_fontData.m_fontColour.IsOk())
         m_previewer->SetForegroundColour(m_fontData.m_fontColour);
 
     m_previewer->Refresh();
