@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     19.08.03
-// RCS-ID:      $Id: listbkg.cpp 63632 2010-03-05 23:55:19Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -36,12 +36,6 @@
 #include "wx/statline.h"
 #include "wx/imaglist.h"
 
-// FIXME: native OS X wxListCtrl hangs if this code is used for it so disable
-//        it for now
-#if !defined(__WXMAC__)
-    #define CAN_USE_REPORT_VIEW
-#endif
-
 // ----------------------------------------------------------------------------
 // various wxWidgets macros
 // ----------------------------------------------------------------------------
@@ -55,8 +49,8 @@
 
 IMPLEMENT_DYNAMIC_CLASS(wxListbook, wxBookCtrlBase)
 
-wxDEFINE_EVENT( wxEVT_COMMAND_LISTBOOK_PAGE_CHANGING, wxBookCtrlEvent );
-wxDEFINE_EVENT( wxEVT_COMMAND_LISTBOOK_PAGE_CHANGED,  wxBookCtrlEvent );
+wxDEFINE_EVENT( wxEVT_LISTBOOK_PAGE_CHANGING, wxBookCtrlEvent );
+wxDEFINE_EVENT( wxEVT_LISTBOOK_PAGE_CHANGED,  wxBookCtrlEvent );
 
 BEGIN_EVENT_TABLE(wxListbook, wxBookCtrlBase)
     EVT_SIZE(wxListbook::OnSize)
@@ -71,11 +65,6 @@ END_EVENT_TABLE()
 // wxListbook creation
 // ----------------------------------------------------------------------------
 
-void wxListbook::Init()
-{
-    m_selection = wxNOT_FOUND;
-}
-
 bool
 wxListbook::Create(wxWindow *parent,
                    wxWindowID id,
@@ -108,17 +97,11 @@ wxListbook::Create(wxWindow *parent,
                     wxID_ANY,
                     wxDefaultPosition,
                     wxDefaultSize,
-                    wxLC_SINGLE_SEL |
-#ifdef CAN_USE_REPORT_VIEW
-                    GetListCtrlReportViewFlags()
-#else // !CAN_USE_REPORT_VIEW
-                    GetListCtrlIconViewFlags()
-#endif // CAN_USE_REPORT_VIEW/!CAN_USE_REPORT_VIEW
+                    GetListCtrlFlags()
                  );
 
-#ifdef CAN_USE_REPORT_VIEW
-    GetListView()->InsertColumn(0, wxT("Pages"));
-#endif // CAN_USE_REPORT_VIEW
+    if ( GetListView()->InReportView() )
+        GetListView()->InsertColumn(0, wxS("Pages"));
 
 #ifdef __WXMSW__
     // On XP with themes enabled the GetViewRect used in GetControllerSize() to
@@ -137,20 +120,42 @@ wxListbook::Create(wxWindow *parent,
 // wxListCtrl flags
 // ----------------------------------------------------------------------------
 
-long wxListbook::GetListCtrlIconViewFlags() const
+long wxListbook::GetListCtrlFlags() const
 {
-    return (IsVertical() ? wxLC_ALIGN_LEFT : wxLC_ALIGN_TOP) | wxLC_ICON;
+    // We'd like to always use wxLC_ICON mode but it doesn't work with the
+    // native wxListCtrl under MSW unless we do have icons for all the items,
+    // so we can't use it if we have no image list. In this case we'd like to
+    // use wxLC_LIST mode because it works correctly for both horizontally and
+    // vertically laid out controls, but MSW native wxListCtrl insists on
+    // creating multiple columns if there are too many items and there doesn't
+    // seem anything to do about it, so we have to use wxLC_REPORT mode in this
+    // case there.
+
+    long flags = IsVertical() ? wxLC_ALIGN_LEFT : wxLC_ALIGN_TOP;
+    if ( GetImageList() )
+    {
+        flags |= wxLC_ICON;
+    }
+    else // No images.
+    {
+#ifdef __WXMSW__
+        if ( !IsVertical() )
+        {
+            // Notice that we intentionally overwrite the alignment flags here
+            // by not using "|=", alignment isn't used for report view.
+            flags = wxLC_REPORT | wxLC_NO_HEADER;
+        }
+        else
+#endif // __WXMSW__
+        {
+            flags |= wxLC_LIST;
+        }
+    }
+
+    // Use single selection in any case.
+    return flags | wxLC_SINGLE_SEL;
 }
 
-#ifdef CAN_USE_REPORT_VIEW
-
-long wxListbook::GetListCtrlReportViewFlags() const
-{
-    return wxLC_REPORT | wxLC_NO_HEADER;
-}
-
-#endif // CAN_USE_REPORT_VIEW
-
 // ----------------------------------------------------------------------------
 // wxListbook geometry management
 // ----------------------------------------------------------------------------
@@ -258,39 +263,31 @@ bool wxListbook::SetPageImage(size_t n, int imageId)
 
 void wxListbook::SetImageList(wxImageList *imageList)
 {
-#ifdef CAN_USE_REPORT_VIEW
+    const long flagsOld = GetListCtrlFlags();
+
+    wxBookCtrlBase::SetImageList(imageList);
+
+    const long flagsNew = GetListCtrlFlags();
+
     wxListView * const list = GetListView();
 
-    // If imageList presence has changed, we update the list control view
-    if ( (imageList != NULL) != (GetImageList() != NULL) )
+    // We may need to change the list control mode if the image list presence
+    // has changed.
+    if ( flagsNew != flagsOld )
     {
         // Preserve the selection which is lost when changing the mode
         const int oldSel = GetSelection();
 
-        // Update the style to use icon view for images, report view otherwise
-        long style = wxLC_SINGLE_SEL;
-        if ( imageList )
-        {
-            style |= GetListCtrlIconViewFlags();
-        }
-        else // no image list
-        {
-            style |= GetListCtrlReportViewFlags();
-        }
-
-        list->SetWindowStyleFlag(style);
-        if ( !imageList )
-            list->InsertColumn(0, wxT("Pages"));
+        list->SetWindowStyleFlag(flagsNew);
+        if ( list->InReportView() )
+            list->InsertColumn(0, wxS("Pages"));
 
         // Restore selection
         if ( oldSel != wxNOT_FOUND )
-           SetSelection(oldSel);
+            SetSelection(oldSel);
     }
 
     list->SetImageList(imageList, wxIMAGE_LIST_NORMAL);
-#endif // CAN_USE_REPORT_VIEW
-
-    wxBookCtrlBase::SetImageList(imageList);
 }
 
 // ----------------------------------------------------------------------------
@@ -304,19 +301,14 @@ void wxListbook::UpdateSelectedPage(size_t newsel)
     GetListView()->Focus(newsel);
 }
 
-int wxListbook::GetSelection() const
-{
-    return m_selection;
-}
-
 wxBookCtrlEvent* wxListbook::CreatePageChangingEvent() const
 {
-    return new wxBookCtrlEvent(wxEVT_COMMAND_LISTBOOK_PAGE_CHANGING, m_windowId);
+    return new wxBookCtrlEvent(wxEVT_LISTBOOK_PAGE_CHANGING, m_windowId);
 }
 
 void wxListbook::MakeChangedEvent(wxBookCtrlEvent &event)
 {
-    event.SetEventType(wxEVT_COMMAND_LISTBOOK_PAGE_CHANGED);
+    event.SetEventType(wxEVT_LISTBOOK_PAGE_CHANGED);
 }
 
 
@@ -346,20 +338,9 @@ wxListbook::InsertPage(size_t n,
         GetListView()->Focus(m_selection);
     }
 
-    // some page should be selected: either this one or the first one if there
-    // is still no selection
-    int selNew = -1;
-    if ( bSelect )
-        selNew = n;
-    else if ( m_selection == -1 )
-        selNew = 0;
-
-    if ( selNew != m_selection )
+    if ( !DoSetSelectionAfterInsertion(n, bSelect) )
         page->Hide();
 
-    if ( selNew != -1 )
-        SetSelection(selNew);
-
     UpdateSize();
 
     return true;
@@ -367,28 +348,13 @@ wxListbook::InsertPage(size_t n,
 
 wxWindow *wxListbook::DoRemovePage(size_t page)
 {
-    const size_t page_count = GetPageCount();
     wxWindow *win = wxBookCtrlBase::DoRemovePage(page);
 
     if ( win )
     {
         GetListView()->DeleteItem(page);
 
-        if (m_selection >= (int)page)
-        {
-            // force new sel valid if possible
-            int sel = m_selection - 1;
-            if (page_count == 1)
-                sel = wxNOT_FOUND;
-            else if ((page_count == 2) || (sel == -1))
-                sel = 0;
-
-            // force sel invalid if deleting current page - don't try to hide it
-            m_selection = (m_selection == (int)page) ? wxNOT_FOUND : m_selection - 1;
-
-            if ((sel != wxNOT_FOUND) && (sel != m_selection))
-                SetSelection(sel);
-        }
+        DoSetSelectionAfterRemoval(page);
 
         GetListView()->Arrange();
         UpdateSize();
@@ -404,8 +370,6 @@ bool wxListbook::DeleteAllPages()
     if (!wxBookCtrlBase::DeleteAllPages())
         return false;
 
-    m_selection = -1;
-
     UpdateSize();
 
     return true;
