@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
 // Name:        src/generic/progdlgg.cpp
-// Purpose:     wxProgressDialog class
+// Purpose:     wxGenericProgressDialog class
 // Author:      Karsten Ballueder
 // Modified by:
 // Created:     09.05.1999
-// RCS-ID:      $Id: progdlgg.cpp 64656 2010-06-20 18:18:23Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Karsten Ballueder
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -66,58 +66,127 @@
 
 static const int wxID_SKIP = 32000;  // whatever
 
-// ----------------------------------------------------------------------------
-// private functions
-// ----------------------------------------------------------------------------
-
-// update the label to show the given time (in seconds)
-static void SetTimeLabel(unsigned long val, wxStaticText *label);
-
 // ----------------------------------------------------------------------------
 // event tables
 // ----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(wxProgressDialog, wxDialog)
-    EVT_BUTTON(wxID_CANCEL, wxProgressDialog::OnCancel)
-    EVT_BUTTON(wxID_SKIP, wxProgressDialog::OnSkip)
+BEGIN_EVENT_TABLE(wxGenericProgressDialog, wxDialog)
+    EVT_BUTTON(wxID_CANCEL, wxGenericProgressDialog::OnCancel)
+    EVT_BUTTON(wxID_SKIP, wxGenericProgressDialog::OnSkip)
 
-    EVT_CLOSE(wxProgressDialog::OnClose)
+    EVT_CLOSE(wxGenericProgressDialog::OnClose)
 END_EVENT_TABLE()
 
-IMPLEMENT_CLASS(wxProgressDialog, wxDialog)
-
 // ============================================================================
-// wxProgressDialog implementation
+// wxGenericProgressDialog implementation
 // ============================================================================
 
+wxIMPLEMENT_CLASS(wxProgressDialog, wxDialog)
+
 // ----------------------------------------------------------------------------
-// wxProgressDialog creation
+// wxGenericProgressDialog creation
 // ----------------------------------------------------------------------------
 
-wxProgressDialog::wxProgressDialog(const wxString& title,
-                                   const wxString& message,
-                                   int maximum,
-                                   wxWindow *parent,
-                                   int style)
-                : wxDialog(GetParentForModalDialog(parent, style), wxID_ANY, title),
-                  m_skip(false),
-                  m_delay(3),
-                  m_hasAbortButton(false),
-                  m_hasSkipButton(false)
+void wxGenericProgressDialog::Init()
 {
     // we may disappear at any moment, let the others know about it
     SetExtraStyle(GetExtraStyle() | wxWS_EX_TRANSIENT);
-    m_windowStyle |= style;
 
-    m_hasAbortButton = (style & wxPD_CAN_ABORT) != 0;
-    m_hasSkipButton = (style & wxPD_CAN_SKIP) != 0;
+    // Initialize all our members that we always use (even when we don't
+    // create a valid window in this class).
+
+    m_pdStyle = 0;
+    m_parentTop = NULL;
+
+    m_gauge = NULL;
+    m_msg = NULL;
+    m_elapsed =
+    m_estimated =
+    m_remaining = NULL;
+
+    m_state = Uncancelable;
+    m_maximum = 0;
+
+    m_timeStart = wxGetCurrentTime();
+    m_timeStop = (unsigned long)-1;
+    m_break = 0;
+
+    m_skip = false;
+
+#if !defined(__SMARTPHONE__)
+    m_btnAbort =
+    m_btnSkip = NULL;
+#endif
+
+    m_display_estimated =
+    m_last_timeupdate =
+    m_ctdelay = 0;
+
+    m_delay = 3;
+
+    m_winDisabler = NULL;
+    m_tempEventLoop = NULL;
+}
+
+wxGenericProgressDialog::wxGenericProgressDialog()
+                       : wxDialog()
+{
+    Init();
+}
+
+wxGenericProgressDialog::wxGenericProgressDialog(const wxString& title,
+                                                 const wxString& message,
+                                                 int maximum,
+                                                 wxWindow *parent,
+                                                 int style)
+                       : wxDialog()
+{
+    Init();
+
+    Create( title, message, maximum, parent, style );
+}
+
+void wxGenericProgressDialog::SetTopParent(wxWindow* parent)
+{
+    m_parentTop = GetParentForModalDialog(parent, GetWindowStyle());
+}
+
+bool wxGenericProgressDialog::Create( const wxString& title,
+                                      const wxString& message,
+                                      int maximum,
+                                      wxWindow *parent,
+                                      int style )
+{
+    SetTopParent(parent);
+
+    m_parentTop = wxGetTopLevelParent(parent);
+    m_pdStyle = style;
+
+    wxWindow* const
+        realParent = GetParentForModalDialog(parent, GetWindowStyle());
+
+    if (!wxDialog::Create(realParent, wxID_ANY, title))
+        return false;
+
+    SetMaximum(maximum);
+
+    // We need a running event loop in order to update the dialog and be able
+    // to process clicks on its buttons, so ensure that there is one running
+    // even if this means we have to start it ourselves (this happens most
+    // commonly during the program initialization, e.g. for the progress
+    // dialogs shown from overridden wxApp::OnInit()).
+    if ( !wxEventLoopBase::GetActive() )
+    {
+        m_tempEventLoop = new wxEventLoop;
+        wxEventLoop::SetActive(m_tempEventLoop);
+    }
 
 #if defined(__WXMSW__) && !defined(__WXUNIVERSAL__)
     // we have to remove the "Close" button from the title bar then as it is
     // confusing to have it - it doesn't work anyhow
     //
     // FIXME: should probably have a (extended?) window style for this
-    if ( !m_hasAbortButton )
+    if ( !HasPDFlag(wxPD_CAN_ABORT) )
     {
         EnableCloseButton(false);
     }
@@ -127,17 +196,7 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
     SetLeftMenu();
 #endif
 
-    m_state = m_hasAbortButton ? Continue : Uncancelable;
-    m_maximum = maximum;
-
-#if defined(__WXMSW__) || defined(__WXPM__)
-    // we can't have values > 65,536 in the progress control under Windows, so
-    // scale everything down
-    m_factor = m_maximum / 65536 + 1;
-    m_maximum /= m_factor;
-#endif // __WXMSW__
-
-    m_parentTop = wxGetTopLevelParent(parent);
+    m_state = HasPDFlag(wxPD_CAN_ABORT) ? Continue : Uncancelable;
 
     // top-level sizerTop
     wxSizer * const sizerTop = new wxBoxSizer(wxVERTICAL);
@@ -145,38 +204,32 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
     m_msg = new wxStaticText(this, wxID_ANY, message);
     sizerTop->Add(m_msg, 0, wxLEFT | wxTOP, 2*LAYOUT_MARGIN);
 
-    if ( maximum > 0 )
-    {
-        int gauge_style = wxGA_HORIZONTAL;
-        if ( style & wxPD_SMOOTH )
-            gauge_style |= wxGA_SMOOTH;
-        m_gauge = new wxGauge
-                      (
-                        this,
-                        wxID_ANY,
-                        m_maximum,
-                        wxDefaultPosition,
-                        // make the progress bar sufficiently long
-                        wxSize(wxMin(wxGetClientDisplayRect().width/3, 300), -1),
-                        gauge_style
-                      );
+    int gauge_style = wxGA_HORIZONTAL;
+    if ( style & wxPD_SMOOTH )
+        gauge_style |= wxGA_SMOOTH;
 
-        sizerTop->Add(m_gauge, 0, wxLEFT | wxRIGHT | wxTOP | wxEXPAND, 2*LAYOUT_MARGIN);
-        m_gauge->SetValue(0);
-    }
-    else
-    {
-        m_gauge = NULL;
-    }
+#ifdef __WXMSW__
+    maximum /= m_factor;
+#endif
+
+    m_gauge = new wxGauge
+                  (
+                    this,
+                    wxID_ANY,
+                    maximum,
+                    wxDefaultPosition,
+                    // make the progress bar sufficiently long
+                    wxSize(wxMin(wxGetClientDisplayRect().width/3, 300), -1),
+                    gauge_style
+                  );
+
+    sizerTop->Add(m_gauge, 0, wxLEFT | wxRIGHT | wxTOP | wxEXPAND, 2*LAYOUT_MARGIN);
+    m_gauge->SetValue(0);
 
     // create the estimated/remaining/total time zones if requested
     m_elapsed =
     m_estimated =
     m_remaining = NULL;
-    m_display_estimated =
-    m_last_timeupdate =
-    m_break = 0;
-    m_ctdelay = 0;
 
     // also count how many labels we really have
     size_t nTimeLabels = 0;
@@ -187,34 +240,28 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
     {
         nTimeLabels++;
 
-        m_elapsed = CreateLabel(_("Elapsed time:"), sizerLabels);
+        m_elapsed = CreateLabel(GetElapsedLabel(), sizerLabels);
     }
 
     if ( style & wxPD_ESTIMATED_TIME )
     {
         nTimeLabels++;
 
-        m_estimated = CreateLabel(_("Estimated time:"), sizerLabels);
+        m_estimated = CreateLabel(GetEstimatedLabel(), sizerLabels);
     }
 
     if ( style & wxPD_REMAINING_TIME )
     {
         nTimeLabels++;
 
-        m_remaining = CreateLabel(_("Remaining time:"), sizerLabels);
+        m_remaining = CreateLabel(GetRemainingLabel(), sizerLabels);
     }
     sizerTop->Add(sizerLabels, 0, wxALIGN_CENTER_HORIZONTAL | wxTOP, LAYOUT_MARGIN);
 
-    if ( nTimeLabels > 0 )
-    {
-        // set it to the current time
-        m_timeStart = wxGetCurrentTime();
-    }
-
 #if defined(__SMARTPHONE__)
-    if ( m_hasSkipButton )
+    if ( HasPDFlag(wxPD_CAN_SKIP) )
         SetRightMenu(wxID_SKIP, _("Skip"));
-    if ( m_hasAbortButton )
+    if ( HasPDFlag(wxPD_CAN_ABORT) )
         SetLeftMenu(wxID_CANCEL);
 #else
     m_btnAbort =
@@ -224,27 +271,30 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
 
     // Windows dialogs usually have buttons in the lower right corner
     const int sizerFlags =
-#if defined(__WXMSW__) || defined(__WXPM__)
+#if defined(__WXMSW__) || defined(__WXPM__) || defined(__WXOSX__)
                            wxALIGN_RIGHT | wxALL
 #else // !MSW
                            wxALIGN_CENTER_HORIZONTAL | wxBOTTOM | wxTOP
 #endif // MSW/!MSW
                            ;
 
-    if ( m_hasSkipButton )
+    if ( HasPDFlag(wxPD_CAN_SKIP) )
     {
         m_btnSkip = new wxButton(this, wxID_SKIP, _("&Skip"));
 
         buttonSizer->Add(m_btnSkip, 0, sizerFlags, LAYOUT_MARGIN);
     }
 
-    if ( m_hasAbortButton )
+    if ( HasPDFlag(wxPD_CAN_ABORT) )
     {
         m_btnAbort = new wxButton(this, wxID_CANCEL);
 
         buttonSizer->Add(m_btnAbort, 0, sizerFlags, LAYOUT_MARGIN);
     }
 
+    if ( !HasPDFlag(wxPD_CAN_SKIP | wxPD_CAN_ABORT) )
+        buttonSizer->AddSpacer(LAYOUT_MARGIN);
+
     sizerTop->Add(buttonSizer, 0, sizerFlags, LAYOUT_MARGIN );
 #endif // __SMARTPHONE__/!__SMARTPHONE__
 
@@ -252,16 +302,7 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
 
     Centre(wxCENTER_FRAME | wxBOTH);
 
-    if ( style & wxPD_APP_MODAL )
-    {
-        m_winDisabler = new wxWindowDisabler(this);
-    }
-    else
-    {
-        if ( m_parentTop )
-            m_parentTop->Disable();
-        m_winDisabler = NULL;
-    }
+    DisableOtherWindows();
 
     Show();
     Enable();
@@ -275,10 +316,85 @@ wxProgressDialog::wxProgressDialog(const wxString& title,
     }
 
     Update();
+    return true;
+}
+
+void wxGenericProgressDialog::UpdateTimeEstimates(int value,
+                                                  unsigned long &elapsedTime,
+                                                  unsigned long &estimatedTime,
+                                                  unsigned long &remainingTime)
+{
+    unsigned long elapsed = wxGetCurrentTime() - m_timeStart;
+    if ( value != 0 && (m_last_timeupdate < elapsed || value == m_maximum) )
+    {
+        m_last_timeupdate = elapsed;
+        unsigned long estimated = m_break +
+                (unsigned long)(( (double) (elapsed-m_break) * m_maximum ) / ((double)value)) ;
+        if (    estimated > m_display_estimated
+                && m_ctdelay >= 0
+            )
+        {
+            ++m_ctdelay;
+        }
+        else if (    estimated < m_display_estimated
+                    && m_ctdelay <= 0
+                )
+        {
+            --m_ctdelay;
+        }
+        else
+        {
+            m_ctdelay = 0;
+        }
+        if (    m_ctdelay >= m_delay          // enough confirmations for a higher value
+                || m_ctdelay <= (m_delay*-1)     // enough confirmations for a lower value
+                || value == m_maximum            // to stay consistent
+                || elapsed > m_display_estimated // to stay consistent
+                || ( elapsed > 0 && elapsed < 4 ) // additional updates in the beginning
+            )
+        {
+            m_display_estimated = estimated;
+            m_ctdelay = 0;
+        }
+    }
+
+    if ( value != 0 )
+    {
+        long display_remaining = m_display_estimated - elapsed;
+        if ( display_remaining < 0 )
+        {
+            display_remaining = 0;
+        }
+
+        estimatedTime = m_display_estimated;
+        remainingTime = display_remaining;
+    }
+
+    elapsedTime = elapsed;
+}
+
+// static
+wxString wxGenericProgressDialog::GetFormattedTime(unsigned long timeInSec)
+{
+    wxString timeAsHMS;
+
+    if ( timeInSec == (unsigned long)-1 )
+    {
+        timeAsHMS = _("Unknown");
+    }
+    else
+    {
+        unsigned hours = timeInSec / 3600;
+        unsigned minutes = (timeInSec % 3600) / 60;
+        unsigned seconds = timeInSec % 60;
+        timeAsHMS.Printf("%u:%02u:%02u", hours, minutes, seconds);
+    }
+
+    return timeAsHMS;
 }
 
 wxStaticText *
-wxProgressDialog::CreateLabel(const wxString& text, wxSizer *sizer)
+wxGenericProgressDialog::CreateLabel(const wxString& text, wxSizer *sizer)
 {
     wxStaticText *label = new wxStaticText(this, wxID_ANY, text);
     wxStaticText *value = new wxStaticText(this, wxID_ANY, _("unknown"));
@@ -302,16 +418,16 @@ wxProgressDialog::CreateLabel(const wxString& text, wxSizer *sizer)
 }
 
 // ----------------------------------------------------------------------------
-// wxProgressDialog operations
+// wxGenericProgressDialog operations
 // ----------------------------------------------------------------------------
 
 bool
-wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
+wxGenericProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
 {
     if ( !DoBeforeUpdate(skip) )
         return false;
 
-    wxASSERT_MSG( value == -1 || m_gauge, wxT("cannot update non existent dialog") );
+    wxCHECK_MSG( m_gauge, false, "dialog should be fully created" );
 
 #ifdef __WXMSW__
     value /= m_factor;
@@ -319,54 +435,19 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
 
     wxASSERT_MSG( value <= m_maximum, wxT("invalid progress value") );
 
-    if ( m_gauge )
-        m_gauge->SetValue(value);
+    m_gauge->SetValue(value);
 
     UpdateMessage(newmsg);
 
     if ( (m_elapsed || m_remaining || m_estimated) && (value != 0) )
     {
-        unsigned long elapsed = wxGetCurrentTime() - m_timeStart;
-        if (    m_last_timeupdate < elapsed
-             || value == m_maximum
-           )
-        {
-            m_last_timeupdate = elapsed;
-            unsigned long estimated = m_break +
-                  (unsigned long)(( (double) (elapsed-m_break) * m_maximum ) / ((double)value)) ;
-            if (    estimated > m_display_estimated
-                 && m_ctdelay >= 0
-               )
-            {
-                ++m_ctdelay;
-            }
-            else if (    estimated < m_display_estimated
-                      && m_ctdelay <= 0
-                    )
-            {
-                --m_ctdelay;
-            }
-            else
-            {
-                m_ctdelay = 0;
-            }
-            if (    m_ctdelay >= m_delay          // enough confirmations for a higher value
-                 || m_ctdelay <= (m_delay*-1)     // enough confirmations for a lower value
-                 || value == m_maximum            // to stay consistent
-                 || elapsed > m_display_estimated // to stay consistent
-                 || ( elapsed > 0 && elapsed < 4 ) // additional updates in the beginning
-               )
-            {
-                m_display_estimated = estimated;
-                m_ctdelay = 0;
-            }
-        }
+        unsigned long elapsed;
+        unsigned long display_remaining;
 
-        long display_remaining = m_display_estimated - elapsed;
-        if ( display_remaining < 0 )
-        {
-            display_remaining = 0;
-        }
+        UpdateTimeEstimates( value,
+                             elapsed,
+                             m_display_estimated,
+                             display_remaining );
 
         SetTimeLabel(elapsed, m_elapsed);
         SetTimeLabel(m_display_estimated, m_estimated);
@@ -387,7 +468,7 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
         // so that we return true below and that out [Cancel] handler knew what
         // to do
         m_state = Finished;
-        if( !HasFlag(wxPD_AUTO_HIDE) )
+        if( !HasPDFlag(wxPD_AUTO_HIDE) )
         {
             EnableClose();
             DisableSkip();
@@ -401,9 +482,6 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
                 m_msg->SetLabel(_("Done."));
             }
 
-            wxCHECK_MSG(wxEventLoopBase::GetActive(), false,
-                        "wxProgressDialog::Update needs a running event loop");
-
             // allow the window to repaint:
             // NOTE: since we yield only for UI events with this call, there
             //       should be no side-effects
@@ -435,12 +513,12 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
     return m_state != Canceled;
 }
 
-bool wxProgressDialog::Pulse(const wxString& newmsg, bool *skip)
+bool wxGenericProgressDialog::Pulse(const wxString& newmsg, bool *skip)
 {
     if ( !DoBeforeUpdate(skip) )
         return false;
 
-    wxASSERT_MSG( m_gauge, wxT("cannot update non existent dialog") );
+    wxCHECK_MSG( m_gauge, false, "dialog should be fully created" );
 
     // show a bit of progress
     m_gauge->Pulse();
@@ -461,11 +539,8 @@ bool wxProgressDialog::Pulse(const wxString& newmsg, bool *skip)
     return m_state != Canceled;
 }
 
-bool wxProgressDialog::DoBeforeUpdate(bool *skip)
+bool wxGenericProgressDialog::DoBeforeUpdate(bool *skip)
 {
-    wxCHECK_MSG(wxEventLoopBase::GetActive(), false,
-                "wxProgressDialog::DoBeforeUpdate needs a running event loop");
-
     // we have to yield because not only we want to update the display but
     // also to process the clicks on the cancel and skip buttons
     // NOTE: using YieldFor() this call shouldn't give re-entrancy problems
@@ -484,18 +559,15 @@ bool wxProgressDialog::DoBeforeUpdate(bool *skip)
     return m_state != Canceled;
 }
 
-void wxProgressDialog::DoAfterUpdate()
+void wxGenericProgressDialog::DoAfterUpdate()
 {
-    wxCHECK_RET(wxEventLoopBase::GetActive(),
-                "wxProgressDialog::DoAfterUpdate needs a running event loop");
-
     // allow the window to repaint:
     // NOTE: since we yield only for UI events with this call, there
     //       should be no side-effects
     wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI);
 }
 
-void wxProgressDialog::Resume()
+void wxGenericProgressDialog::Resume()
 {
     m_state = Continue;
     m_ctdelay = m_delay; // force an update of the elapsed/estimated/remaining time
@@ -506,7 +578,7 @@ void wxProgressDialog::Resume()
     m_skip = false;
 }
 
-bool wxProgressDialog::Show( bool show )
+bool wxGenericProgressDialog::Show( bool show )
 {
     // reenable other windows before hiding this one because otherwise
     // Windows wouldn't give the focus back to the window which had
@@ -517,58 +589,83 @@ bool wxProgressDialog::Show( bool show )
     return wxDialog::Show(show);
 }
 
-int wxProgressDialog::GetValue() const
+int wxGenericProgressDialog::GetValue() const
 {
-    if (m_gauge)
-        return m_gauge->GetValue();
-    return wxNOT_FOUND;
+    wxCHECK_MSG( m_gauge, -1, "dialog should be fully created" );
+
+    return m_gauge->GetValue();
 }
 
-int wxProgressDialog::GetRange() const
+int wxGenericProgressDialog::GetRange() const
 {
-    if (m_gauge)
-        return m_gauge->GetRange();
-    return wxNOT_FOUND;
+    return m_maximum;
 }
 
-wxString wxProgressDialog::GetMessage() const
+wxString wxGenericProgressDialog::GetMessage() const
 {
     return m_msg->GetLabel();
 }
 
-void wxProgressDialog::SetRange(int maximum)
+void wxGenericProgressDialog::SetRange(int maximum)
 {
-    wxASSERT_MSG(m_gauge, "The dialog should have been constructed with a range > 0");
-    wxASSERT_MSG(maximum > 0, "Invalid range");
+    wxCHECK_RET( m_gauge, "dialog should be fully created" );
+
+    wxCHECK_RET( maximum > 0, "Invalid range" );
 
     m_gauge->SetRange(maximum);
+
+    SetMaximum(maximum);
+}
+
+void wxGenericProgressDialog::SetMaximum(int maximum)
+{
     m_maximum = maximum;
 
 #if defined(__WXMSW__) || defined(__WXPM__)
     // we can't have values > 65,536 in the progress control under Windows, so
     // scale everything down
     m_factor = m_maximum / 65536 + 1;
-    m_maximum /= m_factor;
 #endif // __WXMSW__
 }
 
 
-bool wxProgressDialog::WasCancelled() const
+bool wxGenericProgressDialog::WasCancelled() const
 {
-    return m_hasAbortButton && m_state == Canceled;
+    return HasPDFlag(wxPD_CAN_ABORT) && m_state == Canceled;
 }
 
-bool wxProgressDialog::WasSkipped() const
+bool wxGenericProgressDialog::WasSkipped() const
 {
-    return m_hasSkipButton && m_skip;
+    return HasPDFlag(wxPD_CAN_SKIP) && m_skip;
 }
 
+// static
+void wxGenericProgressDialog::SetTimeLabel(unsigned long val,
+                                           wxStaticText *label)
+{
+    if ( label )
+    {
+        wxString s;
+
+        if (val != (unsigned long)-1)
+        {
+            s = GetFormattedTime(val);
+        }
+        else
+        {
+            s = _("Unknown");
+        }
+
+        if ( s != label->GetLabel() )
+            label->SetLabel(s);
+    }
+}
 
 // ----------------------------------------------------------------------------
 // event handlers
 // ----------------------------------------------------------------------------
 
-void wxProgressDialog::OnCancel(wxCommandEvent& event)
+void wxGenericProgressDialog::OnCancel(wxCommandEvent& event)
 {
     if ( m_state == Finished )
     {
@@ -592,13 +689,13 @@ void wxProgressDialog::OnCancel(wxCommandEvent& event)
     }
 }
 
-void wxProgressDialog::OnSkip(wxCommandEvent& WXUNUSED(event))
+void wxGenericProgressDialog::OnSkip(wxCommandEvent& WXUNUSED(event))
 {
     DisableSkip();
     m_skip = true;
 }
 
-void wxProgressDialog::OnClose(wxCloseEvent& event)
+void wxGenericProgressDialog::OnClose(wxCloseEvent& event)
 {
     if ( m_state == Uncancelable )
     {
@@ -625,15 +722,35 @@ void wxProgressDialog::OnClose(wxCloseEvent& event)
 // destruction
 // ----------------------------------------------------------------------------
 
-wxProgressDialog::~wxProgressDialog()
+wxGenericProgressDialog::~wxGenericProgressDialog()
 {
     // normally this should have been already done, but just in case
     ReenableOtherWindows();
+
+    if ( m_tempEventLoop )
+    {
+        wxEventLoopBase::SetActive(NULL);
+        delete m_tempEventLoop;
+    }
 }
 
-void wxProgressDialog::ReenableOtherWindows()
+void wxGenericProgressDialog::DisableOtherWindows()
 {
-    if ( HasFlag(wxPD_APP_MODAL) )
+    if ( HasPDFlag(wxPD_APP_MODAL) )
+    {
+        m_winDisabler = new wxWindowDisabler(this);
+    }
+    else
+    {
+        if ( m_parentTop )
+            m_parentTop->Disable();
+        m_winDisabler = NULL;
+    }
+}
+
+void wxGenericProgressDialog::ReenableOtherWindows()
+{
+    if ( HasPDFlag(wxPD_APP_MODAL) )
     {
         wxDELETE(m_winDisabler);
     }
@@ -648,32 +765,9 @@ void wxProgressDialog::ReenableOtherWindows()
 // private functions
 // ----------------------------------------------------------------------------
 
-static void SetTimeLabel(unsigned long val, wxStaticText *label)
+void wxGenericProgressDialog::EnableSkip(bool enable)
 {
-    if ( label )
-    {
-        wxString s;
-
-        if (val != (unsigned long)-1)
-        {
-            unsigned long hours = val / 3600;
-            unsigned long minutes = (val % 3600) / 60;
-            unsigned long seconds = val % 60;
-            s.Printf(wxT("%lu:%02lu:%02lu"), hours, minutes, seconds);
-        }
-        else
-        {
-            s = _("Unknown");
-        }
-
-        if ( s != label->GetLabel() )
-            label->SetLabel(s);
-    }
-}
-
-void wxProgressDialog::EnableSkip(bool enable)
-{
-    if(m_hasSkipButton)
+    if ( HasPDFlag(wxPD_CAN_SKIP) )
     {
 #ifdef __SMARTPHONE__
         if(enable)
@@ -687,9 +781,9 @@ void wxProgressDialog::EnableSkip(bool enable)
     }
 }
 
-void wxProgressDialog::EnableAbort(bool enable)
+void wxGenericProgressDialog::EnableAbort(bool enable)
 {
-    if(m_hasAbortButton)
+    if( HasPDFlag(wxPD_CAN_ABORT) )
     {
 #ifdef __SMARTPHONE__
         if(enable)
@@ -703,9 +797,9 @@ void wxProgressDialog::EnableAbort(bool enable)
     }
 }
 
-void wxProgressDialog::EnableClose()
+void wxGenericProgressDialog::EnableClose()
 {
-    if(m_hasAbortButton)
+    if(HasPDFlag(wxPD_CAN_ABORT))
     {
 #ifdef __SMARTPHONE__
         SetLeftMenu(wxID_CANCEL, _("Close"));
@@ -719,11 +813,8 @@ void wxProgressDialog::EnableClose()
     }
 }
 
-void wxProgressDialog::UpdateMessage(const wxString &newmsg)
+void wxGenericProgressDialog::UpdateMessage(const wxString &newmsg)
 {
-    wxCHECK_RET(wxEventLoopBase::GetActive(),
-                "wxProgressDialog::UpdateMessage needs a running event loop");
-
     if ( !newmsg.empty() && newmsg != m_msg->GetLabel() )
     {
         m_msg->SetLabel(newmsg);
