@@ -2,7 +2,7 @@
 // Name:        src/gtk/filedlg.cpp
 // Purpose:     native implementation of wxFileDialog
 // Author:      Robert Roebling, Zbigniew Zagorski, Mart Raudsepp
-// Id:          $Id: filedlg.cpp 64381 2010-05-22 12:07:54Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling, 2004 Zbigniew Zagorski, 2005 Mart Raudsepp
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -22,11 +22,14 @@
 #include <gtk/gtk.h>
 #include "wx/gtk/private.h"
 
+#ifdef __UNIX__
 #include <unistd.h> // chdir
+#endif
 
 #include "wx/filename.h" // wxFilename
 #include "wx/tokenzr.h" // wxStringTokenizer
 #include "wx/filefn.h" // ::wxGetCwd
+#include "wx/modalhook.h"
 
 //-----------------------------------------------------------------------------
 // "clicked" for OK-button
@@ -39,6 +42,7 @@ static void gtk_filedialog_ok_callback(GtkWidget *widget, wxFileDialog *dialog)
     wxGtkString filename(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget)));
 
     // gtk version numbers must be identical with the one in ctor (that calls set_do_overwrite_confirmation)
+#ifndef __WXGTK3__
 #if GTK_CHECK_VERSION(2,7,3)
     if (gtk_check_version(2, 7, 3) != NULL)
 #endif
@@ -60,6 +64,7 @@ static void gtk_filedialog_ok_callback(GtkWidget *widget, wxFileDialog *dialog)
             }
         }
     }
+#endif
 
     if (style & wxFD_FILE_MUST_EXIST)
     {
@@ -80,7 +85,7 @@ static void gtk_filedialog_ok_callback(GtkWidget *widget, wxFileDialog *dialog)
         chdir(folder);
     }
 
-    wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK);
+    wxCommandEvent event(wxEVT_BUTTON, wxID_OK);
     event.SetEventObject(dialog);
     dialog->HandleWindowEvent(event);
 }
@@ -111,6 +116,15 @@ static void gtk_filedialog_response_callback(GtkWidget *w,
         gtk_filedialog_cancel_callback(w, dialog);
 }
 
+static void gtk_filedialog_selchanged_callback(GtkFileChooser *chooser,
+                                               wxFileDialog *dialog)
+{
+    wxGtkString filename(gtk_file_chooser_get_preview_filename(chooser));
+
+    dialog->GTKSelectionChanged(wxString::FromUTF8(filename));
+}
+
+
 static void gtk_filedialog_update_preview_callback(GtkFileChooser *chooser,
                                                    gpointer user_data)
 {
@@ -133,22 +147,12 @@ static void gtk_filedialog_update_preview_callback(GtkFileChooser *chooser,
 
 } // extern "C"
 
-//-----------------------------------------------------------------------------
-// "size_request" from m_extraControl
-//-----------------------------------------------------------------------------
-
-extern "C" {
-static void extra_widget_size_request(GtkWidget*, GtkRequisition* req, wxWindow* win)
-{
-    // allow dialog to be resized smaller horizontally
-    req->width = win->GetMinWidth();
-}
-}
-
 void wxFileDialog::AddChildGTK(wxWindowGTK* child)
 {
-    g_signal_connect_after(child->m_widget, "size_request",
-        G_CALLBACK(extra_widget_size_request), child);
+    // allow dialog to be resized smaller horizontally
+    gtk_widget_set_size_request(
+        child->m_widget, child->GetMinWidth(), child->m_height);
+
     gtk_file_chooser_set_extra_widget(
         GTK_FILE_CHOOSER(m_widget), child->m_widget);
 }
@@ -172,13 +176,24 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
                            const wxSize& sz,
                            const wxString& name)
     : wxFileDialogBase()
+{
+    Create(parent, message, defaultDir, defaultFileName, wildCard, style, pos, sz, name);
+}
+
+bool wxFileDialog::Create(wxWindow *parent, const wxString& message,
+                           const wxString& defaultDir,
+                           const wxString& defaultFileName,
+                           const wxString& wildCard,
+                           long style, const wxPoint& pos,
+                           const wxSize& sz,
+                           const wxString& name)
 {
     parent = GetParentForModalDialog(parent, style);
 
     if (!wxFileDialogBase::Create(parent, message, defaultDir, defaultFileName,
                                   wildCard, style, pos, sz, name))
     {
-        return;
+        return false;
     }
 
     if (!PreCreation(parent, pos, wxDefaultSize) ||
@@ -186,7 +201,7 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
                 wxDefaultValidator, wxT("filedialog")))
     {
         wxFAIL_MSG( wxT("wxFileDialog creation failed") );
-        return;
+        return false;
     }
 
     GtkFileChooserAction gtk_action;
@@ -235,7 +250,7 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
     // local-only property could be set to false to allow non-local files to be
     // loaded. In that case get/set_uri(s) should be used instead of
     // get/set_filename(s) everywhere and the GtkFileChooserDialog should
-    // probably also be created with a backend, e.g "gnome-vfs", "default", ...
+    // probably also be created with a backend, e.g. "gnome-vfs", "default", ...
     // (gtk_file_chooser_dialog_new_with_backend). Currently local-only is kept
     // as the default - true:
     // gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(m_widget), true);
@@ -243,6 +258,8 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
     g_signal_connect (m_widget, "response",
         G_CALLBACK (gtk_filedialog_response_callback), this);
 
+    g_signal_connect (m_widget, "selection-changed",
+        G_CALLBACK (gtk_filedialog_selchanged_callback), this);
 
     // deal with extensions/filters
     SetWildcard(wildCard);
@@ -251,11 +268,18 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
     if ( !wildCard.empty() && !defaultFileName.empty() &&
             !wxFileName(defaultFileName).HasExt() )
     {
-        // append the default extension to the initial file name: GTK won't do
-        // it for us by default (unlike e.g. MSW)
-        const wxString defaultExt = m_fc.GetCurrentWildCard().AfterFirst('.');
-        if ( defaultExt.find_first_of("?*") == wxString::npos )
-            defaultFileNameWithExt += "." + defaultExt;
+        // append the default extension, if any, to the initial file name: GTK
+        // won't do it for us by default (unlike e.g. MSW)
+        const wxFileName fnWC(m_fc.GetCurrentWildCard());
+        if ( fnWC.HasExt() )
+        {
+            // Notice that we shouldn't append the extension if it's a wildcard
+            // because this is not useful: the user would need to change it to use
+            // some fixed extension anyhow.
+            const wxString& ext = fnWC.GetExt();
+            if ( ext.find_first_of("?*") == wxString::npos )
+                defaultFileNameWithExt << "." << ext;
+        }
     }
 
 
@@ -275,7 +299,7 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
     const wxString dir = fn.GetPath();
     if ( !dir.empty() )
     {
-        gtk_file_chooser_set_current_folder(file_chooser, dir.fn_str());
+        gtk_file_chooser_set_current_folder(file_chooser, wxGTK_CONV_FN(dir));
     }
 
     const wxString fname = fn.GetFullName();
@@ -283,12 +307,18 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
     {
         if ( !fname.empty() )
         {
-            gtk_file_chooser_set_current_name(file_chooser, fname.fn_str());
+            gtk_file_chooser_set_current_name(file_chooser, wxGTK_CONV_FN(fname));
         }
 
 #if GTK_CHECK_VERSION(2,7,3)
-        if ((style & wxFD_OVERWRITE_PROMPT) && !gtk_check_version(2,7,3))
+        if ((style & wxFD_OVERWRITE_PROMPT)
+#ifndef __WXGTK3__
+            && gtk_check_version(2,7,3) == NULL
+#endif
+            )
+        {
             gtk_file_chooser_set_do_overwrite_confirmation(file_chooser, true);
+        }
 #endif
     }
     else // wxFD_OPEN
@@ -296,7 +326,7 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
         if ( !fname.empty() )
         {
             gtk_file_chooser_set_filename(file_chooser,
-                                          fn.GetFullPath().fn_str());
+                                          wxGTK_CONV_FN(fn.GetFullPath()));
         }
     }
 
@@ -309,6 +339,8 @@ wxFileDialog::wxFileDialog(wxWindow *parent, const wxString& message,
                          G_CALLBACK(gtk_filedialog_update_preview_callback),
                          previewImage);
     }
+
+    return true;
 }
 
 wxFileDialog::~wxFileDialog()
@@ -324,11 +356,19 @@ wxFileDialog::~wxFileDialog()
 
 void wxFileDialog::OnFakeOk(wxCommandEvent& WXUNUSED(event))
 {
+    // Update the current directory from here, accessing it later may not work
+    // due to the strange way GtkFileChooser works.
+    wxGtkString
+        str(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(m_widget)));
+    m_dir = wxString::FromUTF8(str);
+
     EndDialog(wxID_OK);
 }
 
 int wxFileDialog::ShowModal()
 {
+    WX_HOOK_MODAL_DIALOG();
+
     CreateExtraControl();
 
     return wxDialog::ShowModal();
@@ -369,41 +409,36 @@ void wxFileDialog::SetMessage(const wxString& message)
 
 void wxFileDialog::SetPath(const wxString& path)
 {
+    wxFileDialogBase::SetPath(path);
+
+    // Don't do anything if no path is specified, in particular don't set the
+    // path to m_dir below as this would result in opening the dialog in the
+    // parent directory of this one instead of m_dir itself.
+    if ( path.empty() )
+        return;
+
     // we need an absolute path for GTK native chooser so ensure that we have
-    // it
+    // it: use the initial directory if it was set or just CWD otherwise (this
+    // is the default behaviour if m_dir is empty)
     wxFileName fn(path);
-    fn.MakeAbsolute();
+    fn.MakeAbsolute(m_dir);
     m_fc.SetPath(fn.GetFullPath());
 }
 
 void wxFileDialog::SetDirectory(const wxString& dir)
 {
-    if (m_fc.SetDirectory( dir ))
-    {
-        // Cache the dir, as gtk_file_chooser_get_current_folder()
-        // doesn't return anything until the dialog has been shown
-        m_dir = dir;
-    }
-}
+    wxFileDialogBase::SetDirectory(dir);
 
-wxString wxFileDialog::GetDirectory() const
-{
-    wxString currentDir( m_fc.GetDirectory() );
-    if (currentDir.empty())
-    {
-        // m_fc.GetDirectory() will return empty until the dialog has been shown
-        // in which case use any previously provided value
-        currentDir = m_dir;
-    }
-    return currentDir;
+    m_fc.SetDirectory(dir);
 }
 
 void wxFileDialog::SetFilename(const wxString& name)
 {
+    wxFileDialogBase::SetFilename(name);
+
     if (HasFdFlag(wxFD_SAVE))
     {
         gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_widget), wxGTK_CONV(name));
-        m_fileName = name;
     }
 
     else
@@ -415,7 +450,6 @@ void wxFileDialog::SetFilename(const wxString& name)
             return;
         }
         SetPath(wxFileName(path, name).GetFullPath());
-        m_fileName = name;
     }
 }
 
@@ -447,4 +481,12 @@ int wxFileDialog::GetFilterIndex() const
     return m_fc.GetFilterIndex();
 }
 
+void wxFileDialog::GTKSelectionChanged(const wxString& filename)
+{
+    m_currentlySelectedFilename = filename;
+
+    if (m_extraControl)
+        m_extraControl->UpdateWindowUI(wxUPDATE_UI_RECURSE);
+}
+
 #endif // wxUSE_FILEDLG
