@@ -3,7 +3,7 @@
 // Purpose:
 // Author:      Robert Roebling
 // Modified By: Ryan Norton (GtkTreeView implementation)
-// Id:          $Id: listbox.cpp 64436 2010-05-30 17:45:40Z RR $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,15 +25,20 @@
     #include "wx/arrstr.h"
 #endif
 
-#include "wx/gtk/private.h"
-#include "wx/gtk/treeentry_gtk.h"
-
 #if wxUSE_TOOLTIPS
     #include "wx/tooltip.h"
 #endif
 
 #include <gtk/gtk.h>
+#include "wx/gtk/private.h"
+#include "wx/gtk/private/gtk2-compat.h"
+#include "wx/gtk/private/object.h"
+#include "wx/gtk/private/treeentry_gtk.h"
+
 #include <gdk/gdkkeysyms.h>
+#ifdef __WXGTK3__
+#include <gdk/gdkkeysyms-compat.h>
+#endif
 
 //-----------------------------------------------------------------------------
 // data
@@ -56,6 +61,29 @@ extern bool           g_blockEventsOnScroll;
 
 #define WXLISTBOX_DATACOLUMN    WXLISTBOX_DATACOLUMN_ARG(this)
 
+// ----------------------------------------------------------------------------
+// helper functions
+// ----------------------------------------------------------------------------
+
+namespace
+{
+
+// Return the entry for the given listbox item.
+wxTreeEntry *
+GetEntry(GtkListStore *store, GtkTreeIter *iter, const wxListBox *listbox)
+{
+    wxTreeEntry* entry;
+    gtk_tree_model_get(GTK_TREE_MODEL(store),
+                       iter,
+                       WXLISTBOX_DATACOLUMN_ARG(listbox),
+                       &entry,
+                       -1);
+    g_object_unref(entry);
+    return entry;
+}
+
+} // anonymous namespace
+
 //-----------------------------------------------------------------------------
 // "row-activated"
 //-----------------------------------------------------------------------------
@@ -74,37 +102,7 @@ gtk_listbox_row_activated_callback(GtkTreeView        * WXUNUSED(treeview),
 
     int sel = gtk_tree_path_get_indices(path)[0];
 
-    wxCommandEvent event(wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, listbox->GetId() );
-    event.SetEventObject( listbox );
-
-    if (listbox->IsSelected(sel))
-    {
-        GtkTreeEntry* entry = listbox->GTKGetEntry(sel);
-
-        if (entry)
-        {
-            event.SetInt(sel);
-            event.SetString(wxConvUTF8.cMB2WX(gtk_tree_entry_get_label(entry)));
-
-            if ( listbox->HasClientObjectData() )
-                event.SetClientObject( (wxClientData*) gtk_tree_entry_get_userdata(entry) );
-            else if ( listbox->HasClientUntypedData() )
-                event.SetClientData( gtk_tree_entry_get_userdata(entry) );
-
-            g_object_unref (entry);
-        }
-        else
-        {
-            wxLogSysError(wxT("Internal error - could not get entry for double-click"));
-            event.SetInt(-1);
-        }
-    }
-    else
-    {
-        event.SetInt(-1);
-    }
-
-    listbox->HandleWindowEvent( event );
+    listbox->GTKOnActivated(sel);
 }
 }
 
@@ -119,49 +117,9 @@ gtk_listitem_changed_callback(GtkTreeSelection * WXUNUSED(selection),
 {
     if (g_blockEventsOnDrag) return;
 
-    if (listbox->HasFlag(wxLB_MULTIPLE | wxLB_EXTENDED))
-    {
-        listbox->CalcAndSendEvent();
-    }
-    else // single selection
-    {
-        wxCommandEvent event(wxEVT_COMMAND_LISTBOX_SELECTED, listbox->GetId() );
-        event.SetEventObject( listbox );
-
-        int index = listbox->GetSelection();
-        if (index == wxNOT_FOUND)
-        {
-            // indicate that this is a deselection
-            event.SetExtraLong( 0 );
-            event.SetInt( -1 );
-
-            listbox->HandleWindowEvent( event );
-
-            return;
-        }
-        else
-        {
-            GtkTreeEntry* entry = listbox->GTKGetEntry( index );
-
-            // indicate that this is a selection
-            event.SetExtraLong( 1 );
-
-            event.SetInt( index );
-            event.SetString(wxConvUTF8.cMB2WX(gtk_tree_entry_get_label(entry)));
-
-            if ( listbox->HasClientObjectData() )
-                event.SetClientObject(
-                    (wxClientData*) gtk_tree_entry_get_userdata(entry)
-                                 );
-            else if ( listbox->HasClientUntypedData() )
-                event.SetClientData( gtk_tree_entry_get_userdata(entry) );
-
-            listbox->HandleWindowEvent( event );
-
-            g_object_unref (entry);
-        }
-    }
+    listbox->GTKOnSelectionChanged();
 }
+
 }
 
 //-----------------------------------------------------------------------------
@@ -191,27 +149,7 @@ gtk_listbox_key_press_callback( GtkWidget *WXUNUSED(widget),
 
         if (index != wxNOT_FOUND)
         {
-            wxCommandEvent event(wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, listbox->GetId() );
-            event.SetEventObject( listbox );
-
-            GtkTreeEntry* entry = listbox->GTKGetEntry( index );
-
-            // indicate that this is a selection
-            event.SetExtraLong( 1 );
-
-            event.SetInt( index );
-            event.SetString(wxConvUTF8.cMB2WX(gtk_tree_entry_get_label(entry)));
-
-            if ( listbox->HasClientObjectData() )
-                event.SetClientObject(
-                    (wxClientData*) gtk_tree_entry_get_userdata(entry)
-                                 );
-            else if ( listbox->HasClientUntypedData() )
-                event.SetClientData( gtk_tree_entry_get_userdata(entry) );
-
-            /* bool ret = */ listbox->HandleWindowEvent( event );
-
-            g_object_unref (entry);
+            listbox->GTKOnActivated(index);
 
 //          wxMac and wxMSW always invoke default action
 //          if (!ret)
@@ -241,12 +179,12 @@ gtk_listbox_key_press_callback( GtkWidget *WXUNUSED(widget),
 //-----------------------------------------------------------------------------
 
 extern "C" {
-static void gtk_tree_entry_destroy_cb(GtkTreeEntry* entry,
+static void tree_entry_destroy_cb(wxTreeEntry* entry,
                                       wxListBox* listbox)
 {
     if (listbox->HasClientObjectData())
     {
-        gpointer userdata = gtk_tree_entry_get_userdata(entry);
+        void* userdata = wx_tree_entry_get_userdata(entry);
         if (userdata)
             delete (wxClientData *)userdata;
     }
@@ -263,29 +201,16 @@ static gint gtk_listbox_sort_callback(GtkTreeModel * WXUNUSED(model),
                                       GtkTreeIter  *b,
                                       wxListBox    *listbox)
 {
-    GtkTreeEntry* entry;
-    GtkTreeEntry* entry2;
+    wxTreeEntry* entry1 = GetEntry(listbox->m_liststore, a, listbox);
+    wxCHECK_MSG(entry1, 0, wxT("Could not get first entry"));
 
-    gtk_tree_model_get(GTK_TREE_MODEL(listbox->m_liststore),
-                             a,
-                             WXLISTBOX_DATACOLUMN_ARG(listbox),
-                             &entry, -1);
-    gtk_tree_model_get(GTK_TREE_MODEL(listbox->m_liststore),
-                             b,
-                             WXLISTBOX_DATACOLUMN_ARG(listbox),
-                             &entry2, -1);
-    wxCHECK_MSG(entry, 0, wxT("Could not get entry"));
-    wxCHECK_MSG(entry2, 0, wxT("Could not get entry2"));
+    wxTreeEntry* entry2 = GetEntry(listbox->m_liststore, b, listbox);
+    wxCHECK_MSG(entry2, 0, wxT("Could not get second entry"));
 
     //We compare collate keys here instead of calling g_utf8_collate
-    //as it is rather slow (and even the docs reccommend this)
-    int ret = strcmp(gtk_tree_entry_get_collate_key(entry),
-                     gtk_tree_entry_get_collate_key(entry2));
-
-    g_object_unref (entry);
-    g_object_unref (entry2);
-
-    return ret;
+    //as it is rather slow (and even the docs recommend this)
+    return strcmp(wx_tree_entry_get_collate_key(entry1),
+                  wx_tree_entry_get_collate_key(entry2)) >= 0;
 }
 }
 
@@ -300,21 +225,12 @@ static gboolean gtk_listbox_searchequal_callback(GtkTreeModel * WXUNUSED(model),
                                                  GtkTreeIter* iter,
                                                  wxListBox* listbox)
 {
-    GtkTreeEntry* entry;
-
-    gtk_tree_model_get(GTK_TREE_MODEL(listbox->m_liststore),
-                             iter,
-                             WXLISTBOX_DATACOLUMN_ARG(listbox),
-                             &entry, -1);
+    wxTreeEntry* entry = GetEntry(listbox->m_liststore, iter, listbox);
     wxCHECK_MSG(entry, 0, wxT("Could not get entry"));
+
     wxGtkString keycollatekey(g_utf8_collate_key(key, -1));
 
-    int ret = strcmp(keycollatekey,
-                     gtk_tree_entry_get_collate_key(entry));
-
-    g_object_unref (entry);
-
-    return ret != 0;
+    return strcmp(keycollatekey, wx_tree_entry_get_collate_key(entry)) != 0;
 }
 }
 
@@ -322,8 +238,6 @@ static gboolean gtk_listbox_searchequal_callback(GtkTreeModel * WXUNUSED(model),
 // wxListBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxControlWithItems)
-
 // ----------------------------------------------------------------------------
 // construction
 // ----------------------------------------------------------------------------
@@ -398,10 +312,10 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
 #if wxUSE_CHECKLISTBOX
     if(m_hasCheckBoxes)
         m_liststore = gtk_list_store_new(2, G_TYPE_BOOLEAN,
-                                            GTK_TYPE_TREE_ENTRY);
+                                            WX_TYPE_TREE_ENTRY);
     else
 #endif
-        m_liststore = gtk_list_store_new(1, GTK_TYPE_TREE_ENTRY);
+        m_liststore = gtk_list_store_new(1, WX_TYPE_TREE_ENTRY);
 
     gtk_tree_view_set_model(m_treeview, GTK_TREE_MODEL(m_liststore));
 
@@ -430,7 +344,12 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
     else // no multi-selection flags specified
     {
         m_windowStyle |= wxLB_SINGLE;
-        mode = GTK_SELECTION_SINGLE;
+
+        // Notice that we must use BROWSE and not GTK_SELECTION_SINGLE because
+        // the latter allows to not select any items at all while a single
+        // selection listbox is supposed to always have a selection (at least
+        // once the user selected something, it might not have any initially).
+        mode = GTK_SELECTION_BROWSE;
     }
 
     GtkTreeSelection* selection = gtk_tree_view_get_selection( m_treeview );
@@ -482,7 +401,13 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
 
 wxListBox::~wxListBox()
 {
-    m_hasVMT = false;
+    if (m_treeview)
+    {
+        GTKDisconnect(m_treeview);
+        GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
+        if (selection)
+            GTKDisconnect(selection);
+    }
 
     Clear();
 }
@@ -511,7 +436,7 @@ void wxListBox::Update()
     wxWindow::Update();
 
     if (m_treeview)
-        gdk_window_process_updates(GTK_WIDGET(m_treeview)->window, TRUE);
+        gdk_window_process_updates(gtk_widget_get_window(GTK_WIDGET(m_treeview)), true);
 }
 
 // ----------------------------------------------------------------------------
@@ -526,40 +451,26 @@ int wxListBox::DoInsertItems(const wxArrayStringsAdapter& items,
     wxCHECK_MSG( m_treeview != NULL, wxNOT_FOUND, wxT("invalid listbox") );
 
     InvalidateBestSize();
-
-    GtkTreeIter* pIter = NULL; // append by default
-    GtkTreeIter iter;
-    if ( pos != GetCount() )
-    {
-        wxCHECK_MSG( GTKGetIteratorFor(pos, &iter), wxNOT_FOUND,
-                     wxT("internal wxListBox error in insertion") );
-
-        pIter = &iter;
-    }
-
-    const unsigned int numItems = items.GetCount();
-    for ( unsigned int i = 0; i < numItems; ++i )
-    {
-        GtkTreeEntry* entry = gtk_tree_entry_new();
-        gtk_tree_entry_set_label(entry, wxGTK_CONV(items[i]));
-        gtk_tree_entry_set_destroy_func(entry,
-                (GtkTreeEntryDestroy)gtk_tree_entry_destroy_cb,
-                            this);
-
-        GtkTreeIter itercur;
-        gtk_list_store_insert_before(m_liststore, &itercur, pIter);
-
-        GTKSetItem(itercur, entry);
-
-        g_object_unref (entry);
-
-        if (clientData)
-            AssignNewItemClientData(GTKGetIndexFor(itercur), clientData, i, type);
-    }
-
+    int n = DoInsertItemsInLoop(items, pos, clientData, type);
     UpdateOldSelections();
+    return n;
+}
 
-    return pos + numItems - 1;
+int wxListBox::DoInsertOneItem(const wxString& item, unsigned int pos)
+{
+    wxTreeEntry* entry = wx_tree_entry_new();
+    wx_tree_entry_set_label(entry, wxGTK_CONV(item));
+    wx_tree_entry_set_destroy_func(entry, (wxTreeEntryDestroy)tree_entry_destroy_cb, this);
+
+#if wxUSE_CHECKLISTBOX
+    int entryCol = int(m_hasCheckBoxes);
+#else
+    int entryCol = 0;
+#endif
+    gtk_list_store_insert_with_values(m_liststore, NULL, pos, entryCol, entry, -1);
+    g_object_unref(entry);
+
+    return pos;
 }
 
 // ----------------------------------------------------------------------------
@@ -577,6 +488,8 @@ void wxListBox::DoClear()
     gtk_list_store_clear( m_liststore ); /* well, THAT was easy :) */
 
     GTKEnableEvents();
+
+    UpdateOldSelections();
 }
 
 void wxListBox::DoDeleteOneItem(unsigned int n)
@@ -630,35 +543,13 @@ int wxListBox::GTKGetIndexFor(GtkTreeIter& iter) const
 }
 
 // get GtkTreeEntry from position (note: you need to g_unref it if valid)
-GtkTreeEntry *wxListBox::GTKGetEntry(unsigned n) const
+wxTreeEntry* wxListBox::GTKGetEntry(unsigned n) const
 {
     GtkTreeIter iter;
     if ( !GTKGetIteratorFor(n, &iter) )
         return NULL;
 
-
-    GtkTreeEntry* entry = NULL;
-    gtk_tree_model_get(GTK_TREE_MODEL(m_liststore), &iter,
-                       WXLISTBOX_DATACOLUMN, &entry, -1);
-
-    return entry;
-}
-
-void wxListBox::GTKSetItem(GtkTreeIter& iter, const GtkTreeEntry *entry)
-{
-#if wxUSE_CHECKLISTBOX
-    if ( m_hasCheckBoxes )
-    {
-        gtk_list_store_set(m_liststore, &iter,
-                           0, FALSE, // FALSE == not toggled
-                           1, entry,
-                           -1);
-    }
-    else
-#endif // wxUSE_CHECKLISTBOX
-    {
-        gtk_list_store_set(m_liststore, &iter, 0, entry, -1);
-    }
+    return GetEntry(m_liststore, &iter, this);
 }
 
 // ----------------------------------------------------------------------------
@@ -667,27 +558,18 @@ void wxListBox::GTKSetItem(GtkTreeIter& iter, const GtkTreeEntry *entry)
 
 void* wxListBox::DoGetItemClientData(unsigned int n) const
 {
-    wxCHECK_MSG( IsValid(n), NULL,
-                 wxT("Invalid index passed to GetItemClientData") );
-
-    GtkTreeEntry* entry = GTKGetEntry(n);
+    wxTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_MSG(entry, NULL, wxT("could not get entry"));
 
-    void* userdata = gtk_tree_entry_get_userdata( entry );
-    g_object_unref (entry);
-    return userdata;
+    return wx_tree_entry_get_userdata(entry);
 }
 
 void wxListBox::DoSetItemClientData(unsigned int n, void* clientData)
 {
-    wxCHECK_RET( IsValid(n),
-                 wxT("Invalid index passed to SetItemClientData") );
-
-    GtkTreeEntry* entry = GTKGetEntry(n);
+    wxTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_RET(entry, wxT("could not get entry"));
 
-    gtk_tree_entry_set_userdata( entry, clientData );
-    g_object_unref (entry);
+    wx_tree_entry_set_userdata(entry, clientData);
 }
 
 // ----------------------------------------------------------------------------
@@ -696,35 +578,30 @@ void wxListBox::DoSetItemClientData(unsigned int n, void* clientData)
 
 void wxListBox::SetString(unsigned int n, const wxString& label)
 {
-    wxCHECK_RET( IsValid(n), wxT("invalid index in wxListBox::SetString") );
     wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
 
-    GtkTreeEntry* entry = GTKGetEntry(n);
-    wxCHECK_RET( entry, wxT("wrong listbox index") );
+    GtkTreeIter iter;
+    wxCHECK_RET(GTKGetIteratorFor(n, &iter), "invalid index");
+    wxTreeEntry* entry = GetEntry(m_liststore, &iter, this);
 
     // update the item itself
-    gtk_tree_entry_set_label(entry, wxGTK_CONV(label));
+    wx_tree_entry_set_label(entry, wxGTK_CONV(label));
 
-    // and update the model which will refresh the tree too
-    GtkTreeIter iter;
-    wxCHECK_RET( GTKGetIteratorFor(n, &iter), wxT("failed to get iterator") );
-
-    // FIXME: this resets the checked status of a wxCheckListBox item
-
-    GTKSetItem(iter, entry);
+    // signal row changed
+    GtkTreeModel* tree_model = GTK_TREE_MODEL(m_liststore);
+    GtkTreePath* path = gtk_tree_model_get_path(tree_model, &iter);
+    gtk_tree_model_row_changed(tree_model, path, &iter);
+    gtk_tree_path_free(path);
 }
 
 wxString wxListBox::GetString(unsigned int n) const
 {
     wxCHECK_MSG( m_treeview != NULL, wxEmptyString, wxT("invalid listbox") );
 
-    GtkTreeEntry* entry = GTKGetEntry(n);
+    wxTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_MSG( entry, wxEmptyString, wxT("wrong listbox index") );
 
-    wxString label = wxGTK_CONV_BACK( gtk_tree_entry_get_label(entry) );
-
-    g_object_unref (entry);
-    return label;
+    return wxGTK_CONV_BACK(wx_tree_entry_get_label(entry));
 }
 
 unsigned int wxListBox::GetCount() const
@@ -756,6 +633,25 @@ int wxListBox::FindString( const wxString &item, bool bCase ) const
 // selection
 // ----------------------------------------------------------------------------
 
+void wxListBox::GTKOnActivated(int item)
+{
+    SendEvent(wxEVT_LISTBOX_DCLICK, item, IsSelected(item));
+}
+
+void wxListBox::GTKOnSelectionChanged()
+{
+    if ( HasFlag(wxLB_MULTIPLE | wxLB_EXTENDED) )
+    {
+        CalcAndSendEvent();
+    }
+    else // single selection
+    {
+        const int item = GetSelection();
+        if ( DoChangeSingleSelection(item) )
+            SendEvent(wxEVT_LISTBOX, item, true);
+    }
+}
+
 int wxListBox::GetSelection() const
 {
     wxCHECK_MSG( m_treeview != NULL, wxNOT_FOUND, wxT("invalid listbox"));
@@ -805,7 +701,7 @@ bool wxListBox::IsSelected( int n ) const
     GtkTreeIter iter;
     wxCHECK_MSG( GTKGetIteratorFor(n, &iter), false, wxT("Invalid index") );
 
-    return gtk_tree_selection_iter_is_selected(selection, &iter);
+    return gtk_tree_selection_iter_is_selected(selection, &iter) != 0;
 }
 
 void wxListBox::DoSetSelection( int n, bool select )
@@ -851,7 +747,7 @@ void wxListBox::DoScrollToCell(int n, float alignY, float alignX)
     wxCHECK_RET( IsValid(n), wxT("invalid index"));
 
     //RN: I have no idea why this line is needed...
-    if (gdk_pointer_is_grabbed () && GTK_WIDGET_HAS_GRAB (m_treeview))
+    if (gtk_widget_has_grab(GTK_WIDGET(m_treeview)))
         return;
 
     GtkTreeIter iter;
@@ -892,7 +788,7 @@ int wxListBox::DoListHitTest(const wxPoint& point) const
     // need to translate from master window since it is in client coords
     gint binx, biny;
     gdk_window_get_geometry(gtk_tree_view_get_bin_window(m_treeview),
-                            &binx, &biny, NULL, NULL, NULL);
+                            &binx, &biny, NULL, NULL);
 
     GtkTreePath* path;
     if ( !gtk_tree_view_get_path_at_pos
@@ -919,14 +815,6 @@ int wxListBox::DoListHitTest(const wxPoint& point) const
 // helpers
 // ----------------------------------------------------------------------------
 
-#if wxUSE_TOOLTIPS
-void wxListBox::GTKApplyToolTip( GtkTooltips *tips, const gchar *tip )
-{
-    // RN: Is this needed anymore?
-    gtk_tooltips_set_tip( tips, GTK_WIDGET( m_treeview ), tip, NULL );
-}
-#endif // wxUSE_TOOLTIPS
-
 GtkWidget *wxListBox::GetConnectWidget()
 {
     // the correct widget for listbox events (such as mouse clicks for example)
@@ -941,7 +829,10 @@ GdkWindow *wxListBox::GTKGetWindow(wxArrayGdkWindows& WXUNUSED(windows)) const
 
 void wxListBox::DoApplyWidgetStyle(GtkRcStyle *style)
 {
-    if (m_hasBgCol && m_backgroundColour.Ok())
+#ifdef __WXGTK3__
+    // don't know if this is even necessary, or how to do it
+#else
+    if (m_hasBgCol && m_backgroundColour.IsOk())
     {
         GdkWindow *window = gtk_tree_view_get_bin_window(m_treeview);
         if (window)
@@ -951,8 +842,9 @@ void wxListBox::DoApplyWidgetStyle(GtkRcStyle *style)
             gdk_window_clear( window );
         }
     }
+#endif
 
-    gtk_widget_modify_style( GTK_WIDGET(m_treeview), style );
+    GTKApplyStyle(GTK_WIDGET(m_treeview), style);
 }
 
 wxSize wxListBox::DoGetBestSize() const
@@ -1005,7 +897,7 @@ wxSize wxListBox::DoGetBestSize() const
 wxVisualAttributes
 wxListBox::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
 {
-    return GetDefaultAttributesFromGTKWidget(gtk_tree_view_new, true);
+    return GetDefaultAttributesFromGTKWidget(gtk_tree_view_new(), true);
 }
 
 #endif // wxUSE_LISTBOX
