@@ -2,7 +2,7 @@
 // Name:        src/gtk/minifram.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: minifram.cpp 64404 2010-05-26 17:37:55Z RR $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -20,9 +20,14 @@
     #include "wx/image.h"
 #endif
 
+#ifdef __WXGTK3__
+#include "wx/gtk/dc.h"
+#else
 #include "wx/gtk/dcclient.h"
+#endif
 
 #include <gtk/gtk.h>
+#include "wx/gtk/private/gtk2-compat.h"
 
 //-----------------------------------------------------------------------------
 // data
@@ -58,29 +63,46 @@ static wxColor LightContrastColour(const wxColour& c)
 }
 
 extern "C" {
-static gboolean gtk_window_own_expose_callback(GtkWidget* widget, GdkEventExpose* gdk_event, wxMiniFrame* win)
+#ifdef __WXGTK3__
+static gboolean draw(GtkWidget* widget, cairo_t* cr, wxMiniFrame* win)
+#else
+static gboolean expose_event(GtkWidget* widget, GdkEventExpose* gdk_event, wxMiniFrame* win)
+#endif
 {
-    if (!win->m_hasVMT || gdk_event->count > 0 ||
-        gdk_event->window != widget->window)
+#ifdef __WXGTK3__
+    if (!gtk_cairo_should_draw_window(cr, gtk_widget_get_window(widget)))
+        return false;
+
+    GtkStyleContext* sc = gtk_widget_get_style_context(widget);
+    gtk_style_context_save(sc);
+    gtk_style_context_add_class(sc, GTK_STYLE_CLASS_BUTTON);
+    gtk_render_frame(sc, cr, 0, 0, win->m_width, win->m_height);
+    gtk_style_context_restore(sc);
+
+    wxGTKCairoDC dc(cr);
+#else
+    if (gdk_event->count > 0 ||
+        gdk_event->window != gtk_widget_get_window(widget))
     {
         return false;
     }
 
-    gtk_paint_shadow (widget->style,
-                      widget->window,
+    gtk_paint_shadow (gtk_widget_get_style(widget),
+                      gtk_widget_get_window(widget),
                       GTK_STATE_NORMAL,
                       GTK_SHADOW_OUT,
                       NULL, NULL, NULL, // FIXME: No clipping?
                       0, 0,
                       win->m_width, win->m_height);
 
-    int style = win->GetWindowStyle();
-
     wxClientDC dc(win);
 
     wxDCImpl *impl = dc.GetImpl();
     wxClientDCImpl *gtk_impl = wxDynamicCast( impl, wxClientDCImpl );
-    gtk_impl->m_gdkwindow = widget->window; // Hack alert
+    gtk_impl->m_gdkwindow = gtk_widget_get_window(widget); // Hack alert
+#endif
+
+    int style = win->GetWindowStyle();
 
     if (style & wxRESIZE_BORDER)
     {
@@ -120,7 +142,7 @@ extern "C" {
 static gboolean
 gtk_window_button_press_callback(GtkWidget* widget, GdkEventButton* gdk_event, wxMiniFrame* win)
 {
-    if (!win->m_hasVMT || gdk_event->window != widget->window)
+    if (gdk_event->window != gtk_widget_get_window(widget))
         return false;
     if (g_blockEventsOnDrag) return TRUE;
     if (g_blockEventsOnScroll) return TRUE;
@@ -137,7 +159,7 @@ gtk_window_button_press_callback(GtkWidget* widget, GdkEventButton* gdk_event, w
     {
         GtkWidget *ancestor = gtk_widget_get_toplevel( widget );
 
-        GdkWindow *source = widget->window;
+        GdkWindow *source = gtk_widget_get_window(widget);
 
         int org_x = 0;
         int org_y = 0;
@@ -165,19 +187,22 @@ gtk_window_button_press_callback(GtkWidget* widget, GdkEventButton* gdk_event, w
     if (y >= win->m_miniEdge + win->m_miniTitle)
         return true;
 
-    gdk_window_raise( win->m_widget->window );
+    gdk_window_raise(gtk_widget_get_window(win->m_widget));
 
-    gdk_pointer_grab( widget->window, FALSE,
-                      (GdkEventMask)
-                         (GDK_BUTTON_PRESS_MASK |
-                          GDK_BUTTON_RELEASE_MASK |
-                          GDK_POINTER_MOTION_MASK        |
-                          GDK_POINTER_MOTION_HINT_MASK  |
-                          GDK_BUTTON_MOTION_MASK        |
-                          GDK_BUTTON1_MOTION_MASK),
-                      NULL,
-                      NULL,
-                      (unsigned int) GDK_CURRENT_TIME );
+    const GdkEventMask mask = GdkEventMask(
+        GDK_BUTTON_PRESS_MASK |
+        GDK_BUTTON_RELEASE_MASK |
+        GDK_POINTER_MOTION_MASK |
+        GDK_POINTER_MOTION_HINT_MASK |
+        GDK_BUTTON_MOTION_MASK |
+        GDK_BUTTON1_MOTION_MASK);
+#ifdef __WXGTK3__
+    gdk_device_grab(
+        gdk_event->device, gdk_event->window, GDK_OWNERSHIP_NONE,
+        false, mask, NULL, gdk_event->time);
+#else
+    gdk_pointer_grab(gdk_event->window, false, mask, NULL, NULL, gdk_event->time);
+#endif
 
     win->m_diffX = x;
     win->m_diffY = y;
@@ -198,7 +223,7 @@ extern "C" {
 static gboolean
 gtk_window_button_release_callback(GtkWidget* widget, GdkEventButton* gdk_event, wxMiniFrame* win)
 {
-    if (!win->m_hasVMT || gdk_event->window != widget->window)
+    if (gdk_event->window != gtk_widget_get_window(widget))
         return false;
     if (g_blockEventsOnDrag) return TRUE;
     if (g_blockEventsOnScroll) return TRUE;
@@ -209,10 +234,14 @@ gtk_window_button_release_callback(GtkWidget* widget, GdkEventButton* gdk_event,
     int x = (int)gdk_event->x;
     int y = (int)gdk_event->y;
 
-    gdk_pointer_ungrab ( (guint32)GDK_CURRENT_TIME );
+#ifdef __WXGTK3__
+    gdk_device_ungrab(gdk_event->device, gdk_event->time);
+#else
+    gdk_pointer_ungrab(gdk_event->time);
+#endif
     int org_x = 0;
     int org_y = 0;
-    gdk_window_get_origin( widget->window, &org_x, &org_y );
+    gdk_window_get_origin(gtk_widget_get_window(widget), &org_x, &org_y);
     x += org_x - win->m_diffX;
     y += org_y - win->m_diffY;
     win->m_x = x;
@@ -231,14 +260,13 @@ extern "C" {
 static gboolean
 gtk_window_leave_callback(GtkWidget *widget,
                           GdkEventCrossing* gdk_event,
-                          wxMiniFrame *win)
+                          wxMiniFrame*)
 {
-    if (!win->m_hasVMT) return FALSE;
     if (g_blockEventsOnDrag) return FALSE;
-    if (gdk_event->window != widget->window)
+    if (gdk_event->window != gtk_widget_get_window(widget))
         return false;
 
-    gdk_window_set_cursor( widget->window, NULL );
+    gdk_window_set_cursor(gtk_widget_get_window(widget), NULL);
 
     return FALSE;
 }
@@ -252,35 +280,31 @@ extern "C" {
 static gboolean
 gtk_window_motion_notify_callback( GtkWidget *widget, GdkEventMotion *gdk_event, wxMiniFrame *win )
 {
-    if (!win->m_hasVMT || gdk_event->window != widget->window)
+    if (gdk_event->window != gtk_widget_get_window(widget))
         return false;
     if (g_blockEventsOnDrag) return TRUE;
     if (g_blockEventsOnScroll) return TRUE;
 
+    int x = int(gdk_event->x);
+    int y = int(gdk_event->y);
+
     if (gdk_event->is_hint)
     {
-       int x = 0;
-       int y = 0;
-       GdkModifierType state;
-       gdk_window_get_pointer(gdk_event->window, &x, &y, &state);
-       gdk_event->x = x;
-       gdk_event->y = y;
-       gdk_event->state = state;
+#ifdef __WXGTK3__
+       gdk_window_get_device_position(gdk_event->window, gdk_event->device, &x, &y, NULL);
+#else
+       gdk_window_get_pointer(gdk_event->window, &x, &y, NULL);
+#endif
     }
 
-    int style = win->GetWindowStyle();
-
-    int x = (int)gdk_event->x;
-    int y = (int)gdk_event->y;
-
     if (!win->m_isDragging)
     {
-        if (style & wxRESIZE_BORDER)
+        if (win->GetWindowStyle() & wxRESIZE_BORDER)
         {
             if ((x > win->m_width-14) && (y > win->m_height-14))
-               gdk_window_set_cursor( widget->window, gdk_cursor_new( GDK_BOTTOM_RIGHT_CORNER ) );
+               gdk_window_set_cursor(gtk_widget_get_window(widget), gdk_cursor_new(GDK_BOTTOM_RIGHT_CORNER));
             else
-               gdk_window_set_cursor( widget->window, NULL );
+               gdk_window_set_cursor(gtk_widget_get_window(widget), NULL);
             win->GTKUpdateCursor(false);
         }
         return TRUE;
@@ -291,7 +315,7 @@ gtk_window_motion_notify_callback( GtkWidget *widget, GdkEventMotion *gdk_event,
 
     int org_x = 0;
     int org_y = 0;
-    gdk_window_get_origin( widget->window, &org_x, &org_y );
+    gdk_window_get_origin(gtk_widget_get_window(widget), &org_x, &org_y);
     x += org_x - win->m_diffX;
     y += org_y - win->m_diffY;
     win->m_x = x;
@@ -314,6 +338,15 @@ static unsigned char close_bits[]={
 
 IMPLEMENT_DYNAMIC_CLASS(wxMiniFrame,wxFrame)
 
+wxMiniFrame::~wxMiniFrame()
+{
+    if (m_widget)
+    {
+        GtkWidget* eventbox = gtk_bin_get_child(GTK_BIN(m_widget));
+        GTKDisconnect(eventbox);
+    }
+}
+
 bool wxMiniFrame::Create( wxWindow *parent, wxWindowID id, const wxString &title,
       const wxPoint &pos, const wxSize &size,
       long style, const wxString &name )
@@ -332,6 +365,14 @@ bool wxMiniFrame::Create( wxWindow *parent, wxWindowID id, const wxString &title
     m_diffX = 0;
     m_diffY = 0;
 
+    // don't allow sizing smaller than decorations
+    int minWidth = 2 * m_miniEdge;
+    int minHeight = 2 * m_miniEdge + m_miniTitle;
+    if (m_minWidth < minWidth)
+        m_minWidth = minWidth;
+    if (m_minHeight < minHeight)
+        m_minHeight = minHeight;
+
     wxFrame::Create( parent, id, title, pos, size, style, name );
 
     // Use a GtkEventBox for the title and borders. Using m_widget for this
@@ -359,12 +400,6 @@ bool wxMiniFrame::Create( wxWindow *parent, wxWindowID id, const wxString &title
     m_decorSize.Set(0, 0);
     m_deferShow = false;
 
-    // don't allow sizing smaller than decorations
-    GdkGeometry geom;
-    geom.min_width  = 2 * m_miniEdge;
-    geom.min_height = 2 * m_miniEdge + m_miniTitle;
-    gtk_window_set_geometry_hints(GTK_WINDOW(m_widget), NULL, &geom, GDK_HINT_MIN_SIZE);
-
     if (m_parent && (GTK_IS_WINDOW(m_parent->m_widget)))
     {
         gtk_window_set_transient_for( GTK_WINDOW(m_widget), GTK_WINDOW(m_parent->m_widget) );
@@ -379,8 +414,11 @@ bool wxMiniFrame::Create( wxWindow *parent, wxWindowID id, const wxString &title
     }
 
     /* these are called when the borders are drawn */
-    g_signal_connect_after(eventbox, "expose_event",
-                      G_CALLBACK (gtk_window_own_expose_callback), this );
+#ifdef __WXGTK3__
+    g_signal_connect_after(eventbox, "draw", G_CALLBACK(draw), this);
+#else
+    g_signal_connect_after(eventbox, "expose_event", G_CALLBACK(expose_event), this);
+#endif
 
     /* these are required for dragging the mini frame around */
     g_signal_connect (eventbox, "button_press_event",
@@ -397,6 +435,10 @@ bool wxMiniFrame::Create( wxWindow *parent, wxWindowID id, const wxString &title
 void wxMiniFrame::DoGetClientSize(int* width, int* height) const
 {
     wxFrame::DoGetClientSize(width, height);
+
+    if (m_useCachedClientSize)
+        return;
+
     if (width)
     {
         *width -= 2 * m_miniEdge;
@@ -423,9 +465,9 @@ void wxMiniFrame::SetTitle( const wxString &title )
 {
     wxFrame::SetTitle( title );
 
-    GtkWidget* widget = GTK_BIN(m_widget)->child;
-    if (widget->window)
-        gdk_window_invalidate_rect(widget->window, NULL, false);
+    GdkWindow* window = gtk_widget_get_window(gtk_bin_get_child(GTK_BIN(m_widget)));
+    if (window)
+        gdk_window_invalidate_rect(window, NULL, false);
 }
 
 #endif // wxUSE_MINIFRAME
