@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     20.07.2003
-// RCS-ID:      $Id: renderer.cpp 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -34,10 +34,13 @@
 #endif
 
 #include "wx/dcgraph.h"
+#ifndef __WXGTK3__
 #include "wx/gtk/dc.h"
-#include "wx/gtk/private.h"
+#endif
 
 #include <gtk/gtk.h>
+#include "wx/gtk/private.h"
+#include "wx/gtk/private/gtk2-compat.h"
 
 // ----------------------------------------------------------------------------
 // wxRendererGTK: our wxRendererNative implementation
@@ -56,6 +59,8 @@ public:
 
     virtual int GetHeaderButtonHeight(wxWindow *win);
 
+    virtual int GetHeaderButtonMargin(wxWindow *win);
+
 
     // draw the expanded/collapsed icon for a tree control item
     virtual void DrawTreeItemButton(wxWindow *win,
@@ -138,24 +143,43 @@ wxRendererNative& wxRendererNative::GetDefault()
     return s_rendererGTK;
 }
 
-static GdkWindow* wxGetGdkWindowForDC(wxWindow* win, wxDC& dc)
+#ifdef __WXGTK3__
+#define NULL_RECT
+typedef cairo_t wxGTKDrawable;
+
+static cairo_t* wxGetGTKDrawable(wxWindow*, const wxDC& dc)
+{
+    wxGraphicsContext* gc = dc.GetGraphicsContext();
+    wxCHECK_MSG(gc, NULL, "cannot use wxRendererNative on wxDC of this type");
+    return static_cast<cairo_t*>(gc->GetNativeContext());
+}
+
+static const GtkStateFlags stateTypeToFlags[] = {
+    GTK_STATE_FLAG_NORMAL, GTK_STATE_FLAG_ACTIVE, GTK_STATE_FLAG_PRELIGHT,
+    GTK_STATE_FLAG_SELECTED, GTK_STATE_FLAG_INSENSITIVE, GTK_STATE_FLAG_INCONSISTENT,
+    GTK_STATE_FLAG_FOCUSED
+};
+
+#else
+#define NULL_RECT NULL,
+typedef GdkWindow wxGTKDrawable;
+
+static GdkWindow* wxGetGTKDrawable(wxWindow* win, wxDC& dc)
 {
     GdkWindow* gdk_window = NULL;
 
 #if wxUSE_GRAPHICS_CONTEXT
-    if ( dc.IsKindOf( CLASSINFO(wxGCDC) ) )
+    if ( wxDynamicCast(&dc, wxGCDC) )
         gdk_window = win->GTKGetDrawingWindow();
     else
 #endif
     {
-#if wxUSE_NEW_DC
         wxDCImpl *impl = dc.GetImpl();
         wxGTKDCImpl *gtk_impl = wxDynamicCast( impl, wxGTKDCImpl );
         if (gtk_impl)
             gdk_window = gtk_impl->GetGDKWindow();
-#else
-        gdk_window = dc.GetGDKWindow();
-#endif
+        else
+            wxFAIL_MSG("cannot use wxRendererNative on wxDC of this type");
     }
 
 #if !wxUSE_GRAPHICS_CONTEXT
@@ -164,6 +188,7 @@ static GdkWindow* wxGetGdkWindowForDC(wxWindow* win, wxDC& dc)
 
     return gdk_window;
 }
+#endif
 
 // ----------------------------------------------------------------------------
 // list/tree controls drawing
@@ -177,17 +202,12 @@ wxRendererGTK::DrawHeaderButton(wxWindow *win,
                                 wxHeaderSortIconType sortArrow,
                                 wxHeaderButtonParams* params)
 {
-
     GtkWidget *button = wxGTKPrivate::GetHeaderButtonWidget();
     if (flags & wxCONTROL_SPECIAL)
         button = wxGTKPrivate::GetHeaderButtonWidgetFirst();
     if (flags & wxCONTROL_DIRTY)
         button = wxGTKPrivate::GetHeaderButtonWidgetLast();
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
-
     int x_diff = 0;
     if (win->GetLayoutDirection() == wxLayout_RightToLeft)
         x_diff = rect.width;
@@ -201,9 +221,22 @@ wxRendererGTK::DrawHeaderButton(wxWindow *win,
             state = GTK_STATE_PRELIGHT;
     }
 
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        GtkStyleContext* sc = gtk_widget_get_style_context(button);
+        gtk_style_context_save(sc);
+        gtk_style_context_set_state(sc, stateTypeToFlags[state]);
+        gtk_render_background(sc, cr, rect.x - x_diff+4, rect.y+4, rect.width-8, rect.height-8);
+        gtk_render_frame(sc, cr, rect.x - x_diff+4, rect.y+4, rect.width-8, rect.height-8);
+        gtk_style_context_restore(sc);
+    }
+#else
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
     gtk_paint_box
     (
-        button->style,
+        gtk_widget_get_style(button),
         gdk_window,
         state,
         GTK_SHADOW_OUT,
@@ -212,6 +245,7 @@ wxRendererGTK::DrawHeaderButton(wxWindow *win,
         "button",
         dc.LogicalToDeviceX(rect.x) - x_diff, rect.y, rect.width, rect.height
     );
+#endif
 
     return DrawHeaderButtonContents(win, dc, rect, flags, sortArrow, params);
 }
@@ -221,11 +255,21 @@ int wxRendererGTK::GetHeaderButtonHeight(wxWindow *WXUNUSED(win))
     GtkWidget *button = wxGTKPrivate::GetHeaderButtonWidget();
 
     GtkRequisition req;
+#ifdef __WXGTK3__
+    gtk_widget_get_preferred_height(button, NULL, &req.height);
+#else
     GTK_WIDGET_GET_CLASS(button)->size_request(button, &req);
+#endif
 
     return req.height;
 }
 
+int wxRendererGTK::GetHeaderButtonMargin(wxWindow *WXUNUSED(win))
+{
+    wxFAIL_MSG( "GetHeaderButtonMargin() not implemented" );
+    return -1;
+}
+
 
 // draw a ">" or "v" button
 void
@@ -234,10 +278,6 @@ wxRendererGTK::DrawTreeItemButton(wxWindow* win,
 {
     GtkWidget *tree = wxGTKPrivate::GetTreeWidget();
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
-
     GtkStateType state;
     if ( flags & wxCONTROL_CURRENT )
         state = GTK_STATE_PRELIGHT;
@@ -248,21 +288,33 @@ wxRendererGTK::DrawTreeItemButton(wxWindow* win,
     if (win->GetLayoutDirection() == wxLayout_RightToLeft)
         x_diff = rect.width;
 
-    // VZ: I don't know how to get the size of the expander so as to centre it
-    //     in the given rectangle, +2/3 below is just what looks good here...
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        gtk_widget_set_state_flags(tree, stateTypeToFlags[state], true);
+        GtkStyleContext* sc = gtk_widget_get_style_context(tree);
+        gtk_render_expander(sc, cr, rect.x - x_diff, rect.y, rect.width, rect.height);
+    }
+#else
+    // x and y parameters specify the center of the expander
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
+    if (gdk_window == NULL)
+        return;
     gtk_paint_expander
     (
-        tree->style,
+        gtk_widget_get_style(tree),
         gdk_window,
         state,
         NULL,
         tree,
         "treeview",
-        dc.LogicalToDeviceX(rect.x) + 6 - x_diff,
-        dc.LogicalToDeviceY(rect.y) + 3,
+        dc.LogicalToDeviceX(rect.x) + rect.width / 2 - x_diff,
+        dc.LogicalToDeviceY(rect.y) + rect.height / 2,
         flags & wxCONTROL_EXPANDED ? GTK_EXPANDER_EXPANDED
                                    : GTK_EXPANDER_COLLAPSED
     );
+#endif
 }
 
 
@@ -307,21 +359,22 @@ wxRendererGTK::DrawSplitterSash(wxWindow* win,
                                 wxOrientation orient,
                                 int flags)
 {
-    if ( !win->m_wxwindow->window )
+    if (gtk_widget_get_window(win->m_wxwindow) == NULL)
     {
         // window not realized yet
         return;
     }
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
-
-    wxCoord full_size = GetGtkSplitterFullSize(wxGTKPrivate::GetSplitterWidget());
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
     // are we drawing vertical or horizontal splitter?
     const bool isVert = orient == wxVERTICAL;
 
+    GtkWidget* widget = wxGTKPrivate::GetSplitterWidget(orient);
+    const int full_size = GetGtkSplitterFullSize(widget);
+
     GdkRectangle rect;
 
     if ( isVert )
@@ -343,9 +396,21 @@ wxRendererGTK::DrawSplitterSash(wxWindow* win,
     if (win->GetLayoutDirection() == wxLayout_RightToLeft)
         x_diff = rect.width;
 
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        gtk_widget_set_state_flags(widget, stateTypeToFlags[flags & wxCONTROL_CURRENT ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL], true);
+        GtkStyleContext* sc = gtk_widget_get_style_context(widget);
+        gtk_render_handle(sc, cr, rect.x - x_diff, rect.y, rect.width, rect.height);
+    }
+#else
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
+    if (gdk_window == NULL)
+        return;
     gtk_paint_handle
     (
-        win->m_wxwindow->style,
+        gtk_widget_get_style(win->m_wxwindow),
         gdk_window,
         flags & wxCONTROL_CURRENT ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL,
         GTK_SHADOW_NONE,
@@ -358,6 +423,7 @@ wxRendererGTK::DrawSplitterSash(wxWindow* win,
         rect.height,
         isVert ? GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL
     );
+#endif
 }
 
 void
@@ -373,19 +439,12 @@ wxRendererGTK::DrawDropArrow(wxWindow* win,
     // work for wxMemoryDC. So that is why we assume wxDC
     // is wxWindowDC (wxClientDC, wxMemoryDC and wxPaintDC
     // are derived from it) and use its m_window.
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
 
     // draw arrow so that there is even space horizontally
     // on both sides
-    int arrowX = rect.width/4 + 1;
-    int arrowWidth = rect.width - (arrowX*2);
-
-    // scale arrow's height accoording to the width
-    int arrowHeight = rect.width/3;
-    int arrowY = (rect.height-arrowHeight)/2 +
-                 ((rect.height-arrowHeight) & 1);
+    const int size = rect.width / 2;
+    const int x = rect.x + (size + 1) / 2;
+    const int y = rect.y + (rect.height - size + 1) / 2;
 
     GtkStateType state;
 
@@ -398,10 +457,22 @@ wxRendererGTK::DrawDropArrow(wxWindow* win,
     else
         state = GTK_STATE_NORMAL;
 
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        gtk_widget_set_state_flags(button, stateTypeToFlags[state], true);
+        GtkStyleContext* sc = gtk_widget_get_style_context(button);
+        gtk_render_arrow(sc, cr, G_PI, x, y, size);
+    }
+#else
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
+    if (gdk_window == NULL)
+        return;
     // draw arrow on button
     gtk_paint_arrow
     (
-        button->style,
+        gtk_widget_get_style(button),
         gdk_window,
         state,
         flags & wxCONTROL_PRESSED ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
@@ -410,11 +481,10 @@ wxRendererGTK::DrawDropArrow(wxWindow* win,
         "arrow",
         GTK_ARROW_DOWN,
         FALSE,
-        rect.x + arrowX,
-        rect.y + arrowY,
-        arrowWidth,
-        arrowHeight
+        x, y,
+        size, size
     );
+#endif
 }
 
 void
@@ -448,16 +518,13 @@ wxRendererGTK::DrawCheckBox(wxWindow* win,
 {
     GtkWidget *button = wxGTKPrivate::GetCheckButtonWidget();
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
-
     gint indicator_size, indicator_spacing;
     gtk_widget_style_get(button,
                          "indicator_size", &indicator_size,
                          "indicator_spacing", &indicator_spacing,
                          NULL);
 
+#ifndef __WXGTK3__
     GtkStateType state;
 
     if ( flags & wxCONTROL_PRESSED )
@@ -469,12 +536,50 @@ wxRendererGTK::DrawCheckBox(wxWindow* win,
     else
         state = GTK_STATE_NORMAL;
 
+    GtkShadowType shadow_type;
+
+    if ( flags & wxCONTROL_UNDETERMINED )
+        shadow_type = GTK_SHADOW_ETCHED_IN;
+    else if ( flags & wxCONTROL_CHECKED )
+        shadow_type = GTK_SHADOW_IN;
+    else
+        shadow_type = GTK_SHADOW_OUT;
+#endif
+
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        int stateFlags = GTK_STATE_FLAG_NORMAL;
+        if (flags & wxCONTROL_CHECKED)
+            stateFlags = GTK_STATE_FLAG_ACTIVE;
+        if (flags & wxCONTROL_DISABLED)
+            stateFlags |= GTK_STATE_FLAG_INSENSITIVE;
+        if (flags & wxCONTROL_UNDETERMINED)
+            stateFlags |= GTK_STATE_FLAG_INCONSISTENT;
+        if (flags & wxCONTROL_CURRENT)
+            stateFlags |= GTK_STATE_FLAG_PRELIGHT;
+        GtkStyleContext* sc = gtk_widget_get_style_context(button);
+        gtk_style_context_save(sc);
+        gtk_style_context_set_state(sc, GtkStateFlags(stateFlags));
+        gtk_style_context_add_class(sc, GTK_STYLE_CLASS_CHECK);
+        gtk_render_check(sc, cr,
+            rect.x + (rect.width - indicator_size) / 2,
+            rect.y + (rect.height - indicator_size) / 2,
+            indicator_size, indicator_size);
+        gtk_style_context_restore(sc);
+    }
+#else
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
+    if (gdk_window == NULL)
+        return;
+
     gtk_paint_check
     (
-        button->style,
+        gtk_widget_get_style(button),
         gdk_window,
         state,
-        flags & wxCONTROL_CHECKED ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
+        shadow_type,
         NULL,
         button,
         "cellcheck",
@@ -482,6 +587,7 @@ wxRendererGTK::DrawCheckBox(wxWindow* win,
         dc.LogicalToDeviceY(rect.y) + indicator_spacing,
         indicator_size, indicator_size
     );
+#endif
 }
 
 void
@@ -492,10 +598,6 @@ wxRendererGTK::DrawPushButton(wxWindow* win,
 {
     GtkWidget *button = wxGTKPrivate::GetButtonWidget();
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
-
     // draw button
     GtkStateType state;
 
@@ -508,9 +610,25 @@ wxRendererGTK::DrawPushButton(wxWindow* win,
     else
         state = GTK_STATE_NORMAL;
 
+#ifdef __WXGTK3__
+    cairo_t* cr = wxGetGTKDrawable(win, dc);
+    if (cr)
+    {
+        GtkStyleContext* sc = gtk_widget_get_style_context(button);
+        gtk_style_context_save(sc);
+        gtk_style_context_set_state(sc, stateTypeToFlags[state]);
+        gtk_render_background(sc, cr, rect.x, rect.y, rect.width, rect.height);
+        gtk_render_frame(sc, cr, rect.x, rect.y, rect.width, rect.height);
+        gtk_style_context_restore(sc);
+    }
+#else
+    GdkWindow* gdk_window = wxGetGTKDrawable(win, dc);
+    if (gdk_window == NULL)
+        return;
+
     gtk_paint_box
     (
-        button->style,
+        gtk_widget_get_style(button),
         gdk_window,
         state,
         flags & wxCONTROL_PRESSED ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
@@ -522,6 +640,7 @@ wxRendererGTK::DrawPushButton(wxWindow* win,
         rect.width,
         rect.height
     );
+#endif
 }
 
 void
@@ -530,9 +649,9 @@ wxRendererGTK::DrawItemSelectionRect(wxWindow* win,
                                      const wxRect& rect,
                                      int flags )
 {
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
     if (flags & wxCONTROL_SELECTED)
     {
@@ -540,19 +659,30 @@ wxRendererGTK::DrawItemSelectionRect(wxWindow* win,
         if (win->GetLayoutDirection() == wxLayout_RightToLeft)
             x_diff = rect.width;
 
+        GtkWidget* treeWidget = wxGTKPrivate::GetTreeWidget();
+
+#ifdef __WXGTK3__
+        GtkStyleContext* sc = gtk_widget_get_style_context(treeWidget);
+        gtk_style_context_save(sc);
+        gtk_style_context_set_state(sc, GTK_STATE_FLAG_SELECTED);
+        gtk_style_context_add_class(sc, GTK_STYLE_CLASS_CELL);
+        gtk_render_background(sc, drawable, rect.x - x_diff, rect.y, rect.width, rect.height);
+        gtk_style_context_restore(sc);
+#else
         // the wxCONTROL_FOCUSED state is deduced
         // directly from the m_wxwindow by GTK+
-        gtk_paint_flat_box(wxGTKPrivate::GetTreeWidget()->style,
-                        gdk_window,
+        gtk_paint_flat_box(gtk_widget_get_style(treeWidget),
+                        drawable,
                         GTK_STATE_SELECTED,
                         GTK_SHADOW_NONE,
-                        NULL,
+                        NULL_RECT
                         win->m_wxwindow,
                         "cell_even",
                         dc.LogicalToDeviceX(rect.x) - x_diff,
                         dc.LogicalToDeviceY(rect.y),
                         rect.width,
                         rect.height );
+#endif
     }
 
     if ((flags & wxCONTROL_CURRENT) && (flags & wxCONTROL_FOCUSED))
@@ -561,9 +691,9 @@ wxRendererGTK::DrawItemSelectionRect(wxWindow* win,
 
 void wxRendererGTK::DrawFocusRect(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
 {
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
-    wxASSERT_MSG( gdk_window,
-                  wxT("cannot use wxRendererNative on wxDC of this type") );
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
     GtkStateType state;
     if (flags & wxCONTROL_SELECTED)
@@ -571,41 +701,56 @@ void wxRendererGTK::DrawFocusRect(wxWindow* win, wxDC& dc, const wxRect& rect, i
     else
         state = GTK_STATE_NORMAL;
 
-    gtk_paint_focus( win->m_widget->style,
-                     gdk_window,
+#ifdef __WXGTK3__
+    GtkStyleContext* sc = gtk_widget_get_style_context(win->m_widget);
+    gtk_style_context_save(sc);
+    gtk_style_context_set_state(sc, stateTypeToFlags[state]);
+    gtk_render_focus(sc, drawable, rect.x, rect.y, rect.width, rect.height);
+    gtk_style_context_restore(sc);
+#else
+    gtk_paint_focus( gtk_widget_get_style(win->m_widget),
+                     drawable,
                      state,
-                     NULL,
+                     NULL_RECT
                      win->m_wxwindow,
                      NULL,
                      dc.LogicalToDeviceX(rect.x),
                      dc.LogicalToDeviceY(rect.y),
                      rect.width,
                      rect.height );
+#endif
 }
 
 // Uses the theme to draw the border and fill for something like a wxTextCtrl
 void wxRendererGTK::DrawTextCtrl(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
 {
-    GtkWidget *entry = wxGTKPrivate::GetTextEntryWidget();
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
+    GtkWidget* entry = wxGTKPrivate::GetTextEntryWidget();
 
     GtkStateType state = GTK_STATE_NORMAL;
     if ( flags & wxCONTROL_DISABLED )
         state = GTK_STATE_INSENSITIVE;
 
-    if (flags & wxCONTROL_CURRENT )
-        GTK_WIDGET_SET_FLAGS( entry, GTK_HAS_FOCUS );
-    else
-        GTK_WIDGET_UNSET_FLAGS( entry, GTK_HAS_FOCUS );
+    gtk_widget_set_can_focus(entry, (flags & wxCONTROL_CURRENT) != 0);
 
+#ifdef __WXGTK3__
+    GtkStyleContext* sc = gtk_widget_get_style_context(entry);
+    gtk_style_context_save(sc);
+    gtk_style_context_set_state(sc, stateTypeToFlags[state]);
+    gtk_render_background(sc, drawable, rect.x, rect.y, rect.width, rect.height);
+    gtk_render_frame(sc, drawable, rect.x, rect.y, rect.width, rect.height);
+    gtk_style_context_restore(sc);
+#else
     gtk_paint_shadow
     (
-        entry->style,
-        gdk_window,
+        gtk_widget_get_style(entry),
+        drawable,
         state,
         GTK_SHADOW_OUT,
-        NULL,
+        NULL_RECT
         entry,
         "entry",
         dc.LogicalToDeviceX(rect.x),
@@ -613,31 +758,43 @@ void wxRendererGTK::DrawTextCtrl(wxWindow* win, wxDC& dc, const wxRect& rect, in
         rect.width,
         rect.height
   );
+#endif
 }
 
-// Draw the equivallent of a wxComboBox
+// Draw the equivalent of a wxComboBox
 void wxRendererGTK::DrawComboBox(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
 {
-    GtkWidget *combo = wxGTKPrivate::GetComboBoxWidget();
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
+    GtkWidget* combo = wxGTKPrivate::GetComboBoxWidget();
 
     GtkStateType state = GTK_STATE_NORMAL;
     if ( flags & wxCONTROL_DISABLED )
        state = GTK_STATE_INSENSITIVE;
 
-    if (flags & wxCONTROL_CURRENT )
-        GTK_WIDGET_SET_FLAGS( combo, GTK_HAS_FOCUS );
-    else
-        GTK_WIDGET_UNSET_FLAGS( combo, GTK_HAS_FOCUS );
+    gtk_widget_set_can_focus(combo, (flags & wxCONTROL_CURRENT) != 0);
 
+#ifdef __WXGTK3__
+    GtkStyleContext* sc = gtk_widget_get_style_context(combo);
+    gtk_style_context_save(sc);
+    gtk_style_context_set_state(sc, stateTypeToFlags[state]);
+    gtk_render_background(sc, drawable, rect.x, rect.y, rect.width, rect.height);
+    gtk_render_frame(sc, drawable, rect.x, rect.y, rect.width, rect.height);
+    gtk_style_context_restore(sc);
+    wxRect r = rect;
+    r.x += r.width - r.height;
+    r.width = r.height;
+    DrawComboBoxDropButton(win, dc, r, flags);
+#else
     gtk_paint_shadow
     (
-        combo->style,
-        gdk_window,
+        gtk_widget_get_style(combo),
+        drawable,
         state,
         GTK_SHADOW_OUT,
-        NULL,
+        NULL_RECT
         combo,
         "combobox",
         dc.LogicalToDeviceX(rect.x),
@@ -655,11 +812,11 @@ void wxRendererGTK::DrawComboBox(wxWindow* win, wxDC& dc, const wxRect& rect, in
 
     gtk_paint_arrow
     (
-        combo->style,
-        gdk_window,
+        gtk_widget_get_style(combo),
+        drawable,
         state,
         GTK_SHADOW_OUT,
-        NULL,
+        NULL_RECT
         combo,
         "arrow",
         GTK_ARROW_DOWN,
@@ -676,11 +833,11 @@ void wxRendererGTK::DrawComboBox(wxWindow* win, wxDC& dc, const wxRect& rect, in
 
     gtk_paint_box
     (
-        combo->style,
-        gdk_window,
+        gtk_widget_get_style(combo),
+        drawable,
         state,
         GTK_SHADOW_ETCHED_OUT,
-        NULL,
+        NULL_RECT
         combo,
         "vseparator",
         dc.LogicalToDeviceX(r.x),
@@ -688,9 +845,9 @@ void wxRendererGTK::DrawComboBox(wxWindow* win, wxDC& dc, const wxRect& rect, in
         r.width,
         r.height-2
     );
+#endif
 }
 
-
 void wxRendererGTK::DrawChoice(wxWindow* win, wxDC& dc,
                            const wxRect& rect, int flags)
 {
@@ -701,10 +858,28 @@ void wxRendererGTK::DrawChoice(wxWindow* win, wxDC& dc,
 // Draw a themed radio button
 void wxRendererGTK::DrawRadioBitmap(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
 {
-    GtkWidget *button = wxGTKPrivate::GetRadioButtonWidget();
+    wxGTKDrawable* drawable = wxGetGTKDrawable(win, dc);
+    if (drawable == NULL)
+        return;
 
-    GdkWindow* gdk_window = wxGetGdkWindowForDC(win, dc);
+    GtkWidget* button = wxGTKPrivate::GetRadioButtonWidget();
 
+#ifdef __WXGTK3__
+    int state = GTK_STATE_FLAG_NORMAL;
+    if (flags & wxCONTROL_CHECKED)
+        state = GTK_STATE_FLAG_ACTIVE;
+    else if (flags & wxCONTROL_UNDETERMINED)
+        state = GTK_STATE_FLAG_INCONSISTENT;
+    if (flags & wxCONTROL_DISABLED)
+        state |= GTK_STATE_FLAG_INSENSITIVE;
+
+    GtkStyleContext* sc = gtk_widget_get_style_context(button);
+    gtk_style_context_save(sc);
+    gtk_style_context_add_class(sc, GTK_STYLE_CLASS_RADIO);
+    gtk_style_context_set_state(sc, GtkStateFlags(state));
+    gtk_render_option(sc, drawable, rect.x, rect.y, rect.width, rect.height); 
+    gtk_style_context_restore(sc);
+#else
     GtkShadowType shadow_type = GTK_SHADOW_OUT;
     if ( flags & wxCONTROL_CHECKED )
         shadow_type = GTK_SHADOW_IN;
@@ -723,15 +898,16 @@ void wxRendererGTK::DrawRadioBitmap(wxWindow* win, wxDC& dc, const wxRect& rect,
 
     gtk_paint_option
     (
-        button->style,
-        gdk_window,
+        gtk_widget_get_style(button),
+        drawable,
         state,
         shadow_type,
-        NULL,
+        NULL_RECT
         button,
         "radiobutton",
         dc.LogicalToDeviceX(rect.x),
         dc.LogicalToDeviceY(rect.y),
         rect.width, rect.height
     );
+#endif
 }
