@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        gtk/statbox.cpp
+// Name:        src/gtk/statbox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: statbox.cpp 62789 2009-12-05 19:57:58Z PC $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -13,9 +13,10 @@
 #if wxUSE_STATBOX
 
 #include "wx/statbox.h"
-#include "wx/gtk/private/win_gtk.h"     // for wxPizza
 
 #include <gtk/gtk.h>
+#include "wx/gtk/private/gtk2-compat.h"
+#include "wx/gtk/private/win_gtk.h"
 
 // constants taken from GTK sources
 #define LABEL_PAD 1
@@ -25,31 +26,32 @@
 // "size_allocate" from m_widget
 //-----------------------------------------------------------------------------
 
+#ifndef __WXGTK3__
 extern "C" {
 static void size_allocate(GtkWidget* widget, GtkAllocation* alloc, void*)
 {
     // clip label as GTK >= 2.12 does
     GtkWidget* label_widget = gtk_frame_get_label_widget(GTK_FRAME(widget));
     int w = alloc->width -
-        2 * widget->style->xthickness - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD;
+        2 * gtk_widget_get_style(widget)->xthickness - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD;
     if (w < 0)
         w = 0;
 
-    if (label_widget->allocation.width > w)
+    GtkAllocation a;
+    gtk_widget_get_allocation(label_widget, &a);
+    if (a.width > w)
     {
-        GtkAllocation alloc2 = label_widget->allocation;
-        alloc2.width = w;
-        gtk_widget_size_allocate(label_widget, &alloc2);
+        a.width = w;
+        gtk_widget_size_allocate(label_widget, &a);
     }
 }
 }
+#endif
 
 //-----------------------------------------------------------------------------
 // wxStaticBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxStaticBox, wxControl)
-
 wxStaticBox::wxStaticBox()
 {
 }
@@ -99,11 +101,13 @@ bool wxStaticBox::Create( wxWindow *parent,
 
     gtk_frame_set_label_align(GTK_FRAME(m_widget), xalign, 0.5);
 
+#ifndef __WXGTK3__
     if (gtk_check_version(2, 12, 0))
     {
         // we connect this signal to perform label-clipping as GTK >= 2.12 does
         g_signal_connect(m_widget, "size_allocate", G_CALLBACK(size_allocate), NULL);
     }
+#endif
 
     return true;
 }
@@ -148,16 +152,14 @@ void wxStaticBox::GTKWidgetDoSetMnemonic(GtkWidget* w)
 wxVisualAttributes
 wxStaticBox::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
 {
-    return GetDefaultAttributesFromGTKWidget(gtk_frame_new);
+    return GetDefaultAttributesFromGTKWidget(gtk_frame_new(""));
 }
 
 
 void wxStaticBox::GetBordersForSizer(int *borderTop, int *borderOther) const
 {
-    const int BORDER = 5; // FIXME: hardcoded value
-
-    *borderTop = GetLabel().empty() ? 2*BORDER : GetCharHeight();
-    *borderOther = BORDER;
+    *borderTop = GetCharHeight();
+    *borderOther = GetCharWidth()/2;
 }
 
 #endif // wxUSE_STATBOX
