@@ -2,7 +2,7 @@
 // Name:        src/gtk1/notebook.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: notebook.cpp 61508 2009-07-23 20:30:22Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling, Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -100,14 +100,14 @@ static void gtk_notebook_page_change_callback(GtkNotebook *WXUNUSED(widget),
 
     if (notebook->m_skipNextPageChangeEvent)
     {
-        // this event was programatically generated by ChangeSelection() and thus must
+        // this event was programmatically generated by ChangeSelection() and thus must
         // be skipped
         notebook->m_skipNextPageChangeEvent = false;
 
         // make wxNotebook::GetSelection() return the correct (i.e. consistent
         // with wxBookCtrlEvent::GetSelection()) value even though the page is
         // not really changed in GTK+
-        notebook->m_selection = page;
+        notebook->SetSelection(page);
     }
     else
     {
@@ -121,7 +121,7 @@ static void gtk_notebook_page_change_callback(GtkNotebook *WXUNUSED(widget),
             // make wxNotebook::GetSelection() return the correct (i.e. consistent
             // with wxBookCtrlEvent::GetSelection()) value even though the page is
             // not really changed in GTK+
-            notebook->m_selection = page;
+            notebook->SetSelection(page);
 
             notebook->SendPageChangedEvent(old);
         }
@@ -227,7 +227,7 @@ static gint gtk_notebook_key_press_callback( GtkWidget *widget, GdkEventKey *gdk
     if ((gdk_event->keyval == GDK_Tab) || (gdk_event->keyval == GDK_ISO_Left_Tab))
     {
         int sel = notebook->GetSelection();
-        if (sel == -1)
+        if (sel == wxNOT_FOUND)
             return TRUE;
         wxGtkNotebookPage *nb_page = notebook->GetNotebookPage(sel);
         wxCHECK_MSG( nb_page, FALSE, wxT("invalid selection in wxNotebook") );
@@ -280,8 +280,6 @@ static void wxInsertChildInNotebook( wxNotebook* parent, wxWindow* child )
 // wxNotebook
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxNotebook,wxBookCtrlBase)
-
 BEGIN_EVENT_TABLE(wxNotebook, wxBookCtrlBase)
     EVT_NAVIGATION_KEY(wxNotebook::OnNavigationKey)
 END_EVENT_TABLE()
@@ -291,8 +289,6 @@ void wxNotebook::Init()
     m_padding = 0;
     m_inSwitchPage = false;
 
-    m_imageList = NULL;
-    m_selection = -1;
     m_themeEnabled = true;
 }
 
@@ -362,9 +358,9 @@ bool wxNotebook::Create(wxWindow *parent, wxWindowID id,
 
 int wxNotebook::GetSelection() const
 {
-    wxCHECK_MSG( m_widget != NULL, -1, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != NULL, wxNOT_FOUND, wxT("invalid notebook") );
 
-    if ( m_selection == -1 )
+    if ( m_selection == wxNOT_FOUND )
     {
         GList *nb_pages = GTK_NOTEBOOK(m_widget)->children;
 
@@ -375,8 +371,8 @@ int wxNotebook::GetSelection() const
             gpointer cur = notebook->cur_page;
             if ( cur != NULL )
             {
-                wxConstCast(this, wxNotebook)->m_selection =
-                    g_list_index( nb_pages, cur );
+                const_cast<wxNotebook *>(this)->
+                    SetSelection(g_list_index( nb_pages, cur ));
             }
         }
     }
@@ -417,7 +413,7 @@ wxGtkNotebookPage* wxNotebook::GetNotebookPage( int page ) const
 
 int wxNotebook::DoSetSelection( size_t page, int flags )
 {
-    wxCHECK_MSG( m_widget != NULL, -1, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != NULL, wxNOT_FOUND, wxT("invalid notebook") );
 
     wxCHECK_MSG( page < m_pagesData.GetCount(), -1, wxT("invalid notebook index") );
 
@@ -514,10 +510,10 @@ bool wxNotebook::SetPageImage( size_t page, int image )
     }
 
     /* Only cases 3) and 4) left */
-    wxASSERT( m_imageList != NULL ); /* Just in case */
+    wxASSERT( HasImageList() ); /* Just in case */
 
     /* Construct the new pixmap */
-    const wxBitmap *bmp = m_imageList->GetBitmapPtr(image);
+    const wxBitmap *bmp = GetImageList()->GetBitmapPtr(image);
     GdkPixmap *pixmap = bmp->GetPixmap();
     GdkBitmap *mask = NULL;
     if ( bmp->GetMask() )
@@ -597,10 +593,10 @@ bool wxNotebook::DeleteAllPages()
 
 wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
 {
-    if ( m_selection != -1 && (size_t)m_selection >= page )
+    if ( m_selection != wxNOT_FOUND && (size_t)m_selection >= page )
     {
         // the index will become invalid after the page is deleted
-        m_selection = -1;
+        m_selection = wxNOT_FOUND;
     }
 
     wxNotebookPage *client = wxNotebookBase::DoRemovePage(page);
@@ -683,9 +679,9 @@ bool wxNotebook::InsertPage( size_t position,
 
     if (imageId != -1)
     {
-        wxASSERT( m_imageList != NULL );
+        wxASSERT( HasImageList() );
 
-        const wxBitmap *bmp = m_imageList->GetBitmapPtr(imageId);
+        const wxBitmap *bmp = GetImageList()->GetBitmapPtr(imageId);
         GdkPixmap *pixmap = bmp->GetPixmap();
         GdkBitmap *mask = NULL;
         if ( bmp->GetMask() )
