@@ -2,7 +2,7 @@
 // Name:        src/gtk1/scrolbar.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: scrolbar.cpp 58227 2009-01-19 13:55:27Z VZ $
+// Id:          $Id$
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -79,7 +79,7 @@ static void gtk_scrollbar_callback( GtkAdjustment *adjust,
     win->HandleWindowEvent( event );
 
 /*
-    wxCommandEvent cevent( wxEVT_COMMAND_SCROLLBAR_UPDATED, win->GetId() );
+    wxCommandEvent cevent( wxEVT_SCROLLBAR, win->GetId() );
     cevent.SetEventObject( win );
     win->ProcessEvent( cevent );
 */
@@ -165,8 +165,6 @@ gtk_scrollbar_button_release_callback( GtkRange *WXUNUSED(widget),
 // wxScrollBar
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxScrollBar,wxControl)
-
 wxScrollBar::~wxScrollBar()
 {
 }
