@@ -2,7 +2,7 @@
 // Name:        src/html/htmltag.cpp
 // Purpose:     wxHtmlTag class (represents single tag)
 // Author:      Vaclav Slavik
-// RCS-ID:      $Id: htmltag.cpp 64656 2010-06-20 18:18:23Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -85,117 +85,131 @@ wxHtmlTagsCache::wxHtmlTagsCache(const wxString& source)
     const wxString::const_iterator end = source.end();
     for ( wxString::const_iterator pos = source.begin(); pos < end; ++pos )
     {
-        if (*pos == wxT('<'))   // tag found:
+        if (*pos != wxT('<'))
+            continue;
+
+        // possible tag start found:
+
+        // don't cache comment tags
+        if ( wxHtmlParser::SkipCommentTag(pos, end) )
+            continue;
+
+        // Remember the starting tag position.
+        wxString::const_iterator stpos = pos++;
+
+        // And look for the ending one.
+        int i;
+        for ( i = 0;
+              pos < end && i < (int)WXSIZEOF(tagBuffer) - 1 &&
+              *pos != wxT('>') && !wxIsspace(*pos);
+              ++i, ++pos )
         {
-            // don't cache comment tags
-            if ( wxHtmlParser::SkipCommentTag(pos, source.end()) )
-                continue;
+            tagBuffer[i] = (wxChar)wxToupper(*pos);
+        }
+        tagBuffer[i] = wxT('\0');
 
-            size_t tg = Cache().size();
-            Cache().push_back(wxHtmlCacheItem());
+        while (pos < end && *pos != wxT('>'))
+            ++pos;
 
-            wxString::const_iterator stpos = pos++;
-            Cache()[tg].Key = stpos;
+        if ( pos == end )
+        {
+            // We didn't find a closing bracket, this is not a valid tag after
+            // all. Notice that we need to roll back pos to avoid creating an
+            // invalid iterator when "++pos" is done in the loop statement.
+            --pos;
 
-            int i;
-            for ( i = 0;
-                  pos < end && i < (int)WXSIZEOF(tagBuffer) - 1 &&
-                  *pos != wxT('>') && !wxIsspace(*pos);
-                  ++i, ++pos )
+            continue;
+        }
+
+        // We have a valid tag, add it to the cache.
+        size_t tg = Cache().size();
+        Cache().push_back(wxHtmlCacheItem());
+        Cache()[tg].Key = stpos;
+        Cache()[tg].Name = new wxChar[i+1];
+        memcpy(Cache()[tg].Name, tagBuffer, (i+1)*sizeof(wxChar));
+
+        if ((stpos+1) < end && *(stpos+1) == wxT('/')) // ending tag:
+        {
+            Cache()[tg].type = wxHtmlCacheItem::Type_EndingTag;
+            // find matching begin tag:
+            for (i = tg; i >= 0; i--)
             {
-                tagBuffer[i] = (wxChar)wxToupper(*pos);
-            }
-            tagBuffer[i] = wxT('\0');
-
-            Cache()[tg].Name = new wxChar[i+1];
-            memcpy(Cache()[tg].Name, tagBuffer, (i+1)*sizeof(wxChar));
-
-            while (pos < end && *pos != wxT('>'))
-                ++pos;
-
-            if ((stpos+1) < end && *(stpos+1) == wxT('/')) // ending tag:
-            {
-                Cache()[tg].type = wxHtmlCacheItem::Type_EndingTag;
-                // find matching begin tag:
-                for (i = tg; i >= 0; i--)
+                if ((Cache()[i].type == wxHtmlCacheItem::Type_NoMatchingEndingTag) && (wxStrcmp(Cache()[i].Name, tagBuffer+1) == 0))
                 {
-                    if ((Cache()[i].type == wxHtmlCacheItem::Type_NoMatchingEndingTag) && (wxStrcmp(Cache()[i].Name, tagBuffer+1) == 0))
-                    {
-                        Cache()[i].type = wxHtmlCacheItem::Type_Normal;
-                        Cache()[i].End1 = stpos;
-                        Cache()[i].End2 = pos + 1;
-                        break;
-                    }
+                    Cache()[i].type = wxHtmlCacheItem::Type_Normal;
+                    Cache()[i].End1 = stpos;
+                    Cache()[i].End2 = pos + 1;
+                    break;
                 }
             }
-            else
+        }
+        else
+        {
+            Cache()[tg].type = wxHtmlCacheItem::Type_NoMatchingEndingTag;
+
+            if (wxIsCDATAElement(tagBuffer))
             {
-                Cache()[tg].type = wxHtmlCacheItem::Type_NoMatchingEndingTag;
+                // store the orig pos in case we are missing the closing
+                // tag (see below)
+                const wxString::const_iterator old_pos = pos;
+                bool foundCloseTag = false;
 
-                if (wxIsCDATAElement(tagBuffer))
+                // find next matching tag
+                int tag_len = wxStrlen(tagBuffer);
+                while (pos < end)
                 {
-                    // store the orig pos in case we are missing the closing
-                    // tag (see below)
-                    const wxString::const_iterator old_pos = pos;
-                    bool foundCloseTag = false;
+                    // find the ending tag
+                    while (pos + 1 < end &&
+                           (*pos != '<' || *(pos+1) != '/'))
+                        ++pos;
+                    if (*pos == '<')
+                        ++pos;
 
-                    // find next matching tag
-                    int tag_len = wxStrlen(tagBuffer);
-                    while (pos < end)
+                    // see if it matches
+                    int match_pos = 0;
+                    while (pos < end && match_pos < tag_len )
                     {
-                        // find the ending tag
-                        while (pos + 1 < end &&
-                               (*pos != '<' || *(pos+1) != '/'))
-                            ++pos;
-                        if (*pos == '<')
-                            ++pos;
-
-                        // see if it matches
-                        int match_pos = 0;
-                        while (pos < end && match_pos < tag_len )
-                        {
-                            wxChar c = *pos;
-                            if ( c == '>' || c == '<' )
-                                break;
-
-                            // cast to wxChar needed to suppress warning in
-                            // Unicode build
-                            if ((wxChar)wxToupper(c) == tagBuffer[match_pos])
-                            {
-                                ++match_pos;
-                            }
-                            else if (c == wxT(' ') || c == wxT('\n') ||
-                                c == wxT('\r') || c == wxT('\t'))
-                            {
-                                // need to skip over these
-                            }
-                            else
-                            {
-                                match_pos = 0;
-                            }
-                            ++pos;
-                        }
-
-                        // found a match
-                        if (match_pos == tag_len)
-                        {
-                            pos = pos - tag_len - 3;
-                            foundCloseTag = true;
+                        wxChar c = *pos;
+                        if ( c == '>' || c == '<' )
                             break;
-                        }
-                        else // keep looking for the closing tag
+
+                        // cast to wxChar needed to suppress warning in
+                        // Unicode build
+                        if ((wxChar)wxToupper(c) == tagBuffer[match_pos])
                         {
-                            ++pos;
+                            ++match_pos;
                         }
+                        else if (c == wxT(' ') || c == wxT('\n') ||
+                            c == wxT('\r') || c == wxT('\t'))
+                        {
+                            // need to skip over these
+                        }
+                        else
+                        {
+                            match_pos = 0;
+                        }
+                        ++pos;
                     }
-                    if (!foundCloseTag)
+
+                    // found a match
+                    if (match_pos == tag_len)
                     {
-                        // we didn't find closing tag; this means the markup
-                        // is incorrect and the best thing we can do is to
-                        // ignore the unclosed tag and continue parsing as if
-                        // it didn't exist:
-                        pos = old_pos;
+                        pos = pos - tag_len - 3;
+                        foundCloseTag = true;
+                        break;
                     }
+                    else // keep looking for the closing tag
+                    {
+                        ++pos;
+                    }
+                }
+                if (!foundCloseTag)
+                {
+                    // we didn't find closing tag; this means the markup
+                    // is incorrect and the best thing we can do is to
+                    // ignore the unclosed tag and continue parsing as if
+                    // it didn't exist:
+                    pos = old_pos;
                 }
             }
         }
@@ -221,7 +235,12 @@ void wxHtmlTagsCache::QueryTag(const wxString::const_iterator& at,
                                bool *hasEnding)
 {
     if (Cache().empty())
+    {
+        *end1 =
+        *end2 = inputEnd;
+        *hasEnding = true;
         return;
+    }
 
     if (Cache()[m_CachePos].Key != at)
     {
@@ -445,6 +464,7 @@ wxHtmlTag::wxHtmlTag(wxHtmlTag *parent,
         { "width",              "WIDTH"         },
         { "vertical-align",     "VALIGN"        },
         { "background",         "BGCOLOR"       },
+        { "background-color",   "BGCOLOR"       },
     };
 
     wxHtmlStyleParams styleParams(*this);
@@ -567,6 +587,38 @@ bool wxHtmlTag::GetParamAsInt(const wxString& par, int *clr) const
     return true;
 }
 
+bool
+wxHtmlTag::GetParamAsIntOrPercent(const wxString& par,
+                                  int* value,
+                                  bool& isPercent) const
+{
+    const wxString param = GetParam(par);
+    if ( param.empty() )
+        return false;
+
+    wxString num;
+    if ( param.EndsWith("%", &num) )
+    {
+        isPercent = true;
+    }
+    else
+    {
+        isPercent = false;
+        num = param;
+    }
+
+    long lValue;
+    if ( !num.ToLong(&lValue) )
+        return false;
+
+    if ( lValue > INT_MAX || lValue < INT_MIN )
+        return false;
+
+    *value = static_cast<int>(lValue);
+
+    return true;
+}
+
 wxString wxHtmlTag::GetAllParams() const
 {
     // VS: this function is for backward compatibility only,
