@@ -2,7 +2,7 @@
 // Name:        src/html/m_layout.cpp
 // Purpose:     wxHtml module for basic paragraphs/layout handling
 // Author:      Vaclav Slavik
-// RCS-ID:      $Id: m_layout.cpp 63991 2010-04-16 10:43:18Z VS $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -71,7 +71,8 @@ public:
     wxHtmlPageBreakCell() {}
 
     bool AdjustPagebreak(int* pagebreak,
-                         wxArrayInt& known_pagebreaks) const;
+                         const wxArrayInt& known_pagebreaks,
+                         int pageHeight) const;
 
     void Draw(wxDC& WXUNUSED(dc),
               int WXUNUSED(x), int WXUNUSED(y),
@@ -82,7 +83,10 @@ private:
     wxDECLARE_NO_COPY_CLASS(wxHtmlPageBreakCell);
 };
 
-bool wxHtmlPageBreakCell::AdjustPagebreak(int* pagebreak, wxArrayInt& known_pagebreaks) const
+bool
+wxHtmlPageBreakCell::AdjustPagebreak(int* pagebreak,
+                                     const wxArrayInt& known_pagebreaks,
+                                     int WXUNUSED(pageHeight)) const
 {
     // When we are counting pages, 'known_pagebreaks' is non-NULL.
     // That's the only time we change 'pagebreak'. Otherwise, pages
@@ -340,7 +344,7 @@ TAG_HANDLER_BEGIN(BODY, "BODY")
                 if ( is )
                 {
                     wxImage image(*is);
-                    if ( image.Ok() )
+                    if ( image.IsOk() )
                         winIface->SetHTMLBackgroundImage(image);
                 }
 
@@ -351,7 +355,7 @@ TAG_HANDLER_BEGIN(BODY, "BODY")
         if (tag.GetParamAsColour(wxT("BGCOLOR"), &clr))
         {
             m_WParser->GetContainer()->InsertCell(
-                new wxHtmlColourCell(clr, wxHTML_CLR_BACKGROUND));
+                new wxHtmlColourCell(clr, wxHTML_CLR_TRANSPARENT_BACKGROUND));
             winIface->SetHTMLBackgroundColour(clr);
         }
 
