@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.03.03 (replaces the old file with the same name)
-// RCS-ID:      $Id: dib.cpp 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -45,9 +45,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#if !defined(__MWERKS__)
-    #include <memory.h>
-#endif
+#include <memory.h>
 
 #include "wx/msw/dib.h"
 
@@ -137,20 +135,10 @@ bool wxDIB::Create(int width, int height, int depth)
     return true;
 }
 
-bool wxDIB::Create(const wxBitmap& bmp)
-{
-    wxCHECK_MSG( bmp.Ok(), false, wxT("wxDIB::Create(): invalid bitmap") );
-
-    if ( !Create(GetHbitmapOf(bmp)) )
-        return false;
-
-    m_hasAlpha = bmp.HasAlpha();
-
-    return true;
-}
-
 bool wxDIB::Create(HBITMAP hbmp)
 {
+    wxCHECK_MSG( hbmp, false, wxT("wxDIB::Create(): invalid bitmap") );
+
     // this bitmap could already be a DIB section in which case we don't need
     // to convert it to DIB
     DIBSECTION ds;
@@ -280,7 +268,7 @@ bool wxDIB::Load(const wxString& filename)
     m_handle = (HBITMAP)::LoadImage
                          (
                             wxGetInstance(),
-                            filename.fn_str(),
+                            filename.t_str(),
                             IMAGE_BITMAP,
                             0, 0, // don't specify the size
                             LR_CREATEDIBSECTION | LR_LOADFROMFILE
@@ -587,7 +575,7 @@ HGLOBAL wxDIB::ConvertFromBitmap(HBITMAP hbmp)
 wxPalette *wxDIB::CreatePalette() const
 {
     // GetDIBColorTable not available in eVC3
-#if defined(_WIN32_WCE) && _WIN32_WCE < 400
+#if !defined(__WXMSW__) || defined(_WIN32_WCE) && _WIN32_WCE < 400
     return NULL;
 #else
     wxCHECK_MSG( m_handle, NULL, wxT("wxDIB::CreatePalette(): invalid object") );
@@ -668,17 +656,17 @@ wxPalette *wxDIB::CreatePalette() const
 
 #if wxUSE_IMAGE
 
-bool wxDIB::Create(const wxImage& image)
+bool wxDIB::Create(const wxImage& image, PixelFormat pf)
 {
-    wxCHECK_MSG( image.Ok(), false, wxT("invalid wxImage in wxDIB ctor") );
+    wxCHECK_MSG( image.IsOk(), false, wxT("invalid wxImage in wxDIB ctor") );
 
     const int h = image.GetHeight();
     const int w = image.GetWidth();
 
     // if we have alpha channel, we need to create a 32bpp RGBA DIB, otherwise
     // a 24bpp RGB is sufficient
-    m_hasAlpha = image.HasAlpha();
-    const int bpp = m_hasAlpha ? 32 : 24;
+    const bool hasAlpha = image.HasAlpha();
+    const int bpp = hasAlpha ? 32 : 24;
 
     if ( !Create(w, h, bpp) )
         return false;
@@ -688,32 +676,53 @@ bool wxDIB::Create(const wxImage& image)
     const int srcBytesPerLine = w * 3;
     const int dstBytesPerLine = GetLineSize(w, bpp);
     const unsigned char *src = image.GetData() + ((h - 1) * srcBytesPerLine);
-    const unsigned char *alpha = m_hasAlpha ? image.GetAlpha() + (h - 1)*w
-                                            : NULL;
+    const unsigned char *alpha = hasAlpha ? image.GetAlpha() + (h - 1)*w
+                                          : NULL;
     unsigned char *dstLineStart = (unsigned char *)m_data;
     for ( int y = 0; y < h; y++ )
     {
-        // copy one DIB line
+        // Copy one DIB line. Note that RGB components order is reversed in
+        // Windows bitmaps compared to wxImage and is actually BGR.
         unsigned char *dst = dstLineStart;
         if ( alpha )
         {
-            for ( int x = 0; x < w; x++ )
+            int x;
+
+            switch ( pf )
             {
-                // RGB order is reversed, and we need to premultiply
-                // all channels by alpha value for use with ::AlphaBlend.
-                const unsigned char a = *alpha++;
-                *dst++ = (unsigned char)((src[2] * a + 127) / 255);
-                *dst++ = (unsigned char)((src[1] * a + 127) / 255);
-                *dst++ = (unsigned char)((src[0] * a + 127) / 255);
-                *dst++ = a;
-                src += 3;
+                case PixelFormat_PreMultiplied:
+                    // Pre-multiply pixel values so that the DIB could be used
+                    // with ::AlphaBlend().
+                    for ( x = 0; x < w; x++ )
+                    {
+                        const unsigned char a = *alpha++;
+                        *dst++ = (unsigned char)((src[2] * a + 127) / 255);
+                        *dst++ = (unsigned char)((src[1] * a + 127) / 255);
+                        *dst++ = (unsigned char)((src[0] * a + 127) / 255);
+                        *dst++ = a;
+                        src += 3;
+                    }
+                    break;
+
+                case PixelFormat_NotPreMultiplied:
+                    // Just copy pixel data without changing it.
+                    for ( x = 0; x < w; x++ )
+                    {
+                        *dst++ = src[2];
+                        *dst++ = src[1];
+                        *dst++ = src[0];
+
+                        *dst++ = *alpha++;
+                        src += 3;
+                    }
+                    break;
             }
+
         }
         else // no alpha channel
         {
             for ( int x = 0; x < w; x++ )
             {
-                // RGB order is reversed.
                 *dst++ = src[2];
                 *dst++ = src[1];
                 *dst++ = src[0];
@@ -742,26 +751,36 @@ wxImage wxDIB::ConvertToImage() const
     const int w = GetWidth();
     const int h = GetHeight();
     wxImage image(w, h, false /* don't bother clearing memory */);
-    if ( !image.Ok() )
+    if ( !image.IsOk() )
     {
         wxFAIL_MSG( wxT("could not allocate data for image") );
         return wxNullImage;
     }
 
-    if ( m_hasAlpha )
+    const int bpp = GetDepth();
+
+    // Remember if we have any "real" transparency, i.e. either any partially
+    // transparent pixels or not all pixels are fully opaque or fully
+    // transparent.
+    bool hasAlpha = false;
+    bool hasOpaque = false;
+    bool hasTransparent = false;
+
+    if ( bpp == 32 )
     {
+        // 32 bit bitmaps may be either 0RGB or ARGB and we don't know in
+        // advance which one do we have so suppose we have alpha of them and
+        // get rid of it later if it turns out we didn't.
         image.SetAlpha();
     }
 
     // this is the same loop as in Create() just above but with copy direction
     // reversed
-    const int bpp = GetDepth();
     const int dstBytesPerLine = w * 3;
     const int srcBytesPerLine = GetLineSize(w, bpp);
     unsigned char *dst = image.GetData() + ((h - 1) * dstBytesPerLine);
     unsigned char *alpha = image.HasAlpha() ? image.GetAlpha() + (h - 1)*w
                                             : NULL;
-    const bool is32bit = bpp == 32;
     const unsigned char *srcLineStart = (unsigned char *)GetData();
     for ( int y = 0; y < h; y++ )
     {
@@ -773,20 +792,36 @@ wxImage wxDIB::ConvertToImage() const
             dst[1] = *src++;
             dst[0] = *src++;
 
-            if ( is32bit )
+            if ( bpp == 32 )
             {
-                if ( alpha )
+                // wxImage uses non premultiplied alpha so undo
+                // premultiplication done in Create() above
+                const unsigned char a = *src;
+                *alpha++ = a;
+
+                // Check what kind of alpha do we have.
+                switch ( a )
                 {
-                    // wxImage uses non premultiplied alpha so undo
-                    // premultiplication done in Create() above
-                    const unsigned char a = *src;
-                    *alpha++ = a;
-                    if ( a > 0 )
-                    {
-                        dst[0] = (dst[0] * 255) / a;
-                        dst[1] = (dst[1] * 255) / a;
-                        dst[2] = (dst[2] * 255) / a;
-                    }
+                    case 0:
+                        hasTransparent = true;
+                        break;
+
+                    default:
+                        // Anything in between means we have real transparency
+                        // and must use alpha channel.
+                        hasAlpha = true;
+                        break;
+
+                    case 255:
+                        hasOpaque = true;
+                        break;
+                }
+
+                if ( a > 0 )
+                {
+                    dst[0] = (dst[0] * 255) / a;
+                    dst[1] = (dst[1] * 255) / a;
+                    dst[2] = (dst[2] * 255) / a;
                 }
 
                 src++;
@@ -804,6 +839,12 @@ wxImage wxDIB::ConvertToImage() const
         srcLineStart += srcBytesPerLine;
     }
 
+    if ( hasOpaque && hasTransparent )
+        hasAlpha = true;
+
+    if ( !hasAlpha && image.HasAlpha() )
+        image.ClearAlpha();
+
     return image;
 }
 
