@@ -4,7 +4,7 @@
 // Author:      Ryan Norton <wxprojects@comcast.net>
 // Modified by:
 // Created:     11/07/04
-// RCS-ID:      $Id: mediactrl_wmp10.cpp 61508 2009-07-23 20:30:22Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Ryan Norton
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -28,7 +28,7 @@
 // is about to be played - and if the user didn't change the state of the
 // media (m_bWasStateChanged), when set it back to the stop state.
 //
-// The ActiveX control itself is particularily stubborn, calling
+// The ActiveX control itself is particularly stubborn, calling
 // IOleInPlaceSite::OnPosRectChange every file change trying to set itself
 // to something different then what we told it to before.
 //
@@ -724,7 +724,9 @@ wxWMP10MediaBackend::wxWMP10MediaBackend()
 #ifndef WXTEST_ATL
                 m_pAX(NULL),
 #endif
-                m_pWMPPlayer(NULL)
+                m_pWMPPlayer(NULL),
+                m_pWMPSettings(NULL),
+                m_pWMPControls(NULL)
 
 {
     m_evthandler = NULL;
@@ -752,8 +754,10 @@ wxWMP10MediaBackend::~wxWMP10MediaBackend()
 #endif
 
         m_pWMPPlayer->Release();
-        m_pWMPSettings->Release();
-        m_pWMPControls->Release();
+        if (m_pWMPSettings)
+            m_pWMPSettings->Release();
+        if (m_pWMPControls)
+            m_pWMPControls->Release();
     }
 }
 
@@ -857,7 +861,7 @@ bool wxWMP10MediaBackend::CreateControl(wxControl* ctrl, wxWindow* parent,
     IWMPPlayer2* pWMPPlayer2; // Only 2 has windowless video and stretchtofit
     if(m_pWMPPlayer->QueryInterface(IID_IWMPPlayer2, (void**)&pWMPPlayer2) == 0)
     {
-        // We don't check errors here as these arn't particularily important
+        // We don't check errors here as these arn't particularly important
         // and may not be implemented (i.e. stretchToFit on CE)
         pWMPPlayer2->put_windowlessVideo(VARIANT_TRUE);
         pWMPPlayer2->put_stretchToFit(VARIANT_TRUE);
@@ -1452,9 +1456,11 @@ void wxWMP10MediaEvtHandler::OnActiveX(wxActiveXEvent& event)
 }
 
 #endif
-// in source file that contains stuff you don't directly use
-#include "wx/html/forcelnk.h"
-FORCE_LINK_ME(wxmediabackend_wmp10)
+
+// Allow the user code to use wxFORCE_LINK_MODULE() to ensure that this object
+// file is not discarded by the linker.
+#include "wx/link.h"
+wxFORCE_LINK_THIS_MODULE(wxmediabackend_wmp10)
 
 #if 0 // Windows Media Player Mobile 9 hacks
 
@@ -1466,7 +1472,7 @@ FORCE_LINK_ME(wxmediabackend_wmp10)
 // you playing a file through WMP.
 //
 // Here are those hacks. They do indeed "work" as expected - just call
-// SendMessage with one of those myterious values layed out in
+// SendMessage with one of those myterious values laid out in
 // Peter Foot's Friday, May 21, 2004 Blog Post on the issue.
 // (He says they are in a registery section entitled "Pendant Bus")
 //
