@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        minifram.cpp
+// Name:        src/msw/minifram.cpp
 // Purpose:     wxMiniFrame
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: minifram.cpp 35650 2005-09-23 12:56:45Z MR $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,10 +16,10 @@
 #pragma hdrstop
 #endif
 
-#ifndef WX_PRECOMP
-#endif
+#if wxUSE_MINIFRAME
 
 #include "wx/minifram.h"
 
 IMPLEMENT_DYNAMIC_CLASS(wxMiniFrame, wxFrame)
 
+#endif // wxUSE_MINIFRAME
