@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     20.07.2003
-// RCS-ID:      $Id: renderer.cpp 64940 2010-07-13 13:29:13Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -31,6 +31,7 @@
     #include "wx/settings.h"
 #endif //WX_PRECOMP
 
+#include "wx/dcgraph.h"
 #include "wx/scopeguard.h"
 #include "wx/splitter.h"
 #include "wx/renderer.h"
@@ -133,6 +134,26 @@ public:
                                 wxDC& dc,
                                 const wxRect& rect,
                                 int flags = 0);
+
+    void DrawChoice(wxWindow* win,
+                     wxDC& dc,
+                     const wxRect& rect,
+                     int flags = 0);
+
+    void DrawComboBox(wxWindow* win,
+                       wxDC& dc,
+                       const wxRect& rect,
+                       int flags = 0);
+
+    virtual void DrawComboBoxDropButton(wxWindow *win,
+                                         wxDC& dc,
+                                         const wxRect& rect,
+                                         int flags = 0) = 0;
+
+    virtual void DrawTextCtrl(wxWindow* win,
+                               wxDC& dc,
+                               const wxRect& rect,
+                               int flags = 0) = 0;
 };
 
 // ----------------------------------------------------------------------------
@@ -164,16 +185,6 @@ public:
                                 const wxRect& rect,
                                 int flags = 0);
 
-    virtual void DrawChoice(wxWindow* win,
-                            wxDC& dc,
-                            const wxRect& rect,
-                            int flags = 0);
-
-    virtual void DrawComboBox(wxWindow* win,
-                              wxDC& dc,
-                              const wxRect& rect,
-                              int flags = 0);
-
     virtual void DrawTextCtrl(wxWindow* win,
                               wxDC& dc,
                               const wxRect& rect,
@@ -197,6 +208,8 @@ public:
 
     virtual int GetHeaderButtonHeight(wxWindow *win);
 
+    virtual int GetHeaderButtonMargin(wxWindow *win);
+
 private:
     // wrapper of DrawFrameControl()
     void DoDrawFrameControl(UINT type,
@@ -276,6 +289,11 @@ public:
             m_rendererNative.DrawPushButton(win, dc, rect, flags);
     }
 
+    virtual void DrawTextCtrl(wxWindow* win,
+                              wxDC& dc,
+                              const wxRect& rect,
+                              int flags = 0);
+
     virtual void DrawRadioBitmap(wxWindow *win,
                                  wxDC& dc,
                                  const wxRect& rect,
@@ -361,6 +379,31 @@ void wxRendererMSWBase::DrawItemSelectionRect(wxWindow *win,
         DrawFocusRect( win, dc, rect, flags );
 }
 
+void wxRendererMSWBase::DrawChoice(wxWindow* win,
+                                   wxDC& dc,
+                                   const wxRect& rect,
+                                   int flags)
+{
+    DrawComboBox(win, dc, rect, flags);
+}
+
+void wxRendererMSWBase::DrawComboBox(wxWindow* win,
+                                     wxDC& dc,
+                                     const wxRect& rect,
+                                     int flags)
+{
+    // Draw the main part of the control same as TextCtrl
+    DrawTextCtrl(win, dc, rect, flags);
+
+    // Draw the button inside the border, on the right side
+    wxRect br(rect);
+    br.height -= 2;
+    br.x += br.width - br.height - 1;
+    br.width = br.height;
+    br.y += 1;
+
+    DrawComboBoxDropButton(win, dc, br, flags);
+}
 
 // ============================================================================
 // wxRendererNative and wxRendererMSW implementation
@@ -392,8 +435,12 @@ wxRendererMSW::DrawComboBoxDropButton(wxWindow * WXUNUSED(win),
                                       const wxRect& rect,
                                       int flags)
 {
+    wxCHECK_RET( dc.GetImpl(), wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+    
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     int style = DFCS_SCROLLCOMBOBOX;
     if ( flags & wxCONTROL_DISABLED )
@@ -412,8 +459,12 @@ wxRendererMSW::DoDrawFrameControl(UINT type,
                                   const wxRect& rect,
                                   int flags)
 {
+    wxCHECK_RET( dc.GetImpl(), wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     int style = kind;
     if ( flags & wxCONTROL_CHECKED )
@@ -426,6 +477,11 @@ wxRendererMSW::DoDrawFrameControl(UINT type,
         style |= DFCS_PUSHED;
     if ( flags & wxCONTROL_CURRENT )
         style |= DFCS_HOT;
+    if ( flags & wxCONTROL_UNDETERMINED )
+        // Using DFCS_BUTTON3STATE here doesn't work (as might be expected),
+        // use the following two styles to get the same look of a check box
+        // in the undetermined state.
+        style |= DFCS_INACTIVE | DFCS_CHECKED;
 
     ::DrawFrameControl(GetHdcOf(dc.GetTempHDC()), &r, type, style);
 }
@@ -517,67 +573,30 @@ int wxRendererMSW::GetHeaderButtonHeight(wxWindow * WXUNUSED(win))
     return Header_Layout(hwndHeader, &hdl) ? wp.cy : DEFAULT_HEIGHT;
 }
 
+int wxRendererMSW::GetHeaderButtonMargin(wxWindow *WXUNUSED(win))
+{
+    return 10;
+}
+
 // Uses the theme to draw the border and fill for something like a wxTextCtrl
-void wxRendererMSW::DrawTextCtrl(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
+void wxRendererMSW::DrawTextCtrl(wxWindow* WXUNUSED(win),
+                                 wxDC& dc,
+                                 const wxRect& rect,
+                                 int WXUNUSED(flags))
 {
     wxColour fill;
     wxColour bdr;
-    COLORREF cref;
-
-#if wxUSE_UXTHEME
-    wxUxThemeHandle hTheme(win, L"EDIT");
-    if (hTheme)
-    {
-        wxUxThemeEngine::Get()->GetThemeColor(hTheme, EP_EDITTEXT,
-                                              ETS_NORMAL, TMT_FILLCOLOR, &cref);
-        fill = wxRGBToColour(cref);
-
-        int etsState;
-        if ( flags & wxCONTROL_DISABLED )
-            etsState = ETS_DISABLED;
-        else
-            etsState = ETS_NORMAL;
-
-        wxUxThemeEngine::Get()->GetThemeColor(hTheme, EP_EDITTEXT,
-                                              etsState, TMT_BORDERCOLOR, &cref);
-        bdr = wxRGBToColour(cref);
-    }
-    else
-#endif
     {
         fill = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW);
         bdr = *wxBLACK;
     }
 
-    dc.SetPen( bdr );
-    dc.SetBrush( fill );
+    dc.SetPen(bdr);
+    dc.SetBrush(fill);
     dc.DrawRectangle(rect);
 }
 
 
-// Draw the equivalent of a wxComboBox
-void wxRendererMSW::DrawComboBox(wxWindow* win, wxDC& dc, const wxRect& rect, int flags)
-{
-    // Draw the main part of the control same as TextCtrl
-    DrawTextCtrl(win, dc, rect, flags);
-
-    // Draw the button inside the border, on the right side
-    wxRect br(rect);
-    br.height -= 2;
-    br.x += br.width - br.height - 1;
-    br.width = br.height;
-    br.y += 1;
-
-    DrawComboBoxDropButton(win, dc, br, flags);
-}
-
-
-void wxRendererMSW::DrawChoice(wxWindow* win, wxDC& dc,
-                           const wxRect& rect, int flags)
-{
-    DrawComboBox(win, dc, rect, flags);
-}
-
 // ============================================================================
 // wxRendererXP implementation
 // ============================================================================
@@ -608,8 +627,12 @@ wxRendererXP::DrawComboBoxDropButton(wxWindow * win,
         return;
     }
 
+    wxCHECK_RET( dc.GetImpl(), wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     int state;
     if ( flags & wxCONTROL_PRESSED )
@@ -647,8 +670,12 @@ wxRendererXP::DrawHeaderButton(wxWindow *win,
         return m_rendererNative.DrawHeaderButton(win, dc, rect, flags, sortArrow, params);
     }
 
+    wxCHECK_MSG( dc.GetImpl(), -1, wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     int state;
     if ( flags & wxCONTROL_PRESSED )
@@ -689,8 +716,12 @@ wxRendererXP::DrawTreeItemButton(wxWindow *win,
         return;
     }
 
+    wxCHECK_RET( dc.GetImpl(), wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     int state = flags & wxCONTROL_EXPANDED ? GLPS_OPENED : GLPS_CLOSED;
     wxUxThemeEngine::Get()->DrawThemeBackground
@@ -727,8 +758,12 @@ wxRendererXP::DoDrawButtonLike(HTHEME htheme,
                                const wxRect& rect,
                                int flags)
 {
+    wxCHECK_RET( dc.GetImpl(), wxT("Invalid wxDC") );
+
+    wxRect adjustedRect = dc.GetImpl()->MSWApplyGDIPlusTransform(rect);
+
     RECT r;
-    wxCopyRectToRECT(rect, r);
+    wxCopyRectToRECT(adjustedRect, r);
 
     // the base state is always 1, whether it is PBS_NORMAL,
     // {CBS,RBS}_UNCHECKEDNORMAL or CBS_NORMAL
@@ -818,6 +853,42 @@ wxRendererXP::DrawTitleBarBitmap(wxWindow *win,
     DoDrawButtonLike(hTheme, part, dc, rect, flags);
 }
 
+// Uses the theme to draw the border and fill for something like a wxTextCtrl
+void wxRendererXP::DrawTextCtrl(wxWindow* win,
+                                wxDC& dc,
+                                const wxRect& rect,
+                                int flags)
+{
+    wxUxThemeHandle hTheme(win, L"EDIT");
+    if ( !hTheme )
+    {
+        m_rendererNative.DrawTextCtrl(win,dc,rect,flags);
+        return;
+    }
+
+    wxColour fill;
+    wxColour bdr;
+    COLORREF cref;
+
+    wxUxThemeEngine::Get()->GetThemeColor(hTheme, EP_EDITTEXT,
+                                          ETS_NORMAL, TMT_FILLCOLOR, &cref);
+    fill = wxRGBToColour(cref);
+
+    int etsState;
+    if ( flags & wxCONTROL_DISABLED )
+        etsState = ETS_DISABLED;
+    else
+        etsState = ETS_NORMAL;
+
+    wxUxThemeEngine::Get()->GetThemeColor(hTheme, EP_EDITTEXT,
+                                              etsState, TMT_BORDERCOLOR, &cref);
+    bdr = wxRGBToColour(cref);
+
+    dc.SetPen( bdr );
+    dc.SetBrush( fill );
+    dc.DrawRectangle(rect);
+}
+
 // ----------------------------------------------------------------------------
 // splitter drawing
 // ----------------------------------------------------------------------------
