@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     04.04.98
-// RCS-ID:      $Id: statusbar.cpp 64656 2010-06-20 18:18:23Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -84,29 +84,15 @@ wxStatusBar::wxStatusBar()
     m_pDC = NULL;
 }
 
-bool wxStatusBar::Create(wxWindow *parent,
-                         wxWindowID id,
-                         long style,
-                         const wxString& name)
+WXDWORD wxStatusBar::MSWGetStyle(long style, WXDWORD *exstyle) const
 {
-    wxCHECK_MSG( parent, false, "status bar must have a parent" );
-
-    SetName(name);
-    SetWindowStyleFlag(style);
-    SetParent(parent);
-
-    parent->AddChild(this);
-
-    m_windowId = id == wxID_ANY ? NewControlId() : id;
-
-    DWORD wstyle = WS_CHILD | WS_VISIBLE;
-
-    if ( style & wxCLIP_SIBLINGS )
-        wstyle |= WS_CLIPSIBLINGS;
+    WXDWORD msStyle = wxStatusBarBase::MSWGetStyle(style, exstyle);
 
     // wxSTB_SIZEGRIP is part of our default style but it doesn't make sense to
-    // show size grip if this is the status bar of a non-resizeable TLW so turn
+    // show size grip if this is the status bar of a non-resizable TLW so turn
     // it off in such case
+    wxWindow * const parent = GetParent();
+    wxCHECK_MSG( parent, msStyle, wxS("Status bar must have a parent") );
     if ( parent->IsTopLevel() && !parent->HasFlag(wxRESIZE_BORDER) )
         style &= ~wxSTB_SIZEGRIP;
 
@@ -117,47 +103,38 @@ bool wxStatusBar::Create(wxWindow *parent,
     // is not given
     if ( !(style & wxSTB_SIZEGRIP) )
     {
-        wstyle |= CCS_TOP;
+        msStyle |= CCS_TOP;
     }
     else
     {
 #ifndef __WXWINCE__
         // may be some versions of comctl32.dll do need it - anyhow, it won't
         // do any harm
-        wstyle |= SBARS_SIZEGRIP;
+       msStyle |= SBARS_SIZEGRIP;
 #endif
     }
 
-    m_hWnd = CreateWindow
-             (
-                STATUSCLASSNAME,
-                wxT(""),
-                wstyle,
-                0, 0, 0, 0,
-                GetHwndOf(parent),
-                (HMENU)wxUIntToPtr(m_windowId.GetValue()),
-                wxGetInstance(),
-                NULL
-             );
-    if ( m_hWnd == 0 )
-    {
-        wxLogSysError(_("Failed to create a status bar."));
+    return msStyle;
+}
 
+bool wxStatusBar::Create(wxWindow *parent,
+                         wxWindowID id,
+                         long style,
+                         const wxString& name)
+{
+    if ( !CreateControl(parent, id, wxDefaultPosition, wxDefaultSize,
+                        style, wxDefaultValidator, name) )
+        return false;
+
+    if ( !MSWCreateControl(STATUSCLASSNAME, wxString(),
+                           wxDefaultPosition, wxDefaultSize) )
         return false;
-    }
 
     SetFieldsCount(1);
-    SubclassWin(m_hWnd);
 
     // cache the DC instance used by DoUpdateStatusText:
-    // NOTE: create the DC before calling InheritAttributes() since
-    //       it may result in a call to our SetFont()
     m_pDC = new wxClientDC(this);
 
-    InheritAttributes();
-
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_MENUBAR));
-
     // we must refresh the frame size when the statusbar is created, because
     // its client area might change
     //
@@ -176,11 +153,13 @@ wxStatusBar::~wxStatusBar()
     // occupy
     PostSizeEventToParent();
 
+#if wxUSE_TOOLTIPS
     // delete existing tooltips
     for (size_t i=0; i<m_tooltips.size(); i++)
     {
         wxDELETE(m_tooltips[i]);
     }
+#endif // wxUSE_TOOLTIPS
 
     wxDELETE(m_pDC);
 }
@@ -199,12 +178,9 @@ void wxStatusBar::SetFieldsCount(int nFields, const int *widths)
     // this is a Windows limitation
     wxASSERT_MSG( (nFields > 0) && (nFields < 255), "too many fields" );
 
-    wxStatusBarBase::SetFieldsCount(nFields, widths);
-
-    MSWUpdateFieldsWidths();
-
     // keep in synch also our m_tooltips array
 
+#if wxUSE_TOOLTIPS
     // reset all current tooltips
     for (size_t i=0; i<m_tooltips.size(); i++)
     {
@@ -212,7 +188,12 @@ void wxStatusBar::SetFieldsCount(int nFields, const int *widths)
     }
 
     // shrink/expand the array:
-    m_tooltips.resize(m_panes.GetCount(), NULL);
+    m_tooltips.resize(nFields, NULL);
+#endif // wxUSE_TOOLTIPS
+
+    wxStatusBarBase::SetFieldsCount(nFields, widths);
+
+    MSWUpdateFieldsWidths();
 }
 
 void wxStatusBar::SetStatusWidths(int n, const int widths[])
@@ -236,8 +217,11 @@ void wxStatusBar::MSWUpdateFieldsWidths()
     widthAvailable -= extraWidth*(count - 1);   // extra space between fields
     widthAvailable -= MSWGetMetrics().textMargin;   // and for the last field
 
-    if ( HasFlag(wxSTB_SIZEGRIP) )
-        widthAvailable -= MSWGetMetrics().gripWidth;
+    // Deal with the grip: we shouldn't overflow onto the space occupied by it
+    // so the effectively available space is smaller.
+    const int gripWidth = HasFlag(wxSTB_SIZEGRIP) ? MSWGetMetrics().gripWidth
+                                                  : 0;
+    widthAvailable -= gripWidth;
 
     // distribute the available space (client width) among the various fields:
 
@@ -249,21 +233,30 @@ void wxStatusBar::MSWUpdateFieldsWidths()
     int *pWidths = new int[count];
 
     int nCurPos = 0;
-    for ( int i = 0; i < count; i++ )
+    int i;
+    for ( i = 0; i < count; i++ )
     {
         nCurPos += widthsAbs[i] + extraWidth;
         pWidths[i] = nCurPos;
     }
 
+    // The total width of the panes passed to Windows must be equal to the
+    // total width available, including the grip. Otherwise we get an extra
+    // separator line just before it.
+    pWidths[count - 1] += gripWidth;
+
     if ( !StatusBar_SetParts(GetHwnd(), count, pWidths) )
     {
         wxLogLastError("StatusBar_SetParts");
     }
 
+    // Now that all parts have been created, set their text.
+    for ( i = 0; i < count; i++ )
+    {
+        DoUpdateStatusText(i);
+    }
+
     delete [] pWidths;
-
-
-    // FIXME: we may want to call DoUpdateStatusText() here since we may need to (de)ellipsize status texts
 }
 
 void wxStatusBar::DoUpdateStatusText(int nField)
@@ -282,6 +275,7 @@ void wxStatusBar::DoUpdateStatusText(int nField)
         style = SBT_NOBORDERS;
         break;
 
+    case wxSB_SUNKEN:
     case wxSB_NORMAL:
     default:
         style = 0;
@@ -324,11 +318,12 @@ void wxStatusBar::DoUpdateStatusText(int nField)
 
     // Set the status text in the native control passing both field number and style.
     // NOTE: MSDN library doesn't mention that nField and style have to be 'ORed'
-    if ( !StatusBar_SetText(GetHwnd(), nField | style, text.wx_str()) )
+    if ( !StatusBar_SetText(GetHwnd(), nField | style, text.t_str()) )
     {
         wxLogLastError("StatusBar_SetText");
     }
 
+#if wxUSE_TOOLTIPS
     if (HasFlag(wxSTB_SHOW_TIPS))
     {
         wxASSERT(m_tooltips.size() == m_panes.GetCount());
@@ -357,6 +352,7 @@ void wxStatusBar::DoUpdateStatusText(int nField)
             //else: leave m_tooltips[nField]==NULL
         }
     }
+#endif // wxUSE_TOOLTIPS
 }
 
 wxStatusBar::MSWBorders wxStatusBar::MSWGetBorders() const
@@ -389,7 +385,7 @@ int wxStatusBar::MSWGetBorderWidth() const
 /* static */
 const wxStatusBar::MSWMetrics& wxStatusBar::MSWGetMetrics()
 {
-    static MSWMetrics s_metrics = { 0 };
+    static MSWMetrics s_metrics = { 0, 0 };
     if ( !s_metrics.textMargin )
     {
         // Grip size should be self explanatory (the only problem with it is
@@ -399,12 +395,14 @@ const wxStatusBar::MSWMetrics& wxStatusBar::MSWGetMetrics()
         // into account to make sure the text drawn by user fits inside the
         // pane. Notice that it's not the value returned by SB_GETBORDERS
         // which, at least on this Windows 2003 system, returns {0, 2, 2}
+#if wxUSE_UXTHEME
         if ( wxUxThemeEngine::GetIfActive() )
         {
             s_metrics.gripWidth = 20;
             s_metrics.textMargin = 8;
         }
         else // classic/unthemed look
+#endif // wxUSE_UXTHEME
         {
             s_metrics.gripWidth = 18;
             s_metrics.textMargin = 4;
@@ -416,7 +414,18 @@ const wxStatusBar::MSWMetrics& wxStatusBar::MSWGetMetrics()
 
 void wxStatusBar::SetMinHeight(int height)
 {
-    SendMessage(GetHwnd(), SB_SETMINHEIGHT, height + 2*GetBorderY(), 0);
+    // It looks like we need to count the border twice to really make the
+    // controls taking exactly height pixels fully fit in the status bar:
+    // at least under Windows 7 the checkbox in the custom status bar of the
+    // statbar sample gets truncated otherwise.
+    height += 4*GetBorderY();
+
+    // We need to set the size and not the size to reflect the height because
+    // wxFrame uses our size and not the minimal size as it assumes that the
+    // size of a status bar never changes anyhow.
+    SetSize(-1, height);
+
+    SendMessage(GetHwnd(), SB_SETMINHEIGHT, height, 0);
 
     // we have to send a (dummy) WM_SIZE to redraw it now
     SendMessage(GetHwnd(), WM_SIZE, 0, 0);
@@ -452,14 +461,6 @@ bool wxStatusBar::GetFieldRect(int i, wxRect& rect) const
 
     wxCopyRECTToRect(r, rect);
 
-    // Windows seems to under-report the size of the last field rectangle,
-    // presumably in order to prevent the buggy applications from overflowing
-    // onto the size grip but we want to return the real size to wx users
-    if ( HasFlag(wxSTB_SIZEGRIP) && i == (int)m_panes.GetCount() - 1 )
-    {
-        rect.width += MSWGetMetrics().gripWidth - MSWGetBorderWidth();
-    }
-
     return true;
 }
 
@@ -494,11 +495,10 @@ wxSize wxStatusBar::DoGetBestSize() const
         width = 2*DEFAULT_FIELD_WIDTH;
     }
 
-    // calculate height
-    int height;
-    wxGetCharSize(GetHWND(), NULL, &height, GetFont());
-    height = EDIT_HEIGHT_FROM_CHAR_HEIGHT(height);
-    height += borders.vert;
+    // calculate height: by default it should be just big enough to show text
+    // (see SetMinHeight() for the explanation of 4 factor)
+    int height = GetCharHeight();
+    height += 4*borders.vert;
 
     wxSize best(width, height);
     CacheBestSize(best);
@@ -525,9 +525,6 @@ void wxStatusBar::DoMoveWindow(int x, int y, int width, int height)
                        );
     }
 
-    // adjust fields widths to the new size
-    MSWUpdateFieldsWidths();
-
     // we have to trigger wxSizeEvent if there are children window in status
     // bar because GetFieldRect returned incorrect (not updated) values up to
     // here, which almost certainly resulted in incorrectly redrawn statusbar
@@ -557,6 +554,7 @@ void wxStatusBar::SetStatusStyles(int n, const int styles[])
         case wxSB_FLAT:
             style = SBT_NOBORDERS;
             break;
+        case wxSB_SUNKEN:
         case wxSB_NORMAL:
         default:
             style = 0;
@@ -567,7 +565,7 @@ void wxStatusBar::SetStatusStyles(int n, const int styles[])
         // the fields' styles.
         // NOTE: MSDN library doesn't mention that nField and style have to be 'ORed'
         wxString text = GetStatusText(i);
-        if (!StatusBar_SetText(GetHwnd(), style | i, text.wx_str()))
+        if (!StatusBar_SetText(GetHwnd(), style | i, text.t_str()))
         {
             wxLogLastError("StatusBar_SetText");
         }
@@ -618,15 +616,21 @@ wxStatusBar::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
     }
 #endif
 
-    bool needsEllipsization = HasFlag(wxSTB_ELLIPSIZE_START) ||
-                              HasFlag(wxSTB_ELLIPSIZE_MIDDLE) ||
-                              HasFlag(wxSTB_ELLIPSIZE_END);
-    if ( nMsg == WM_SIZE && needsEllipsization )
+    if ( nMsg == WM_SIZE )
     {
-        for (int i=0; i<GetFieldsCount(); i++)
-            DoUpdateStatusText(i);
-            // re-set the field text, in case we need to ellipsize
-            // (or de-ellipsize) some parts of it
+        MSWUpdateFieldsWidths();
+
+        if ( HasFlag(wxSTB_ELLIPSIZE_START) ||
+                HasFlag(wxSTB_ELLIPSIZE_MIDDLE) ||
+                    HasFlag(wxSTB_ELLIPSIZE_END) )
+        {
+            for (int i=0; i<GetFieldsCount(); i++)
+            {
+                // re-set the field text, in case we need to ellipsize
+                // (or de-ellipsize) some parts of it
+                DoUpdateStatusText(i);
+            }
+        }
     }
 
     return wxStatusBarBase::MSWWindowProc(nMsg, wParam, lParam);
