@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin (use hash map instead of list, global rewrite)
 // Created:     04/01/98
-// RCS-ID:      $Id: timer.cpp 63486 2010-02-15 17:34:21Z RD $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -31,6 +31,7 @@
 #endif
 
 #include "wx/msw/private.h"
+#include "wx/msw/private/hiddenwin.h"
 
 // ----------------------------------------------------------------------------
 // private globals
@@ -76,11 +77,6 @@ UINT_PTR GetNewTimerId(wxMSWTimerImpl *t)
 LRESULT APIENTRY _EXPORT wxTimerWndProc(HWND hWnd, UINT message,
                                         WPARAM wParam, LPARAM lParam);
 
-// implemented in utils.cpp
-extern "C" WXDLLIMPEXP_BASE HWND
-wxCreateHiddenWindow(LPCTSTR *pclassname, LPCTSTR classname, WNDPROC wndproc);
-
-
 // ----------------------------------------------------------------------------
 // wxTimerHiddenWindowModule: used to manage the hidden window used for
 // catching timer messages (we need a module to ensure that the window is
@@ -173,15 +169,17 @@ LRESULT APIENTRY _EXPORT wxTimerWndProc(HWND hWnd, UINT message,
     {
         wxTimerMap::iterator node = TimerMap().find(wParam);
 
-        wxCHECK_MSG( node != TimerMap().end(), 0, wxT("bogus timer id in wxTimerProc") );
+        if ( node != TimerMap().end() )
+        {
+            wxProcessTimer(*(node->second));
 
-        wxProcessTimer(*(node->second));
+            return 0;
+        }
+        //else: Unknown timer, probably one of our timers that had fired just
+        //      before being removed from the timers map by Stop().
     }
-    else
-    {
-        return ::DefWindowProc(hWnd, message, wParam, lParam);
-    }
-    return 0;
+
+    return ::DefWindowProc(hWnd, message, wParam, lParam);
 }
 
 // ----------------------------------------------------------------------------
