@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2003-07-12
-// RCS-ID:      $Id: tbarwce.cpp 61724 2009-08-21 10:41:26Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -372,7 +372,7 @@ bool wxToolMenuBar::Realize()
 #endif // 0
 
     bool lastWasRadio = false;
-    wxToolBarToolsList::Node* node;
+    wxToolBarToolsList::compatibility_iterator node;
     for ( node = m_tools.GetFirst(); node; node = node->GetNext() )
     {
         wxToolMenuBarTool *tool = (wxToolMenuBarTool*) node->GetData();
@@ -403,7 +403,7 @@ bool wxToolMenuBar::Realize()
                     const wxString& label = tool->GetLabel();
                     if ( !label.empty() )
                     {
-                        button.iString = (int)label.wx_str();
+                        button.iString = (int) wxMSW_CONV_LPCTSTR(label);
                     }
                 }
 
@@ -429,7 +429,7 @@ bool wxToolMenuBar::Realize()
                 }
 
                 int n = 0;
-                if ( bmpToUse.Ok() )
+                if ( bmpToUse.IsOk() )
                 {
                     n = ::CommandBar_AddBitmap( (HWND) GetHWND(), NULL, (int) (HBITMAP) bmpToUse.GetHBITMAP(),
                                                     1, 16, 16 );
@@ -504,7 +504,7 @@ bool wxToolMenuBar::MSWCommand(WXUINT WXUNUSED(cmd), WXWORD id_)
             }
         }
 
-        wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED);
+        wxCommandEvent event(wxEVT_MENU);
         event.SetEventObject(this);
         event.SetId(id);
         event.SetInt(checked);
@@ -525,7 +525,7 @@ bool wxToolMenuBar::MSWCommand(WXUINT WXUNUSED(cmd), WXWORD id_)
     if ( !tool->CanBeToggled() || tool->GetKind() != wxITEM_RADIO || toggled )
     {
         // OnLeftClick() can veto the button state change - for buttons which
-        // may be toggled only, of couse
+        // may be toggled only, of course.
         if ( !OnLeftClick((int)id, toggled) && tool->CanBeToggled() )
         {
             // revert back
