@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     1998-01-01
-// RCS-ID:      $Id: app.cpp 64343 2010-05-19 08:26:59Z SC $
+// RCS-ID:      $Id$
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -130,7 +130,7 @@ pascal OSErr AEHandleGURL( const AppleEvent *event , AppleEvent *reply , SRefCon
 }
 
 
-// AEODoc Calls MacOpenFile on each of the files passed
+// AEODoc Calls MacOpenFiles with all of the files passed
 
 short wxApp::MacHandleAEODoc(const WXEVENTREF event, WXEVENTREF WXUNUSED(reply))
 {
@@ -158,16 +158,23 @@ short wxApp::MacHandleAEODoc(const WXEVENTREF event, WXEVENTREF WXUNUSED(reply))
     wxString fName ;
     FSRef theRef ;
 
+    wxArrayString fileNames;
     for (i = 1; i <= itemsInList; i++)
     {
-        AEGetNthPtr(
+        err = AEGetNthPtr(
             &docList, i, typeFSRef, &keywd, &returnedType,
             (Ptr)&theRef, sizeof(theRef), &actualSize);
+        
+        if ( err != noErr)
+            return err;
+        
         fName = wxMacFSRefToPath( &theRef ) ;
 
-        MacOpenFile(fName);
+        fileNames.Add(fName);
     }
 
+    MacOpenFiles(fileNames);
+
     return noErr;
 }
 
@@ -226,9 +233,13 @@ short wxApp::MacHandleAEPDoc(const WXEVENTREF event , WXEVENTREF WXUNUSED(reply)
 
     for (i = 1; i <= itemsInList; i++)
     {
-        AEGetNthPtr(
+        err = AEGetNthPtr(
             &docList, i, typeFSRef, &keywd, &returnedType,
             (Ptr)&theRef, sizeof(theRef), &actualSize);
+        
+        if ( err != noErr)
+            return err;
+        
         fName = wxMacFSRefToPath( &theRef ) ;
 
         MacPrintFile(fName);
@@ -274,6 +285,16 @@ short wxApp::MacHandleAERApp(const WXEVENTREF WXUNUSED(event) , WXEVENTREF WXUNU
 // Support Routines linking the Mac...File Calls to the Document Manager
 //----------------------------------------------------------------------
 
+void wxApp::MacOpenFiles(const wxArrayString & fileNames )
+{
+    size_t i;
+    const size_t fileCount = fileNames.GetCount();
+    for (i = 0; i < fileCount; i++)
+    {
+        MacOpenFile(fileNames[i]);
+    }
+}
+
 void wxApp::MacOpenFile(const wxString & fileName )
 {
 #if wxUSE_DOC_VIEW_ARCHITECTURE
@@ -336,7 +357,7 @@ void wxApp::MacReopenApp()
     // if some windows are not hidden -> do nothing
 
     wxWindowList::compatibility_iterator node = wxTopLevelWindows.GetFirst();
-    if ( node == NULL )
+    if ( !node )
     {
         MacNewFile() ;
     }
@@ -369,8 +390,14 @@ void wxApp::MacReopenApp()
 
         if ( firstIconized )
             firstIconized->Iconize( false ) ;
+        
+        // showing hidden windows is not really always a good solution, also non-modal dialogs when closed end up
+        // as hidden tlws, we don't want to reshow those, so let's just reopen the minimized a.k.a. iconized tlws
+        // unless we find a regression ...
+#if 0
         else if ( firstHidden )
             firstHidden->Show( true );
+#endif
     }
 }
 
@@ -697,22 +724,7 @@ pascal OSStatus wxMacAppEventHandler( EventHandlerCallRef handler , EventRef eve
             break ;
 #endif
         case kEventClassAppleEvent :
-            {
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
-                if ( AEProcessEvent != NULL )
-                {
-                    result = AEProcessEvent(event);
-                }
-#endif
-#if MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
-                {
-                    EventRecord rec ;
-
-                    wxMacConvertEventToRecord( event , &rec ) ;
-                    result = AEProcessAppleEvent( &rec ) ;
-                }
-#endif
-            }
+            result = AEProcessEvent(event);
             break ;
 
         default :
@@ -794,7 +806,37 @@ bool wxApp::Initialize(int& argc, wxChar **argv)
         {
             // remove this argument
             --argc;
-            memmove(argv + 1, argv + 2, argc * sizeof(char *));
+            memmove(argv + 1, argv + 2, argc * sizeof(wxChar*));
+        }
+    }
+
+    /*
+     Cocoa supports -Key value options which set the user defaults key "Key"
+     to the value "value"  Some of them are very handy for debugging like
+     -NSShowAllViews YES.  Cocoa picks these up from the real argv so
+     our removal of them from the wx copy of it does not affect Cocoa's
+     ability to see them.
+     
+     We basically just assume that any "-NS" option and its following
+     argument needs to be removed from argv.  We hope that user code does
+     not expect to see -NS options and indeed it's probably a safe bet
+     since most user code accepting options is probably using the
+     double-dash GNU-style syntax.
+     */
+    for(int i=1; i < argc; ++i)
+    {
+        static const wxChar *ARG_NS = wxT("-NS");
+        if( wxStrncmp(argv[i], ARG_NS, wxStrlen(ARG_NS)) == 0 )
+        {
+            // Only eat this option if it has an argument
+            if( (i + 1) < argc )
+            {
+                memmove(argv + i, argv + i + 2, (argc-i-1)*sizeof(wxChar*));
+                argc -= 2;
+                // drop back one position so the next run through the loop
+                // reprocesses the argument at our current index.
+                --i;
+            }
         }
     }
 
@@ -939,98 +981,6 @@ void wxApp::CleanUp()
 // misc initialization stuff
 //----------------------------------------------------------------------
 
-#if wxOSX_USE_CARBON && MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5
-bool wxMacConvertEventToRecord( EventRef event , EventRecord *rec)
-{
-    OSStatus err = noErr ;
-    bool converted = ConvertEventRefToEventRecord( event, rec) ;
-
-    if ( !converted )
-    {
-        switch ( GetEventClass( event ) )
-        {
-            case kEventClassKeyboard :
-            {
-                converted = true ;
-                switch ( GetEventKind(event) )
-                {
-                    case kEventRawKeyDown :
-                        rec->what = keyDown ;
-                        break ;
-
-                    case kEventRawKeyRepeat :
-                        rec->what = autoKey ;
-                        break ;
-
-                    case kEventRawKeyUp :
-                        rec->what = keyUp ;
-                        break ;
-
-                    case kEventRawKeyModifiersChanged :
-                        rec->what = nullEvent ;
-                        break ;
-
-                    default :
-                        converted = false ;
-                        break ;
-                }
-
-                if ( converted )
-                {
-                    UInt32 keyCode ;
-                    unsigned char charCode ;
-                    UInt32 modifiers ;
-                    GetMouse( &rec->where) ;
-                    err = GetEventParameter(event, kEventParamKeyModifiers, typeUInt32, NULL, 4, NULL, &modifiers);
-                    err = GetEventParameter(event, kEventParamKeyCode, typeUInt32, NULL, 4, NULL, &keyCode);
-                    err = GetEventParameter(event, kEventParamKeyMacCharCodes, typeChar, NULL, 1, NULL, &charCode);
-                    rec->modifiers = modifiers ;
-                    rec->message = (keyCode << 8 ) + charCode ;
-                }
-            }
-            break ;
-
-            case kEventClassTextInput :
-            {
-                switch ( GetEventKind( event ) )
-                {
-                    case kEventTextInputUnicodeForKeyEvent :
-                        {
-                            EventRef rawEvent ;
-                            err = GetEventParameter(
-                                event, kEventParamTextInputSendKeyboardEvent, typeEventRef, NULL,
-                                sizeof(rawEvent), NULL, &rawEvent ) ;
-                            converted = true ;
-
-                            {
-                                UInt32 keyCode, modifiers;
-                                unsigned char charCode ;
-                                GetMouse( &rec->where) ;
-                                rec->what = keyDown ;
-                                err = GetEventParameter(rawEvent, kEventParamKeyModifiers, typeUInt32, NULL, 4, NULL, &modifiers);
-                                err = GetEventParameter(rawEvent, kEventParamKeyCode, typeUInt32, NULL, 4, NULL, &keyCode);
-                                err = GetEventParameter(rawEvent, kEventParamKeyMacCharCodes, typeChar, NULL, 1, NULL, &charCode);
-                                rec->modifiers = modifiers ;
-                                rec->message = (keyCode << 8 ) + charCode ;
-                            }
-                       }
-                       break ;
-
-                    default :
-                        break ;
-                }
-            }
-            break ;
-
-            default :
-                break ;
-        }
-    }
-
-    return converted ;
-}
-#endif
-
 wxApp::wxApp()
 {
     m_printMode = wxPRINT_WINDOWS;
@@ -1124,7 +1074,137 @@ void wxApp::MacHandleUnhandledEvent( WXEVENTREF WXUNUSED(evr) )
     // Override to process unhandled events as you please
 }
 
-#if wxOSX_USE_CARBON
+#if wxOSX_USE_COCOA_OR_CARBON
+
+CGKeyCode wxCharCodeWXToOSX(wxKeyCode code)
+{
+    CGKeyCode keycode;
+    
+    switch (code)
+    {
+        // Clang warns about switch values not of the same type as (enumerated)
+        // switch controlling expression. This is generally useful but here we
+        // really want to be able to use letters and digits without making them
+        // part of wxKeyCode enum.
+#ifdef __clang__
+    #pragma clang diagnostic push
+    #pragma clang diagnostic ignored "-Wswitch"
+#endif // __clang__
+
+        case 'a': case 'A':   keycode = kVK_ANSI_A; break;
+        case 'b': case 'B':   keycode = kVK_ANSI_B; break;
+        case 'c': case 'C':   keycode = kVK_ANSI_C; break;
+        case 'd': case 'D':   keycode = kVK_ANSI_D; break;
+        case 'e': case 'E':   keycode = kVK_ANSI_E; break;
+        case 'f': case 'F':   keycode = kVK_ANSI_F; break;
+        case 'g': case 'G':   keycode = kVK_ANSI_G; break;
+        case 'h': case 'H':   keycode = kVK_ANSI_H; break;
+        case 'i': case 'I':   keycode = kVK_ANSI_I; break;
+        case 'j': case 'J':   keycode = kVK_ANSI_J; break;
+        case 'k': case 'K':   keycode = kVK_ANSI_K; break;
+        case 'l': case 'L':   keycode = kVK_ANSI_L; break;
+        case 'm': case 'M':   keycode = kVK_ANSI_M; break;
+        case 'n': case 'N':   keycode = kVK_ANSI_N; break;
+        case 'o': case 'O':   keycode = kVK_ANSI_O; break;
+        case 'p': case 'P':   keycode = kVK_ANSI_P; break;
+        case 'q': case 'Q':   keycode = kVK_ANSI_Q; break;
+        case 'r': case 'R':   keycode = kVK_ANSI_R; break;
+        case 's': case 'S':   keycode = kVK_ANSI_S; break;
+        case 't': case 'T':   keycode = kVK_ANSI_T; break;
+        case 'u': case 'U':   keycode = kVK_ANSI_U; break;
+        case 'v': case 'V':   keycode = kVK_ANSI_V; break;
+        case 'w': case 'W':   keycode = kVK_ANSI_W; break;
+        case 'x': case 'X':   keycode = kVK_ANSI_X; break;
+        case 'y': case 'Y':   keycode = kVK_ANSI_Y; break;
+        case 'z': case 'Z':   keycode = kVK_ANSI_Z; break;
+            
+        case '0':             keycode = kVK_ANSI_0; break;
+        case '1':             keycode = kVK_ANSI_1; break;
+        case '2':             keycode = kVK_ANSI_2; break;
+        case '3':             keycode = kVK_ANSI_3; break;
+        case '4':             keycode = kVK_ANSI_4; break;
+        case '5':             keycode = kVK_ANSI_5; break;
+        case '6':             keycode = kVK_ANSI_6; break;
+        case '7':             keycode = kVK_ANSI_7; break;
+        case '8':             keycode = kVK_ANSI_8; break;
+        case '9':             keycode = kVK_ANSI_9; break;
+
+#ifdef __clang__
+    #pragma clang diagnostic pop
+#endif // __clang__
+            
+        case WXK_BACK:        keycode = kVK_Delete; break;
+        case WXK_TAB:         keycode = kVK_Tab; break;
+        case WXK_RETURN:      keycode = kVK_Return; break;
+        case WXK_ESCAPE:      keycode = kVK_Escape; break;
+        case WXK_SPACE:       keycode = kVK_Space; break;
+        case WXK_DELETE:      keycode = kVK_ForwardDelete; break;
+            
+        case WXK_SHIFT:       keycode = kVK_Shift; break;
+        case WXK_ALT:         keycode = kVK_Option; break;
+        case WXK_RAW_CONTROL: keycode = kVK_Control; break;
+        case WXK_CONTROL:     keycode = kVK_Command; break;
+            
+        case WXK_CAPITAL:     keycode = kVK_CapsLock; break;
+        case WXK_END:         keycode = kVK_End; break;
+        case WXK_HOME:        keycode = kVK_Home; break;
+        case WXK_LEFT:        keycode = kVK_LeftArrow; break;
+        case WXK_UP:          keycode = kVK_UpArrow; break;
+        case WXK_RIGHT:       keycode = kVK_RightArrow; break;
+        case WXK_DOWN:        keycode = kVK_DownArrow; break;
+            
+        case WXK_HELP:        keycode = kVK_Help; break;
+            
+            
+        case WXK_NUMPAD0:     keycode = kVK_ANSI_Keypad0; break;
+        case WXK_NUMPAD1:     keycode = kVK_ANSI_Keypad1; break;
+        case WXK_NUMPAD2:     keycode = kVK_ANSI_Keypad2; break;
+        case WXK_NUMPAD3:     keycode = kVK_ANSI_Keypad3; break;
+        case WXK_NUMPAD4:     keycode = kVK_ANSI_Keypad4; break;
+        case WXK_NUMPAD5:     keycode = kVK_ANSI_Keypad5; break;
+        case WXK_NUMPAD6:     keycode = kVK_ANSI_Keypad6; break;
+        case WXK_NUMPAD7:     keycode = kVK_ANSI_Keypad7; break;
+        case WXK_NUMPAD8:     keycode = kVK_ANSI_Keypad8; break;
+        case WXK_NUMPAD9:     keycode = kVK_ANSI_Keypad9; break;
+        case WXK_F1:          keycode = kVK_F1; break;
+        case WXK_F2:          keycode = kVK_F2; break;
+        case WXK_F3:          keycode = kVK_F3; break;
+        case WXK_F4:          keycode = kVK_F4; break;
+        case WXK_F5:          keycode = kVK_F5; break;
+        case WXK_F6:          keycode = kVK_F6; break;
+        case WXK_F7:          keycode = kVK_F7; break;
+        case WXK_F8:          keycode = kVK_F8; break;
+        case WXK_F9:          keycode = kVK_F9; break;
+        case WXK_F10:         keycode = kVK_F10; break;
+        case WXK_F11:         keycode = kVK_F11; break;
+        case WXK_F12:         keycode = kVK_F12; break;
+        case WXK_F13:         keycode = kVK_F13; break;
+        case WXK_F14:         keycode = kVK_F14; break;
+        case WXK_F15:         keycode = kVK_F15; break;
+        case WXK_F16:         keycode = kVK_F16; break;
+        case WXK_F17:         keycode = kVK_F17; break;
+        case WXK_F18:         keycode = kVK_F18; break;
+        case WXK_F19:         keycode = kVK_F19; break;
+        case WXK_F20:         keycode = kVK_F20; break;
+            
+        case WXK_PAGEUP:      keycode = kVK_PageUp; break;
+        case WXK_PAGEDOWN:    keycode = kVK_PageDown; break;
+            
+        case WXK_NUMPAD_DELETE:    keycode = kVK_ANSI_KeypadClear; break;
+        case WXK_NUMPAD_EQUAL:     keycode = kVK_ANSI_KeypadEquals; break;
+        case WXK_NUMPAD_MULTIPLY:  keycode = kVK_ANSI_KeypadMultiply; break;
+        case WXK_NUMPAD_ADD:       keycode = kVK_ANSI_KeypadPlus; break;
+        case WXK_NUMPAD_SUBTRACT:  keycode = kVK_ANSI_KeypadMinus; break;
+        case WXK_NUMPAD_DECIMAL:   keycode = kVK_ANSI_KeypadDecimal; break;
+        case WXK_NUMPAD_DIVIDE:    keycode = kVK_ANSI_KeypadDivide; break;
+            
+        default:
+            wxLogDebug( "Unrecognised keycode %d", code );
+            keycode = static_cast<CGKeyCode>(-1);
+    }
+    
+    return keycode;
+}
 
 long wxMacTranslateKey(unsigned char key, unsigned char code)
 {
@@ -1273,6 +1353,7 @@ int wxMacKeyCodeToModifier(wxKeyCode key)
     {
     case WXK_START:
     case WXK_MENU:
+    case WXK_COMMAND:
         return cmdKey;
 
     case WXK_SHIFT:
@@ -1284,7 +1365,7 @@ int wxMacKeyCodeToModifier(wxKeyCode key)
     case WXK_ALT:
         return optionKey;
 
-    case WXK_CONTROL:
+    case WXK_RAW_CONTROL:
         return controlKey;
 
     default:
@@ -1313,10 +1394,10 @@ wxMouseState wxGetMouseState()
     ms.SetRightDown( (buttons & 0x02) != 0 );
 
     UInt32 modifiers = GetCurrentKeyModifiers();
-    ms.SetControlDown(modifiers & controlKey);
+    ms.SetRawControlDown(modifiers & controlKey);
     ms.SetShiftDown(modifiers & shiftKey);
     ms.SetAltDown(modifiers & optionKey);
-    ms.SetMetaDown(modifiers & cmdKey);
+    ms.SetControlDown(modifiers & cmdKey);
 
     return ms;
 }
@@ -1325,53 +1406,49 @@ wxMouseState wxGetMouseState()
 
 // TODO : once the new key/char handling is tested, move all the code to wxWindow
 
-bool wxApp::MacSendKeyDownEvent( wxWindow* focus , long keymessage , long modifiers , long when , short wherex , short wherey , wxChar uniChar )
+bool wxApp::MacSendKeyDownEvent( wxWindow* focus , long keymessage , long modifiers , long when , wxChar uniChar )
 {
     if ( !focus )
         return false ;
 
     wxKeyEvent event(wxEVT_KEY_DOWN) ;
-    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , wherex , wherey , uniChar ) ;
+    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , uniChar ) ;
 
     return focus->OSXHandleKeyEvent(event);
 }
 
-bool wxApp::MacSendKeyUpEvent( wxWindow* focus , long keymessage , long modifiers , long when , short wherex , short wherey , wxChar uniChar )
+bool wxApp::MacSendKeyUpEvent( wxWindow* focus , long keymessage , long modifiers , long when , wxChar uniChar )
 {
     if ( !focus )
         return false ;
 
     wxKeyEvent event( wxEVT_KEY_UP ) ;
-    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , wherex , wherey , uniChar ) ;
+    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , uniChar ) ;
 
     return focus->OSXHandleKeyEvent(event) ;
 }
 
-bool wxApp::MacSendCharEvent( wxWindow* focus , long keymessage , long modifiers , long when , short wherex , short wherey , wxChar uniChar )
+bool wxApp::MacSendCharEvent( wxWindow* focus , long keymessage , long modifiers , long when , wxChar uniChar )
 {
     if ( !focus )
         return false ;
     wxKeyEvent event(wxEVT_CHAR) ;
-    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , wherex , wherey , uniChar ) ;
+    MacCreateKeyEvent( event, focus , keymessage , modifiers , when , uniChar ) ;
 
     bool handled = false ;
 
 #if wxOSX_USE_CARBON
     long keyval = event.m_keyCode ;
-    wxNonOwnedWindow *tlw = focus->MacGetTopLevelWindow() ;
 
-    if (tlw)
     {
-        event.SetEventType( wxEVT_CHAR_HOOK );
-        handled = tlw->HandleWindowEvent( event );
-        if ( handled && event.GetSkipped() )
+        wxKeyEvent eventCharHook(wxEVT_CHAR_HOOK, event);
+        handled = focus->HandleWindowEvent( eventCharHook );
+        if ( handled && eventCharHook.IsNextEventAllowed() )
             handled = false ;
     }
 
     if ( !handled )
     {
-        event.SetEventType( wxEVT_CHAR );
-        event.Skip( false ) ;
         handled = focus->HandleWindowEvent( event ) ;
     }
 
@@ -1414,7 +1491,7 @@ bool wxApp::MacSendCharEvent( wxWindow* focus , long keymessage , long modifiers
                     wxButton *def = wxDynamicCast(tlw->GetDefaultItem(), wxButton);
                     if ( def && def->IsEnabled() )
                     {
-                        wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, def->GetId() );
+                        wxCommandEvent event(wxEVT_BUTTON, def->GetId() );
                         event.SetEventObject(def);
                         def->Command(event);
 
@@ -1425,7 +1502,7 @@ bool wxApp::MacSendCharEvent( wxWindow* focus , long keymessage , long modifiers
             else if (keyval == WXK_ESCAPE || (keyval == '.' && modifiers & cmdKey ) )
             {
                 // generate wxID_CANCEL if command-. or <esc> has been pressed (typically in dialogs)
-                wxCommandEvent new_event(wxEVT_COMMAND_BUTTON_CLICKED,wxID_CANCEL);
+                wxCommandEvent new_event(wxEVT_BUTTON,wxID_CANCEL);
                 new_event.SetEventObject( focus );
                 handled = focus->HandleWindowEvent( new_event );
             }
@@ -1437,9 +1514,10 @@ bool wxApp::MacSendCharEvent( wxWindow* focus , long keymessage , long modifiers
 }
 
 // This method handles common code for SendKeyDown, SendKeyUp, and SendChar events.
-void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymessage , long modifiers , long when , short wherex , short wherey , wxChar uniChar )
+void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymessage , long modifiers , long when , wxChar uniChar )
 {
-#if wxOSX_USE_CARBON
+#if wxOSX_USE_COCOA_OR_CARBON
+    
     short keycode, keychar ;
 
     keychar = short(keymessage & charCodeMask);
@@ -1508,9 +1586,9 @@ void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymess
     }
 
     event.m_shiftDown = modifiers & shiftKey;
-    event.m_controlDown = modifiers & controlKey;
+    event.m_rawControlDown = modifiers & controlKey;
     event.m_altDown = modifiers & optionKey;
-    event.m_metaDown = modifiers & cmdKey;
+    event.m_controlDown = modifiers & cmdKey;
     event.m_keyCode = keyval ;
 #if wxUSE_UNICODE
     event.m_uniChar = uniChar ;
@@ -1518,8 +1596,6 @@ void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymess
 
     event.m_rawCode = keymessage;
     event.m_rawFlags = modifiers;
-    event.m_x = wherex;
-    event.m_y = wherey;
     event.SetTimestamp(when);
     event.SetEventObject(focus);
 #else
@@ -1528,8 +1604,6 @@ void wxApp::MacCreateKeyEvent( wxKeyEvent& event, wxWindow* focus , long keymess
     wxUnusedVar(keymessage);
     wxUnusedVar(modifiers);
     wxUnusedVar(when);
-    wxUnusedVar(wherex);
-    wxUnusedVar(wherey);
     wxUnusedVar(uniChar);
 #endif
 }
