@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        src/osx/corefoundation/joystick.cpp
+// Name:        src/osx/core/joystick.cpp
 // Purpose:     wxJoystick class
 // Author:      Ryan Norton
 // Modified by:
 // Created:     2/13/2005
-// RCS-ID:      $Id: hidjoystick.cpp 64656 2010-06-20 18:18:23Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) Ryan Norton
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -191,25 +191,25 @@ wxPoint wxJoystick::GetPosition() const
 int wxJoystick::GetZPosition() const
 {
     if (m_thread)
-        return m_thread->m_axe[wxJS_AXIS_Z];
+        return m_thread->m_axe[wxJS_AXIS_Z-wxJS_AXIS_X];
     return 0;
 }
 int wxJoystick::GetRudderPosition() const
 {
     if (m_thread)
-        return m_thread->m_axe[wxJS_AXIS_RUDDER];
+        return m_thread->m_axe[wxJS_AXIS_RUDDER-wxJS_AXIS_X];
     return 0;
 }
 int wxJoystick::GetUPosition() const
 {
     if (m_thread)
-        return m_thread->m_axe[wxJS_AXIS_U];
+        return m_thread->m_axe[wxJS_AXIS_U-wxJS_AXIS_X];
     return 0;
 }
 int wxJoystick::GetVPosition() const
 {
     if (m_thread)
-        return m_thread->m_axe[wxJS_AXIS_V];
+        return m_thread->m_axe[wxJS_AXIS_V-wxJS_AXIS_X];
     return 0;
 }
 
@@ -566,7 +566,7 @@ void wxHIDJoystick::BuildCookies(CFArrayRef Array)
 
     //
     // I wasted two hours of my life on this line :(
-    // accidently removed it during some source cleaning...
+    // accidentally removed it during some source cleaning...
     //
     MakeCookies(Array);
 
@@ -778,11 +778,11 @@ void* wxJoystickThread::Entry()
 //---------------------------------------------------------------------------
 // wxJoystickThread::HIDCallback (static)
 //
-// Callback for the native HID device when it recieves input.
+// Callback for the native HID device when it receives input.
 //
 // This is where the REAL dirty work gets done.
 //
-// 1) Loops through each event the queue has recieved
+// 1) Loops through each event the queue has received
 // 2) First, checks if the thread that is running the loop for
 //    the polling has ended - if so it breaks out
 // 3) Next, it checks if there was an error getting this event from
