@@ -6,7 +6,7 @@
 # Author:       Robin Dunn
 #
 # Created:      5-Sept-2000
-# RCS-ID:       $Id: gen_iface.py 64940 2010-07-13 13:29:13Z VZ $
+# RCS-ID:       $Id$
 # Copyright:    (c) 2000 by Total Control Software
 # Licence:      wxWindows licence
 #----------------------------------------------------------------------------
@@ -18,13 +18,15 @@ from fileinput import FileInput
 
 IFACE         = os.path.abspath('./scintilla/include/Scintilla.iface')
 H_TEMPLATE    = os.path.abspath('./stc.h.in')
+IH_TEMPLATE   = os.path.abspath('./stc.interface.h.in')
 CPP_TEMPLATE  = os.path.abspath('./stc.cpp.in')
 H_DEST        = os.path.abspath('../../include/wx/stc/stc.h')
+IH_DEST       = os.path.abspath('../../interface/wx/stc/stc.h')
 CPP_DEST      = os.path.abspath('./stc.cpp')
 if len(sys.argv) > 1 and sys.argv[1] == '--wxpython':
     DOCSTR_DEST   = os.path.abspath('../../../wxPython/src/_stc_gendocs.i')
 else:
-    DOCSTR_DEST   = '/dev/null'
+    DOCSTR_DEST   = None
 
 
 # Value prefixes to convert
@@ -52,6 +54,8 @@ cmdValues = [ 2011,
               (2426, 2442),
               (2450, 2455),
               2518,
+              (2619, 2621),
+              (2628, 2629)
             ]
 
 
@@ -88,8 +92,8 @@ methodOverrideMap = {
                  'void %s(const wxString& text);',
 
                  '''void %s(const wxString& text) {
-                    wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-                    SendMsg(%s, strlen(buf), (sptr_t)(const char*)buf);''',
+                    const wxWX2MBbuf buf = wx2stc(text);
+                    SendMsg(%s, wx2stclen(text, buf), (sptr_t)(const char*)buf);''',
                  0),
 
     'AddStyledText' : (0,
@@ -103,8 +107,8 @@ methodOverrideMap = {
                  'void %s(const wxString& text);',
 
                  '''void %s(const wxString& text) {
-                    wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-                    SendMsg(%s, strlen(buf), (sptr_t)(const char*)buf);''',
+                    const wxWX2MBbuf buf = wx2stc(text);
+                    SendMsg(%s, wx2stclen(text, buf), (sptr_t)(const char*)buf);''',
                  0),
 
     'GetViewWS' : ( 'GetViewWhiteSpace', 0, 0, 0),
@@ -180,6 +184,7 @@ methodOverrideMap = {
 
     'MarkerSetFore' : ('MarkerSetForeground', 0, 0, 0),
     'MarkerSetBack' : ('MarkerSetBackground', 0, 0, 0),
+    'MarkerSetBackSelected' : ('MarkerSetBackgroundSelected', 0,0,0),
 
     'MarkerSymbolDefined' : ('GetMarkerSymbolDefined', 0, 0, 0),
 
@@ -194,9 +199,9 @@ methodOverrideMap = {
                 const wxColour& background) {
 
                 SendMsg(%s, markerNumber, markerSymbol);
-                if (foreground.Ok())
+                if (foreground.IsOk())
                     MarkerSetForeground(markerNumber, foreground);
-                if (background.Ok())
+                if (background.IsOk())
                     MarkerSetBackground(markerNumber, background);''',
 
      ('Set the symbol used for a particular marker number,',
@@ -231,7 +236,9 @@ methodOverrideMap = {
     'GetMarginMaskN' : ('GetMarginMask', 0, 0, 0),
     'SetMarginSensitiveN' : ('SetMarginSensitive', 0, 0, 0),
     'GetMarginSensitiveN' : ('GetMarginSensitive', 0, 0, 0),
-
+    'SetMarginCursorN' : ('SetMarginCursor', 0, 0, 0),
+    'GetMarginCursorN' : ('GetMarginCursor', 0, 0, 0),
+    
     'MarginGetText' :
     (0,
     'wxString %s(int line) const;',
@@ -355,6 +362,8 @@ methodOverrideMap = {
 
     'IndicSetAlpha' : ('IndicatorSetAlpha', 0, 0, 0),
     'IndicGetAlpha' : ('IndicatorGetAlpha', 0, 0, 0),
+    'IndicSetOutlineAlpha' : ('IndicatorSetOutlineAlpha', 0, 0, 0),
+    'IndicGetOutlineAlpha' : ('IndicatorGetOutlineAlpha', 0, 0, 0),
     'IndicSetStyle' : ('IndicatorSetStyle', 0, 0, 0),
     'IndicGetStyle' : ('IndicatorGetStyle', 0, 0, 0),
     'IndicSetFore' : ('IndicatorSetForeground', 0, 0, 0),
@@ -394,6 +403,8 @@ methodOverrideMap = {
     'AutoCSetMaxHeight'     : ('AutoCompSetMaxHeight', 0, 0, 0),
     'AutoCGetMaxHeight'     : ('AutoCompGetMaxHeight', 0, 0, 0),
     'AutoCGetMaxHeight'     : ('AutoCompGetMaxHeight', 0, 0, 0),
+    'AutoCSetCaseInsensitiveBehaviour'     : ('AutoCompSetCaseInsensitiveBehaviour', 0, 0, 0),
+    'AutoCGetCaseInsensitiveBehaviour'     : ('AutoCompGetCaseInsensitiveBehaviour', 0, 0, 0),
 
     'RegisterImage' :
     (0,
@@ -439,7 +450,7 @@ methodOverrideMap = {
             TextToFind  ft;
             ft.chrg.cpMin = minPos;
             ft.chrg.cpMax = maxPos;
-            wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
+            const wxWX2MBbuf buf = wx2stc(text);
             ft.lpstrText = (char*)(const char*)buf;
 
             return SendMsg(%s, flags, (sptr_t)&ft);''',
@@ -509,11 +520,7 @@ methodOverrideMap = {
      'wxString %s();',
 
      '''wxString %s() {
-         long   start;
-         long   end;
-
-         GetSelection(&start, &end);
-         int   len  = end - start;
+         const int len = SendMsg(SCI_GETSELTEXT, 0, (sptr_t)0);
          if (!len) return wxEmptyString;
 
          wxMemoryBuffer mbuf(len+2);
@@ -596,8 +603,8 @@ methodOverrideMap = {
 
      '''
      int %s(const wxString& text) {
-         wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-         return SendMsg(%s, strlen(buf), (sptr_t)(const char*)buf);''',
+         const wxWX2MBbuf buf = wx2stc(text);
+         return SendMsg(%s, wx2stclen(text, buf), (sptr_t)(const char*)buf);''',
      0),
 
     'ReplaceTargetRE' :
@@ -606,8 +613,8 @@ methodOverrideMap = {
 
      '''
      int %s(const wxString& text) {
-         wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-         return SendMsg(%s, strlen(buf), (sptr_t)(const char*)buf);''',
+         const wxWX2MBbuf buf = wx2stc(text);
+         return SendMsg(%s, wx2stclen(text, buf), (sptr_t)(const char*)buf);''',
      0),
 
     'SearchInTarget' :
@@ -616,8 +623,8 @@ methodOverrideMap = {
 
      '''
      int %s(const wxString& text) {
-         wxWX2MBbuf buf = (wxWX2MBbuf)wx2stc(text);
-         return SendMsg(%s, strlen(buf), (sptr_t)(const char*)buf);''',
+         const wxWX2MBbuf buf = wx2stc(text);
+         return SendMsg(%s, wx2stclen(text, buf), (sptr_t)(const char*)buf);''',
      0),
 
     # not sure what to do about these yet
@@ -733,12 +740,181 @@ methodOverrideMap = {
     'SetSelection' : (None, 0, 0, 0),
 
     'GetCharacterPointer' : (0,
-                             'const char* %s();',
-                             'const char* %s() {\n'
+                             'const char* %s() const;',
+                             'const char* %s() const {\n'
                              '    return (const char*)SendMsg(%s, 0, 0);',
                              0),
     
+    'GetRangePointer' : (0,
+                             'const char* %s(int position, int rangeLength) const;',
+                             'const char* %s(int position, int rangeLength) const {\n'
+                             '    return (const char*)SendMsg(%s, position, rangeLength);',
+                             0),
+    
 
+    'GetWordChars' :
+    (0,
+     'wxString %s() const;',
+
+     '''wxString %s() const {
+         int msg = %s;
+         int len = SendMsg(msg, 0, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, 0, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+
+     ('Get the set of characters making up words for when moving or selecting by word.',)),
+
+    'GetTag' :
+    (0,
+     'wxString %s(int tagNumber) const;',
+
+     '''wxString %s(int tagNumber) const {
+         int msg = %s;
+         int len = SendMsg(msg, tagNumber, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, tagNumber, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+
+    'GetWhitespaceChars' :
+    (0,
+     'wxString %s() const;',
+
+     '''wxString %s() const {
+         int msg = %s;
+         int len = SendMsg(msg, 0, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, 0, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+
+
+    'GetPunctuationChars' :
+    (0,
+     'wxString %s() const;',
+
+     '''wxString %s() const {
+         int msg = %s;
+         int len = SendMsg(msg, 0, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, 0, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+
+
+    'PropertyNames' :
+    (0,
+     'wxString %s() const;',
+
+     '''wxString %s() const {
+         int msg = %s;
+         int len = SendMsg(msg, 0, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, 0, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+
+
+
+    'DescribeProperty' :
+    (0,
+     'wxString %s(const wxString& name) const;',
+
+     '''wxString %s(const wxString& name) const {
+         int msg = %s;
+         int len = SendMsg(msg, (sptr_t)(const char*)wx2stc(name), (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, (sptr_t)(const char*)wx2stc(name), (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+
+
+
+    'DescribeKeyWordSets' :
+    (0,
+     'wxString %s() const;',
+
+     '''wxString %s() const {
+         int msg = %s;
+         int len = SendMsg(msg, 0, (sptr_t)NULL);
+         if (!len) return wxEmptyString;
+
+         wxMemoryBuffer mbuf(len+1);
+         char* buf = (char*)mbuf.GetWriteBuf(len+1);
+         SendMsg(msg, 0, (sptr_t)buf);
+         mbuf.UngetWriteBuf(len);
+         mbuf.AppendByte(0);
+         return stc2wx(buf);''',
+     0),
+     
+
+    'MarkerDefineRGBAImage' :
+    (0,
+    'void %s(int markerNumber, const unsigned char* pixels);',
+    '''void %s(int markerNumber, const unsigned char* pixels) {
+           SendMsg(%s, markerNumber, (sptr_t)pixels);''',
+    0),
+
+
+    'RegisterRGBAImage' :
+    (0,
+    'void %s(int type, const unsigned char* pixels);',
+    '''void %s(int type, const unsigned char* pixels) {
+           SendMsg(%s, type, (sptr_t)pixels);''',
+    0),
+
+
+    # I think these are only available on the native OSX backend, so
+    # don't add them to the wx API...
+    'FindIndicatorShow' : (None, 0,0,0),
+    'FindIndicatorFlash' : (None, 0,0,0),
+    'FindIndicatorHide' : (None, 0,0,0),
+
+    'CreateLoader' :
+    (0,
+     'void* %s(int bytes) const;',
+     """void* %s(int bytes) const {
+         return (void*)(sptr_t)SendMsg(%s, bytes); """,
+     0),
+
+     'PrivateLexerCall' :
+     (0,
+      'void* %s(int operation, void* pointer);',
+      """void* %s(int operation, void* pointer) {
+           return (void*)(sptr_t)SendMsg(%s, operation, (sptr_t)pointer); """,
+      0),
+    
     '' : ('', 0, 0, 0),
 
     }
@@ -758,7 +934,7 @@ constNonGetterMethods = (
 
 #----------------------------------------------------------------------------
 
-def processIface(iface, h_tmplt, cpp_tmplt, h_dest, cpp_dest, docstr_dest):
+def processIface(iface, h_tmplt, cpp_tmplt, ih_tmplt, h_dest, cpp_dest, docstr_dest, ih_dest):
     curDocStrings = []
     values = []
     methods = []
@@ -785,7 +961,7 @@ def processIface(iface, h_tmplt, cpp_tmplt, h_dest, cpp_dest, docstr_dest):
             curDocStrings = []
 
         elif op == 'cat ':
-            if string.strip(line[4:]) == 'Deprecated':
+            if line[4:].strip() == 'Deprecated':
                 break    # skip the rest of the file
 
         elif op == 'evt ':
@@ -798,34 +974,38 @@ def processIface(iface, h_tmplt, cpp_tmplt, h_dest, cpp_dest, docstr_dest):
             pass
 
         else:
-            print '***** Unknown line type: ', line
+            print('***** Unknown line type: %s' % line)
 
 
     # process templates
     data = {}
     data['VALUES'] = processVals(values)
     data['CMDS']   = processVals(cmds)
-    defs, imps, docstrings = processMethods(methods)
+    defs, imps, docstrings, idefs = processMethods(methods)
     data['METHOD_DEFS'] = defs
+    data['METHOD_IDEFS'] = idefs
     data['METHOD_IMPS'] = imps
 
     # get template text
     h_text = open(h_tmplt).read()
+    ih_text = open(ih_tmplt).read()
     cpp_text = open(cpp_tmplt).read()
 
     # do the substitutions
     h_text = h_text % data
     cpp_text = cpp_text % data
+    ih_text = ih_text % data
 
     # write out destination files
     open(h_dest, 'w').write(h_text)
     open(cpp_dest, 'w').write(cpp_text)
-    open(docstr_dest, 'w').write(docstrings)
-
+    if docstr_dest:
+        open(docstr_dest, 'w').write(docstrings)
+    open(ih_dest, 'w').write(ih_text)
 
 
 def joinWithNewLines(values):
-    return string.join(values, '\n')
+    return '\n'.join(values)
 
 #----------------------------------------------------------------------------
 
@@ -835,7 +1015,7 @@ def processVals(values):
         if docs:
             text.append('')
             for x in docs:
-                text.append('// ' + x)
+                text.append('/// ' + x)
         text.append('#define %s %s' % (name, value))
     return joinWithNewLines(text)
 
@@ -843,6 +1023,7 @@ def processVals(values):
 
 def processMethods(methods):
     defs = []
+    idefs = []
     imps = []
     dstr = []
 
@@ -872,6 +1053,18 @@ def processMethods(methods):
             theDef = theDef + ';'
         defs.append(theDef)
 
+        # Build the method definition for the interface .h file
+        if docs:
+            idefs.append('')
+            idefs.append('    /**')
+            for x in docs:
+                idefs.append('        ' + x)
+            idefs.append('    */')
+        if name == 'GetCurLine':
+            idefs.append('    wxString GetCurLine(int* linePos=NULL);')
+        else:
+            idefs.append(theDef)
+                     
         # Build the method implementation string
         if docs:
             imps.append('')
@@ -898,14 +1091,14 @@ def processMethods(methods):
         imps.append(theImp)
 
 
-    return joinWithNewLines(defs), joinWithNewLines(imps), joinWithNewLines(dstr)
+    return joinWithNewLines(defs), joinWithNewLines(imps), joinWithNewLines(dstr), joinWithNewLines(idefs)
 
 
 #----------------------------------------------------------------------------
 
 def checkMethodOverride(name, number, docs):
     theDef = theImp = None
-    if methodOverrideMap.has_key(name):
+    if name in methodOverrideMap:
         item = methodOverrideMap[name]
 
         try:
@@ -918,7 +1111,7 @@ def checkMethodOverride(name, number, docs):
             if item[3] != 0:
                 docs = item[3]
         except:
-            print "*************", name
+            print("************* " + name)
             raise
 
     return name, theDef, theImp, docs
@@ -958,7 +1151,7 @@ def makeParamString(param1, param2):
 #----------------------------------------------------------------------------
 
 def parseVal(line, values, docs):
-    name, val = string.split(line, '=')
+    name, val = line.split('=')
 
     # remove prefixes such as SCI, etc.
     for old, new in valPrefixes:
@@ -981,16 +1174,16 @@ funregex = re.compile(r'\s*([a-zA-Z0-9_]+)'  # <ws>return type
 
 def parseFun(line, methods, docs, values, is_const):
     def parseParam(param):
-        param = string.strip(param)
+        param = param.strip()
         if param == '':
             param = None
         else:
-            param = tuple(string.split(param))
+            param = tuple(param.split())
         return param
 
     mo = funregex.match(line)
     if mo is None:
-        print "***** Line doesn't match! : " + line
+        print("***** Line doesn't match! : %s" % line)
 
     retType, name, number, param1, param2 = mo.groups()
 
@@ -998,10 +1191,10 @@ def parseFun(line, methods, docs, values, is_const):
     param2 = parseParam(param2)
 
     # Special case.  For the key command functions we want a value defined too
-    num = string.atoi(number)
+    num = int(number)
     for v in cmdValues:
         if (type(v) == type(()) and v[0] <= num <= v[1]) or v == num:
-            parseVal('CMD_%s=%s' % (string.upper(name), number), values, docs)
+            parseVal('CMD_%s=%s' % (name.upper(), number), values, docs)
 
             # if we are not also doing a function for CMD values, then
             # just return, otherwise fall through to the append blow.
@@ -1019,11 +1212,11 @@ def main(args):
     # TODO: parse command line args to replace default input/output files???
 
     if not os.path.exists(IFACE):
-        print 'Please run this script from src/stc subdirectory.'
+        print('Please run this script from src/stc subdirectory.')
         sys.exit(1)
 
     # Now just do it
-    processIface(IFACE, H_TEMPLATE, CPP_TEMPLATE, H_DEST, CPP_DEST, DOCSTR_DEST)
+    processIface(IFACE, H_TEMPLATE, CPP_TEMPLATE, IH_TEMPLATE, H_DEST, CPP_DEST, DOCSTR_DEST, IH_DEST)
 
 
 
