@@ -15,12 +15,13 @@
 # Author:      Robin Dunn
 #
 # Created:     23-March-2004
-# RCS-ID:      $Id: config.py 65632 2010-09-25 00:22:17Z RD $
+# RCS-ID:      $Id$
 # Copyright:   (c) 2004 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
 
 import sys, os, glob, fnmatch, tempfile
+import subprocess
 
 EGGing = 'bdist_egg' in sys.argv or 'egg_info' in sys.argv
 if not EGGing:
@@ -94,7 +95,7 @@ BUILD_DLLWIDGET = 0# Build a module that enables unknown wx widgets
                    # to be loaded from a DLL and to be used from Python.
 
                    # Internet Explorer wrapper (experimental)
-BUILD_ACTIVEX = (os.name == 'nt') # and os.environ.get('CPU',None) != 'AMD64') 
+BUILD_ACTIVEX = (os.name == 'nt')
 
 
 CORE_ONLY = 0      # if true, don't build any of the above
@@ -328,7 +329,7 @@ def Verify_WX_CONFIG():
 
 
 def getWxConfigValue(flag):
-    cmd = "%s %s" % (WX_CONFIG, flag)
+    cmd = "%s --version=%s.%s  %s" % (WX_CONFIG, VER_MAJOR, VER_MINOR, flag)
     value = os.popen(cmd, 'r').read()[:-1]
     return value
 
@@ -404,13 +405,12 @@ def run_swig(files, dir, gendir, package, USE_SWIG, force, swig_args,
 
     return sources
 
-import subprocess as sp
 
 def swig_version():
     # It may come on either stdout or stderr, depending on the
     # version, so read both.
-    p = sp.Popen(SWIG + ' -version', shell=True, universal_newlines=True,
-                 stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.PIPE)
+    p = subprocess.Popen(SWIG + ' -version', shell=True, universal_newlines=True,
+                 stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
     stext = p.stdout.read() + p.stderr.read()
     import re
     match = re.search(r'[0-9]+\.[0-9]+\.[0-9]+$', stext, re.MULTILINE)
@@ -693,6 +693,31 @@ def getExtraPath(shortVer=True, addOpts=False, addRelease=True):
 
     return ep
 
+
+def getoutput(cmd):
+    sp = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    output = None
+    output = sp.stdout.read()
+    if sys.version_info > (3,):
+        output = output.decode('utf-8')  # TODO: is utf-8 okay here?
+    output = output.rstrip()
+    rval = sp.wait()
+    if rval:
+        # Failed!
+        print("Command '%s' failed with exit code %d." % (cmd, rval))
+        sys.exit(rval)
+    return output
+
+def getVisCVersion():
+    text = getoutput("cl.exe")
+    if 'Version 13' in text:
+        return '71'
+    if 'Version 15' in text:
+        return '90'
+    # TODO: Add more tests to get the other versions...
+    else:
+        return 'UNKNOWN'
+ 
 #----------------------------------------------------------------------
 # These functions and class are copied from distutils in Python 2.5
 # and then grafted back into the distutils modules so we can change
@@ -707,12 +732,21 @@ from distutils.errors import DistutilsExecError, CompileError
 def _darwin_compiler_fixup(compiler_so, cc_args):
     """
     This function will strip '-isysroot PATH' and '-arch ARCH' from the
-    compile flags if the user has specified one them in extra_compile_flags.
+    compile flags if the user has specified one of them in extra_compile_flags.
 
     This is needed because '-arch ARCH' adds another architecture to the
     build, without a way to remove an architecture. Furthermore GCC will
     barf if multiple '-isysroot' arguments are present.
+    
+    Robin: I've further modified our copy of this function to check if there
+    is a -isysroot flag in the CC/CXX values in the environment. If so then we
+    want to make sure that we keep that one and strip the others, instead of
+    stripping it and leaving Python's.
     """
+    
+    ccHasSysroot = '-isysroot' in os.environ.get('CC', '') \
+                 or '-isysroot' in os.environ.get('CXX', '')
+    
     stripArch = stripSysroot = 0
 
     compiler_so = list(compiler_so)
@@ -725,7 +759,7 @@ def _darwin_compiler_fixup(compiler_so, cc_args):
         stripArch = stripSysroot = True
     else:
         stripArch = '-arch' in cc_args
-        stripSysroot = '-isysroot' in cc_args or stripArch  # <== This line changed
+        stripSysroot = '-isysroot' in cc_args or stripArch or ccHasSysroot
 
     if stripArch:
         while 1:
@@ -738,7 +772,10 @@ def _darwin_compiler_fixup(compiler_so, cc_args):
 
     if stripSysroot:
         try:
-            index = compiler_so.index('-isysroot')
+            index = 0
+            if ccHasSysroot:
+                index = compiler_so.index('-isysroot') + 1
+            index = compiler_so.index('-isysroot', index)
             # Strip this argument and the next one:
             del compiler_so[index:index+2]
         except ValueError:
@@ -756,6 +793,7 @@ def _darwin_compiler_fixup(compiler_so, cc_args):
         sysroot = compiler_so[idx+1]
 
     if sysroot and not os.path.isdir(sysroot):
+        from distutils import log
         log.warn("Compiling with an SDK that doesn't seem to exist: %s",
                 sysroot)
         log.warn("Please check your Xcode installation")
@@ -771,8 +809,9 @@ def _darwin_compiler_fixup_24(compiler_so, cc_args):
 class MyUnixCCompiler(distutils.unixccompiler.UnixCCompiler):
     def _compile(self, obj, src, ext, cc_args, extra_postargs, pp_opts):
         compiler_so = self.compiler_so
-        if sys.platform == 'darwin':
+        if sys.platform == 'darwin':           
             compiler_so = _darwin_compiler_fixup(compiler_so, cc_args + extra_postargs)
+
         try:
             self.spawn(compiler_so + cc_args + [src, '-o', obj] +
                        extra_postargs)
@@ -895,10 +934,10 @@ if os.name == 'nt' and  COMPILER == 'msvc':
     WXPLAT = '__WXMSW__'
     GENDIR = 'msw'
 
-    if os.environ.get('CPU', None) == 'AMD64':
-        VCDLL = 'vc_amd64_dll'
+    if os.environ.get('CPU', None) in ['AMD64', 'X64']:
+        VCDLL = 'vc%s_x64_dll' % getVisCVersion()
     else:
-        VCDLL = 'vc_dll'
+        VCDLL = 'vc%s_dll' % getVisCVersion()
         
     includes = ['include', 'src',
                 opj(WXDIR, 'lib', VCDLL, 'msw'  + libFlag()),
@@ -1016,7 +1055,7 @@ elif os.name == 'posix' or COMPILER == 'mingw32':
 
         libs = ['stdc++']
         NO_SCRIPTS = 1
-        if not ARCH == "":
+        if ARCH != "":
             cflags.append("-arch")
             cflags.append(ARCH)
             lflags.append("-arch")
@@ -1025,7 +1064,27 @@ elif os.name == 'posix' or COMPILER == 'mingw32':
         if not os.environ.get('CC') or not os.environ.get('CXX'):
             os.environ["CXX"] = getWxConfigValue('--cxx')
             os.environ["CC"]  = getWxConfigValue('--cc')
-            #os.environ["CPP"] = "cpp-4.0"
+
+            # We want to use the linker command from wx to make sure
+            # we get the right sysroot, but we also need to ensure that
+            # the other linker flags that distutils wants to use are
+            # included as well.
+            LDSHARED = distutils.sysconfig.get_config_var('LDSHARED').split()
+            # remove the compiler command
+            del LDSHARED[0]
+            # remove any -sysroot flags and their arg
+            while 1:
+                try:
+                    index = LDSHARED.index('-isysroot')
+                    # Strip this argument and the next one:
+                    del LDSHARED[index:index+2]
+                except ValueError:
+                    break            
+            LDSHARED = ' '.join(LDSHARED)
+            # Combine with wx's ld command and stash it in the env
+            # where distutils will get it later.
+            LDSHARED = getWxConfigValue('--ld').replace(' -o', '') + ' ' + LDSHARED
+            os.environ["LDSHARED"]  = LDSHARED
 
     else:
         # Set flags for other Unix type platforms
