@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     30-April-2001
-# RCS-ID:      $Id: make_installer.py 65772 2010-10-05 23:37:22Z RD $
+# RCS-ID:      $Id$
 # Copyright:   (c) 2003 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -39,7 +39,7 @@ ISS_Template = r'''
 AppName = wxPython%(SHORTVER)s-%(PYVER)s
 AppVerName = wxPython %(VERSION)s for Python %(PYTHONVER)s
 OutputBaseFilename = wxPython%(SHORTVER)s-win%(BITS)s-%(VERSION)s-%(PYVER)s
-AppCopyright = Copyright 2010 Total Control Software
+AppCopyright = Copyright 2011 Total Control Software
 DefaultDirName = {code:GetInstallDir|c:\DoNotInstallHere}
 DefaultGroupName = wxPython %(VERSION)s for Python %(PYTHONVER)s
 PrivilegesRequired = %(PRIV)s
@@ -71,6 +71,7 @@ LicenseFile = licence\licence.txt
 
 [Components]
 Name: core;     Description: "wxPython modules and library";              Types: full custom;  Flags: fixed
+Name: cairo;    Description: "Cairo runtime DLLs";                        Types: full
 Name: pthfile;  Description: "Make this install be the default wxPython"; Types: full
 
 ;;------------------------------------------------------------
@@ -81,6 +82,7 @@ Source: "%(WXDIR)s\lib\%(VCDLLDIR)s\wx*%(WXDLLVER)s_*.dll";  DestDir: "{app}\%(P
 %(GDIPLUS)s
 %(CPPDLL)s
 %(MSLU)s
+Source: "%(CAIRO_ROOT)s\bin\*.dll";            DestDir: "{app}\%(PKGDIR)s\wx"; Components: cairo; Flags: replacesameversion
 
 
 Source: "wx\_activex.pyd";                     DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
@@ -94,6 +96,7 @@ Source: "wx\_gizmos.pyd";                      DestDir: "{app}\%(PKGDIR)s\wx"; C
 Source: "wx\_glcanvas.pyd";                    DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_grid.pyd";                        DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_html.pyd";                        DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
+Source: "wx\_html2.pyd";                       DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_media.pyd";                       DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_misc_.pyd";                       DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_stc.pyd";                         DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
@@ -113,8 +116,10 @@ Source: "wx\lib\*.idl";                         DestDir: "{app}\%(PKGDIR)s\wx\li
 Source: "wx\lib\*.tlb";                         DestDir: "{app}\%(PKGDIR)s\wx\lib"; Components: core
 Source: "wx\lib\agw\*.py";                      DestDir: "{app}\%(PKGDIR)s\wx\lib\agw"; Components: core
 Source: "wx\lib\agw\aui\*.py";                  DestDir: "{app}\%(PKGDIR)s\wx\lib\agw\aui"; Components: core
+Source: "wx\lib\agw\persist\*.py";              DestDir: "{app}\%(PKGDIR)s\wx\lib\agw\persist"; Components: core
 Source: "wx\lib\agw\ribbon\*.py";               DestDir: "{app}\%(PKGDIR)s\wx\lib\agw\ribbon"; Components: core
-Source: "wx\lib\agw\*.png";                     DestDir: "{app}\%(PKGDIR)s\wx\lib\agw"; Components: core
+Source: "wx\lib\agw\data\*.png";                DestDir: "{app}\%(PKGDIR)s\wx\lib\agw\data"; Components: core
+Source: "wx\lib\agw\data\*.html";               DestDir: "{app}\%(PKGDIR)s\wx\lib\agw\data"; Components: core
 Source: "wx\lib\analogclock\*.py";              DestDir: "{app}\%(PKGDIR)s\wx\lib\analogclock"; Components: core
 Source: "wx\lib\analogclock\lib_setup\*.py";    DestDir: "{app}\%(PKGDIR)s\wx\lib\analogclock\lib_setup"; Components: core
 Source: "wx\lib\art\*.py";                      DestDir: "{app}\%(PKGDIR)s\wx\lib\art"; Components: core
@@ -178,6 +183,7 @@ Source: "wx\tools\Editra\src\extern\pygments\*.py";          DestDir: "{app}\%(P
 Source: "wx\tools\Editra\src\extern\pygments\filters\*.py";    DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\filters"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\formatters\*.py"; DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatters"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\lexers\*.py";     DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers"; Components: core
+Source: "wx\tools\Editra\src\extern\pygments\styles\*.py";     DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles"; Components: core
 Source: "wx\tools\Editra\src\syntax\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax"; Components: core
 Source: "wx\tools\Editra\src\syntax\README";                 DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax"; Components: core
 Source: "wx\tools\Editra\styles\*.ess";                      DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\styles"; Components: core
@@ -250,6 +256,8 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\aui\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\aui\*.pyo";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\persist\*.pyc";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\persist\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\ribbon\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\agw\ribbon\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\analogclock\*.pyc";
@@ -308,6 +316,8 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\*.pyo";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\aui\*.pyc";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\aui\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\*.pyc";      
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\*.pyo";      
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\filters\*.pyc"; 
@@ -316,6 +326,8 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatt
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatters\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers\*.pyc";    
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers\*.pyo";    
+Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles\*.pyc";    
+Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles\*.pyo";    
 
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\autocomp\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\autocomp\*.pyo";
@@ -349,19 +361,19 @@ begin
     (* -------------------------------------------------------------- *)
     (* Figure out what to use as a default installation dir           *)
 
-    if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
+    if not RegQueryStringValue(HKEY_CURRENT_USER,
                                'Software\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                '', PythonDir) then begin
 
-        if not RegQueryStringValue(HKEY_CURRENT_USER,
+        if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
                                    'Software\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                    '', PythonDir) then begin
 
-            if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
+            if not RegQueryStringValue(HKEY_CURRENT_USER,
                                        'Software\Wow6432Node\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                        '', PythonDir) then begin
 
-                if not RegQueryStringValue(HKEY_CURRENT_USER,
+                if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
                                            'Software\Wow6432Node\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                            '', PythonDir) then begin
 
@@ -451,7 +463,7 @@ ISS_DocDemo_Template = r'''
 AppName = wxPython%(SHORTVER)s-docs-demos
 AppVerName = wxPython Docs and Demos %(VERSION)s
 OutputBaseFilename = wxPython%(SHORTVER)s-win32-docs-demos-%(VERSION)s
-AppCopyright = Copyright 2009 Total Control Software
+AppCopyright = Copyright 2011 Total Control Software
 DefaultDirName = {pf}\wxPython%(SHORTVER)s Docs and Demos
 DefaultGroupName = wxPython%(SHORTVER)s Docs Demos and Tools
 PrivilegesRequired = none
@@ -494,6 +506,7 @@ Source: "demo\agw\*.py";                    DestDir: "{app}\demo\agw";
 Source: "demo\agw\bitmaps\*.png";           DestDir: "{app}\demo\agw\bitmaps";
 Source: "demo\agw\bitmaps\*.ico";           DestDir: "{app}\demo\agw\bitmaps";
 Source: "demo\agw\bitmaps\*.gif";           DestDir: "{app}\demo\agw\bitmaps";
+Source: "demo\agw\data\*.xls";              DestDir: "{app}\demo\agw\data";
 
 Source: "demo\bitmaps\*.bmp";               DestDir: "{app}\demo\bitmaps";
 Source: "demo\bitmaps\*.gif";               DestDir: "{app}\demo\bitmaps";
@@ -548,9 +561,9 @@ Source: "docs\screenshots\*.png";           DestDir: "{app}\docs\screenshots";
 
 Source: "samples\doodle\*.py";              DestDir: "{app}\samples\doodle";
 Source: "samples\doodle\*.txt";             DestDir: "{app}\samples\doodle";
-Source: "samples\doodle\*.bat";             DestDir: "{app}\samples\doodle";
+Source: "samples\doodle\*.ico";             DestDir: "{app}\samples\doodle";
+Source: "samples\doodle\*.icns";            DestDir: "{app}\samples\doodle";
 Source: "samples\doodle\sample.ddl";        DestDir: "{app}\samples\doodle";
-Source: "samples\doodle\superdoodle.iss";   DestDir: "{app}\samples\doodle";
 
 Source: "samples\docview\*.py";                DestDir: "{app}\samples\docview";
 Source: "samples\pydocview\*.py";              DestDir: "{app}\samples\pydocview";
@@ -579,6 +592,16 @@ Source: "samples\hangman\*.py";             DestDir: "{app}\samples\hangman";
 
 Source: "samples\mainloop\*.py";            DestDir: "{app}\samples\mainloop";
 
+Source: "samples\pubsub\*.bat";            DestDir: "{app}\samples\pubsub";
+Source: "samples\pubsub\advanced\*.txt";   DestDir: "{app}\samples\advanced";
+Source: "samples\pubsub\advanced\*.py";    DestDir: "{app}\samples\advanced";
+Source: "samples\pubsub\basic_arg1\*.txt";   DestDir: "{app}\samples\basic_arg1";
+Source: "samples\pubsub\basic_arg1\*.py";    DestDir: "{app}\samples\basic_arg1";
+Source: "samples\pubsub\basic_kwargs\*.txt";   DestDir: "{app}\samples\basic_kwargs";
+Source: "samples\pubsub\basic_kwargs\*.py";    DestDir: "{app}\samples\basic_kwargs";
+Source: "samples\pubsub\basic_v1\*.txt";   DestDir: "{app}\samples\basic_v1";
+Source: "samples\pubsub\basic_v1\*.py";    DestDir: "{app}\samples\basic_v1";
+
 Source: "samples\pySketch\*.py";            DestDir: "{app}\samples\pySketch";
 Source: "samples\pySketch\images\*.bmp";    DestDir: "{app}\samples\pySketch\images";
 
@@ -634,7 +657,7 @@ Source: "wx\tools\Editra\pixmaps\editra.ico";   DestDir: "{app}\scripts";
 [Icons]
 Name: "{group}\Run the wxPython DEMO"; Filename: "{app}\demo\demo.pyw";           WorkingDir: "{app}\demo";   IconFilename: "{app}\demo\wxpdemo.ico";
 Name: "{group}\PyCrust";               Filename: "{app}\scripts\pycrust.pyw";     WorkingDir: "c:\";          IconFilename: "{app}\scripts\PyCrust.ico";
-Name: "{group}\Pylices";               Filename: "{app}\scripts\pyslices.pyw";    WorkingDir: "c:\";          IconFilename: "{app}\scripts\PySlices.ico";
+Name: "{group}\PySlices";              Filename: "{app}\scripts\pyslices.pyw";    WorkingDir: "c:\";          IconFilename: "{app}\scripts\PySlices.ico";
 Name: "{group}\PyShell";               Filename: "{app}\scripts\pyshell.pyw";     WorkingDir: "c:\";          IconFilename: "{app}\scripts\PyCrust.ico";
 Name: "{group}\PySlicesShell";         Filename: "{app}\scripts\pysliceshell.pyw";WorkingDir: "c:\";          IconFilename: "{app}\scripts\PySlices.ico";
 Name: "{group}\XRC Resource Editor";   Filename: "{app}\scripts\xrced.pyw";       WorkingDir: "c:\";          IconFilename: "{app}\scripts\xrced.ico";
@@ -677,6 +700,14 @@ Type: files; Name: "{app}\samples\hangman\*.pyo";
 Type: files; Name: "{app}\samples\hangman\*.txt";
 Type: files; Name: "{app}\samples\mainloop\*.pyc";
 Type: files; Name: "{app}\samples\mainloop\*.pyo";
+Type: files; Name: "{app}\samples\pubsub\advanced\*.pyc";
+Type: files; Name: "{app}\samples\pubsub\advanced\*.pyo"; 
+Type: files; Name: "{app}\samples\pubsub\basic_arg1\*.pyc";
+Type: files; Name: "{app}\samples\pubsub\basic_arg1\*.pyo"; 
+Type: files; Name: "{app}\samples\pubsub\basic_kwargs\*.pyc";
+Type: files; Name: "{app}\samples\pubsub\basic_kwargs\*.pyo"; 
+Type: files; Name: "{app}\samples\pubsub\basic_v1\*.pyc";   
+Type: files; Name: "{app}\samples\pubsub\basic_v1\*.pyo";    
 Type: files; Name: "{app}\samples\pySketch\*.pyc";
 Type: files; Name: "{app}\samples\pySketch\*.pyo";
 Type: files; Name: "{app}\samples\simple\*.pyc";
@@ -734,10 +765,10 @@ Type: files; Name: "{app}\samples\wxPIA_book\Chapter-18\*";
 
 def find_DLLs():
 
-    if os.environ.get('CPU', '') == 'AMD64':
+    if os.environ.get('CPU', '') in ['AMD64', 'X64']:
         # Just hard-code it for now until a good solution for finding
         # the right dumpbin can be found...
-        return '291u', sys.version[:3]
+        return '295u', sys.version[:3]
         
     WXDLLVER = PYTHONVER = None
 
@@ -809,7 +840,7 @@ def get_runtime_dlls(PYVER, PKGDIR):
         # bother with installing it ourselves.
         return ('', '')
         
-    if os.environ.get('CPU', '') == 'AMD64':
+    if os.environ.get('CPU', '') in ['AMD64', 'X64']:
         if PYVER == 'py25':
             # For now just pull the DLLs from the system dir, and install
             # them there.  We may eventually want to get more customized
@@ -851,18 +882,18 @@ def main():
     LOCALE          = build_locale_string(PKGDIR)
     EDITRA_LOCALE   = build_editra_locale(PKGDIR)
     RTDLL,CPPDLL    = get_runtime_dlls(PYVER, PKGDIR)
+    CAIRO_ROOT      = os.environ["CAIRO_ROOT"]
 
-    if os.environ.get('CPU', '') == 'AMD64':
+    if os.environ.get('CPU', '') in ['AMD64', 'X64']:
         BITS        = '64'
-        VCDLLDIR    = 'vc_amd64_dll'
+        VCDLLDIR    = 'vc90_x64_dll'
         GDIPLUS     = ''
         ARCH        = 'ArchitecturesInstallIn64BitMode = x64\nArchitecturesAllowed = x64'
-        #ARCH        = ''
         PRIV        = 'admin'
 
     else:
         BITS        = '32'
-        VCDLLDIR    = 'vc_dll'
+        VCDLLDIR    = 'vc90_dll'
         GDIPLUS     = 'Source: "distrib\msw\gdiplus.dll"; DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: replacesameversion' % vars()
         ARCH        = ''
         PRIV        = 'none'
@@ -879,6 +910,7 @@ Building Win32 installer for wxPython:
     WXDIR      = %(WXDIR)s
     WXPYDIR    = %(WXPYDIR)s
     SYSDIR     = %(SYSDIR)s
+    CAIRO_ROOT = %(CAIRO_ROOT)s
     """ % vars()
 
     if PYTHONVER >= "2.2":
