@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Sept-2008
-# RCS-ID:      $Id: wxcairo.py 63525 2010-02-19 22:16:18Z RD $
+# RCS-ID:      $Id$
 # Copyright:   (c) 2008 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -36,7 +36,7 @@ normal 'python setup.py install' dance.
 
 On Windows you can get a Cairo DLL from here:
 
-    http://www.gtk.org/download-windows.html
+    http://www.gtk.org/download/win32.php
 
 You'll also want to get the zlib and libpng binaries from the same
 page.  Once you get those files extract the DLLs from each of the zip
@@ -68,7 +68,7 @@ pycairoAPI = None
 # a convenience funtion, just to save a bit of typing below
 def voidp(ptr):
     """Convert a SWIGged void* type to a ctypes c_void_p"""
-    return ctypes.c_void_p(int(ptr))
+    return ctypes.c_void_p(long(ptr))
 
 #----------------------------------------------------------------------------
 
@@ -85,15 +85,19 @@ def ContextFromDC(dc):
         width, height = dc.GetSize()
 
         # use the CGContextRef of the DC to make the cairo surface
-        cgc = dc.GetCGContext()
+        cgc = dc.GetHandle()
         assert cgc is not None, "Unable to get CGContext from DC."
         cgref = voidp( cgc )
-        surfaceptr = voidp(
-            cairoLib.cairo_quartz_surface_create_for_cg_context(
-                cgref, width, height) )
+        surface_create = cairoLib.cairo_quartz_surface_create_for_cg_context
+        surface_create.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
+        surface_create.restype = ctypes.c_void_p
+        surfaceptr = voidp(surface_create(cgref, width, height))
 
         # create a cairo context for that surface
-        ctxptr = cairoLib.cairo_create(surfaceptr)
+        cairo_create = cairoLib.cairo_create
+        cairo_create.argtypes = [ctypes.c_void_p]
+        cairo_create.restype = ctypes.c_void_p
+        ctxptr = voidp(cairo_create(surfaceptr))
 
         # Turn it into a pycairo context object
         ctx = pycairoAPI.Context_FromContext(ctxptr, pycairoAPI.Context_Type, None)
@@ -105,8 +109,10 @@ def ContextFromDC(dc):
     elif 'wxMSW' in wx.PlatformInfo:
         # This one is easy, just fetch the HDC and use PyCairo to make
         # the surface and context.
-        hdc = dc.GetHDC()
-        surface = cairo.Win32Surface(hdc)
+        hdc = dc.GetHandle()
+        # Ensure the pointer value is clampped into the range of a C signed long
+        hdc = ctypes.c_long(hdc)
+        surface = cairo.Win32Surface(hdc.value)
         ctx = cairo.Context(surface)
     
 
@@ -114,7 +120,7 @@ def ContextFromDC(dc):
         gdkLib = _findGDKLib()
 
         # Get the GdkDrawable from the dc
-        drawable = voidp( dc.GetGdkDrawable() )
+        drawable = voidp( dc.GetHandle() )
 
         # Call a GDK API to create a cairo context
         gdkLib.gdk_cairo_create.restype = ctypes.c_void_p
@@ -138,13 +144,16 @@ def FontFaceFromFont(font):
     """
     
     if 'wxMac' in wx.PlatformInfo:
-        fontfaceptr = voidp(
-            cairoLib.cairo_quartz_font_face_create_for_cgfont(
-                voidp(font.OSXGetCGFont())) )
+        font_face_create = cairoLib.cairo_quartz_font_face_create_for_cgfont
+        font_face_create.argtypes = [ctypes.c_void_p]
+        font_face_create.restype = ctypes.c_void_p
+        
+        fontfaceptr = font_face_create(voidp(font.OSXGetCGFont()))
         fontface = pycairoAPI.FontFace_FromFontFace(fontfaceptr)
 
 
     elif 'wxMSW' in wx.PlatformInfo:
+        cairoLib.cairo_win32_font_face_create_for_hfont.restype = ctypes.c_void_p
         fontfaceptr = voidp( cairoLib.cairo_win32_font_face_create_for_hfont(
             ctypes.c_ulong(font.GetHFONT())) )
         fontface = pycairoAPI.FontFace_FromFontFace(fontfaceptr)
@@ -227,6 +236,7 @@ def ImageSurfaceFromBitmap(bitmap):
     
     surface = cairo.ImageSurface(format, width, height)
     bitmap.CopyToBuffer(surface.get_data(), fmt, stride)
+    surface.mark_dirty()
     return surface
 
 
@@ -256,11 +266,12 @@ def _findCairoLib():
     # appropriate for the system
     for name in names:
         location = ctypes.util.find_library(name) 
-        try:
-            cairoLib = ctypes.CDLL(location)
-            return
-        except:
-            pass
+        if location:
+            try:
+                cairoLib = ctypes.CDLL(location)
+                return
+            except:
+                pass
         
     # If the above didn't find it on OS X then we still have a
     # trick up our sleeve...
@@ -360,7 +371,10 @@ class Pycairo_CAPI(ctypes.Structure):
                                                       ctypes.py_object)),
             ('Check_Status', ctypes.PYFUNCTYPE(ctypes.c_int, ctypes.c_int))]
 
-    elif cairo.version_info < (1,9):  # This structure is known good with pycairo 1.8.4
+    # This structure is known good with pycairo 1.8.4.
+    # We have to also test for (1,10,8) because pycairo 1.8.10 has an
+    # incorrect version_info value
+    elif cairo.version_info < (1,9) or cairo.version_info == (1,10,8):  
         _fields_ = [
             ('Context_Type', ctypes.py_object),
             ('Context_FromContext', ctypes.PYFUNCTYPE(ctypes.py_object,
@@ -398,6 +412,48 @@ class Pycairo_CAPI(ctypes.Structure):
                                                       ctypes.py_object)),
             ('Check_Status', ctypes.PYFUNCTYPE(ctypes.c_int, ctypes.c_int))]
 
+    # This structure is known good with pycairo 1.10.0. They keep adding stuff
+    # to the middle of the structure instead of only adding to the end!
+    elif cairo.version_info < (1,11):  
+        _fields_ = [
+            ('Context_Type', ctypes.py_object),
+            ('Context_FromContext', ctypes.PYFUNCTYPE(ctypes.py_object,
+                                                      ctypes.c_void_p,
+                                                      ctypes.py_object,
+                                                      ctypes.py_object)),
+            ('FontFace_Type', ctypes.py_object),
+            ('ToyFontFace_Type', ctypes.py_object),  
+            ('FontFace_FromFontFace', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p)),
+            ('FontOptions_Type', ctypes.py_object),
+            ('FontOptions_FromFontOptions', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p)),
+            ('Matrix_Type', ctypes.py_object),
+            ('Matrix_FromMatrix', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p)),
+            ('Path_Type', ctypes.py_object),
+            ('Path_FromPath', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p)),
+            ('Pattern_Type', ctypes.py_object),
+            ('SolidPattern_Type', ctypes.py_object),
+            ('SurfacePattern_Type', ctypes.py_object),
+            ('Gradient_Type', ctypes.py_object),
+            ('LinearGradient_Type', ctypes.py_object),
+            ('RadialGradient_Type', ctypes.py_object),
+            ('Pattern_FromPattern', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p,
+                                                      ctypes.py_object)), #** changed in 1.8.4
+            ('ScaledFont_Type', ctypes.py_object),
+            ('ScaledFont_FromScaledFont', ctypes.PYFUNCTYPE(ctypes.py_object, ctypes.c_void_p)),
+            ('Surface_Type', ctypes.py_object),
+            ('ImageSurface_Type', ctypes.py_object),
+            ('PDFSurface_Type', ctypes.py_object),
+            ('PSSurface_Type', ctypes.py_object),
+            ('SVGSurface_Type', ctypes.py_object),
+            ('Win32Surface_Type', ctypes.py_object),
+            ('Win32PrintingSurface_Type', ctypes.py_object),  #** new
+            ('XCBSurface_Type', ctypes.py_object),            #** new
+            ('XlibSurface_Type', ctypes.py_object),
+            ('Surface_FromSurface', ctypes.PYFUNCTYPE(ctypes.py_object,
+                                                      ctypes.c_void_p,
+                                                      ctypes.py_object)),
+            ('Check_Status', ctypes.PYFUNCTYPE(ctypes.c_int, ctypes.c_int))]
+
 
 def _loadPycairoAPI():
     global pycairoAPI
