@@ -33,13 +33,17 @@ PB_STYLE_SQUARE:
 Instead of the default rounded shape use a rectangular shaped button with
 square edges.
 
-PB_STYLE_NB:
+PB_STYLE_NOBG:
 This style only has an effect on Windows but does not cause harm to use on the
 platforms. It should only be used when the control is shown on a panel or other
 window that has a non solid color for a background. i.e a gradient or image is
 painted on the background of the parent window. If used on a background with
 a solid color it may cause the control to loose its transparent appearance.
 
+PB_STYLE_DROPARROW:
+Add a drop button arrow to the button that will send a separate event when
+clicked on.
+
 Other attributes can be configured after the control has been created. The
 settings that are currently available are as follows:
 
@@ -69,21 +73,24 @@ Requirements:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: platebtn.py 57713 2009-01-01 23:36:15Z CJP $"
-__revision__ = "$Revision: 57713 $"
+__svnid__ = "$Id: platebtn.py 74715 2013-08-27 15:33:47Z CJP $"
+__revision__ = "$Revision: 74715 $"
 
-__all__ = ["PlateButton", "AdjustAlpha", "AdjustColor", "GetHighlightColor",
+__all__ = ["PlateButton",
            "PLATE_NORMAL", "PLATE_PRESSED", "PLATE_HIGHLIGHT", 
+
            "PB_STYLE_DEFAULT", "PB_STYLE_GRADIENT", "PB_STYLE_SQUARE",
-           "PB_STYLE_NOBG" ]
+           "PB_STYLE_NOBG", "PB_STYLE_DROPARROW", "PB_STYLE_TOGGLE",
+
+           "EVT_PLATEBTN_DROPARROW_PRESSED"]
 
 #-----------------------------------------------------------------------------#
 # Imports
 import wx
+import wx.lib.newevent
 
-# Used on OSX to get access to carbon api constants
-if wx.Platform == '__WXMAC__':
-    import Carbon.Appearance
+# Local Imports
+from eclutil import *
 
 #-----------------------------------------------------------------------------#
 # Button States
@@ -97,65 +104,14 @@ PB_STYLE_GRADIENT = 2   # Gradient Filled Background
 PB_STYLE_SQUARE   = 4   # Use square corners instead of rounded
 PB_STYLE_NOBG     = 8   # Usefull on Windows to get a transparent appearance
                         # when the control is shown on a non solid background
+PB_STYLE_DROPARROW = 16 # Draw drop arrow and fire EVT_PLATEBTN_DROPRROW_PRESSED event
+PB_STYLE_TOGGLE   = 32  # Stay pressed untill clicked again
 
 #-----------------------------------------------------------------------------#
-# Utility Functions
-def AdjustAlpha(colour, alpha):
-    """Adjust the alpha of a given colour"""
-    return wx.Colour(colour.Red(), colour.Green(), colour.Blue(), alpha)
 
-def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
-    """ Brighten/Darken input colour by percent and adjust alpha
-    channel if needed. Returns the modified color.
-    @param color: color object to adjust
-    @type color: wx.Color
-    @param percent: percent to adjust +(brighten) or -(darken)
-    @type percent: int
-    @keyword alpha: amount to adjust alpha channel
-
-    """
-    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
-                         for val in color.Get() ]
-
-    if percent < 0:
-        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
-    else:
-        radj, gadj, badj = [ val or 255 for val in [radj, gadj, badj] ]
-
-    red = min(color.Red() + radj, 255)
-    green = min(color.Green() + gadj, 255)
-    blue = min(color.Blue() + badj, 255)
-    return wx.Colour(red, green, blue, alpha)
-
-def BestLabelColour(color):
-    """Get the best color to use for the label that will be drawn on
-    top of the given color.
-    @param color: background color that text will be drawn on
-
-    """
-    avg = sum(color.Get()) / 3
-    if avg > 192:
-        txt_color = wx.BLACK
-    elif avg > 128:
-        txt_color = AdjustColour(color, -95)
-    elif avg < 64:
-        txt_color = wx.WHITE
-    else:
-        txt_color = AdjustColour(color, 95)
-    return txt_color
-
-def GetHighlightColour():
-    """Get the default highlight color
-    @return: wx.Color
-
-    """
-    if wx.Platform == '__WXMAC__':
-        brush = wx.Brush(wx.BLACK)
-        # kThemeBrushButtonPressedLightHighlight
-        brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
-        return brush.GetColour()
-    else:
-        return wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT)
+# EVT_BUTTON used for normal event notification
+# EVT_TOGGLE_BUTTON used for toggle button mode notification
+PlateBtnDropArrowPressed, EVT_PLATEBTN_DROPARROW_PRESSED = wx.lib.newevent.NewEvent()
 
 #-----------------------------------------------------------------------------#
 
@@ -164,7 +120,7 @@ class PlateButton(wx.PyControl):
     displaying bitmaps and having an attached dropdown menu.
 
     """
-    def __init__(self, parent, id_=wx.ID_ANY, label='', bmp=None, 
+    def __init__(self, parent, id=wx.ID_ANY, label='', bmp=None, 
                  pos=wx.DefaultPosition, size=wx.DefaultSize,
                  style=PB_STYLE_DEFAULT, name=wx.ButtonNameStr):
         """Create a PlateButton
@@ -173,27 +129,29 @@ class PlateButton(wx.PyControl):
         @keyword style: Button style
 
         """
-        wx.PyControl.__init__(self, parent, id_, pos, size,
-                              wx.BORDER_NONE|wx.TRANSPARENT_WINDOW, name=name)
+        super(PlateButton, self).__init__(parent, id, pos, size,
+                                          wx.BORDER_NONE|wx.TRANSPARENT_WINDOW,
+                                          name=name)
 
         # Attributes
         self.InheritAttributes()
-        self._bmp = dict(enable=bmp)
+        self._bmp = dict(enable=None, disable=None)
         if bmp is not None:
+            assert isinstance(bmp, wx.Bitmap) and bmp.IsOk()
+            self._bmp['enable'] = bmp
             img = bmp.ConvertToImage()
             img = img.ConvertToGreyscale(.795, .073, .026) #(.634, .224, .143)
-            self._bmp['disable'] = img.ConvertToBitmap()
-        else:
-            self._bmp['disable'] = None
+            self._bmp['disable'] = wx.BitmapFromImage(img)
 
         self._menu = None
         self.SetLabel(label)
         self._style = style
         self._state = dict(pre=PLATE_NORMAL, cur=PLATE_NORMAL)
         self._color = self.__InitColors()
+        self._pressed = False
 
         # Setup Initial Size
-        self.SetInitialSize()
+        self.SetInitialSize(size)
 
         # Event Handlers
         self.Bind(wx.EVT_PAINT, lambda evt: self.__DrawButton())
@@ -202,13 +160,13 @@ class PlateButton(wx.PyControl):
         self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
 
         # Mouse Events
+        self.Bind(wx.EVT_LEFT_DCLICK, lambda evt: self._ToggleState())
         self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
         self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
-        self.Bind(wx.EVT_LEFT_DCLICK, lambda evt: self.ToggleState())
         self.Bind(wx.EVT_ENTER_WINDOW,
-                  lambda evt: self.SetState(PLATE_HIGHLIGHT))
+                  lambda evt: self._SetState(PLATE_HIGHLIGHT))
         self.Bind(wx.EVT_LEAVE_WINDOW,
-                  lambda evt: wx.CallLater(80, self.SetState, PLATE_NORMAL))
+                  lambda evt: wx.CallLater(80, self.__LeaveWindow))
 
         # Other events
         self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
@@ -227,7 +185,7 @@ class PlateButton(wx.PyControl):
 
         if bmp is not None and bmp.IsOk():
             bw, bh = bmp.GetSize()
-            ypos = (self.GetSize()[1] - bh) / 2
+            ypos = (self.GetSize()[1] - bh) // 2
             gc.DrawBitmap(bmp, 6, ypos, bmp.GetMask() != None)
             return bw + 6
         else:
@@ -240,7 +198,7 @@ class PlateButton(wx.PyControl):
         @param ypos: y cord to start at
 
         """
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             # Positioning needs a little help on Windows
             if wx.Platform == '__WXMSW__':
                 xpos -= 2
@@ -285,27 +243,25 @@ class PlateButton(wx.PyControl):
 
     def __PostEvent(self):
         """Post a button event to parent of this control"""
-        bevt = wx.CommandEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, self.GetId())
+        if self._style & PB_STYLE_TOGGLE:
+            etype = wx.wxEVT_COMMAND_TOGGLEBUTTON_CLICKED
+        else:
+            etype = wx.wxEVT_COMMAND_BUTTON_CLICKED
+        bevt = wx.CommandEvent(etype, self.GetId())
         bevt.SetEventObject(self)
         bevt.SetString(self.GetLabel())
-        wx.PostEvent(self.GetParent(), bevt)
+        self.GetEventHandler().ProcessEvent(bevt)
 
     def __DrawButton(self):
         """Draw the button"""
-        # TODO using a buffered paintdc on windows with the nobg style
-        #      causes lots of weird drawing. So currently the use of a
-        #      buffered dc is dissabled for this style.
-        if PB_STYLE_NOBG & self._style:
-            dc = wx.PaintDC(self)
-        else:
-            dc = wx.AutoBufferedPaintDCFactory(self)
-
+        dc = wx.PaintDC(self)
         gc = wx.GCDC(dc)
 
         # Setup
         dc.SetBrush(wx.TRANSPARENT_BRUSH)
         gc.SetBrush(wx.TRANSPARENT_BRUSH)
-        gc.SetFont(self.GetFont())
+        gc.SetFont(self.Font)
+        dc.SetFont(self.Font)
         gc.SetBackgroundMode(wx.TRANSPARENT)
 
         # The background needs some help to look transparent on
@@ -316,8 +272,11 @@ class PlateButton(wx.PyControl):
 
         # Calc Object Positions
         width, height = self.GetSize()
-        tw, th = gc.GetTextExtent(self.GetLabel())
-        txt_y = max((height - th) / 2, 1)
+        if wx.Platform == '__WXGTK__':
+            tw, th = dc.GetTextExtent(self.Label)
+        else:
+            tw, th = gc.GetTextExtent(self.Label)
+        txt_y = max((height - th) // 2, 1)
 
         if self._state['cur'] == PLATE_HIGHLIGHT:
             gc.SetTextForeground(self._color['htxt'])
@@ -327,17 +286,19 @@ class PlateButton(wx.PyControl):
         elif self._state['cur'] == PLATE_PRESSED:
             gc.SetTextForeground(self._color['htxt'])
             if wx.Platform == '__WXMAC__':
-                brush = wx.Brush(wx.BLACK)
-                brush.MacSetTheme(Carbon.Appearance.kThemeBrushFocusHighlight)
-                pen = wx.Pen(brush.GetColour(), 1, wx.SOLID)
+                pen = wx.Pen(GetHighlightColour(), 1, wx.SOLID)
             else:
                 pen = wx.Pen(AdjustColour(self._color['press'], -80, 220), 1)
             gc.SetPen(pen)
 
             self.__DrawHighlight(gc, width, height)
             txt_x = self.__DrawBitmap(gc)
-            gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
-            self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
+            t_x = max((width - tw - (txt_x + 2)) // 2, txt_x + 2)
+            if wx.Platform == '__WXGTK__':
+                dc.DrawText(self.Label, t_x, txt_y)
+            else:
+                gc.DrawText(self.Label, t_x, txt_y)
+            self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)
 
         else:
             if self.IsEnabled():
@@ -349,8 +310,12 @@ class PlateButton(wx.PyControl):
         # Draw bitmap and text
         if self._state['cur'] != PLATE_PRESSED:
             txt_x = self.__DrawBitmap(gc)
-            gc.DrawText(self.GetLabel(), txt_x + 2, txt_y)
-            self.__DrawDropArrow(gc, txt_x + tw + 6, (height / 2) - 2)
+            t_x = max((width - tw - (txt_x + 2)) // 2, txt_x + 2)
+            if wx.Platform == '__WXGTK__':
+                dc.DrawText(self.Label, t_x, txt_y)
+            else:
+                gc.DrawText(self.Label, t_x, txt_y)
+            self.__DrawDropArrow(gc, width - 10, (height // 2) - 2)
 
     def __InitColors(self):
         """Initialize the default colors"""
@@ -362,31 +327,62 @@ class PlateButton(wx.PyControl):
                       htxt=BestLabelColour(self.GetForegroundColour()))
         return colors
 
+    def __LeaveWindow(self):
+        """Handle updating the buttons state when the mouse cursor leaves"""
+        if (self._style & PB_STYLE_TOGGLE) and self._pressed:
+            self._SetState(PLATE_PRESSED) 
+        else:
+            self._SetState(PLATE_NORMAL)
+            self._pressed = False
+
+    def _SetState(self, state):
+        """Manually set the state of the button
+        @param state: one of the PLATE_* values
+        @note: the state may be altered by mouse actions
+        @note: Internal use only!
+
+        """
+        self._state['pre'] = self._state['cur']
+        self._state['cur'] = state
+        if wx.Platform == '__WXMSW__':
+            self.Parent.RefreshRect(self.Rect, False)
+        else:
+            self.Refresh()
+
+    def _ToggleState(self):
+        """Toggle button state
+        @note: Internal Use Only!
+
+        """
+        if self._state['cur'] != PLATE_PRESSED:
+            self._SetState(PLATE_PRESSED)
+        else:
+            self._SetState(PLATE_HIGHLIGHT)
+
     #---- End Private Member Function ----#
 
     #---- Public Member Functions ----#
-    def AcceptsFocus(self):
-        """Can this window have the focus?"""
-        return self.IsEnabled()
 
-    @property
-    def BitmapDisabled(self):
-        """Property for accessing the bitmap for the disabled state"""
-        return self._bmp['disable']
-
-    @property
-    def BitmapLabel(self):
-        """Property for accessing the default bitmap"""
-        return self._bmp['enable']
+    BitmapDisabled = property(lambda self: self.GetBitmapDisabled(),
+                              lambda self, bmp: self.SetBitmapDisabled(bmp))
+    BitmapLabel = property(lambda self: self.GetBitmapLabel(),
+                           lambda self, bmp: self.SetBitmap(bmp))
 
     # Aliases
     BitmapFocus = BitmapLabel
     BitmapHover = BitmapLabel
     BitmapSelected = BitmapLabel
 
+    LabelText = property(lambda self: self.GetLabel(),
+                         lambda self, lbl: self.SetLabel(lbl))
+
+    def AcceptsFocus(self):
+        """Can this window have the focus?"""
+        return self.IsEnabled()
+
     def Disable(self):
         """Disable the control"""
-        wx.PyControl.Disable(self)
+        super(PlateButton, self).Disable()
         self.Refresh()
 
     def DoGetBestSize(self):
@@ -396,13 +392,16 @@ class PlateButton(wx.PyControl):
         """
         width = 4
         height = 6
-        if self.GetLabel():
-            lsize = self.GetTextExtent(self.GetLabel())
+        if self.Label:
+            # NOTE: Should measure with a GraphicsContext to get right
+            #       size, but due to random segfaults on linux special
+            #       handling is done in the drawing instead...
+            lsize = self.GetFullTextExtent(self.Label)
             width += lsize[0]
             height += lsize[1]
             
         if self._bmp['enable'] is not None:
-            bsize = self._bmp['enable'].GetSize()
+            bsize = self._bmp['enable'].Size
             width += (bsize[0] + 10)
             if height <= bsize[1]:
                 height = bsize[1] + 6
@@ -411,7 +410,7 @@ class PlateButton(wx.PyControl):
         else:
             width += 10
 
-        if self._menu is not None:
+        if self._menu is not None or self._style & PB_STYLE_DROPARROW:
             width += 12
 
         best = wx.Size(width, height)
@@ -420,7 +419,7 @@ class PlateButton(wx.PyControl):
 
     def Enable(self, enable=True):
         """Enable/Disable the control"""
-        wx.PyControl.Enable(self, enable)
+        super(PlateButton, self).Enable(enable)
         self.Refresh()
 
     def GetBackgroundBrush(self, dc):
@@ -435,11 +434,11 @@ class PlateButton(wx.PyControl):
         bkgrd = self.GetBackgroundColour()
         brush = wx.Brush(bkgrd, wx.SOLID)
         my_attr = self.GetDefaultAttributes()
-        p_attr = self.GetParent().GetDefaultAttributes()
+        p_attr = self.Parent.GetDefaultAttributes()
         my_def = bkgrd == my_attr.colBg
-        p_def = self.GetParent().GetBackgroundColour() == p_attr.colBg
+        p_def = self.Parent.GetBackgroundColour() == p_attr.colBg
         if my_def and not p_def:
-            bkgrd = self.GetParent().GetBackgroundColour()
+            bkgrd = self.Parent.GetBackgroundColour()
             brush = wx.Brush(bkgrd, wx.SOLID)
         return brush
 
@@ -448,14 +447,14 @@ class PlateButton(wx.PyControl):
         @return: wx.Bitmap or None
 
         """
-        return self._bmp['disable']
+        return self.BitmapDisabled
 
     def GetBitmapLabel(self):
         """Get the label bitmap
         @return: wx.Bitmap or None
 
         """
-        return self._bmp['enable']
+        return self.BitmapLabel
 
     # GetBitmap Aliases for BitmapButton api
     GetBitmapFocus = GetBitmapLabel
@@ -469,16 +468,26 @@ class PlateButton(wx.PyControl):
         menu is associated with it.
 
         """
-        return getattr(self, '_menu', None)
+        return self._menu
+
+    def GetState(self):
+        """Get the current state of the button
+        @return: int
+        @see: PLATE_NORMAL, PLATE_HIGHLIGHT, PLATE_PRESSED
+
+        """
+        return self._state['cur']
 
     def HasTransparentBackground(self):
         """Override setting of background fill"""
         return True
 
-    @property
-    def LabelText(self):
-        """Property for getting the label of the button"""
-        return self.GetLabel()
+    def IsPressed(self):
+        """Return if button is pressed (PB_STYLE_TOGGLE)
+        @return: bool
+
+        """
+        return self._pressed
 
     #---- Event Handlers ----#
 
@@ -488,12 +497,13 @@ class PlateButton(wx.PyControl):
         @param evt: wx.EVT_ERASE_BACKGROUND
 
         """
-        pass
+        if not (PB_STYLE_NOBG & self._style):
+            evt.Skip()
 
     def OnFocus(self, evt):
         """Set the visual focus state if need be"""
         if self._state['cur'] == PLATE_NORMAL:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
 
     def OnKeyUp(self, evt):
         """Execute a single button press action when the Return key is pressed
@@ -502,9 +512,9 @@ class PlateButton(wx.PyControl):
 
         """
         if evt.GetKeyCode() == wx.WXK_SPACE:
-            self.SetState(PLATE_PRESSED)
+            self._SetState(PLATE_PRESSED)
             self.__PostEvent()
-            wx.CallLater(100, self.SetState, PLATE_HIGHLIGHT)
+            wx.CallLater(100, self._SetState, PLATE_HIGHLIGHT)
         else:
             evt.Skip()
 
@@ -517,20 +527,29 @@ class PlateButton(wx.PyControl):
         #       handler to prevent ghost highlighting from happening when
         #       quickly changing focus and activating buttons
         if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_NORMAL)
+            self._SetState(PLATE_NORMAL)
 
     def OnLeftDown(self, evt):
         """Sets the pressed state and depending on the click position will
         show the popup menu if one has been set.
 
         """
+        if (self._style & PB_STYLE_TOGGLE):
+            self._pressed = not self._pressed
+
         pos = evt.GetPositionTuple()
-        self.SetState(PLATE_PRESSED)
+        self._SetState(PLATE_PRESSED)
         size = self.GetSizeTuple()
-        if self._menu is not None and pos[0] >= size[0] - 16:
-            self.ShowMenu()
+        if pos[0] >= size[0] - 16:
+            if self._menu is not None:
+                self.ShowMenu()
+            elif self._style & PB_STYLE_DROPARROW:
+                event = PlateBtnDropArrowPressed()
+                event.SetEventObject(self)
+                self.EventHandler.ProcessEvent(event)
+        
         self.SetFocus()
-            
+
     def OnLeftUp(self, evt):
         """Post a button event if the control was previously in a
         pressed state.
@@ -538,8 +557,15 @@ class PlateButton(wx.PyControl):
 
         """
         if self._state['cur'] == PLATE_PRESSED:
-            self.__PostEvent()
-        self.SetState(PLATE_HIGHLIGHT)
+            pos = evt.GetPositionTuple()
+            size = self.GetSizeTuple()
+            if not (self._style & PB_STYLE_DROPARROW and pos[0] >= size[0] - 16):
+                self.__PostEvent()
+
+        if self._pressed:
+            self._SetState(PLATE_PRESSED)
+        else:
+            self._SetState(PLATE_HIGHLIGHT)
 
     def OnMenuClose(self, evt):
         """Refresh the control to a proper state after the menu has been
@@ -549,9 +575,9 @@ class PlateButton(wx.PyControl):
         """
         mpos = wx.GetMousePosition()
         if self.HitTest(self.ScreenToClient(mpos)) != wx.HT_WINDOW_OUTSIDE:
-            self.SetState(PLATE_HIGHLIGHT)
+            self._SetState(PLATE_HIGHLIGHT)
         else:
-            self.SetState(PLATE_NORMAL)
+            self._SetState(PLATE_NORMAL)
         evt.Skip()
 
     #---- End Event Handlers ----#
@@ -583,12 +609,12 @@ class PlateButton(wx.PyControl):
     def SetFocus(self):
         """Set this control to have the focus"""
         if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_HIGHLIGHT)
-        wx.PyControl.SetFocus(self)
+            self._SetState(PLATE_HIGHLIGHT)
+        super(PlateButton, self).SetFocus()
 
     def SetFont(self, font):
         """Adjust size of control when font changes"""
-        wx.PyControl.SetFont(self, font)
+        super(PlateButton, self).SetFont(font)
         self.InvalidateBestSize()
 
     def SetLabel(self, label):
@@ -596,23 +622,24 @@ class PlateButton(wx.PyControl):
         @param label: lable string
 
         """
-        wx.PyControl.SetLabel(self, label)
+        super(PlateButton, self).SetLabel(label)
         self.InvalidateBestSize()
 
-    def SetLabelColor(self, normal, hlight=wx.NullColor):
+    def SetLabelColor(self, normal, hlight=wx.NullColour):
         """Set the color of the label. The optimal label color is usually
         automatically selected depending on the button color. In some
-        cases the colors that are choosen may not be optimal.
+        cases the colors that are chosen may not be optimal.
         
         The normal state must be specified, if the other two params are left
         Null they will be automatically guessed based on the normal color. To
         prevent this automatic color choices from happening either specify
         a color or None for the other params.
 
-        @param normal: Label color for normal state
+        @param normal: Label color for normal state (wx.Colour)
         @keyword hlight: Color for when mouse is hovering over
 
         """
+        assert isinstance(normal, wx.Colour), "Must supply a colour object"
         self._color['default'] = False
         self.SetForegroundColour(normal)
 
@@ -623,7 +650,7 @@ class PlateButton(wx.PyControl):
                 self._color['htxt'] = BestLabelColour(normal)
 
         if wx.Platform == '__WXMSW__':
-            self.GetParent().RefreshRect(self.GetRect(), False)
+            self.Parent.RefreshRect(self.GetRect(), False)
         else:
             self.Refresh()
 
@@ -643,7 +670,7 @@ class PlateButton(wx.PyControl):
 
     def SetPressColor(self, color):
         """Set the color used for highlighting the pressed state
-        @param color: wx.Color
+        @param color: wx.Colour
         @note: also resets all text colours as necessary
 
         """
@@ -656,19 +683,6 @@ class PlateButton(wx.PyControl):
         self._color['htxt'] = BestLabelColour(self._color['hlight'])
         self.Refresh()
 
-    def SetState(self, state):
-        """Manually set the state of the button
-        @param state: one of the PLATE_* values
-        @note: the state may be altered by mouse actions
-
-        """
-        self._state['pre'] = self._state['cur']
-        self._state['cur'] = state
-        if wx.Platform == '__WXMSW__':
-            self.GetParent().RefreshRect(self.GetRect(), False)
-        else:
-            self.Refresh()
-
     def SetWindowStyle(self, style):
         """Sets the window style bytes, the updates take place
         immediately no need to call refresh afterwards.
@@ -680,7 +694,7 @@ class PlateButton(wx.PyControl):
 
     def SetWindowVariant(self, variant):
         """Set the variant/font size of this control"""
-        wx.PyControl.SetWindowVariant(self, variant)
+        super(PlateButton, self).SetWindowVariant(variant)
         self.InvalidateBestSize()
 
     def ShouldInheritColours(self):
@@ -703,11 +717,4 @@ class PlateButton(wx.PyControl):
 
             self.PopupMenu(self._menu, (xpos, size[1] + adj))
 
-    def ToggleState(self):
-        """Toggle button state"""
-        if self._state['cur'] != PLATE_PRESSED:
-            self.SetState(PLATE_PRESSED)
-        else:
-            self.SetState(PLATE_HIGHLIGHT)
-
     #---- End Public Member Functions ----#
